@@ -5,7 +5,7 @@
 
 /* Eve board-specific configuration */
 
-#include "adc_chip.h"
+#include "acpi.h"
 #include "bd99992gw.h"
 #include "board_config.h"
 #include "button.h"
@@ -15,9 +15,10 @@
 #include "charger.h"
 #include "chipset.h"
 #include "console.h"
+#include "device_event.h"
 #include "driver/accel_kionix.h"
 #include "driver/accel_kxcj9.h"
-#include "driver/accelgyro_bmi160.h"
+#include "driver/accelgyro_bmi_common.h"
 #include "driver/als_si114x.h"
 #include "driver/charger/bd9995x.h"
 #include "driver/tcpm/anx74xx.h"
@@ -61,17 +62,20 @@
 
 static void tcpc_alert_event(enum gpio_signal signal)
 {
-	if ((signal == GPIO_USB_C0_PD_INT_ODL) &&
-	    !gpio_get_level(GPIO_USB_C0_PD_RST_L))
-		return;
-	else if ((signal == GPIO_USB_C1_PD_INT_ODL) &&
-		 !gpio_get_level(GPIO_USB_C1_PD_RST_L))
-		return;
+	int port = -1;
 
-#ifdef HAS_TASK_PDCMD
-	/* Exchange status with TCPCs */
-	host_command_pd_send_status(PD_CHARGE_NO_CHANGE);
-#endif
+	switch (signal) {
+	case GPIO_USB_C0_PD_INT_ODL:
+		port = 0;
+		break;
+	case GPIO_USB_C1_PD_INT_ODL:
+		port = 1;
+		break;
+	default:
+		return;
+	}
+
+	schedule_deferred_pd_interrupt(port);
 }
 
 /*
@@ -90,8 +94,13 @@ void tablet_mode_interrupt(enum gpio_signal signal)
 /* Send event to wake AP based on trackpad input */
 void trackpad_interrupt(enum gpio_signal signal)
 {
-	/* TODO(b/36024430): Use device specific wake event */
-	host_set_single_event(EC_HOST_EVENT_KEY_PRESSED);
+	device_set_single_event(EC_DEVICE_EVENT_TRACKPAD);
+}
+
+/* Send event to wake AP based on DSP interrupt */
+void dsp_interrupt(enum gpio_signal signal)
+{
+	device_set_single_event(EC_DEVICE_EVENT_DSP);
 }
 
 #ifdef CONFIG_USB_PD_TCPC_LOW_POWER
@@ -110,7 +119,7 @@ static void anx74xx_c0_cable_det_handler(void)
 	 * and if in normal mode, then there is no need to trigger a tcpc reset.
 	 */
 	if (cable_det && !reset_n)
-		task_set_event(TASK_ID_PD_C0, PD_EVENT_TCPC_RESET, 0);
+		task_set_event(TASK_ID_PD_C0, PD_EVENT_TCPC_RESET);
 }
 DECLARE_DEFERRED(anx74xx_c0_cable_det_handler);
 
@@ -129,7 +138,7 @@ static void anx74xx_c1_cable_det_handler(void)
 	 * and if in normal mode, then there is no need to trigger a tcpc reset.
 	 */
 	if (cable_det && !reset_n)
-		task_set_event(TASK_ID_PD_C1, PD_EVENT_TCPC_RESET, 0);
+		task_set_event(TASK_ID_PD_C1, PD_EVENT_TCPC_RESET);
 }
 DECLARE_DEFERRED(anx74xx_c1_cable_det_handler);
 
@@ -147,19 +156,8 @@ void anx74xx_cable_det_interrupt(enum gpio_signal signal)
 
 #include "gpio_list.h"
 
-/* power signal list.  Must match order of enum power_signal. */
-const struct power_signal_info power_signal_list[] = {
-	{GPIO_PCH_SLP_S0_L,	1, "SLP_S0_DEASSERTED"},
-	{VW_SLP_S3_L,		1, "SLP_S3_DEASSERTED"},
-	{VW_SLP_S4_L,		1, "SLP_S4_DEASSERTED"},
-	{GPIO_PCH_SLP_SUS_L,	1, "SLP_SUS_DEASSERTED"},
-	{GPIO_RSMRST_L_PGOOD,	1, "RSMRST_L_PGOOD"},
-	{GPIO_PMIC_DPWROK,	1, "PMIC_DPWROK"},
-};
-BUILD_ASSERT(ARRAY_SIZE(power_signal_list) == POWER_SIGNAL_COUNT);
-
 /* Keyboard scan. Increase output_settle_us to 80us from default 50us. */
-struct keyboard_scan_config keyscan_config = {
+__override struct keyboard_scan_config keyscan_config = {
 	.output_settle_us = 80,
 	.debounce_down_us = 9 * MSEC,
 	.debounce_up_us = 30 * MSEC,
@@ -194,57 +192,117 @@ const int hibernate_wake_pins_used = ARRAY_SIZE(hibernate_wake_pins);
 
 /* I2C port map */
 const struct i2c_port_t i2c_ports[]  = {
-	{"tcpc0",     I2C_PORT_TCPC0,    400, GPIO_I2C0_0_SCL, GPIO_I2C0_0_SDA},
-	{"tcpc1",     I2C_PORT_TCPC1,    400, GPIO_I2C0_1_SCL, GPIO_I2C0_1_SDA},
-	{"accelgyro", I2C_PORT_GYRO,     400, GPIO_I2C1_SCL,   GPIO_I2C1_SDA},
-	{"sensors",   I2C_PORT_LID_ACCEL, 400, GPIO_I2C2_SCL,  GPIO_I2C2_SDA},
-	{"batt",      I2C_PORT_BATTERY,  100, GPIO_I2C3_SCL,   GPIO_I2C3_SDA},
+	{
+		.name = "tcpc0",
+		.port = I2C_PORT_TCPC0,
+		.kbps = 400,
+		.scl  = GPIO_I2C0_0_SCL,
+		.sda  = GPIO_I2C0_0_SDA
+	},
+	{
+		.name = "tcpc1",
+		.port = I2C_PORT_TCPC1,
+		.kbps = 400,
+		.scl  = GPIO_I2C0_1_SCL,
+		.sda  = GPIO_I2C0_1_SDA
+	},
+	{
+		.name = "accelgyro",
+		.port = I2C_PORT_GYRO,
+		.kbps = 400,
+		.scl  = GPIO_I2C1_SCL,
+		.sda  = GPIO_I2C1_SDA
+	},
+	{
+		.name = "sensors",
+		.port = I2C_PORT_LID_ACCEL,
+		.kbps = 400,
+		.scl  = GPIO_I2C2_SCL,
+		.sda  = GPIO_I2C2_SDA
+	},
+	{
+		.name = "batt",
+		.port = I2C_PORT_BATTERY,
+		.kbps = 100,
+		.scl  = GPIO_I2C3_SCL,
+		.sda  = GPIO_I2C3_SDA
+	},
 };
 const unsigned int i2c_ports_used = ARRAY_SIZE(i2c_ports);
 
 /* TCPC mux configuration */
-const struct tcpc_config_t tcpc_config[CONFIG_USB_PD_PORT_COUNT] = {
-	{I2C_PORT_TCPC0, 0x50, &anx74xx_tcpm_drv, TCPC_ALERT_ACTIVE_LOW},
-	{I2C_PORT_TCPC1, 0x50, &anx74xx_tcpm_drv, TCPC_ALERT_ACTIVE_LOW},
+const struct tcpc_config_t tcpc_config[CONFIG_USB_PD_PORT_MAX_COUNT] = {
+	{
+		.bus_type = EC_BUS_TYPE_I2C,
+		.i2c_info = {
+			.port = I2C_PORT_TCPC0,
+			.addr_flags = ANX74XX_I2C_ADDR1_FLAGS,
+		},
+		.drv = &anx74xx_tcpm_drv,
+	},
+	{
+		.bus_type = EC_BUS_TYPE_I2C,
+		.i2c_info = {
+			.port = I2C_PORT_TCPC1,
+			.addr_flags = ANX74XX_I2C_ADDR1_FLAGS,
+		},
+		.drv = &anx74xx_tcpm_drv,
+	},
 };
 
-struct usb_mux usb_muxes[CONFIG_USB_PD_PORT_COUNT] = {
+const struct usb_mux usb_muxes[CONFIG_USB_PD_PORT_MAX_COUNT] = {
 	{
-		.port_addr = 0,
+		.usb_port = 0,
 		.driver = &anx74xx_tcpm_usb_mux_driver,
 		.hpd_update = &anx74xx_tcpc_update_hpd_status,
 	},
 	{
-		.port_addr = 1,
+		.usb_port = 1,
 		.driver = &anx74xx_tcpm_usb_mux_driver,
 		.hpd_update = &anx74xx_tcpc_update_hpd_status,
 	},
 };
 
-/* called from anx74xx_set_power_mode() */
+const struct charger_config_t chg_chips[] = {
+	{
+		.i2c_port = I2C_PORT_CHARGER,
+		.i2c_addr_flags = BD9995X_ADDR_FLAGS,
+		.drv = &bd9995x_drv,
+	},
+};
+
+/**
+ * Power on (or off) a single TCPC.
+ * minimum on/off delays are included.
+ *
+ * @param port	Port number of TCPC.
+ * @param mode	0: power off, 1: power on.
+ */
 void board_set_tcpc_power_mode(int port, int mode)
 {
 	switch (port) {
 	case 0:
 		if (mode) {
 			gpio_set_level(GPIO_USB_C0_TCPC_PWR, 1);
-			msleep(10);
+			msleep(ANX74XX_PWR_H_RST_H_DELAY_MS);
 			gpio_set_level(GPIO_USB_C0_PD_RST_L, 1);
 		} else {
 			gpio_set_level(GPIO_USB_C0_PD_RST_L, 0);
-			msleep(1);
+			msleep(ANX74XX_RST_L_PWR_L_DELAY_MS);
 			gpio_set_level(GPIO_USB_C0_TCPC_PWR, 0);
+			msleep(ANX74XX_PWR_L_PWR_H_DELAY_MS);
 		}
 		break;
 	case 1:
 		if (mode) {
 			gpio_set_level(GPIO_USB_C1_TCPC_PWR, 1);
-			msleep(10);
+			msleep(ANX74XX_PWR_H_RST_H_DELAY_MS);
 			gpio_set_level(GPIO_USB_C1_PD_RST_L, 1);
 		} else {
 			gpio_set_level(GPIO_USB_C1_PD_RST_L, 0);
-			msleep(1);
+			msleep(ANX74XX_RST_L_PWR_L_DELAY_MS);
 			gpio_set_level(GPIO_USB_C1_TCPC_PWR, 0);
+			msleep(ANX74XX_PWR_L_PWR_H_DELAY_MS);
 		}
 		break;
 	}
@@ -255,15 +313,15 @@ void board_reset_pd_mcu(void)
 	/* Assert reset */
 	gpio_set_level(GPIO_USB_C0_PD_RST_L, 0);
 	gpio_set_level(GPIO_USB_C1_PD_RST_L, 0);
-	msleep(1);
+	msleep(ANX74XX_RST_L_PWR_L_DELAY_MS);
 	/* Disable power */
 	gpio_set_level(GPIO_USB_C0_TCPC_PWR, 0);
 	gpio_set_level(GPIO_USB_C1_TCPC_PWR, 0);
-	msleep(10);
+	msleep(ANX74XX_PWR_L_PWR_H_DELAY_MS);
 	/* Enable power */
 	gpio_set_level(GPIO_USB_C0_TCPC_PWR, 1);
 	gpio_set_level(GPIO_USB_C1_TCPC_PWR, 1);
-	msleep(10);
+	msleep(ANX74XX_PWR_H_RST_H_DELAY_MS);
 	/* Deassert reset */
 	gpio_set_level(GPIO_USB_C0_PD_RST_L, 1);
 	gpio_set_level(GPIO_USB_C1_PD_RST_L, 1);
@@ -271,10 +329,20 @@ void board_reset_pd_mcu(void)
 
 void board_tcpc_init(void)
 {
+	int count = 0;
 	int port;
 
+	/* Wait for disconnected battery to wake up */
+	while (battery_hw_present() == BP_YES &&
+	       battery_is_present() == BP_NO) {
+		usleep(100 * MSEC);
+		/* Give up waiting after 2 seconds */
+		if (++count > 20)
+			break;
+	}
+
 	/* Only reset TCPC if not sysjump */
-	if (!system_jumped_to_this_image())
+	if (!system_jumped_late())
 		board_reset_pd_mcu();
 
 	/* Enable TCPC interrupts */
@@ -291,13 +359,10 @@ void board_tcpc_init(void)
 	 * Initialize HPD to low; after sysjump SOC needs to see
 	 * HPD pulse to enable video path
 	 */
-	for (port = 0; port < CONFIG_USB_PD_PORT_COUNT; port++) {
-		const struct usb_mux *mux = &usb_muxes[port];
-
-		mux->hpd_update(port, 0, 0);
-	}
+	for (port = 0; port < CONFIG_USB_PD_PORT_MAX_COUNT; ++port)
+		usb_mux_hpd_update(port, USB_PD_MUX_HPD_LVL_DEASSERTED |
+					 USB_PD_MUX_HPD_IRQ_DEASSERTED);
 }
-DECLARE_HOOK(HOOK_INIT, board_tcpc_init, HOOK_PRIO_INIT_I2C+1);
 
 uint16_t tcpc_get_alert_status(void)
 {
@@ -317,68 +382,128 @@ uint16_t tcpc_get_alert_status(void)
 }
 
 const struct temp_sensor_t temp_sensors[] = {
-	{"Battery", TEMP_SENSOR_TYPE_BATTERY, charge_get_battery_temp, 0, 4},
+	{"Battery", TEMP_SENSOR_TYPE_BATTERY, charge_get_battery_temp, 0},
 
 	/* These BD99992GW temp sensors are only readable in S0 */
 	{"Ambient", TEMP_SENSOR_TYPE_BOARD, bd99992gw_get_val,
-	 BD99992GW_ADC_CHANNEL_SYSTHERM0, 4},
+	 BD99992GW_ADC_CHANNEL_SYSTHERM0},
 	{"Charger", TEMP_SENSOR_TYPE_BOARD, bd99992gw_get_val,
-	 BD99992GW_ADC_CHANNEL_SYSTHERM1, 4},
+	 BD99992GW_ADC_CHANNEL_SYSTHERM1},
 	{"DRAM", TEMP_SENSOR_TYPE_BOARD, bd99992gw_get_val,
-	 BD99992GW_ADC_CHANNEL_SYSTHERM2, 4},
+	 BD99992GW_ADC_CHANNEL_SYSTHERM2},
 	{"eMMC", TEMP_SENSOR_TYPE_BOARD, bd99992gw_get_val,
-	 BD99992GW_ADC_CHANNEL_SYSTHERM3, 4},
-	{"Gyro", TEMP_SENSOR_TYPE_BOARD, bmi160_get_sensor_temp, BASE_GYRO, 1},
+	 BD99992GW_ADC_CHANNEL_SYSTHERM3},
+	{"Gyro", TEMP_SENSOR_TYPE_BOARD, bmi160_get_sensor_temp, BASE_GYRO},
 };
 BUILD_ASSERT(ARRAY_SIZE(temp_sensors) == TEMP_SENSOR_COUNT);
 
-const struct button_config buttons[CONFIG_BUTTON_COUNT] = {
-	{"Volume Down", KEYBOARD_BUTTON_VOLUME_DOWN, GPIO_VOLUME_DOWN_L,
-	 30 * MSEC, 0},
-	{"Volume Up", KEYBOARD_BUTTON_VOLUME_UP, GPIO_VOLUME_UP_L,
-	 30 * MSEC, 0},
-};
+/*
+ * Check if PMIC fault registers indicate VR fault. If yes, print out fault
+ * register info to console. Additionally, set panic reason so that the OS can
+ * check for fault register info by looking at offset 0x14(PWRSTAT1) and
+ * 0x15(PWRSTAT2) in cros ec panicinfo.
+ */
+static void board_report_pmic_fault(const char *str)
+{
+	int vrfault, pwrstat1 = 0, pwrstat2 = 0;
+	uint32_t info;
+
+	/* RESETIRQ1 -- Bit 4: VRFAULT */
+	if (i2c_read8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x8, &vrfault)
+	    != EC_SUCCESS)
+		return;
+
+	if (!(vrfault & BIT(4)))
+		return;
+
+	/* VRFAULT has occurred, print VRFAULT status bits. */
+
+	/* PWRSTAT1 */
+	i2c_read8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x16, &pwrstat1);
+
+	/* PWRSTAT2 */
+	i2c_read8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x17, &pwrstat2);
+
+	CPRINTS("PMIC VRFAULT: %s", str);
+	CPRINTS("PMIC VRFAULT: PWRSTAT1=0x%02x PWRSTAT2=0x%02x", pwrstat1,
+		pwrstat2);
+
+	/* Clear all faults -- Write 1 to clear. */
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x8, BIT(4));
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x16, pwrstat1);
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x17, pwrstat2);
+
+	/*
+	 * Status of the fault registers can be checked in the OS by looking at
+	 * offset 0x14(PWRSTAT1) and 0x15(PWRSTAT2) in cros ec panicinfo.
+	 */
+	info = ((pwrstat2 & 0xFF) << 8) | (pwrstat1 & 0xFF);
+	panic_set_reason(PANIC_SW_PMIC_FAULT, info, 0);
+}
 
 static void board_pmic_init(void)
 {
-	if (system_jumped_to_this_image())
+	board_report_pmic_fault("SYSJUMP");
+
+	/* Clear power source events */
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x04, 0xff);
+
+	/* Disable power button shutdown timer */
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x14, 0x00);
+
+	/* Disable VCCIO in ALL_SYS_PWRGD for early boards */
+	if (board_get_version() <= BOARD_VERSION_DVTB)
+		i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x18, 0x80);
+
+	if (system_jumped_late())
 		return;
 
+	/* DISCHGCNT2 - enable 100 ohm discharge on V3.3A and V1.8A */
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x3d, 0x05);
+
 	/* DISCHGCNT3 - enable 100 ohm discharge on V1.00A */
-	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992, 0x3e, 0x04);
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x3e, 0x04);
 
 	/* Set CSDECAYEN / VCCIO decays to 0V at assertion of SLP_S0# */
-	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992, 0x30, 0x7a);
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x30, 0x7a);
 
 	/*
 	 * Set V100ACNT / V1.00A Control Register:
 	 * Nominal output = 1.0V.
 	 */
-	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992, 0x37, 0x1a);
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x37, 0x1a);
 
 	/*
 	 * Set V085ACNT / V0.85A Control Register:
 	 * Lower power mode = 0.7V.
 	 * Nominal output = 1.0V.
 	 */
-	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992, 0x38, 0x7a);
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x38, 0x7a);
 
 	/* VRMODECTRL - disable low-power mode for all rails */
-	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992, 0x3b, 0x1f);
+	i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x3b, 0x1f);
 }
-DECLARE_HOOK(HOOK_INIT, board_pmic_init, HOOK_PRIO_DEFAULT);
+DECLARE_DEFERRED(board_pmic_init);
 
 static void board_set_tablet_mode(void)
 {
-	tablet_set_mode(!gpio_get_level(GPIO_TABLET_MODE_L));
+	int flipped_360_mode = !gpio_get_level(GPIO_TABLET_MODE_L);
+
+	tablet_set_mode(flipped_360_mode, TABLET_TRIGGER_LID);
+
+	/* Update DPTF profile based on mode */
+	if (flipped_360_mode)
+		acpi_dptf_set_profile_num(DPTF_PROFILE_FLIPPED_360_MODE);
+	else
+		acpi_dptf_set_profile_num(DPTF_PROFILE_CLAMSHELL);
 }
 
 int board_has_working_reset_flags(void)
 {
 	int version = board_get_version();
 
-	/* board version >= P1b will lose reset flags on power cycle */
-	if (version >= 3)
+	/* board version P1b to EVTb will lose reset flags on power cycle */
+	if (version >= BOARD_VERSION_P1B && version <= BOARD_VERSION_EVTB)
 		return 0;
 
 	/* All other board versions should have working reset flags */
@@ -403,41 +528,61 @@ static void board_init(void)
 	/* Provide AC status to the PCH */
 	gpio_set_level(GPIO_PCH_ACOK, extpower_is_present());
 
-#if defined(CONFIG_KEYBOARD_SCANCODE_MUTABLE) && !defined(TEST_BUILD)
-	if (board_get_version() == 4) {
+#ifndef TEST_BUILD
+	if (board_get_version() == BOARD_VERSION_EVT) {
 		/* Set F13 to new defined key on EVT */
 		CPRINTS("Overriding F13 scan code");
-		scancode_set1[3][9] = 0xe058;
-		scancode_set2[3][9] = 0xe007;
+		set_scancode_set2(3, 9, 0xe007);
+#ifdef CONFIG_KEYBOARD_DEBUG
+		set_keycap_label(3, 9, KLLI_F13);
+#endif
 	}
 #endif
+
+	/* Initialize PMIC */
+	hook_call_deferred(&board_pmic_init_data, 0);
 }
 DECLARE_HOOK(HOOK_INIT, board_init, HOOK_PRIO_DEFAULT);
 
+__override enum pd_dual_role_states pd_get_drp_state_in_suspend(void)
+{
+	/*
+	 * If board is not connected to charger it will disable VBUS
+	 * on all ports that acts as source when going to suspend.
+	 * Change DRP state to force sink, to inform TCPM about that.
+	 */
+	if (!extpower_is_present())
+		return PD_DRP_FORCE_SINK;
+
+	return PD_DRP_TOGGLE_OFF;
+}
+
 /**
  * Buffer the AC present GPIO to the PCH.
+ * Set appropriate DRP state when chipset in suspend
  */
 static void board_extpower(void)
 {
+	enum pd_dual_role_states drp_state;
+	int port;
+
 	gpio_set_level(GPIO_PCH_ACOK, extpower_is_present());
+
+	if (chipset_in_or_transitioning_to_state(CHIPSET_STATE_SUSPEND)) {
+		drp_state = pd_get_drp_state_in_suspend();
+		for (port = 0; port < board_get_usb_pd_port_count(); port++)
+			if (pd_get_dual_role(port) != drp_state)
+				pd_set_dual_role(port, drp_state);
+	}
 }
 DECLARE_HOOK(HOOK_AC_CHANGE, board_extpower, HOOK_PRIO_DEFAULT);
 
 int pd_snk_is_vbus_provided(int port)
 {
-	enum bd9995x_charge_port bd9995x_port;
-
-	switch (port) {
-	case 0:
-	case 1:
-		bd9995x_port = bd9995x_pd_port_to_chg_port(port);
-		break;
-	default:
+	if (port != 0 && port != 1)
 		panic("Invalid charge port\n");
-		break;
-	}
 
-	return bd9995x_is_vbus_provided(bd9995x_port);
+	return bd9995x_is_vbus_provided(port);
 }
 
 /**
@@ -452,17 +597,6 @@ int board_set_active_charge_port(int charge_port)
 {
 	enum bd9995x_charge_port bd9995x_port;
 	int bd9995x_port_select = 1;
-	static int initialized;
-
-	/*
-	 * Reject charge port disable if our battery is critical and we
-	 * have yet to initialize a charge port - continue to charge using
-	 * charger ROM / POR settings.
-	 */
-	if (!initialized &&
-	    charge_port == CHARGE_PORT_NONE &&
-	    charge_get_percent() < 2)
-		return -1;
 
 	switch (charge_port) {
 	case 0:
@@ -471,7 +605,7 @@ int board_set_active_charge_port(int charge_port)
 		if (board_vbus_source_enabled(charge_port))
 			return -1;
 
-		bd9995x_port = bd9995x_pd_port_to_chg_port(charge_port);
+		bd9995x_port = charge_port;
 		break;
 	case CHARGE_PORT_NONE:
 		bd9995x_port_select = 0;
@@ -491,7 +625,6 @@ int board_set_active_charge_port(int charge_port)
 	}
 
 	CPRINTS("New chg p%d", charge_port);
-	initialized = 1;
 
 	return bd9995x_select_input_port(bd9995x_port, bd9995x_port_select);
 }
@@ -521,46 +654,28 @@ void board_set_charge_limit(int port, int supplier, int charge_ma,
 				   CONFIG_CHARGER_INPUT_CURRENT), charge_mv);
 }
 
-/**
- * Return whether ramping is allowed for given supplier
- */
-int board_is_ramp_allowed(int supplier)
-{
-	/* Don't allow ramping in RO when write protected */
-	if (system_get_image_copy() != SYSTEM_IMAGE_RW
-		&& system_is_locked())
-		return 0;
-	else
-		return (supplier == CHARGE_SUPPLIER_BC12_DCP ||
-			supplier == CHARGE_SUPPLIER_BC12_SDP ||
-			supplier == CHARGE_SUPPLIER_BC12_CDP ||
-			supplier == CHARGE_SUPPLIER_OTHER);
-}
-
-/**
- * Return the maximum allowed input current
- */
-int board_get_ramp_current_limit(int supplier, int sup_curr)
-{
-	return bd9995x_get_bc12_ilim(supplier);
-}
-
-/**
- * Return if board is consuming full amount of input current
- */
-int board_is_consuming_full_charge(void)
-{
-	int chg_perc = charge_get_percent();
-
-	return chg_perc > 2 && chg_perc < 95;
-}
-
 /**
  * Return if VBUS is sagging too low
  */
 int board_is_vbus_too_low(int port, enum chg_ramp_vbus_state ramp_state)
 {
-	return charger_get_vbus_voltage(port) < BD9995X_BC12_MIN_VOLTAGE;
+	int voltage;
+
+	if (charger_get_vbus_voltage(port, &voltage))
+		voltage = 0;
+
+	return voltage < BD9995X_BC12_MIN_VOLTAGE;
+}
+
+/* Clear pending interrupts and enable DSP for wake */
+static void dsp_wake_enable(int enable)
+{
+	if (enable) {
+		gpio_clear_pending_interrupt(GPIO_MIC_DSP_IRQ_1V8_L);
+		gpio_enable_interrupt(GPIO_MIC_DSP_IRQ_1V8_L);
+	} else {
+		gpio_disable_interrupt(GPIO_MIC_DSP_IRQ_1V8_L);
+	}
 }
 
 /* Clear pending interrupts and enable trackpad for wake */
@@ -598,14 +713,14 @@ static void enable_input_devices(void)
 }
 
 /* Enable or disable input devices, based on chipset state and tablet mode */
-#ifndef TEST_BUILD
-void lid_angle_peripheral_enable(int enable)
+__override void lid_angle_peripheral_enable(int enable)
 {
 	/*
-	 * If the lid is in 360 position, ignore the lid angle,
+	 * If suspended and the lid is in 360 position, ignore the lid angle,
 	 * which might be faulty. Disable keyboard and trackpad wake.
 	 */
-	if (tablet_get_mode() || chipset_in_state(CHIPSET_STATE_ANY_OFF))
+	if (chipset_in_state(CHIPSET_STATE_ANY_OFF) ||
+	   (tablet_get_mode() && chipset_in_state(CHIPSET_STATE_SUSPEND)))
 		enable = 0;
 	keyboard_scan_enable(enable, KB_SCAN_DISABLE_LID_ANGLE);
 
@@ -614,7 +729,6 @@ void lid_angle_peripheral_enable(int enable)
 		enable = 0;
 	trackpad_wake_enable(enable);
 }
-#endif
 
 /* Called on AP S5 -> S3 transition */
 static void board_chipset_startup(void)
@@ -628,8 +742,9 @@ DECLARE_HOOK(HOOK_CHIPSET_STARTUP, board_chipset_startup, HOOK_PRIO_DEFAULT);
 /* Called on AP S3 -> S5 transition */
 static void board_chipset_shutdown(void)
 {
-	/* Disable Trackpad */
+	/* Disable Trackpad and DSP wake in S5 */
 	trackpad_wake_enable(0);
+	dsp_wake_enable(0);
 	gpio_set_level(GPIO_TRACKPAD_SHDN_L, 0);
 	hook_call_deferred(&enable_input_devices_data, 0);
 }
@@ -639,8 +754,14 @@ DECLARE_HOOK(HOOK_CHIPSET_SHUTDOWN, board_chipset_shutdown, HOOK_PRIO_DEFAULT);
 static void board_chipset_suspend(void)
 {
 	gpio_set_level(GPIO_ENABLE_BACKLIGHT, 0);
-	if (!tablet_get_mode() && lid_is_open())
-		trackpad_wake_enable(1);
+	if (lid_is_open()) {
+		/* Enable DSP wake if suspended with lid open */
+		dsp_wake_enable(1);
+
+		/* Enable trackpad wake if suspended and not in tablet mode */
+		if (!tablet_get_mode())
+			trackpad_wake_enable(1);
+	}
 }
 DECLARE_HOOK(HOOK_CHIPSET_SUSPEND, board_chipset_suspend, HOOK_PRIO_DEFAULT);
 
@@ -648,15 +769,25 @@ DECLARE_HOOK(HOOK_CHIPSET_SUSPEND, board_chipset_suspend, HOOK_PRIO_DEFAULT);
 static void board_chipset_resume(void)
 {
 	gpio_set_level(GPIO_ENABLE_BACKLIGHT, 1);
+	dsp_wake_enable(0);
 	trackpad_wake_enable(0);
 }
 DECLARE_HOOK(HOOK_CHIPSET_RESUME, board_chipset_resume, HOOK_PRIO_DEFAULT);
 
+static void board_chipset_reset(void)
+{
+	board_report_pmic_fault("CHIPSET RESET");
+}
+DECLARE_HOOK(HOOK_CHIPSET_RESET, board_chipset_reset, HOOK_PRIO_DEFAULT);
+
 /* Called on lid change */
 static void board_lid_change(void)
 {
-	if (!lid_is_open())
+	/* Disable trackpad and DSP wake if lid is closed */
+	if (!lid_is_open()) {
 		trackpad_wake_enable(0);
+		dsp_wake_enable(0);
+	}
 }
 DECLARE_HOOK(HOOK_LID_CHANGE, board_lid_change, HOOK_PRIO_DEFAULT);
 
@@ -671,7 +802,7 @@ void board_hibernate(void)
 	/* Shut down PMIC */
 	CPRINTS("Triggering PMIC shutdown");
 	uart_flush_output();
-	if (i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992, 0x49, 0x01)) {
+	if (i2c_write8(I2C_PORT_PMIC, I2C_ADDR_BD99992_FLAGS, 0x49, 0x01)) {
 		/*
 		 * If we can't tell the PMIC to shutdown, instead reset
 		 * and don't start the AP. Hopefully we'll be able to
@@ -709,8 +840,6 @@ int board_get_version(void)
 
 void sensor_board_proc_double_tap(void)
 {
-	/* TODO: Call led update function */
-	CPRINTS("Call LED status update");
 	led_register_double_tap();
 }
 
@@ -721,7 +850,7 @@ static struct mutex g_base_mutex;
 static struct mutex g_lid_mutex;
 
 static struct kionix_accel_data g_kxcj9_data;
-static struct bmi160_drv_data_t g_bmi160_data;
+static struct bmi_drv_data_t g_bmi160_data;
 
 static struct si114x_drv_data_t g_si114x_data = {
 	.state = SI114X_NOT_READY,
@@ -732,8 +861,9 @@ static struct si114x_drv_data_t g_si114x_data = {
 		},
 		/* light */
 		{
-			.base_data_reg = SI114X_REG_ALSVIS_DATA0,
-			.irq_flags = SI114X_ALS_INT_FLAG,
+			.base_data_reg = SI114X_ALS_VIS_DATA0,
+			.irq_flags = SI114X_IRQ_ENABLE_ALS_IE_INT0 |
+				     SI114X_IRQ_ENABLE_ALS_IE_INT1,
 			.scale = 1,
 			.offset = -256,
 		}
@@ -741,20 +871,19 @@ static struct si114x_drv_data_t g_si114x_data = {
 };
 
 /* Matrix to rotate accelrator into standard reference frame */
-const matrix_3x3_t mag_standard_ref = {
+const mat33_fp_t mag_standard_ref = {
 	{ FLOAT_TO_FP(-1), 0, 0},
 	{ 0,  FLOAT_TO_FP(1), 0},
 	{ 0, 0, FLOAT_TO_FP(-1)}
 };
 
-const matrix_3x3_t lid_standard_ref = {
+const mat33_fp_t lid_standard_ref = {
 	{FLOAT_TO_FP(-1),  0,  0},
 	{ 0,  FLOAT_TO_FP(-1), 0},
 	{ 0,  0, FLOAT_TO_FP(1)}
 };
 
 struct motion_sensor_t motion_sensors[] = {
-
 	[LID_ACCEL] = {
 	 .name = "Lid Accel",
 	 .active_mask = SENSOR_ACTIVE_S0_S3,
@@ -765,33 +894,22 @@ struct motion_sensor_t motion_sensors[] = {
 	 .mutex = &g_lid_mutex,
 	 .drv_data = &g_kxcj9_data,
 	 .port = I2C_PORT_LID_ACCEL,
-	 .addr = KXCJ9_ADDR0,
+	 .i2c_spi_addr_flags = KXCJ9_ADDR0_FLAGS,
 	 .rot_standard_ref = &lid_standard_ref,
-	 .default_range = 2, /* g, enough for laptop. */
+	 .default_range = 2, /* g, enough for lid angle calculation. */
 	 .min_frequency = KXCJ9_ACCEL_MIN_FREQ,
 	 .max_frequency = KXCJ9_ACCEL_MAX_FREQ,
 	 .config = {
-		/* AP: by default use EC settings */
-		[SENSOR_CONFIG_AP] = {
-			.odr = 0,
-			.ec_rate = 0,
-		},
 		/* EC use accel for angle detection */
 		[SENSOR_CONFIG_EC_S0] = {
 			.odr = 10000 | ROUND_UP_FLAG,
-			.ec_rate = 0,
 		},
 		 /* Sensor on for lid angle detection */
 		[SENSOR_CONFIG_EC_S3] = {
 			.odr = 10000 | ROUND_UP_FLAG,
-			.ec_rate = 0,
-		},
-		[SENSOR_CONFIG_EC_S5] = {
-			.odr = 0,
-			.ec_rate = 0,
 		},
 	 },
-		},
+	},
 
 	[BASE_ACCEL] = {
 	 .name = "Base Accel",
@@ -803,17 +921,12 @@ struct motion_sensor_t motion_sensors[] = {
 	 .mutex = &g_base_mutex,
 	 .drv_data = &g_bmi160_data,
 	 .port = I2C_PORT_GYRO,
-	 .addr = BMI160_ADDR0,
+	 .i2c_spi_addr_flags = BMI160_ADDR0_FLAGS,
 	 .rot_standard_ref = NULL,
-	 .default_range = 2,  /* g, enough for laptop. */
-	 .min_frequency = BMI160_ACCEL_MIN_FREQ,
-	 .max_frequency = BMI160_ACCEL_MAX_FREQ,
+	 .default_range = 4,  /* g, to meet CDD 7.3.1/C-1-4 reqs */
+	 .min_frequency = BMI_ACCEL_MIN_FREQ,
+	 .max_frequency = BMI_ACCEL_MAX_FREQ,
 	 .config = {
-		 /* AP: by default use EC settings */
-		 [SENSOR_CONFIG_AP] = {
-			.odr = 0,
-			.ec_rate = 0,
-		 },
 		 /* EC use accel for angle detection */
 		 [SENSOR_CONFIG_EC_S0] = {
 			.odr = TAP_ODR,
@@ -824,7 +937,7 @@ struct motion_sensor_t motion_sensors[] = {
 			.odr = TAP_ODR,
 			.ec_rate = 100 * MSEC,
 		 },
-		 /* Sensor off in S5 */
+		 /* Sensor on in S5 for battery detection */
 		 [SENSOR_CONFIG_EC_S5] = {
 			.odr = TAP_ODR,
 			.ec_rate = 100 * MSEC,
@@ -834,7 +947,7 @@ struct motion_sensor_t motion_sensors[] = {
 
 	[BASE_GYRO] = {
 	 .name = "Base Gyro",
-	 .active_mask = SENSOR_ACTIVE_S0,
+	 .active_mask = SENSOR_ACTIVE_S0_S3_S5,
 	 .chip = MOTIONSENSE_CHIP_BMI160,
 	 .type = MOTIONSENSE_TYPE_GYRO,
 	 .location = MOTIONSENSE_LOC_BASE,
@@ -842,38 +955,16 @@ struct motion_sensor_t motion_sensors[] = {
 	 .mutex = &g_base_mutex,
 	 .drv_data = &g_bmi160_data,
 	 .port = I2C_PORT_GYRO,
-	 .addr = BMI160_ADDR0,
+	 .i2c_spi_addr_flags = BMI160_ADDR0_FLAGS,
 	 .default_range = 1000, /* dps */
 	 .rot_standard_ref = NULL,
-	 .min_frequency = BMI160_GYRO_MIN_FREQ,
-	 .max_frequency = BMI160_GYRO_MAX_FREQ,
-	 .config = {
-		 /* AP: by default shutdown all sensors */
-		 [SENSOR_CONFIG_AP] = {
-			.odr = 0,
-			.ec_rate = 0,
-		 },
-		 /* EC does not need in S0 */
-		 [SENSOR_CONFIG_EC_S0] = {
-			.odr = 0,
-			.ec_rate = 0,
-		 },
-		 /* Sensor off in S3/S5 */
-		 [SENSOR_CONFIG_EC_S3] = {
-			.odr = 0,
-			.ec_rate = 0,
-		 },
-		 /* Sensor off in S3/S5 */
-		 [SENSOR_CONFIG_EC_S5] = {
-			.odr = 0,
-			.ec_rate = 0,
-		 },
-	 },
+	 .min_frequency = BMI_GYRO_MIN_FREQ,
+	 .max_frequency = BMI_GYRO_MAX_FREQ,
 	},
 
 	[BASE_MAG] = {
 	 .name = "Base Mag",
-	 .active_mask = SENSOR_ACTIVE_S0,
+	 .active_mask = SENSOR_ACTIVE_S0_S3_S5,
 	 .chip = MOTIONSENSE_CHIP_BMI160,
 	 .type = MOTIONSENSE_TYPE_MAG,
 	 .location = MOTIONSENSE_LOC_BASE,
@@ -881,33 +972,11 @@ struct motion_sensor_t motion_sensors[] = {
 	 .mutex = &g_base_mutex,
 	 .drv_data = &g_bmi160_data,
 	 .port = I2C_PORT_GYRO,
-	 .addr = BMI160_ADDR0,
-	 .default_range = 1 << 11, /* 16LSB / uT, fixed */
+	 .i2c_spi_addr_flags = BMI160_ADDR0_FLAGS,
+	 .default_range = BIT(11), /* 16LSB / uT, fixed */
 	 .rot_standard_ref = &mag_standard_ref,
 	 .min_frequency = BMM150_MAG_MIN_FREQ,
-	 .max_frequency = BMM150_MAG_MAX_FREQ,
-	 .config = {
-		 /* AP: by default shutdown all sensors */
-		 [SENSOR_CONFIG_AP] = {
-			.odr = 0,
-			.ec_rate = 0,
-		 },
-		 /* EC does not need in S0 */
-		 [SENSOR_CONFIG_EC_S0] = {
-			.odr = 0,
-			.ec_rate = 0,
-		 },
-		 /* Sensor off in S3/S5 */
-		 [SENSOR_CONFIG_EC_S3] = {
-			.odr = 0,
-			.ec_rate = 0,
-		 },
-		 /* Sensor off in S3/S5 */
-		 [SENSOR_CONFIG_EC_S5] = {
-			 .odr = 0,
-			 .ec_rate = 0,
-		 },
-	 },
+	 .max_frequency = BMM150_MAG_MAX_FREQ(SPECIAL),
 	},
 
 	[LID_LIGHT] = {
@@ -920,31 +989,15 @@ struct motion_sensor_t motion_sensors[] = {
 	 .mutex = &g_lid_mutex,
 	 .drv_data = &g_si114x_data,
 	 .port = I2C_PORT_ALS,
-	 .addr = SI114X_ADDR,
+	 .i2c_spi_addr_flags = SI114X_ADDR_FLAGS,
 	 .rot_standard_ref = NULL,
-	 .default_range = 3088, /* 30.88%: int = 0 - frac = 3088/10000 */
+	 .default_range = 6000, /* 60.00%: int = 0 - frac = 6000/10000 */
 	 .min_frequency = SI114X_LIGHT_MIN_FREQ,
 	 .max_frequency = SI114X_LIGHT_MAX_FREQ,
 	 .config = {
-		 /* AP: by default shutdown all sensors */
-		 [SENSOR_CONFIG_AP] = {
-			 .odr = 0,
-			 .ec_rate = 0,
-		 },
 		 /* Run ALS sensor in S0 */
 		 [SENSOR_CONFIG_EC_S0] = {
 			 .odr = 1000,
-			 .ec_rate = 0,
-		 },
-		 /* Sensor off in S3/S5 */
-		 [SENSOR_CONFIG_EC_S3] = {
-			 .odr = 0,
-			 .ec_rate = 0,
-		 },
-		 /* Sensor off in S3/S5 */
-		 [SENSOR_CONFIG_EC_S5] = {
-			 .odr = 0,
-			 .ec_rate = 0,
 		 },
 	 },
 	},
@@ -956,4 +1009,3 @@ const struct motion_sensor_t *motion_als_sensors[] = {
 	&motion_sensors[LID_LIGHT],
 };
 BUILD_ASSERT(ARRAY_SIZE(motion_als_sensors) == ALS_COUNT);
-
