@@ -1,4 +1,4 @@
-/* Copyright (c) 2014 The Chromium OS Authors. All rights reserved.
+/* Copyright 2014 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -10,7 +10,7 @@
 
 #ifndef __ASSEMBLER__
 
-#undef CONFIG_FLASH /* TODO: implement me */
+#undef CONFIG_FLASH_CROS /* TODO: implement me */
 #undef CONFIG_FLASH_PHYSICAL /* TODO: implement me */
 #undef CONFIG_FMAP /* TODO: implement me */
 #undef CONFIG_WATCHDOG
@@ -37,4 +37,3 @@
 #endif /* !__ASSEMBLER__ */
 
 #endif /* __CROS_EC_BOARD_H */
-
