@@ -6,7 +6,6 @@
 /* reef_it8320 board-specific configuration */
 
 #include "adc.h"
-#include "adc_chip.h"
 #include "button.h"
 #include "charge_manager.h"
 #include "charge_ramp.h"
@@ -17,8 +16,8 @@
 #include "driver/charger/bd9995x.h"
 #include "driver/tcpm/it83xx_pd.h"
 #include "driver/tcpm/tcpm.h"
+#include "driver/usb_mux/pi3usb3x532.h"
 #include "extpower.h"
-#include "ec2i_chip.h"
 #include "gpio.h"
 #include "hooks.h"
 #include "host_command.h"
@@ -40,7 +39,7 @@
 #include "tablet_mode.h"
 #include "task.h"
 #include "temp_sensor.h"
-#include "thermistor.h"
+#include "temp_sensor/thermistor.h"
 #include "timer.h"
 #include "uart.h"
 #include "usb_charge.h"
@@ -58,46 +57,48 @@
 
 #include "gpio_list.h"
 
-/* power signal list.  Must match order of enum power_signal. */
-const struct power_signal_info power_signal_list[] = {
-	{GPIO_RSMRST_L_PGOOD,     1, "RSMRST_L"},
-	{GPIO_PCH_SLP_S3_L,       1, "SLP_S3_DEASSERTED"},
-	{GPIO_PCH_SLP_S4_L,       1, "SLP_S4_DEASSERTED"},
-	{GPIO_SUSPWRNACK,         1, "SUSPWRNACK_DEASSERTED"},
-
-	{GPIO_ALL_SYS_PGOOD,      1, "ALL_SYS_PGOOD"},
-	{GPIO_PP3300_PG,          1, "PP3300_PG"},
-	{GPIO_PP5000_PG,          1, "PP5000_PG"},
-};
-BUILD_ASSERT(ARRAY_SIZE(power_signal_list) == POWER_SIGNAL_COUNT);
-
-/* ADC channels */
 const struct adc_t adc_channels[] = {
 	/* Convert to mV (3000mV/1024). */
-	{"CHARGER",     3000, 1024, 0, 1}, /* GPI1 */
-	{"AMBIENT",     3000, 1024, 0, 2}, /* GPI2 */
-	{"BRD_ID",      3000, 1024, 0, 3}, /* GPI3 */
+	{"CHARGER",     3000, 1024, 0, CHIP_ADC_CH1}, /* GPI1 */
+	{"AMBIENT",     3000, 1024, 0, CHIP_ADC_CH2}, /* GPI2 */
+	{"BRD_ID",      3000, 1024, 0, CHIP_ADC_CH3}, /* GPI3 */
 };
 BUILD_ASSERT(ARRAY_SIZE(adc_channels) == ADC_CH_COUNT);
 
 const struct i2c_port_t i2c_ports[]  = {
-	{"mux",       IT83XX_I2C_CH_C, 400,
-		GPIO_EC_I2C_C_SCL, GPIO_EC_I2C_C_SDA},
-	{"batt",      IT83XX_I2C_CH_E, 100,
-		GPIO_EC_I2C_E_SCL, GPIO_EC_I2C_E_SDA},
+	{
+		.name = "mux",
+		.port = IT83XX_I2C_CH_C,
+		.kbps = 400,
+		.scl  = GPIO_EC_I2C_C_SCL,
+		.sda  = GPIO_EC_I2C_C_SDA
+	},
+	{
+		.name = "batt",
+		.port = IT83XX_I2C_CH_E,
+		.kbps = 100,
+		.scl  = GPIO_EC_I2C_E_SCL,
+		.sda  = GPIO_EC_I2C_E_SDA
+	},
 };
 const unsigned int i2c_ports_used = ARRAY_SIZE(i2c_ports);
 
-const struct tcpc_config_t tcpc_config[CONFIG_USB_PD_PORT_COUNT] = {
-	{-1, -1, &it83xx_tcpm_drv, 0},
-	{-1, -1, &it83xx_tcpm_drv, 0},
+const struct tcpc_config_t tcpc_config[CONFIG_USB_PD_PORT_MAX_COUNT] = {
+	{
+		.bus_type = EC_BUS_TYPE_EMBEDDED,
+		.drv = &it83xx_tcpm_drv
+	},
+	{
+		.bus_type = EC_BUS_TYPE_EMBEDDED,
+		.drv = &it83xx_tcpm_drv
+	},
 };
 
-void board_pd_vconn_ctrl(int port, int cc_pin, int enabled)
+void board_pd_vconn_ctrl(int port, enum usbpd_cc_pin cc_pin, int enabled)
 {
 	int cc1_enabled = 0, cc2_enabled = 0;
 
-	if (cc_pin)
+	if (cc_pin != USBPD_CC_PIN_1)
 		cc2_enabled = enabled;
 	else
 		cc1_enabled = enabled;
@@ -111,38 +112,6 @@ void board_pd_vconn_ctrl(int port, int cc_pin, int enabled)
 	}
 }
 
-/*
- * PD host event status for host command
- * Note: this variable must be aligned on 4-byte boundary because we pass the
- * address to atomic_ functions which use assembly to access them.
- */
-static uint32_t pd_host_event_status __aligned(4);
-
-static int hc_pd_host_event_status(struct host_cmd_handler_args *args)
-{
-	struct ec_response_host_event_status *r = args->response;
-
-	/* Read and clear the host event status to return to AP */
-	r->status = atomic_read_clear(&pd_host_event_status);
-
-	args->response_size = sizeof(*r);
-	return EC_RES_SUCCESS;
-}
-DECLARE_HOST_COMMAND(EC_CMD_PD_HOST_EVENT_STATUS, hc_pd_host_event_status,
-		     EC_VER_MASK(0));
-
-/* Send host event up to AP */
-void pd_send_host_event(int mask)
-{
-	/* mask must be set */
-	if (!mask)
-		return;
-
-	atomic_or(&pd_host_event_status, mask);
-	/* interrupt the AP */
-	host_set_single_event(EC_HOST_EVENT_PD_MCU);
-}
-
 const enum gpio_signal hibernate_wake_pins[] = {
 	GPIO_AC_PRESENT,
 	GPIO_LID_OPEN,
@@ -151,10 +120,18 @@ const enum gpio_signal hibernate_wake_pins[] = {
 
 const int hibernate_wake_pins_used = ARRAY_SIZE(hibernate_wake_pins);
 
-static void it83xx_tcpc_update_hpd_status(int port, int hpd_lvl, int hpd_irq)
+static void it83xx_tcpc_update_hpd_status(const struct usb_mux *me,
+					  mux_state_t mux_state,
+					  bool *ack_required)
 {
+	int hpd_lvl = (mux_state & USB_PD_MUX_HPD_LVL) ? 1 : 0;
+	int hpd_irq = (mux_state & USB_PD_MUX_HPD_IRQ) ? 1 : 0;
 	enum gpio_signal gpio =
-		port ? GPIO_USB_C1_HPD_1P8_ODL : GPIO_USB_C0_HPD_1P8_ODL;
+		me->usb_port ? GPIO_USB_C1_HPD_1P8_ODL
+			     : GPIO_USB_C0_HPD_1P8_ODL;
+
+	/* This driver does not use host command ACKs */
+	*ack_required = false;
 
 	hpd_lvl = !hpd_lvl;
 
@@ -166,14 +143,18 @@ static void it83xx_tcpc_update_hpd_status(int port, int hpd_lvl, int hpd_irq)
 	}
 }
 
-struct usb_mux usb_muxes[CONFIG_USB_PD_PORT_COUNT] = {
+const struct usb_mux usb_muxes[CONFIG_USB_PD_PORT_MAX_COUNT] = {
 	{
-		.port_addr = 0xa8,
-		.driver = &pi3usb30532_usb_mux_driver,
+		.usb_port = 0,
+		.i2c_port = I2C_PORT_USB_MUX,
+		.i2c_addr_flags = PI3USB3X532_I2C_ADDR0,
+		.driver = &pi3usb3x532_usb_mux_driver,
 		.hpd_update = &it83xx_tcpc_update_hpd_status,
 	},
 	{
-		.port_addr = 0x20,
+		.usb_port = 1,
+		.i2c_port = I2C_PORT_USB_MUX,
+		.i2c_addr_flags = 0x10,
 		.driver = &ps8740_usb_mux_driver,
 		.hpd_update = &it83xx_tcpc_update_hpd_status,
 	},
@@ -183,103 +164,32 @@ const int usb_port_enable[CONFIG_USB_PORT_POWER_SMART_PORT_COUNT] = {
 	GPIO_USB1_ENABLE,
 };
 
-/*
- * Data derived from Seinhart-Hart equation in a resistor divider circuit with
- * Vdd=3300mV, R = 13.7Kohm, and Murata NCP15WB-series thermistor (B = 4050,
- * T0 = 298.15, nominal resistance (R0) = 47Kohm).
- */
-#define CHARGER_THERMISTOR_SCALING_FACTOR 13
-static const struct thermistor_data_pair charger_thermistor_data[] = {
-	{ 3044 / CHARGER_THERMISTOR_SCALING_FACTOR, 0 },
-	{ 2890 / CHARGER_THERMISTOR_SCALING_FACTOR, 10 },
-	{ 2680 / CHARGER_THERMISTOR_SCALING_FACTOR, 20 },
-	{ 2418 / CHARGER_THERMISTOR_SCALING_FACTOR, 30 },
-	{ 2117 / CHARGER_THERMISTOR_SCALING_FACTOR, 40 },
-	{ 1800 / CHARGER_THERMISTOR_SCALING_FACTOR, 50 },
-	{ 1490 / CHARGER_THERMISTOR_SCALING_FACTOR, 60 },
-	{ 1208 / CHARGER_THERMISTOR_SCALING_FACTOR, 70 },
-	{ 966 / CHARGER_THERMISTOR_SCALING_FACTOR, 80 },
-	{ 860 / CHARGER_THERMISTOR_SCALING_FACTOR, 85 },
-	{ 766 / CHARGER_THERMISTOR_SCALING_FACTOR, 90 },
-	{ 679 / CHARGER_THERMISTOR_SCALING_FACTOR, 95 },
-	{ 603 / CHARGER_THERMISTOR_SCALING_FACTOR, 100 },
-};
-
-static const struct thermistor_info charger_thermistor_info = {
-	.scaling_factor = CHARGER_THERMISTOR_SCALING_FACTOR,
-	.num_pairs = ARRAY_SIZE(charger_thermistor_data),
-	.data = charger_thermistor_data,
-};
-
-int board_get_charger_temp(int idx, int *temp_ptr)
-{
-	int mv = adc_read_channel(ADC_TEMP_SENSOR_CHARGER);
-
-	if (mv < 0)
-		return -1;
-
-	*temp_ptr = thermistor_linear_interpolate(mv, &charger_thermistor_info);
-	*temp_ptr = C_TO_K(*temp_ptr);
-	return 0;
-}
-
-/*
- * Data derived from Seinhart-Hart equation in a resistor divider circuit with
- * Vdd=3300mV, R = 51.1Kohm, and Murata NCP15WB-series thermistor (B = 4050,
- * T0 = 298.15, nominal resistance (R0) = 47Kohm).
- */
-#define AMB_THERMISTOR_SCALING_FACTOR 11
-static const struct thermistor_data_pair amb_thermistor_data[] = {
-	{ 2512 / AMB_THERMISTOR_SCALING_FACTOR, 0 },
-	{ 2158 / AMB_THERMISTOR_SCALING_FACTOR, 10 },
-	{ 1772 / AMB_THERMISTOR_SCALING_FACTOR, 20 },
-	{ 1398 / AMB_THERMISTOR_SCALING_FACTOR, 30 },
-	{ 1070 / AMB_THERMISTOR_SCALING_FACTOR, 40 },
-	{ 803 / AMB_THERMISTOR_SCALING_FACTOR, 50 },
-	{ 597 / AMB_THERMISTOR_SCALING_FACTOR, 60 },
-	{ 443 / AMB_THERMISTOR_SCALING_FACTOR, 70 },
-	{ 329 / AMB_THERMISTOR_SCALING_FACTOR, 80 },
-	{ 285 / AMB_THERMISTOR_SCALING_FACTOR, 85 },
-	{ 247 / AMB_THERMISTOR_SCALING_FACTOR, 90 },
-	{ 214 / AMB_THERMISTOR_SCALING_FACTOR, 95 },
-	{ 187 / AMB_THERMISTOR_SCALING_FACTOR, 100 },
-};
-
-static const struct thermistor_info amb_thermistor_info = {
-	.scaling_factor = AMB_THERMISTOR_SCALING_FACTOR,
-	.num_pairs = ARRAY_SIZE(amb_thermistor_data),
-	.data = amb_thermistor_data,
-};
-
-int board_get_ambient_temp(int idx, int *temp_ptr)
-{
-	int mv = adc_read_channel(ADC_TEMP_SENSOR_AMB);
-
-	if (mv < 0)
-		return -1;
-
-	*temp_ptr = thermistor_linear_interpolate(mv, &amb_thermistor_info);
-	*temp_ptr = C_TO_K(*temp_ptr);
-	return 0;
-}
-
 const struct temp_sensor_t temp_sensors[] = {
-	/* FIXME(dhendrix): tweak action_delay_sec */
-	{"Battery", TEMP_SENSOR_TYPE_BATTERY, charge_get_battery_temp, 0, 1},
-	{"Ambient", TEMP_SENSOR_TYPE_BOARD, board_get_ambient_temp, 0, 5},
-	{"Charger", TEMP_SENSOR_TYPE_BOARD, board_get_charger_temp, 1, 1},
+	[TEMP_SENSOR_BATTERY] = {.name = "Battery",
+				 .type = TEMP_SENSOR_TYPE_BATTERY,
+				 .read = charge_get_battery_temp,
+				 .idx = 0},
+	[TEMP_SENSOR_AMBIENT] = {.name = "Ambient",
+				 .type = TEMP_SENSOR_TYPE_BOARD,
+				 .read = get_temp_3v3_51k1_47k_4050b,
+				 .idx = ADC_TEMP_SENSOR_AMB},
+	[TEMP_SENSOR_CHARGER] = {.name = "Charger",
+				 .type = TEMP_SENSOR_TYPE_BOARD,
+				 .read = get_temp_3v3_13k7_47k_4050b,
+				 .idx = ADC_TEMP_SENSOR_CHARGER},
 };
 BUILD_ASSERT(ARRAY_SIZE(temp_sensors) == TEMP_SENSOR_COUNT);
 
-const struct button_config buttons[CONFIG_BUTTON_COUNT] = {
-	{"Volume Down", KEYBOARD_BUTTON_VOLUME_DOWN, GPIO_EC_VOLDN_BTN_ODL,
-	 30 * MSEC, 0},
-	{"Volume Up", KEYBOARD_BUTTON_VOLUME_UP, GPIO_EC_VOLUP_BTN_ODL,
-	 30 * MSEC, 0},
+const struct charger_config_t chg_chips[] = {
+	{
+		.i2c_port = I2C_PORT_CHARGER,
+		.i2c_addr_flags = BD9995X_ADDR_FLAGS,
+		.drv = &bd9995x_drv,
+	},
 };
 
 /* Called by APL power state machine when transitioning from G3 to S5 */
-static void chipset_pre_init(void)
+void chipset_pre_init_callback(void)
 {
 	/*
 	 * No need to re-init PMIC since settings are sticky across sysjump.
@@ -306,13 +216,20 @@ static void chipset_pre_init(void)
 	/* Enable PMIC */
 	gpio_set_level(GPIO_PMIC_EN, 1);
 }
-DECLARE_HOOK(HOOK_CHIPSET_PRE_INIT, chipset_pre_init, HOOK_PRIO_DEFAULT);
+
+static void board_set_tablet_mode(void)
+{
+	/*
+	 * Always report device isn't in tablet mode because
+	 * our id is clamshell and no TABLET_MODE_L pin
+	 */
+	tablet_set_mode(0, TABLET_TRIGGER_LID);
+}
 
 /* Initialize board. */
 static void board_init(void)
 {
-	int port;
-
+	board_set_tablet_mode();
 	/* Enable charger interrupts */
 	gpio_enable_interrupt(GPIO_CHARGER_INT_L);
 
@@ -320,26 +237,18 @@ static void board_init(void)
 	* Initialize HPD to low; after sysjump SOC needs to see
 	* HPD pulse to enable video path
 	*/
-	for (port = 0; port < CONFIG_USB_PD_PORT_COUNT; port++)
-		usb_muxes[port].hpd_update(port, 0, 0);
+	for (int port = 0; port < CONFIG_USB_PD_PORT_MAX_COUNT; ++port)
+		usb_mux_hpd_update(port, USB_PD_MUX_HPD_LVL_DEASSERTED |
+					 USB_PD_MUX_HPD_IRQ_DEASSERTED);
 }
 DECLARE_HOOK(HOOK_INIT, board_init, HOOK_PRIO_INIT_I2C + 1);
 
 int pd_snk_is_vbus_provided(int port)
 {
-	enum bd9995x_charge_port bd9995x_port = 0;
-
-	switch (port) {
-	case 0:
-	case 1:
-		bd9995x_port = bd9995x_pd_port_to_chg_port(port);
-		break;
-	default:
+	if (port != 0 && port != 1)
 		panic("Invalid charge port\n");
-		break;
-	}
 
-	return bd9995x_is_vbus_provided(bd9995x_port);
+	return bd9995x_is_vbus_provided(port);
 }
 
 /**
@@ -354,17 +263,6 @@ int board_set_active_charge_port(int charge_port)
 {
 	enum bd9995x_charge_port bd9995x_port = 0;
 	int bd9995x_port_select = 1;
-	static int initialized;
-
-	/*
-	 * Reject charge port disable if our battery is critical and we
-	 * have yet to initialize a charge port - continue to charge using
-	 * charger ROM / POR settings.
-	 */
-	if (!initialized &&
-	    charge_port == CHARGE_PORT_NONE &&
-	    charge_get_percent() < 2)
-		return -1;
 
 	switch (charge_port) {
 	case 0:
@@ -373,7 +271,7 @@ int board_set_active_charge_port(int charge_port)
 		if (board_vbus_source_enabled(charge_port))
 			return -1;
 
-		bd9995x_port = bd9995x_pd_port_to_chg_port(charge_port);
+		bd9995x_port = charge_port;
 		break;
 	case CHARGE_PORT_NONE:
 		bd9995x_port_select = 0;
@@ -393,7 +291,6 @@ int board_set_active_charge_port(int charge_port)
 	}
 
 	CPRINTS("New chg p%d", charge_port);
-	initialized = 1;
 
 	return bd9995x_select_input_port(bd9995x_port, bd9995x_port_select);
 }
@@ -423,46 +320,17 @@ void board_set_charge_limit(int port, int supplier, int charge_ma,
 				   CONFIG_CHARGER_INPUT_CURRENT), charge_mv);
 }
 
-/**
- * Return whether ramping is allowed for given supplier
- */
-int board_is_ramp_allowed(int supplier)
-{
-	/* Don't allow ramping in RO when write protected */
-	if (system_get_image_copy() != SYSTEM_IMAGE_RW
-		&& system_is_locked())
-		return 0;
-	else
-		return (supplier == CHARGE_SUPPLIER_BC12_DCP ||
-			supplier == CHARGE_SUPPLIER_BC12_SDP ||
-			supplier == CHARGE_SUPPLIER_BC12_CDP ||
-			supplier == CHARGE_SUPPLIER_OTHER);
-}
-
-/**
- * Return the maximum allowed input current
- */
-int board_get_ramp_current_limit(int supplier, int sup_curr)
-{
-	return bd9995x_get_bc12_ilim(supplier);
-}
-
-/**
- * Return if board is consuming full amount of input current
- */
-int board_is_consuming_full_charge(void)
-{
-	int chg_perc = charge_get_percent();
-
-	return chg_perc > 2 && chg_perc < 95;
-}
-
 /**
  * Return if VBUS is sagging too low
  */
 int board_is_vbus_too_low(int port, enum chg_ramp_vbus_state ramp_state)
 {
-	return charger_get_vbus_voltage(port) < BD9995X_BC12_MIN_VOLTAGE;
+	int voltage;
+
+	if (charger_get_vbus_voltage(port, &voltage))
+		voltage = 0;
+
+	return voltage < BD9995X_BC12_MIN_VOLTAGE;
 }
 
 /* Called on AP S5 -> S3 transition */
@@ -620,7 +488,7 @@ int board_get_version(void)
 }
 
 /* Keyboard scan setting */
-struct keyboard_scan_config keyscan_config = {
+__override struct keyboard_scan_config keyscan_config = {
 	/*
 	 * F3 key scan cycle completed but scan input is not
 	 * charging to logic high when EC start scan next
@@ -638,92 +506,3 @@ struct keyboard_scan_config keyscan_config = {
 		0xa4, 0xff, 0xfe, 0x55, 0xfa, 0xca  /* full set */
 	},
 };
-
-/* PNPCFG settings */
-const struct ec2i_t pnpcfg_settings[] = {
-	/* Select logical device 06h(keyboard) */
-	{HOST_INDEX_LDN, LDN_KBC_KEYBOARD},
-	/* Set IRQ=01h for logical device */
-	{HOST_INDEX_IRQNUMX, 0x01},
-	/* Enable logical device */
-	{HOST_INDEX_LDA, 0x01},
-
-	/* Select logical device 05h(mouse) */
-	{HOST_INDEX_LDN, LDN_KBC_MOUSE},
-	/* Set IRQ=0Ch for logical device */
-	{HOST_INDEX_IRQNUMX, 0x0C},
-	/* Enable logical device */
-	{HOST_INDEX_LDA, 0x01},
-
-	/* Select logical device 11h(PM1 ACPI) */
-	{HOST_INDEX_LDN, LDN_PMC1},
-	/* Set IRQ=00h for logical device */
-	{HOST_INDEX_IRQNUMX, 0x00},
-	/* Enable logical device */
-	{HOST_INDEX_LDA, 0x01},
-
-	/* Select logical device 12h(PM2) */
-	{HOST_INDEX_LDN, LDN_PMC2},
-	/* I/O Port Base Address 200h/204h */
-	{HOST_INDEX_IOBAD0_MSB, 0x02},
-	{HOST_INDEX_IOBAD0_LSB, 0x00},
-	{HOST_INDEX_IOBAD1_MSB, 0x02},
-	{HOST_INDEX_IOBAD1_LSB, 0x04},
-	/* Set IRQ=00h for logical device */
-	{HOST_INDEX_IRQNUMX, 0x00},
-	/* Enable logical device */
-	{HOST_INDEX_LDA, 0x01},
-
-	/* Select logical device 0Fh(SMFI) */
-	{HOST_INDEX_LDN, LDN_SMFI},
-	/* H2RAM LPC I/O cycle Dxxx */
-	{HOST_INDEX_DSLDC6, 0x00},
-	/* Enable H2RAM LPC I/O cycle */
-	{HOST_INDEX_DSLDC7, 0x01},
-	/* Enable logical device */
-	{HOST_INDEX_LDA, 0x01},
-
-	/* Select logical device 17h(PM3) */
-	{HOST_INDEX_LDN, LDN_PMC3},
-	/* I/O Port Base Address 80h */
-	{HOST_INDEX_IOBAD0_MSB, 0x00},
-	{HOST_INDEX_IOBAD0_LSB, 0x80},
-	{HOST_INDEX_IOBAD1_MSB, 0x00},
-	{HOST_INDEX_IOBAD1_LSB, 0x00},
-	/* Set IRQ=00h for logical device */
-	{HOST_INDEX_IRQNUMX, 0x00},
-	/* Enable logical device */
-	{HOST_INDEX_LDA, 0x01},
-	/* Select logical device 10h(RTCT) */
-	{HOST_INDEX_LDN, LDN_RTCT},
-	/* P80L Begin Index */
-	{HOST_INDEX_DSLDC4, P80L_P80LB},
-	/* P80L End Index */
-	{HOST_INDEX_DSLDC5, P80L_P80LE},
-	/* P80L Current Index */
-	{HOST_INDEX_DSLDC6, P80L_P80LC},
-#ifdef CONFIG_UART_HOST
-	/* Select logical device 2h(UART2) */
-	{HOST_INDEX_LDN, LDN_UART2},
-	/*
-	 * I/O port base address is 2F8h.
-	 * Host can use LPC I/O port 0x2F8 ~ 0x2FF to access UART2.
-	 * See specification 7.24.4 for more detial.
-	 */
-	{HOST_INDEX_IOBAD0_MSB, 0x02},
-	{HOST_INDEX_IOBAD0_LSB, 0xF8},
-	/* IRQ number is 3 */
-	{HOST_INDEX_IRQNUMX, 0x03},
-	/*
-	 * Interrupt Request Type Select
-	 * bit1, 0: IRQ request is buffered and applied to SERIRQ.
-	 *       1: IRQ request is inverted before being applied to SERIRQ.
-	 * bit0, 0: Edge triggered mode.
-	 *       1: Level triggered mode.
-	 */
-	{HOST_INDEX_IRQTP, 0x02},
-	/* Enable logical device */
-	{HOST_INDEX_LDA, 0x01},
-#endif
-};
-BUILD_ASSERT(ARRAY_SIZE(pnpcfg_settings) == EC2I_SETTING_COUNT);
