@@ -1,4 +1,4 @@
-/* Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+/* Copyright 2012 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -8,6 +8,8 @@
 #ifndef __CROS_EC_ADC_CHIP_H
 #define __CROS_EC_ADC_CHIP_H
 
+#include <stdint.h>
+
 enum lm4_adc_sequencer {
 	LM4_ADC_SEQ0 = 0,
 	LM4_ADC_SEQ1,
@@ -29,12 +31,6 @@ struct adc_t {
 	uint8_t gpio_mask;
 };
 
-/*
- * Boards must provide this list of ADC channel definitions.  This must match
- * the enum adc_channel list provided by the board.
- */
-extern const struct adc_t adc_channels[];
-
 /* Minimum and maximum values returned by raw ADC read. */
 #define ADC_READ_MIN 0
 #define ADC_READ_MAX 4095
@@ -42,7 +38,7 @@ extern const struct adc_t adc_channels[];
 /* Just plain id mapping for code readability */
 #define LM4_AIN(x) (x)
 
-/* Dummy value for "channel" in adc_t if we don't have an external channel. */
+/* Mock value for "channel" in adc_t if we don't have an external channel. */
 #define LM4_AIN_NONE (-1)
 
 #endif /* __CROS_EC_ADC_CHIP_H */
