@@ -1,4 +1,4 @@
-/* Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+/* Copyright 2012 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -6,11 +6,10 @@
 /* Temperature sensor module for Chrome EC */
 
 #include "adc.h"
-#include "adc_chip.h"
 #include "common.h"
 #include "hooks.h"
 
-/* Initialize temperature reading to a sane value (27 C) */
+/* Initialize temperature reading to a valid value (27 C) */
 static int last_val = C_TO_K(27);
 
 static void chip_temp_sensor_poll(void)
