@@ -1,4 +1,4 @@
-/* Copyright (c) 2013 The Chromium OS Authors. All rights reserved.
+/* Copyright 2013 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -54,10 +54,6 @@
 #define CPUTS(outstr) cputs(CC_LPC, outstr)
 #define CPRINTS(format, args...) cprints(CC_LPC, format, ## args)
 
-#define LPC_SYSJUMP_TAG 0x4c50  /* "LP" */
-
-static uint32_t host_events;     /* Currently pending SCI/SMI events */
-static uint32_t event_mask[3];   /* Event masks for each type */
 static struct host_packet lpc_packet;
 static struct host_cmd_handler_args host_cmd_args;
 static uint8_t host_cmd_flags;   /* Flags from host command */
@@ -153,6 +149,8 @@ static inline void keyboard_irq_assert(void)
  */
 static void lpc_generate_smi(void)
 {
+	host_event_t smi;
+
 	/* Enforce signal-high for long enough to debounce high */
 	gpio_set_level(GPIO_PCH_SMI_L, 1);
 	udelay(65);
@@ -162,9 +160,9 @@ static void lpc_generate_smi(void)
 	/* Set signal high, now that we've generated the edge */
 	gpio_set_level(GPIO_PCH_SMI_L, 1);
 
-	if (host_events & event_mask[LPC_HOST_EVENT_SMI])
-		CPRINTS("smi 0x%08x",
-			host_events & event_mask[LPC_HOST_EVENT_SMI]);
+	smi = lpc_get_host_events_by_type(LPC_HOST_EVENT_SMI);
+	if (smi)
+		HOST_EVENT_CPRINTS("smi", smi);
 }
 
 /**
@@ -172,6 +170,8 @@ static void lpc_generate_smi(void)
  */
 static void lpc_generate_sci(void)
 {
+	host_event_t sci;
+
 #ifdef CONFIG_SCI_GPIO
 	/* Enforce signal-high for long enough to debounce high */
 	gpio_set_level(CONFIG_SCI_GPIO, 1);
@@ -185,9 +185,9 @@ static void lpc_generate_sci(void)
 	LM4_LPC_LPCCTL |= LM4_LPC_SCI_START;
 #endif
 
-	if (host_events & event_mask[LPC_HOST_EVENT_SCI])
-		CPRINTS("sci 0x%08x",
-			host_events & event_mask[LPC_HOST_EVENT_SCI]);
+	sci = lpc_get_host_events_by_type(LPC_HOST_EVENT_SCI);
+	if (sci)
+		HOST_EVENT_CPRINTS("sci", sci);
 }
 
 /**
@@ -195,7 +195,7 @@ static void lpc_generate_sci(void)
  *
  * @param wake_events	Currently asserted wake events
  */
-static void lpc_update_wake(uint32_t wake_events)
+static void lpc_update_wake(uint64_t wake_events)
 {
 	/*
 	 * Mask off power button event, since the AP gets that through a
@@ -341,7 +341,7 @@ void lpc_comx_put_char(int c)
  *   - SMI pulse via EC_SMI_L GPIO
  *   - SCI pulse via LPC0SCI
  */
-static void update_host_event_status(void)
+void lpc_update_host_event_status(void)
 {
 	int need_sci = 0;
 	int need_smi = 0;
@@ -352,7 +352,7 @@ static void update_host_event_status(void)
 	/* Disable LPC interrupt while updating status register */
 	task_disable_irq(LM4_IRQ_LPC);
 
-	if (host_events & event_mask[LPC_HOST_EVENT_SMI]) {
+	if (lpc_get_host_events_by_type(LPC_HOST_EVENT_SMI)) {
 		/* Only generate SMI for first event */
 		if (!(LM4_LPC_ST(LPC_CH_ACPI) & LM4_LPC_ST_SMI))
 			need_smi = 1;
@@ -360,7 +360,7 @@ static void update_host_event_status(void)
 	} else
 		LM4_LPC_ST(LPC_CH_ACPI) &= ~LM4_LPC_ST_SMI;
 
-	if (host_events & event_mask[LPC_HOST_EVENT_SCI]) {
+	if (lpc_get_host_events_by_type(LPC_HOST_EVENT_SCI)) {
 		/* Generate SCI for every event */
 		need_sci = 1;
 		LM4_LPC_ST(LPC_CH_ACPI) |= LM4_LPC_ST_SCI;
@@ -368,12 +368,13 @@ static void update_host_event_status(void)
 		LM4_LPC_ST(LPC_CH_ACPI) &= ~LM4_LPC_ST_SCI;
 
 	/* Copy host events to mapped memory */
-	*(uint32_t *)host_get_memmap(EC_MEMMAP_HOST_EVENTS) = host_events;
+	*(host_event_t *)host_get_memmap(EC_MEMMAP_HOST_EVENTS) =
+				lpc_get_host_events();
 
 	task_enable_irq(LM4_IRQ_LPC);
 
 	/* Process the wake events. */
-	lpc_update_wake(host_events & event_mask[LPC_HOST_EVENT_WAKE]);
+	lpc_update_wake(lpc_get_host_events_by_type(LPC_HOST_EVENT_WAKE));
 
 	/* Send pulse on SMI signal if needed */
 	if (need_smi)
@@ -384,54 +385,6 @@ static void update_host_event_status(void)
 		lpc_generate_sci();
 }
 
-void lpc_set_host_event_state(uint32_t mask)
-{
-	if (mask != host_events) {
-		host_events = mask;
-		update_host_event_status();
-	}
-}
-
-int lpc_query_host_event_state(void)
-{
-	const uint32_t any_mask = event_mask[0] | event_mask[1] | event_mask[2];
-	int evt_index = 0;
-	int i;
-
-	for (i = 0; i < 32; i++) {
-		const uint32_t e = (1 << i);
-
-		if (host_events & e) {
-			host_clear_events(e);
-
-			/*
-			 * If host hasn't unmasked this event, drop it.  We do
-			 * this at query time rather than event generation time
-			 * so that the host has a chance to unmask events
-			 * before they're dropped by a query.
-			 */
-			if (!(e & any_mask))
-				continue;
-
-			evt_index = i + 1;	/* Events are 1-based */
-			break;
-		}
-	}
-
-	return evt_index;
-}
-
-void lpc_set_host_event_mask(enum lpc_host_event_type type, uint32_t mask)
-{
-	event_mask[type] = mask;
-	update_host_event_status();
-}
-
-uint32_t lpc_get_host_event_mask(enum lpc_host_event_type type)
-{
-	return event_mask[type];
-}
-
 void lpc_set_acpi_status_mask(uint8_t mask)
 {
 	uint32_t set_mask = 0;
@@ -452,7 +405,7 @@ void lpc_clear_acpi_status_mask(uint8_t mask)
 
 int lpc_get_pltrst_asserted(void)
 {
-	return (LM4_LPC_LPCSTS & (1<<10)) ? 1 : 0;
+	return (LM4_LPC_LPCSTS & BIT(10)) ? 1 : 0;
 }
 
 /**
@@ -589,7 +542,7 @@ DECLARE_DEFERRED(lpc_chipset_reset);
 /**
  * LPC interrupt handler
  */
-void lpc_interrupt(void)
+static void lpc_interrupt(void)
 {
 	uint32_t mis = LM4_LPC_LPCMIS;
 	uint32_t st;
@@ -641,8 +594,8 @@ void lpc_interrupt(void)
 #endif
 
 	/* Debugging: print changes to LPC0RESET */
-	if (mis & (1 << 31)) {
-		if (LM4_LPC_LPCSTS & (1 << 10)) {
+	if (mis & BIT(31)) {
+		if (LM4_LPC_LPCSTS & BIT(10)) {
 			int i;
 
 			/* Store port 80 reset event */
@@ -667,32 +620,6 @@ void lpc_interrupt(void)
 }
 DECLARE_IRQ(LM4_IRQ_LPC, lpc_interrupt, 2);
 
-/**
- * Preserve event masks across a sysjump.
- */
-static void lpc_sysjump(void)
-{
-	system_add_jump_tag(LPC_SYSJUMP_TAG, 1,
-			    sizeof(event_mask), event_mask);
-}
-DECLARE_HOOK(HOOK_SYSJUMP, lpc_sysjump, HOOK_PRIO_DEFAULT);
-
-/**
- * Restore event masks after a sysjump.
- */
-static void lpc_post_sysjump(void)
-{
-	const uint32_t *prev_mask;
-	int size, version;
-
-	prev_mask = (const uint32_t *)system_get_jump_tag(LPC_SYSJUMP_TAG,
-							  &version, &size);
-	if (!prev_mask || version != 1 || size != sizeof(event_mask))
-		return;
-
-	memcpy(event_mask, prev_mask, sizeof(event_mask));
-}
-
 /* Enable LPC ACPI-EC interrupts */
 void lpc_enable_acpi_interrupts(void)
 {
@@ -755,7 +682,7 @@ static void lpc_init(void)
 	 * data writes, pool bytes 0(data)/1(cmd)
 	 */
 	LM4_LPC_ADR(LPC_CH_KEYBOARD) = 0x60;
-	LM4_LPC_CTL(LPC_CH_KEYBOARD) = (1 << 24/* IRQSEL1 */) |
+	LM4_LPC_CTL(LPC_CH_KEYBOARD) = (BIT(24)/* IRQSEL1 */) |
 		(0 << 18/* IRQEN1 */) | (LPC_POOL_OFFS_KEYBOARD << (5 - 1));
 	LM4_LPC_ST(LPC_CH_KEYBOARD) = 0;
 	/* Unmask interrupt for host command/data writes and data reads */
@@ -816,32 +743,33 @@ static void lpc_init(void)
 	 * Unmask LPC bus reset interrupt.  This lets us monitor the PCH
 	 * PLTRST# signal for debugging.
 	 */
-	LM4_LPC_LPCIM |= (1 << 31);
+	LM4_LPC_LPCIM |= BIT(31);
 
 	/* Enable LPC channels */
 	LM4_LPC_LPCCTL = LM4_LPC_SCI_CLK_1 |
-		(1 << LPC_CH_ACPI) |
-		(1 << LPC_CH_PORT80) |
-		(1 << LPC_CH_CMD_DATA) |
-		(1 << LPC_CH_KEYBOARD) |
-		(1 << LPC_CH_CMD) |
-		(1 << LPC_CH_MEMMAP);
+		BIT(LPC_CH_ACPI) |
+		BIT(LPC_CH_PORT80) |
+		BIT(LPC_CH_CMD_DATA) |
+		BIT(LPC_CH_KEYBOARD) |
+		BIT(LPC_CH_CMD) |
+		BIT(LPC_CH_MEMMAP);
 
 #ifdef CONFIG_UART_HOST
 	LM4_LPC_LPCCTL |= 1 << LPC_CH_COMX;
 #endif
 
 	/*
-	 * Ensure the EC (slave) has control of the memory-mapped I/O space.
-	 * Once the EC has won arbitration for the memory-mapped space, it will
-	 * keep control of it until it writes the last byte in the space.
-	 * (That never happens; we can't use the last byte in the space because
-	 * ACPI can't see it anyway.)
+	 * Ensure the EC (peripheral) has control of the memory-mapped
+	 * I/O space.  Once the EC has won arbitration for the
+	 * memory-mapped space, it will keep control of it until it
+	 * writes the last byte in the space.  (That never happens; we
+	 * can't use the last byte in the space because ACPI can't see
+	 * it anyway.)
 	 */
 	while (!(LM4_LPC_ST(LPC_CH_MEMMAP) & 0x10)) {
 		/* Clear HW1ST */
 		LM4_LPC_ST(LPC_CH_MEMMAP) &= ~0x40;
-		/* Do a dummy slave write; this should cause SW1ST to be set */
+		/* Do a peripheral write; this should cause SW1ST to be set */
 		*LPC_POOL_MEMMAP = *LPC_POOL_MEMMAP;
 	}
 
@@ -862,14 +790,11 @@ static void lpc_init(void)
 	uart_comx_enable();
 #endif
 
-	/* Restore event masks if needed */
-	lpc_post_sysjump();
-
 	/* Sufficiently initialized */
 	init_done = 1;
 
 	/* Update host events now that we can copy them to memmap */
-	update_host_event_status();
+	lpc_update_host_event_status();
 }
 /*
  * Set prio to higher than default; this way LPC memory mapped data is ready
@@ -877,18 +802,6 @@ static void lpc_init(void)
  */
 DECLARE_HOOK(HOOK_INIT, lpc_init, HOOK_PRIO_INIT_LPC);
 
-static void lpc_resume(void)
-{
-	/* Mask all host events until the host unmasks them itself.  */
-	lpc_set_host_event_mask(LPC_HOST_EVENT_SMI, 0);
-	lpc_set_host_event_mask(LPC_HOST_EVENT_SCI, 0);
-	lpc_set_host_event_mask(LPC_HOST_EVENT_WAKE, 0);
-
-	/* Store port 80 event so we know where resume happened */
-	port_80_write(PORT_80_EVENT_RESUME);
-}
-DECLARE_HOOK(HOOK_CHIPSET_RESUME, lpc_resume, HOOK_PRIO_DEFAULT);
-
 static void lpc_tick(void)
 {
 	/*
@@ -903,12 +816,12 @@ DECLARE_HOOK(HOOK_TICK, lpc_tick, HOOK_PRIO_DEFAULT);
 /**
  * Get protocol information
  */
-static int lpc_get_protocol_info(struct host_cmd_handler_args *args)
+static enum ec_status lpc_get_protocol_info(struct host_cmd_handler_args *args)
 {
 	struct ec_response_get_protocol_info *r = args->response;
 
 	memset(r, 0, sizeof(*r));
-	r->protocol_versions = (1 << 2) | (1 << 3);
+	r->protocol_versions = BIT(2) | BIT(3);
 	r->max_request_packet_size = EC_LPC_HOST_PACKET_SIZE;
 	r->max_response_packet_size = EC_LPC_HOST_PACKET_SIZE;
 	r->flags = 0;
