@@ -1,4 +1,4 @@
-/* Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+/* Copyright 2012 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -25,14 +25,15 @@ enum hibdata_index {
 #ifdef CONFIG_SOFTWARE_PANIC
 	HIBDATA_INDEX_SAVED_PANIC_REASON,   /* Saved panic reason */
 	HIBDATA_INDEX_SAVED_PANIC_INFO,     /* Saved panic data */
-	HIBDATA_INDEX_SAVED_PANIC_EXCEPTION /* Saved panic exception code */
+	HIBDATA_INDEX_SAVED_PANIC_EXCEPTION,/* Saved panic exception code */
+	HIBDATA_INDEX_SAVED_PANIC_FLAGS,    /* Saved panic flags */
 #endif
 };
 
 /* Flags for HIBDATA_INDEX_WAKE */
-#define HIBDATA_WAKE_RTC        (1 << 0)  /* RTC alarm */
-#define HIBDATA_WAKE_HARD_RESET (1 << 1)  /* Hard reset via short RTC alarm */
-#define HIBDATA_WAKE_PIN        (1 << 2)  /* Wake pin */
+#define HIBDATA_WAKE_RTC        BIT(0)  /* RTC alarm */
+#define HIBDATA_WAKE_HARD_RESET BIT(1)  /* Hard reset via short RTC alarm */
+#define HIBDATA_WAKE_PIN        BIT(2)  /* Wake pin */
 
 /*
  * Time to hibernate to trigger a power-on reset.  50 ms is sufficient for the
@@ -103,6 +104,16 @@ static int hibdata_write(enum hibdata_index index, uint32_t value)
 	return wait_for_hibctl_wc();
 }
 
+uint32_t chip_read_reset_flags(void)
+{
+	return hibdata_read(HIBDATA_INDEX_SAVED_RESET_FLAGS);
+}
+
+void chip_save_reset_flags(uint32_t flags)
+{
+	hibdata_write(HIBDATA_INDEX_SAVED_RESET_FLAGS, flags);
+}
+
 static void check_reset_cause(void)
 {
 	uint32_t hib_status = LM4_HIBERNATE_HIBRIS;
@@ -123,7 +134,7 @@ static void check_reset_cause(void)
 		 * Note that this is also triggered by hibernation, because
 		 * that de-powers the chip.
 		 */
-		flags |= RESET_FLAG_POWER_ON;
+		flags |= EC_RESET_FLAG_POWER_ON;
 	} else if (!flags && (raw_reset_cause & 0x01)) {
 		/*
 		 * LM4 signals the reset pin in RESC for all power-on resets,
@@ -131,46 +142,46 @@ static void check_reset_cause(void)
 		 * this flag mutually-exclusive with power on flag, so we can
 		 * use it to indicate a keyboard-triggered reset.
 		 */
-		flags |= RESET_FLAG_RESET_PIN;
+		flags |= EC_RESET_FLAG_RESET_PIN;
 	}
 
 	if (raw_reset_cause & 0x04)
-		flags |= RESET_FLAG_BROWNOUT;
+		flags |= EC_RESET_FLAG_BROWNOUT;
 
 	if (raw_reset_cause & 0x10)
-		flags |= RESET_FLAG_SOFT;
+		flags |= EC_RESET_FLAG_SOFT;
 
 	if (raw_reset_cause & 0x28) {
 		/* Watchdog timer 0 or 1 */
-		flags |= RESET_FLAG_WATCHDOG;
+		flags |= EC_RESET_FLAG_WATCHDOG;
 	}
 
 	/* Handle other raw reset causes */
 	if (raw_reset_cause && !flags)
-		flags |= RESET_FLAG_OTHER;
+		flags |= EC_RESET_FLAG_OTHER;
 
 
 	if ((hib_status & 0x09) &&
 	    (hib_wake_flags & HIBDATA_WAKE_HARD_RESET)) {
 		/* Hibernation caused by software-triggered hard reset */
-		flags |= RESET_FLAG_HARD;
+		flags |= EC_RESET_FLAG_HARD;
 
 		/* Consume the hibernate reasons so we don't see them below */
 		hib_status &= ~0x09;
 	}
 
 	if ((hib_status & 0x01) && (hib_wake_flags & HIBDATA_WAKE_RTC))
-		flags |= RESET_FLAG_RTC_ALARM;
+		flags |= EC_RESET_FLAG_RTC_ALARM;
 
 	if ((hib_status & 0x08) && (hib_wake_flags & HIBDATA_WAKE_PIN))
-		flags |= RESET_FLAG_WAKE_PIN;
+		flags |= EC_RESET_FLAG_WAKE_PIN;
 
 	if (hib_status & 0x04)
-		flags |= RESET_FLAG_LOW_BATTERY;
+		flags |= EC_RESET_FLAG_LOW_BATTERY;
 
 	/* Restore then clear saved reset flags */
-	flags |= hibdata_read(HIBDATA_INDEX_SAVED_RESET_FLAGS);
-	hibdata_write(HIBDATA_INDEX_SAVED_RESET_FLAGS, 0);
+	flags |= chip_read_reset_flags();
+	chip_save_reset_flags(0);
 
 	system_set_reset_flags(flags);
 }
@@ -323,7 +334,7 @@ void system_reset_rtc_alarm(void)
 /**
  * Hibernate module interrupt
  */
-void __hibernate_irq(void)
+static void __hibernate_irq(void)
 {
 	system_reset_rtc_alarm();
 }
@@ -388,12 +399,48 @@ void system_hibernate(uint32_t seconds, uint32_t microseconds)
 	hibernate(seconds, microseconds, HIBDATA_WAKE_PIN);
 }
 
+void chip_pre_init(void)
+{
+	/* Enable clocks to GPIO block C in run and sleep modes. */
+	clock_enable_peripheral(CGC_OFFSET_GPIO, 0x0004, CGC_MODE_ALL);
+
+	/*
+	 * Ensure PC0:3 are set to JTAG function.  They should be set this way
+	 * on a cold boot, but on a warm reboot a previous misbehaving image
+	 * could have set them differently.
+	 */
+	if (((LM4_GPIO_PCTL(LM4_GPIO_C) & 0x0000ffff) == 0x00001111) &&
+	    ((LM4_GPIO_AFSEL(LM4_GPIO_C) & 0x0f) == 0x0f) &&
+	    ((LM4_GPIO_DEN(LM4_GPIO_C) & 0x0f) == 0x0f) &&
+	    ((LM4_GPIO_PUR(LM4_GPIO_C) & 0x0f) == 0x0f))
+		return;  /* Already properly configured */
+
+	/* Unlock commit register for JTAG pins */
+	LM4_GPIO_LOCK(LM4_GPIO_C) = LM4_GPIO_LOCK_UNLOCK;
+	LM4_GPIO_CR(LM4_GPIO_C) |= 0x0f;
+
+	/* Reset JTAG pins */
+	LM4_GPIO_PCTL(LM4_GPIO_C) =
+		(LM4_GPIO_PCTL(LM4_GPIO_C) & 0xffff0000) | 0x00001111;
+	LM4_GPIO_AFSEL(LM4_GPIO_C) |= 0x0f;
+	LM4_GPIO_DEN(LM4_GPIO_C) |= 0x0f;
+	LM4_GPIO_PUR(LM4_GPIO_C) |= 0x0f;
+
+	/* Set interrupt on either edge of the JTAG signals */
+	LM4_GPIO_IS(LM4_GPIO_C) &= ~0x0f;
+	LM4_GPIO_IBE(LM4_GPIO_C) |= 0x0f;
+
+	/* Re-lock commit register */
+	LM4_GPIO_CR(LM4_GPIO_C) &= ~0x0f;
+	LM4_GPIO_LOCK(LM4_GPIO_C) = 0;
+}
+
 void system_pre_init(void)
 {
 	uint32_t hibctl;
 #ifdef CONFIG_SOFTWARE_PANIC
 	uint32_t reason, info;
-	uint8_t exception;
+	uint8_t exception, panic_flags;
 #endif
 
 	/*
@@ -456,11 +503,15 @@ void system_pre_init(void)
 	reason = hibdata_read(HIBDATA_INDEX_SAVED_PANIC_REASON);
 	info = hibdata_read(HIBDATA_INDEX_SAVED_PANIC_INFO);
 	exception = hibdata_read(HIBDATA_INDEX_SAVED_PANIC_EXCEPTION);
+	panic_flags = hibdata_read(HIBDATA_INDEX_SAVED_PANIC_FLAGS);
+
 	if (reason || info || exception) {
 		panic_set_reason(reason, info, exception);
+		panic_get_data()->flags = panic_flags;
 		hibdata_write(HIBDATA_INDEX_SAVED_PANIC_REASON, 0);
 		hibdata_write(HIBDATA_INDEX_SAVED_PANIC_INFO, 0);
 		hibdata_write(HIBDATA_INDEX_SAVED_PANIC_EXCEPTION, 0);
+		hibdata_write(HIBDATA_INDEX_SAVED_PANIC_FLAGS, 0);
 	}
 #endif
 
@@ -488,23 +539,26 @@ void system_reset(int flags)
 
 	/* Save current reset reasons if necessary */
 	if (flags & SYSTEM_RESET_PRESERVE_FLAGS)
-		save_flags = system_get_reset_flags() | RESET_FLAG_PRESERVED;
+		save_flags = system_get_reset_flags() | EC_RESET_FLAG_PRESERVED;
 
 	if (flags & SYSTEM_RESET_LEAVE_AP_OFF)
-		save_flags |= RESET_FLAG_AP_OFF;
+		save_flags |= EC_RESET_FLAG_AP_OFF;
 
-	hibdata_write(HIBDATA_INDEX_SAVED_RESET_FLAGS, save_flags);
+	chip_save_reset_flags(save_flags);
 
 	if (flags & SYSTEM_RESET_HARD) {
 #ifdef CONFIG_SOFTWARE_PANIC
 		uint32_t reason, info;
-		uint8_t exception;
+		uint8_t exception, panic_flags;
+
+		panic_flags = panic_get_data()->flags;
 
 		/* Panic data will be wiped by hard reset, so save it */
 		panic_get_reason(&reason, &info, &exception);
 		hibdata_write(HIBDATA_INDEX_SAVED_PANIC_REASON, reason);
 		hibdata_write(HIBDATA_INDEX_SAVED_PANIC_INFO, info);
 		hibdata_write(HIBDATA_INDEX_SAVED_PANIC_EXCEPTION, exception);
+		hibdata_write(HIBDATA_INDEX_SAVED_PANIC_FLAGS, panic_flags);
 #endif
 
 		/*
@@ -525,9 +579,10 @@ int system_set_scratchpad(uint32_t value)
 	return hibdata_write(HIBDATA_INDEX_SCRATCHPAD, value);
 }
 
-uint32_t system_get_scratchpad(void)
+int system_get_scratchpad(uint32_t *value)
 {
-	return hibdata_read(HIBDATA_INDEX_SCRATCHPAD);
+	*value = hibdata_read(HIBDATA_INDEX_SCRATCHPAD);
+	return EC_SUCCESS;
 }
 
 const char *system_get_chip_vendor(void)
@@ -624,7 +679,6 @@ const char *system_get_chip_revision(void)
 
 /*****************************************************************************/
 /* Console commands */
-#ifdef CONFIG_CMD_RTC
 void print_system_rtc(enum console_channel ch)
 {
 	uint32_t rtc;
@@ -635,6 +689,7 @@ void print_system_rtc(enum console_channel ch)
 		 rtc, rtcss, rtc, HIB_RTC_SUBSEC_TO_USEC(rtcss));
 }
 
+#ifdef CONFIG_CMD_RTC
 static int command_system_rtc(int argc, char **argv)
 {
 	if (argc == 3 && !strcasecmp(argv[1], "set")) {
@@ -695,7 +750,7 @@ DECLARE_CONSOLE_COMMAND(rtc_alarm, command_rtc_alarm_test,
 /* Host commands */
 
 #ifdef CONFIG_HOSTCMD_RTC
-static int system_rtc_get_value(struct host_cmd_handler_args *args)
+static enum ec_status system_rtc_get_value(struct host_cmd_handler_args *args)
 {
 	struct ec_response_rtc *r = args->response;
 
@@ -708,7 +763,7 @@ DECLARE_HOST_COMMAND(EC_CMD_RTC_GET_VALUE,
 		     system_rtc_get_value,
 		     EC_VER_MASK(0));
 
-static int system_rtc_set_value(struct host_cmd_handler_args *args)
+static enum ec_status system_rtc_set_value(struct host_cmd_handler_args *args)
 {
 	const struct ec_params_rtc *p = args->params;
 
