@@ -61,13 +61,17 @@ struct vw_event_t {
 
 /* Default settings of VWEVMS registers (Please refer Table.43/44) */
 static const struct vwevms_config_t espi_in_list[] = {
-	/* IDX EN ENPL ENESP IE             VW Event Bit 0 - 3 (M->S)         */
-	{0x02,  1,  0,  0,  0},  /* SLP_S3#,   SLP_S4#,    SLP_S5,    Reserve */
-	{0x03,  1,  0,  1,  0},  /* SUS_STAT#, PLTRST#,    ORST_WARN, Reserve */
-	{0x07,  1,  1,  1,  0},  /* HRST_WARN, SMIOUT#,    NMIOUT#,   Reserve */
-	{0x41,  1,  0,  1,  0},  /* SUS_WARN#, SPWRDN_ACK, Reserve,   SLP_A#  */
-	{0x42,  1,  0,  0,  0},  /* SLP_LAN#,  SLP_WAN#,   Reserve,   Reserve */
-	{0x47,  1,  1,  1,  0},  /* HOST_C10,  Reserve,    Reserve,   Reserve */
+	/* IDX EN ENPL ENESP IE/WE          VW Event Bit 0 - 3 (M->S)         */
+#ifdef CONFIG_HOSTCMD_ESPI_RESET_SLP_SX_VW_ON_ESPI_RST
+	{0x02,  1,  0,  1,  1},  /* SLP_S3#,   SLP_S4#,    SLP_S5#,   Reserve */
+#else
+	{0x02,  1,  0,  0,  1},  /* SLP_S3#,   SLP_S4#,    SLP_S5#,   Reserve */
+#endif
+	{0x03,  1,  0,  1,  1},  /* SUS_STAT#, PLTRST#,    ORST_WARN, Reserve */
+	{0x07,  1,  1,  1,  1},  /* HRST_WARN, SMIOUT#,    NMIOUT#,   Reserve */
+	{0x41,  1,  0,  1,  1},  /* SUS_WARN#, SPWRDN_ACK, Reserve,   SLP_A#  */
+	{0x42,  1,  0,  0,  1},  /* SLP_LAN#,  SLP_WAN#,   Reserve,   Reserve */
+	{0x47,  1,  1,  1,  1},  /* HOST_C10,  Reserve,    Reserve,   Reserve */
 };
 
 /* Default settings of VWEVSM registers (Please refer Table.43/44) */
@@ -86,7 +90,7 @@ static const struct vwevsm_config_t espi_out_list[] = {
 /* eSPI interrupts used in MIWU */
 static const struct host_wui_item espi_vw_int_list[] = {
 	/* ESPI_RESET  */
-	{MIWU_TABLE_0, MIWU_GROUP_5, 5, MIWU_EDGE_RISING},
+	{MIWU_TABLE_0, MIWU_GROUP_5, 5, MIWU_EDGE_FALLING},
 	/* SLP_S3 */
 	{MIWU_TABLE_2, MIWU_GROUP_1, 0, MIWU_EDGE_ANYING},
 	/* SLP_S4 */
@@ -105,32 +109,32 @@ static const struct host_wui_item espi_vw_int_list[] = {
 
 /* VW signals used in eSPI */
 static const struct vw_event_t vw_events_list[] = {
-	{VW_SLP_S3_L,               0x02,   0x01},	/* index 02h (In)  */
-	{VW_SLP_S4_L,               0x02,   0x02},
-	{VW_SLP_S5_L,               0x02,   0x04},
-	{VW_SUS_STAT_L,             0x03,   0x01},	/* index 03h (In)  */
-	{VW_PLTRST_L,               0x03,   0x02},
-	{VW_OOB_RST_WARN,           0x03,   0x04},
-	{VW_OOB_RST_ACK,            0x04,   0x01},	/* index 04h (Out) */
-	{VW_WAKE_L,                 0x04,   0x04},
-	{VW_PME_L,                  0x04,   0x08},
-	{VW_ERROR_FATAL,            0x05,   0x02},	/* index 05h (Out) */
-	{VW_ERROR_NON_FATAL,        0x05,   0x04},
-	{VW_SLAVE_BTLD_STATUS_DONE, 0x05,   0x09},
-	{VW_SCI_L,                  0x06,   0x01},	/* index 06h (Out) */
-	{VW_SMI_L,                  0x06,   0x02},
-	{VW_RCIN_L,                 0x06,   0x04},
-	{VW_HOST_RST_ACK,           0x06,   0x08},
-	{VW_HOST_RST_WARN,          0x07,   0x01},	/* index 07h (In)  */
-	{VW_SUS_ACK,                0x40,   0x01},	/* index 40h (Out) */
-	{VW_SUS_WARN_L,             0x41,   0x01},	/* index 41h (In)  */
-	{VW_SUS_PWRDN_ACK_L,        0x41,   0x02},
-	{VW_SLP_A_L,                0x41,   0x08},
-	{VW_SLP_LAN,                0x42,   0x01},	/* index 42h (In)  */
-	{VW_SLP_WLAN,               0x42,   0x02},
+	{VW_SLP_S3_L,                    0x02,   0x01},	/* index 02h (In)  */
+	{VW_SLP_S4_L,                    0x02,   0x02},
+	{VW_SLP_S5_L,                    0x02,   0x04},
+	{VW_SUS_STAT_L,                  0x03,   0x01},	/* index 03h (In)  */
+	{VW_PLTRST_L,                    0x03,   0x02},
+	{VW_OOB_RST_WARN,                0x03,   0x04},
+	{VW_OOB_RST_ACK,                 0x04,   0x01},	/* index 04h (Out) */
+	{VW_WAKE_L,                      0x04,   0x04},
+	{VW_PME_L,                       0x04,   0x08},
+	{VW_ERROR_FATAL,                 0x05,   0x02},	/* index 05h (Out) */
+	{VW_ERROR_NON_FATAL,             0x05,   0x04},
+	{VW_PERIPHERAL_BTLD_STATUS_DONE, 0x05,   0x09},
+	{VW_SCI_L,                       0x06,   0x01},	/* index 06h (Out) */
+	{VW_SMI_L,                       0x06,   0x02},
+	{VW_RCIN_L,                      0x06,   0x04},
+	{VW_HOST_RST_ACK,                0x06,   0x08},
+	{VW_HOST_RST_WARN,               0x07,   0x01},	/* index 07h (In)  */
+	{VW_SUS_ACK,                     0x40,   0x01},	/* index 40h (Out) */
+	{VW_SUS_WARN_L,                  0x41,   0x01},	/* index 41h (In)  */
+	{VW_SUS_PWRDN_ACK_L,             0x41,   0x02},
+	{VW_SLP_A_L,                     0x41,   0x08},
+	{VW_SLP_LAN,                     0x42,   0x01},	/* index 42h (In)  */
+	{VW_SLP_WLAN,                    0x42,   0x02},
 };
 
-/* Flag for SLAVE_BOOT_LOAD siganls */
+/* Flag for boot load signals */
 static uint8_t boot_load_done;
 
 /*****************************************************************************/
@@ -145,7 +149,7 @@ static void espi_reset_recovery(void)
 	boot_load_done = 0;
 }
 
-/* Configure Master-to-Slave virtual wire inputs */
+/* Configure Controller-to-Peripheral virtual wire inputs */
 static void espi_vw_config_in(const struct vwevms_config_t *config)
 {
 	uint32_t val;
@@ -177,7 +181,7 @@ static void espi_vw_config_in(const struct vwevms_config_t *config)
 	}
 }
 
-/* Configure Slave-to-Master virtual wire outputs */
+/* Configure Peripheral-to-Controller virtual wire outputs */
 static void espi_vw_config_out(const struct vwevsm_config_t *config)
 {
 	uint32_t val;
@@ -190,8 +194,8 @@ static void espi_vw_config_out(const struct vwevsm_config_t *config)
 			index = VWEVSM_IDX_GET(NPCX_VWEVSM(i));
 			/* Set VW output register */
 			if (index == config->idx) {
-				/* Get Wire field */
-				val = NPCX_VWEVSM(i) & 0x0F;
+				/* Preserve WIRE(3-0) and HW_WIRE (27-24). */
+				val = NPCX_VWEVSM(i) & 0x0F00000F;
 				val |= VWEVSM_FIELD(config->idx,
 						config->idx_en,
 						config->valid,
@@ -209,7 +213,7 @@ static void espi_vw_config_out(const struct vwevsm_config_t *config)
 	}
 }
 
-/* Config Master-to-Slave VWire interrupt edge type and enable it */
+/* Config Controller-to-Peripheral VWire interrupt edge type and enable it */
 static void espi_enable_vw_int(const struct host_wui_item *vwire_int)
 {
 	uint8_t table = vwire_int->table;
@@ -233,6 +237,9 @@ static void espi_enable_vw_int(const struct host_wui_item *vwire_int)
 		/* enable Any Edge */
 		SET_BIT(NPCX_WKAEDG(table, group), num);
 
+	/* Clear the pending bit */
+	NPCX_WKPCL(table, group) = BIT(num);
+
 	/* Enable wake-up input sources */
 	SET_BIT(NPCX_WKEN(table, group), num);
 }
@@ -257,8 +264,9 @@ static int espi_vw_get_signal_index(enum espi_vw_signal event)
 /* The ISRs of VW signals which used for power sequences */
 void espi_vw_power_signal_interrupt(enum espi_vw_signal signal)
 {
-	/* TODO: Add VW handler in power/common.c */
-	power_signal_interrupt((enum gpio_signal) signal);
+	if (IS_ENABLED(CONFIG_HOST_ESPI_VW_POWER_SIGNAL))
+		/* TODO: Add VW handler in power/common.c */
+		power_signal_interrupt((enum gpio_signal) signal);
 }
 
 /*****************************************************************************/
@@ -273,7 +281,8 @@ void espi_vw_power_signal_interrupt(enum espi_vw_signal signal)
  */
 int espi_vw_set_wire(enum espi_vw_signal signal, uint8_t level)
 {
-	uint8_t offset, sig_idx, value;
+	uint8_t offset, value;
+	int sig_idx;
 
 	/* Get index of vw signal list by signale name */
 	sig_idx = espi_vw_get_signal_index(signal);
@@ -314,7 +323,8 @@ int espi_vw_set_wire(enum espi_vw_signal signal, uint8_t level)
  */
 int espi_vw_get_wire(enum espi_vw_signal signal)
 {
-	uint8_t offset, sig_idx, value;
+	uint8_t offset, value;
+	int sig_idx;
 
 	/* Get index of vw signal list by signale name */
 	sig_idx = espi_vw_get_signal_index(signal);
@@ -402,17 +412,27 @@ void espi_vw_evt_pltrst(void)
 
 	if (pltrst) {
 		/* PLTRST# deasserted */
+#if defined(CHIP_FAMILY_NPCX5)
+		/* See errata 2.22 */
+
 		/* Disable eSPI peripheral channel support first */
 		CLEAR_BIT(NPCX_ESPICFG, NPCX_ESPICFG_PCCHN_SUPP);
 
-		/* Enable eSPI peripheral channel */
-		SET_BIT(NPCX_ESPICFG, NPCX_ESPICFG_PCHANEN);
-
 		/* Initialize host settings */
 		host_register_init();
 
+		/* Enable eSPI peripheral channel */
+		SET_BIT(NPCX_ESPICFG, NPCX_ESPICFG_PCHANEN);
+
 		/* Re-enable eSPI peripheral channel support */
 		SET_BIT(NPCX_ESPICFG, NPCX_ESPICFG_PCCHN_SUPP);
+#else
+		/* Initialize host settings */
+		host_register_init();
+
+		/* Enable eSPI peripheral channel */
+		SET_BIT(NPCX_ESPICFG, NPCX_ESPICFG_PCHANEN);
+#endif
 	} else {
 		/* PLTRST# asserted */
 #ifdef CONFIG_CHIPSET_RESET_HOOK
@@ -482,7 +502,7 @@ void espi_espirst_handler(void)
 }
 
 /* Handle eSPI virtual wire interrupt 1 */
-void __espi_wk2a_interrupt(void)
+static void __espi_wk2a_interrupt(void)
 {
 	uint8_t pending_bits = NPCX_WKPND(MIWU_TABLE_2, MIWU_GROUP_1);
 
@@ -501,10 +521,10 @@ void __espi_wk2a_interrupt(void)
 	if (IS_BIT_SET(pending_bits, 6))
 		espi_vw_evt_oobrst();
 }
-DECLARE_IRQ(NPCX_IRQ_WKINTA_2, __espi_wk2a_interrupt, 2);
+DECLARE_IRQ(NPCX_IRQ_WKINTA_2, __espi_wk2a_interrupt, 3);
 
 /* Handle eSPI virtual wire interrupt 2 */
-void __espi_wk2b_interrupt(void)
+static void __espi_wk2b_interrupt(void)
 {
 	uint8_t pending_bits = NPCX_WKPND(MIWU_TABLE_2, MIWU_GROUP_2);
 
@@ -517,15 +537,23 @@ void __espi_wk2b_interrupt(void)
 	if (IS_BIT_SET(pending_bits, 0))
 		espi_vw_evt_hostrst_warn();
 }
-DECLARE_IRQ(NPCX_IRQ_WKINTB_2, __espi_wk2b_interrupt, 2);
+DECLARE_IRQ(NPCX_IRQ_WKINTB_2, __espi_wk2b_interrupt, 3);
 
 /* Interrupt handler for eSPI status changed */
-void espi_interrupt(void)
+static void espi_interrupt(void)
 {
 	int chan;
 	uint32_t mask, status;
 
+#if NPCX_FAMILY_VERSION >= NPCX_FAMILY_NPCX7
+	/*
+	 * Bit 17 of ESPIIE is reserved. We need to set the same bit in mask
+	 * in case bit 17 in ESPISTS of npcx7 is not cleared in ISR.
+	 */
+	mask = NPCX_ESPIIE | BIT(NPCX_ESPISTS_VWUPDW);
+#else
 	mask = NPCX_ESPIIE;
+#endif
 	status = NPCX_ESPISTS & mask;
 
 	while (status) {
@@ -533,7 +561,8 @@ void espi_interrupt(void)
 		NPCX_ESPISTS = status;
 
 		if (IS_BIT_SET(status, NPCX_ESPISTS_BERR))
-			CPRINTS("eSPI Bus Error");
+			/* Always print eSPI Bus Errors */
+			cprints(CC_LPC, "eSPI Bus Error");
 
 		/* eSPI inband reset(from VW) */
 		if (IS_BIT_SET(status, NPCX_ESPISTS_IBRST)) {
@@ -556,23 +585,24 @@ void espi_interrupt(void)
 			 */
 			for (chan = NPCX_ESPI_CH_VW; chan < NPCX_ESPI_CH_COUNT;
 					chan++) {
-				if (!IS_SLAVE_CHAN_ENABLE(chan) &&
+				if (!IS_PERIPHERAL_CHAN_ENABLE(chan) &&
 						IS_HOST_CHAN_EN(chan))
 					ENABLE_ESPI_CHAN(chan);
-				else if (IS_SLAVE_CHAN_ENABLE(chan) &&
+				else if (IS_PERIPHERAL_CHAN_ENABLE(chan) &&
 						!IS_HOST_CHAN_EN(chan))
 					DISABLE_ESPI_CHAN(chan);
 			}
 
 			/*
-			 * Send SLAVE_BOOTLOAD_DONE and SLAVE_BOOTLOAD_STATUS
+			 * Send BOOTLOAD_DONE and BOOTLOAD_STATUS
 			 * events to host simultaneously. To indicate the
 			 * completion of EC firmware code loading.
 			 */
 			if (boot_load_done == 0 &&
-					IS_SLAVE_CHAN_ENABLE(NPCX_ESPI_CH_VW)) {
+			    IS_PERIPHERAL_CHAN_ENABLE(NPCX_ESPI_CH_VW)) {
 
-				espi_vw_set_wire(VW_SLAVE_BTLD_STATUS_DONE, 1);
+				espi_vw_set_wire(
+					VW_PERIPHERAL_BTLD_STATUS_DONE, 1);
 				boot_load_done = 1;
 			}
 		}
@@ -585,7 +615,7 @@ void espi_interrupt(void)
 		status = NPCX_ESPISTS & mask;
 	}
 }
-DECLARE_IRQ(NPCX_IRQ_ESPI, espi_interrupt, 3);
+DECLARE_IRQ(NPCX_IRQ_ESPI, espi_interrupt, 4);
 
 /*****************************************************************************/
 /* eSPI Initialization functions */
@@ -597,18 +627,18 @@ void espi_init(void)
 	NPCX_ESPICFG |= ESPI_SUPP_CH_ALL;
 
 	/* Support all I/O modes */
-	SET_FIELD(NPCX_ESPICFG, NPCX_ESPICFG_IOMODE_FILED,
+	SET_FIELD(NPCX_ESPICFG, NPCX_ESPICFG_IOMODE_FIELD,
 		NPCX_ESPI_IO_MODE_ALL);
 
-	/* Max freq 66 MHz of eSPI */
-	SET_FIELD(NPCX_ESPICFG, NPCX_ESPICFG_MAXFREQ_FILED,
-			NPCX_ESPI_MAXFREQ_66);
+	/* Set eSPI speed to max supported */
+	SET_FIELD(NPCX_ESPICFG, NPCX_ESPICFG_MAXFREQ_FIELD,
+		  NPCX_ESPI_MAXFREQ_MAX);
 
-	/* Configure Master-to-Slave Virtual Wire indexes (Inputs) */
+	/* Configure Controller-to-Peripheral Virtual Wire indexes (Inputs) */
 	for (i = 0; i < ARRAY_SIZE(espi_in_list); i++)
 		espi_vw_config_in(&espi_in_list[i]);
 
-	/* Configure Slave-to-Master Virtual Wire indexes (Outputs) */
+	/* Configure Peripheral-to-Controller Virtual Wire indexes (Outputs) */
 	for (i = 0; i < ARRAY_SIZE(espi_out_list); i++)
 		espi_vw_config_out(&espi_out_list[i]);
 
@@ -653,7 +683,7 @@ static int command_espi(int argc, char **argv)
 
 		if (*e)
 			return EC_ERROR_PARAM2;
-		if (m < 0 || m > 4)
+		if (m > 4)
 			return EC_ERROR_PARAM2;
 		else if (m == 4)
 			chan = 0x0F;
@@ -672,4 +702,3 @@ static int command_espi(int argc, char **argv)
 DECLARE_CONSOLE_COMMAND(espi, command_espi,
 			"cfg/vms/vsm/en/dis [channel]",
 			"eSPI configurations");
-
