@@ -1,4 +1,4 @@
-/* Copyright (c) 2014 The Chromium OS Authors. All rights reserved.
+/* Copyright 2014 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -21,14 +21,11 @@
 static int all_protected; /* Has all-flash protection been requested? */
 static int addr_prot_start;
 static int addr_prot_length;
-static uint8_t flag_prot_inconsistent;
 
 /* SR regs aren't readable when UMA lock is on, so save a copy */
 static uint8_t saved_sr1;
 static uint8_t saved_sr2;
 
-#define FLASH_ABORT_TIMEOUT     10000
-
 #ifdef CONFIG_EXTERNAL_STORAGE
 #define TRISTATE_FLASH(x)
 #else
@@ -40,6 +37,7 @@ static struct mutex flash_lock;
 
 /*****************************************************************************/
 /* flash internal functions */
+#if !defined(NPCX_INT_FLASH_SUPPORT)
 static void flash_pinmux(int enable)
 {
 	/* Select pin-mux for FIU*/
@@ -57,11 +55,15 @@ static void flash_pinmux(int enable)
 		CLEAR_BIT(NPCX_DEVALT(0), NPCX_DEVALT0_F_SPI_CS1_2);
 	}
 }
+#endif
 
 static void flash_execute_cmd(uint8_t code, uint8_t cts)
 {
-	/* Flash mutex must be held while executing UMA commands. */
-	ASSERT(flash_lock.lock);
+	/*
+	 * Flash mutex must be held while executing UMA commands after
+	 * task_start().
+	 */
+	ASSERT(!task_start_called() || flash_lock.lock);
 
 	/* set UMA_CODE */
 	NPCX_UMA_CODE = code;
@@ -77,18 +79,20 @@ static void flash_cs_level(int level)
 	UPDATE_BIT(NPCX_UMA_ECTS, NPCX_UMA_ECTS_SW_CS1, level);
 }
 
-static int flash_wait_ready(int timeout)
+static int flash_wait_ready(void)
 {
 	uint8_t mask = SPI_FLASH_SR1_BUSY;
-
-	if (timeout <= 0)
-		return EC_ERROR_INVAL;
+	const timestamp_t start = get_time();
+	const uint32_t timeout_us = 10 * SECOND;
+	const timestamp_t deadline = {
+		.val = start.val + timeout_us,
+	};
 
 	/* Chip Select down. */
 	flash_cs_level(0);
 	/* Command for Read status register */
 	flash_execute_cmd(CMD_READ_STATUS_REG, MASK_CMD_ONLY);
-	while (timeout > 0) {
+	do {
 		/* Read status register */
 		NPCX_UMA_CTS  = MASK_RD_1BYTE;
 		while (IS_BIT_SET(NPCX_UMA_CTS, NPCX_UMA_CTS_EXEC_DONE))
@@ -96,14 +100,13 @@ static int flash_wait_ready(int timeout)
 		/* Busy bit is clear */
 		if ((NPCX_UMA_DB0 & mask) == 0)
 			break;
-		if (--timeout > 0)
-			msleep(1);
-	}; /* Wait for Busy clear */
+		usleep(10);
+	} while (!timestamp_expired(deadline, NULL)); /* Wait for Busy clear */
 
 	/* Chip Select high. */
 	flash_cs_level(1);
 
-	if (timeout == 0)
+	if (timestamp_expired(deadline, NULL))
 		return EC_ERROR_TIMEOUT;
 
 	return EC_SUCCESS;
@@ -114,7 +117,7 @@ static int flash_write_enable(void)
 	uint8_t mask = SPI_FLASH_SR1_WEL;
 	int rv;
 	/* Wait for previous operation to complete */
-	rv = flash_wait_ready(FLASH_ABORT_TIMEOUT);
+	rv = flash_wait_ready();
 	if (rv)
 		return rv;
 
@@ -122,7 +125,7 @@ static int flash_write_enable(void)
 	flash_execute_cmd(CMD_WRITE_EN, MASK_CMD_ONLY);
 
 	/* Wait for flash is not busy */
-	rv = flash_wait_ready(FLASH_ABORT_TIMEOUT);
+	rv = flash_wait_ready();
 	if (rv)
 		return rv;
 
@@ -141,64 +144,85 @@ static void flash_set_address(uint32_t dest_addr)
 	NPCX_UMA_AB0 = addr[0];
 }
 
-static uint8_t flash_get_status1(void)
+static void flash_get_status(uint8_t *sr1, uint8_t *sr2)
 {
-	uint8_t ret;
-
-	if (all_protected)
-		return saved_sr1;
+	if (all_protected) {
+		*sr1 = saved_sr1;
+		*sr2 = saved_sr2;
+		return;
+	}
 
 	/* Lock physical flash operations */
-	flash_lock_mapped_storage(1);
+	crec_flash_lock_mapped_storage(1);
 
 	/* Disable tri-state */
 	TRISTATE_FLASH(0);
 	/* Read status register1 */
 	flash_execute_cmd(CMD_READ_STATUS_REG, MASK_CMD_RD_1BYTE);
+	*sr1 = NPCX_UMA_DB0;
+	/* Read status register2 */
+	flash_execute_cmd(CMD_READ_STATUS_REG2, MASK_CMD_RD_1BYTE);
+	*sr2 = NPCX_UMA_DB0;
 	/* Enable tri-state */
 	TRISTATE_FLASH(1);
 
-	ret = NPCX_UMA_DB0;
-
 	/* Unlock physical flash operations */
-	flash_lock_mapped_storage(0);
-
-	return ret;
+	crec_flash_lock_mapped_storage(0);
 }
 
-static uint8_t flash_get_status2(void)
+static void flash_set_status(uint8_t sr1, uint8_t sr2)
 {
-	uint8_t ret;
-
-	if (all_protected)
-		return saved_sr2;
-
 	/* Lock physical flash operations */
-	flash_lock_mapped_storage(1);
+	crec_flash_lock_mapped_storage(1);
 
 	/* Disable tri-state */
 	TRISTATE_FLASH(0);
-	/* Read status register2 */
-	flash_execute_cmd(CMD_READ_STATUS_REG2, MASK_CMD_RD_1BYTE);
+	/* Enable write */
+	flash_write_enable();
+
+	NPCX_UMA_DB0 = sr1;
+	NPCX_UMA_DB1 = sr2;
+
+	/* Write status register 1/2 */
+	flash_execute_cmd(CMD_WRITE_STATUS_REG, MASK_CMD_WR_2BYTE);
 	/* Enable tri-state */
 	TRISTATE_FLASH(1);
 
-	ret = NPCX_UMA_DB0;
-
 	/* Unlock physical flash operations */
-	flash_lock_mapped_storage(0);
+	crec_flash_lock_mapped_storage(0);
+}
 
-	return ret;
+static void flash_set_quad_enable(int enable)
+{
+	uint8_t sr1, sr2;
+
+	flash_get_status(&sr1, &sr2);
+
+	/* If QE is the same value, return directly. */
+	if (!!(sr2 & SPI_FLASH_SR2_QE) == enable)
+		return;
+
+	if (enable)
+		sr2 |= SPI_FLASH_SR2_QE;
+	else
+		sr2 &= ~SPI_FLASH_SR2_QE;
+
+	flash_set_status(sr1, sr2);
 }
 
 #ifdef NPCX_INT_FLASH_SUPPORT
+static int is_int_flash_protected(void)
+{
+	return IS_BIT_SET(NPCX_DEV_CTL4, NPCX_DEV_CTL4_WP_IF);
+}
+
 static void flash_protect_int_flash(int enable)
 {
 	/*
 	 * Please notice the type of WP_IF bit is R/W1S. Once it's set,
 	 * only rebooting EC can clear it.
 	 */
-	if (enable && !IS_BIT_SET(NPCX_DEV_CTL4, NPCX_DEV_CTL4_WP_IF))
+	if (enable && !is_int_flash_protected())
 		SET_BIT(NPCX_DEV_CTL4, NPCX_DEV_CTL4_WP_IF);
 }
 #endif
@@ -208,7 +232,7 @@ static void flash_protect_int_flash(int enable)
 void flash_get_mfr_dev_id(uint8_t *dest)
 {
 	/* Lock physical flash operations */
-	flash_lock_mapped_storage(1);
+	crec_flash_lock_mapped_storage(1);
 
 	/* Disable tri-state */
 	TRISTATE_FLASH(0);
@@ -223,13 +247,15 @@ void flash_get_mfr_dev_id(uint8_t *dest)
 	dest[1] = NPCX_UMA_DB1;
 
 	/* Unlock physical flash operations */
-	flash_lock_mapped_storage(0);
+	crec_flash_lock_mapped_storage(0);
 }
 
+#endif /* CONFIG_HOSTCMD_FLASH_SPI_INFO */
+
 void flash_get_jedec_id(uint8_t *dest)
 {
 	/* Lock physical flash operations */
-	flash_lock_mapped_storage(1);
+	crec_flash_lock_mapped_storage(1);
 
 	/* Disable tri-state */
 	TRISTATE_FLASH(0);
@@ -243,11 +269,9 @@ void flash_get_jedec_id(uint8_t *dest)
 	dest[2] = NPCX_UMA_DB2;
 
 	/* Unlock physical flash operations */
-	flash_lock_mapped_storage(0);
+	crec_flash_lock_mapped_storage(0);
 }
 
-#endif /* CONFIG_HOSTCMD_FLASH_SPI_INFO */
-
 static void flash_uma_lock(int enable)
 {
 	if (enable && !all_protected) {
@@ -255,8 +279,7 @@ static void flash_uma_lock(int enable)
 		 * Store SR1 / SR2 for later use since we're about to lock
 		 * out all access (including read access) to these regs.
 		 */
-		saved_sr1 = flash_get_status1();
-		saved_sr2 = flash_get_status2();
+		flash_get_status(&saved_sr1, &saved_sr2);
 	}
 
 	all_protected = enable;
@@ -270,7 +293,12 @@ static int flash_set_status_for_prot(int reg1, int reg2)
 	 * is deasserted then remove the lock and allow the write.
 	 */
 	if (all_protected) {
-		if (flash_get_protect() & EC_FLASH_PROTECT_GPIO_ASSERTED)
+#ifdef NPCX_INT_FLASH_SUPPORT
+		if (is_int_flash_protected())
+			return EC_ERROR_ACCESS_DENIED;
+#endif
+
+		if (crec_flash_get_protect() & EC_FLASH_PROTECT_GPIO_ASSERTED)
 			return EC_ERROR_ACCESS_DENIED;
 		flash_uma_lock(0);
 	}
@@ -280,27 +308,13 @@ static int flash_set_status_for_prot(int reg1, int reg2)
 	 * internal spi-flash, protect it now before setting them.
 	 */
 #ifdef NPCX_INT_FLASH_SUPPORT
+#ifdef CONFIG_WP_ACTIVE_HIGH
+	flash_protect_int_flash(gpio_get_level(GPIO_WP));
+#else
 	flash_protect_int_flash(!gpio_get_level(GPIO_WP_L));
+#endif /*_CONFIG_WP_ACTIVE_HIGH_*/
 #endif
-
-	/* Lock physical flash operations */
-	flash_lock_mapped_storage(1);
-
-	/* Disable tri-state */
-	TRISTATE_FLASH(0);
-	/* Enable write */
-	flash_write_enable();
-
-	NPCX_UMA_DB0 = reg1;
-	NPCX_UMA_DB1 = reg2;
-
-	/* Write status register 1/2 */
-	flash_execute_cmd(CMD_WRITE_STATUS_REG, MASK_CMD_WR_2BYTE);
-	/* Enable tri-state */
-	TRISTATE_FLASH(1);
-
-	/* Unlock physical flash operations */
-	flash_lock_mapped_storage(0);
+	flash_set_status(reg1, reg2);
 
 	spi_flash_reg_to_protect(reg1, reg2,
 				 &addr_prot_start, &addr_prot_length);
@@ -311,7 +325,7 @@ static int flash_set_status_for_prot(int reg1, int reg2)
 static int flash_check_prot_range(unsigned int offset, unsigned int bytes)
 {
 	/* Invalid value */
-	if (offset + bytes > CONFIG_FLASH_SIZE)
+	if (offset + bytes > CONFIG_FLASH_SIZE_BYTES)
 		return EC_ERROR_INVAL;
 	/* Check if ranges overlap */
 	if (MAX(addr_prot_start, offset) < MIN(addr_prot_start +
@@ -333,17 +347,19 @@ static int flash_check_prot_reg(unsigned int offset, unsigned int bytes)
 	 * internal spi-flash, protect it now.
 	 */
 #ifdef NPCX_INT_FLASH_SUPPORT
+#ifdef CONFIG_WP_ACTIVE_HIGH
+	flash_protect_int_flash(gpio_get_level(GPIO_WP));
+#else
 	flash_protect_int_flash(!gpio_get_level(GPIO_WP_L));
+#endif /* CONFIG_WP_ACTIVE_HIGH */
 #endif
 
-	sr1 = flash_get_status1();
-	sr2 = flash_get_status2();
-
 	/* Invalid value */
-	if (offset + bytes > CONFIG_FLASH_SIZE)
+	if (offset + bytes > CONFIG_FLASH_SIZE_BYTES)
 		return EC_ERROR_INVAL;
 
 	/* Compute current protect range */
+	flash_get_status(&sr1, &sr2);
 	rv = spi_flash_reg_to_protect(sr1, sr2, &start, &len);
 	if (rv)
 		return rv;
@@ -360,14 +376,14 @@ static int flash_write_prot_reg(unsigned int offset, unsigned int bytes,
 				int hw_protect)
 {
 	int rv;
-	uint8_t sr1 = flash_get_status1();
-	uint8_t sr2 = flash_get_status2();
+	uint8_t sr1, sr2;
 
 	/* Invalid values */
-	if (offset + bytes > CONFIG_FLASH_SIZE)
+	if (offset + bytes > CONFIG_FLASH_SIZE_BYTES)
 		return EC_ERROR_INVAL;
 
 	/* Compute desired protect range */
+	flash_get_status(&sr1, &sr2);
 	rv = spi_flash_protect_to_reg(offset, bytes, &sr1, &sr2);
 	if (rv)
 		return rv;
@@ -397,7 +413,7 @@ static void flash_burst_write(unsigned int dest_addr, unsigned int bytes,
 }
 
 static int flash_program_bytes(uint32_t offset, uint32_t bytes,
-	const uint8_t const *data)
+	const uint8_t *data)
 {
 	int write_size;
 	int rv;
@@ -416,7 +432,7 @@ static int flash_program_bytes(uint32_t offset, uint32_t bytes,
 		flash_burst_write(offset, write_size, data);
 
 		/* Wait write completed */
-		rv = flash_wait_ready(FLASH_ABORT_TIMEOUT);
+		rv = flash_wait_ready();
 		if (rv)
 			return rv;
 
@@ -430,13 +446,13 @@ static int flash_program_bytes(uint32_t offset, uint32_t bytes,
 
 /*****************************************************************************/
 
-int flash_physical_read(int offset, int size, char *data)
+int crec_flash_physical_read(int offset, int size, char *data)
 {
 	int dest_addr = offset;
 	uint32_t idx;
 
 	/* Lock physical flash operations */
-	flash_lock_mapped_storage(1);
+	crec_flash_lock_mapped_storage(1);
 
 	/* Disable tri-state */
 	TRISTATE_FLASH(0);
@@ -465,12 +481,12 @@ int flash_physical_read(int offset, int size, char *data)
 	TRISTATE_FLASH(1);
 
 	/* Unlock physical flash operations */
-	flash_lock_mapped_storage(0);
+	crec_flash_lock_mapped_storage(0);
 
 	return EC_SUCCESS;
 }
 
-int flash_physical_write(int offset, int size, const char *data)
+int crec_flash_physical_write(int offset, int size, const char *data)
 {
 	int dest_addr = offset;
 	int write_len;
@@ -486,7 +502,7 @@ int flash_physical_write(int offset, int size, const char *data)
 		return EC_ERROR_ACCESS_DENIED;
 
 	/* Lock physical flash operations */
-	flash_lock_mapped_storage(1);
+	crec_flash_lock_mapped_storage(1);
 
 	/* Disable tri-state */
 	TRISTATE_FLASH(0);
@@ -515,12 +531,12 @@ int flash_physical_write(int offset, int size, const char *data)
 	TRISTATE_FLASH(1);
 
 	/* Unlock physical flash operations */
-	flash_lock_mapped_storage(0);
+	crec_flash_lock_mapped_storage(0);
 
 	return rv;
 }
 
-int flash_physical_erase(int offset, int size)
+int crec_flash_physical_erase(int offset, int size)
 {
 	int rv = EC_SUCCESS;
 	/* check protection */
@@ -528,7 +544,7 @@ int flash_physical_erase(int offset, int size)
 		return EC_ERROR_ACCESS_DENIED;
 
 	/* Lock physical flash operations */
-	flash_lock_mapped_storage(1);
+	crec_flash_lock_mapped_storage(1);
 
 	/* Disable tri-state */
 	TRISTATE_FLASH(0);
@@ -557,10 +573,10 @@ int flash_physical_erase(int offset, int size)
 		/* Set erase address */
 		flash_set_address(offset);
 		/* Start erase */
-		flash_execute_cmd(CMD_SECTOR_ERASE, MASK_CMD_ADR);
+		flash_execute_cmd(NPCX_ERASE_COMMAND, MASK_CMD_ADR);
 
 		/* Wait erase completed */
-		rv = flash_wait_ready(FLASH_ABORT_TIMEOUT);
+		rv = flash_wait_ready();
 		if (rv)
 			break;
 	}
@@ -569,32 +585,42 @@ int flash_physical_erase(int offset, int size)
 	TRISTATE_FLASH(1);
 
 	/* Unlock physical flash operations */
-	flash_lock_mapped_storage(0);
+	crec_flash_lock_mapped_storage(0);
 
 	return rv;
 }
 
-int flash_physical_get_protect(int bank)
+int crec_flash_physical_get_protect(int bank)
 {
 	uint32_t addr = bank * CONFIG_FLASH_BANK_SIZE;
 
 	return flash_check_prot_reg(addr, CONFIG_FLASH_BANK_SIZE);
 }
 
-uint32_t flash_physical_get_protect_flags(void)
+uint32_t crec_flash_physical_get_protect_flags(void)
 {
 	uint32_t flags = 0;
+	uint8_t sr1, sr2;
+	unsigned int start, len;
+	int rv;
 
 	/* Check if WP region is protected in status register */
-	if (flash_check_prot_reg(WP_BANK_OFFSET*CONFIG_FLASH_BANK_SIZE,
-				 WP_BANK_COUNT*CONFIG_FLASH_BANK_SIZE))
+	rv = flash_check_prot_reg(WP_BANK_OFFSET * CONFIG_FLASH_BANK_SIZE,
+				  WP_BANK_COUNT * CONFIG_FLASH_BANK_SIZE);
+	if (rv == EC_ERROR_ACCESS_DENIED)
 		flags |= EC_FLASH_PROTECT_RO_AT_BOOT;
+	else if (rv)
+		return EC_FLASH_PROTECT_ERROR_UNKNOWN;
 
 	/*
-	 * TODO: If status register protects a range, but SRP0 is not set,
-	 * flags should indicate EC_FLASH_PROTECT_ERROR_INCONSISTENT.
+	 * If the status register protects a range, but SRP0 is not set, or QE
+	 * is set, flags should indicate EC_FLASH_PROTECT_ERROR_INCONSISTENT.
 	 */
-	if (flag_prot_inconsistent)
+	flash_get_status(&sr1, &sr2);
+	rv = spi_flash_reg_to_protect(sr1, sr2, &start, &len);
+	if (rv)
+		return EC_FLASH_PROTECT_ERROR_UNKNOWN;
+	if (len && (!(sr1 & SPI_FLASH_SR1_SRP0) || (sr2 & SPI_FLASH_SR2_QE)))
 		flags |= EC_FLASH_PROTECT_ERROR_INCONSISTENT;
 
 	/* Read all-protected state from our shadow copy */
@@ -604,7 +630,7 @@ uint32_t flash_physical_get_protect_flags(void)
 	return flags;
 }
 
-int flash_physical_protect_now(int all)
+int crec_flash_physical_protect_now(int all)
 {
 	if (all) {
 		/*
@@ -620,7 +646,7 @@ int flash_physical_protect_now(int all)
 }
 
 
-int flash_physical_protect_at_boot(uint32_t new_flags)
+int crec_flash_physical_protect_at_boot(uint32_t new_flags)
 {
 	int ret;
 
@@ -644,14 +670,14 @@ int flash_physical_protect_at_boot(uint32_t new_flags)
 	return ret;
 }
 
-uint32_t flash_physical_get_valid_flags(void)
+uint32_t crec_flash_physical_get_valid_flags(void)
 {
 	return EC_FLASH_PROTECT_RO_AT_BOOT |
 			EC_FLASH_PROTECT_RO_NOW |
 			EC_FLASH_PROTECT_ALL_NOW;
 }
 
-uint32_t flash_physical_get_writable_flags(uint32_t cur_flags)
+uint32_t crec_flash_physical_get_writable_flags(uint32_t cur_flags)
 {
 	uint32_t ret = 0;
 
@@ -673,30 +699,41 @@ uint32_t flash_physical_get_writable_flags(uint32_t cur_flags)
 /*****************************************************************************/
 /* High-level APIs */
 
-int flash_pre_init(void)
+int crec_flash_pre_init(void)
 {
-	/*
-	 * Protect status registers of internal spi-flash if WP# is active
-	 * during ec initialization.
-	 */
-#ifdef NPCX_INT_FLASH_SUPPORT
-	flash_protect_int_flash(!gpio_get_level(GPIO_WP_L));
-#endif
-
+#if !defined(NPCX_INT_FLASH_SUPPORT)
 	/* Enable FIU interface */
 	flash_pinmux(1);
+#endif
 
-#ifdef CONFIG_EXTERNAL_STORAGE
+#if defined(CONFIG_EXTERNAL_STORAGE) && !defined(NPCX_INT_FLASH_SUPPORT)
 	/* Disable tristate all the time */
 	CLEAR_BIT(NPCX_DEVCNT, NPCX_DEVCNT_F_SPI_TRIS);
 #endif
 
 	/* Initialize UMA to unlocked */
 	flash_uma_lock(0);
+
+	/*
+	 * Disable flash quad enable to avoid /WP pin function is not
+	 * available. */
+	flash_set_quad_enable(0);
+
+	/*
+	 * Protect status registers of internal spi-flash if WP# is active
+	 * during ec initialization.
+	 */
+#ifdef NPCX_INT_FLASH_SUPPORT
+#ifdef CONFIG_WP_ACTIVE_HIGH
+	flash_protect_int_flash(gpio_get_level(GPIO_WP));
+#else
+	flash_protect_int_flash(!gpio_get_level(GPIO_WP_L));
+#endif /*CONFIG_WP_ACTIVE_HIGH */
+#endif
 	return EC_SUCCESS;
 }
 
-void flash_lock_mapped_storage(int lock)
+void crec_flash_lock_mapped_storage(int lock)
 {
 	if (lock)
 		mutex_lock(&flash_lock);
@@ -710,15 +747,14 @@ void flash_lock_mapped_storage(int lock)
 #if defined(CONFIG_HOSTCMD_FLASH_SPI_INFO) && !defined(BOARD_NPCX_EVB)
 /* NPCX EVB uses implementation from spi_flash.c */
 
-static int flash_command_spi_info(struct host_cmd_handler_args *args)
+static enum ec_status flash_command_spi_info(struct host_cmd_handler_args *args)
 {
 	struct ec_response_flash_spi_info *r = args->response;
 
 	flash_get_jedec_id(r->jedec);
 	r->reserved0 = 0;
 	flash_get_mfr_dev_id(r->mfr_dev_id);
-	r->sr1 = flash_get_status1();
-	r->sr2 = flash_get_status2();
+	flash_get_status(&r->sr1, &r->sr2);
 
 	args->response_size = sizeof(*r);
 	return EC_RES_SUCCESS;
@@ -730,6 +766,9 @@ DECLARE_HOST_COMMAND(EC_CMD_FLASH_SPI_INFO,
 #endif
 
 #ifdef CONFIG_CMD_FLASH_TRISTATE
+#ifdef NPCX_INT_FLASH_SUPPORT
+#error "Flash tristate is not relevant when internal flash is used."
+#endif
 static void flash_tristate(int enable)
 {
 	/* Enable/Disable FIU pins to tri-state */
@@ -762,7 +801,7 @@ static int command_flash_spi_sel_lock(int argc, char **argv)
 	return EC_SUCCESS;
 }
 DECLARE_CONSOLE_COMMAND(flash_spi_lock, command_flash_spi_sel_lock,
-			"[0 | 1]",
+			"[on | off]",
 			"Lock spi flash interface selection");
 
 static int command_flash_tristate(int argc, char **argv)
@@ -779,6 +818,24 @@ static int command_flash_tristate(int argc, char **argv)
 	return EC_SUCCESS;
 }
 DECLARE_CONSOLE_COMMAND(flash_tristate, command_flash_tristate,
-			"[0 | 1]",
+			"[on | off]",
 			"Tristate spi flash pins");
 #endif /* CONFIG_CMD_FLASH_TRISTATE */
+
+static int command_flash_chip(int argc, char **argv)
+{
+	uint8_t jedec_id[3];
+	uint8_t sr1, sr2;
+
+	flash_get_status(&sr1, &sr2);
+	ccprintf("Status 1: 0x%02x, Status 2: 0x%02x\n", sr1, sr2);
+
+	flash_get_jedec_id(jedec_id);
+	ccprintf("Manufacturer: 0x%02x, DID: 0x%02x%02x\n", jedec_id[0],
+			jedec_id[1], jedec_id[2]);
+
+	return EC_SUCCESS;
+}
+DECLARE_CONSOLE_COMMAND(flashchip, command_flash_chip,
+			NULL,
+			"Print flash chip info");
