@@ -1,4 +1,4 @@
-/* Copyright (c) 2014 The Chromium OS Authors. All rights reserved.
+/* Copyright 2014 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -13,15 +13,15 @@
 #include "gpio.h"
 #include "hooks.h"
 #include "host_command.h"
+#include "i8042_protocol.h"
 #include "keyboard_protocol.h"
 #include "lpc.h"
 #include "lpc_chip.h"
 #include "port80.h"
-#include "pwm.h"
 #include "registers.h"
 #include "system.h"
+#include "sib_chip.h"
 #include "task.h"
-#include "timer.h"
 #include "uart.h"
 #include "util.h"
 #include "system_chip.h"
@@ -35,32 +35,13 @@
 #define CPRINTS(format, args...) cprints(CC_LPC, format, ## args)
 #endif
 
-#define LPC_SYSJUMP_TAG 0x4c50          /* "LP" */
-
 /* PM channel definitions */
 #define PMC_ACPI     PM_CHAN_1
 #define PMC_HOST_CMD PM_CHAN_2
 
-/* Super-IO index and register definitions */
-#define SIO_OFFSET      0x4E
-#define INDEX_SID       0x20
-#define INDEX_CHPREV    0x24
-#define INDEX_SRID      0x27
+#define PORT80_MAX_BUF_SIZE    16
+static uint16_t port80_buf[PORT80_MAX_BUF_SIZE];
 
-/*
- * Timeout to wait for host transaction to be completed.
- *
- * For eSPI - it is 200 us.
- * For LPC - it is 5 us.
- */
-#ifdef CONFIG_ESPI
-#define LPC_HOST_TRANSACTION_TIMEOUT_US 200
-#else
-#define LPC_HOST_TRANSACTION_TIMEOUT_US 5
-#endif
-
-static uint32_t host_events;            /* Currently pending SCI/SMI events */
-static uint32_t event_mask[3];          /* Event masks for each type */
 static struct	host_packet lpc_packet;
 static struct	host_cmd_handler_args host_cmd_args;
 static uint8_t	host_cmd_flags;         /* Flags from host command */
@@ -70,8 +51,6 @@ static uint8_t	shm_memmap[256] __aligned(8);
 static uint8_t params_copy[EC_LPC_HOST_PACKET_SIZE] __aligned(4);
 static int init_done;
 
-static uint8_t * const cmd_params = (uint8_t *)shm_mem_host_cmd +
-		EC_LPC_ADDR_HOST_PARAM - EC_LPC_ADDR_HOST_ARGS;
 static struct ec_lpc_host_args * const lpc_host_args =
 		(struct ec_lpc_host_args *)shm_mem_host_cmd;
 
@@ -102,33 +81,39 @@ static void keyboard_irq_assert(void)
 
 static void lpc_task_enable_irq(void)
 {
+#ifdef HAS_TASK_KEYPROTO
 	task_enable_irq(NPCX_IRQ_KBC_IBF);
+#endif
 	task_enable_irq(NPCX_IRQ_PM_CHAN_IBF);
 	task_enable_irq(NPCX_IRQ_PORT80);
-#ifdef CONFIG_ESPI
+#ifdef CONFIG_HOST_INTERFACE_ESPI
 	task_enable_irq(NPCX_IRQ_ESPI);
 	/* Virtual Wire: SLP_S3/4/5, SUS_STAT, PLTRST, OOB_RST_WARN */
 	task_enable_irq(NPCX_IRQ_WKINTA_2);
 	/* Virtual Wire: HOST_RST_WARN, SUS_WARN, SUS_PWRDN_ACK, SLP_A */
 	task_enable_irq(NPCX_IRQ_WKINTB_2);
-	/* Enable eSPI module interrupts */
+	/* Enable eSPI module interrupts and wake-up functionalities */
 	NPCX_ESPIIE |= (ESPIIE_GENERIC | ESPIIE_VW);
+	NPCX_ESPIWE |= (ESPIWE_GENERIC | ESPIWE_VW);
 #endif
 }
 
 static void lpc_task_disable_irq(void)
 {
+#ifdef HAS_TASK_KEYPROTO
 	task_disable_irq(NPCX_IRQ_KBC_IBF);
+#endif
 	task_disable_irq(NPCX_IRQ_PM_CHAN_IBF);
 	task_disable_irq(NPCX_IRQ_PORT80);
-#ifdef CONFIG_ESPI
+#ifdef CONFIG_HOST_INTERFACE_ESPI
 	task_disable_irq(NPCX_IRQ_ESPI);
 	/* Virtual Wire: SLP_S3/4/5, SUS_STAT, PLTRST, OOB_RST_WARN */
 	task_disable_irq(NPCX_IRQ_WKINTA_2);
 	/* Virtual Wire: HOST_RST_WARN,SUS_WARN, SUS_PWRDN_ACK, SLP_A */
 	task_disable_irq(NPCX_IRQ_WKINTB_2);
-	/* Disable eSPI module interrupts */
+	/* Disable eSPI module interrupts and wake-up functionalities */
 	NPCX_ESPIIE &= ~(ESPIIE_GENERIC | ESPIIE_VW);
+	NPCX_ESPIWE &= ~(ESPIWE_GENERIC | ESPIWE_VW);
 #endif
 }
 /**
@@ -141,6 +126,8 @@ static void lpc_task_disable_irq(void)
  */
 static void lpc_generate_smi(void)
 {
+	host_event_t smi;
+
 #ifdef CONFIG_SCI_GPIO
 	/* Enforce signal-high for long enough to debounce high */
 	gpio_set_level(GPIO_PCH_SMI_L, 1);
@@ -150,7 +137,7 @@ static void lpc_generate_smi(void)
 	udelay(65);
 	/* Set signal high, now that we've generated the edge */
 	gpio_set_level(GPIO_PCH_SMI_L, 1);
-#elif defined(CONFIG_ESPI)
+#elif defined(CONFIG_HOST_INTERFACE_ESPI)
 	/*
 	 * Don't use SET_BIT/CLEAR_BIT macro to toggle SMIB/SCIB to generate
 	 * virtual wire. Use NPCX_VW_SMI/NPCX_VW_SCI macro instead.
@@ -175,9 +162,9 @@ static void lpc_generate_smi(void)
 	/* Set signal high */
 	SET_BIT(NPCX_HIPMIC(PMC_ACPI), NPCX_HIPMIC_SMIB);
 #endif
-	if (host_events & event_mask[LPC_HOST_EVENT_SMI])
-		CPRINTS("smi 0x%08x",
-			host_events & event_mask[LPC_HOST_EVENT_SMI]);
+	smi = lpc_get_host_events_by_type(LPC_HOST_EVENT_SMI);
+	if (smi)
+		HOST_EVENT_CPRINTS("smi", smi);
 }
 
 /**
@@ -185,6 +172,8 @@ static void lpc_generate_smi(void)
  */
 static void lpc_generate_sci(void)
 {
+	host_event_t sci;
+
 #ifdef CONFIG_SCI_GPIO
 	/* Enforce signal-high for long enough to debounce high */
 	gpio_set_level(CONFIG_SCI_GPIO, 1);
@@ -194,7 +183,7 @@ static void lpc_generate_sci(void)
 	udelay(65);
 	/* Set signal high, now that we've generated the edge */
 	gpio_set_level(CONFIG_SCI_GPIO, 1);
-#elif defined(CONFIG_ESPI)
+#elif defined(CONFIG_HOST_INTERFACE_ESPI)
 	/*
 	 * Don't use SET_BIT/CLEAR_BIT macro to toggle SMIB/SCIB to generate
 	 * virtual wire. Use NPCX_VW_SMI/NPCX_VW_SCI macro instead.
@@ -220,9 +209,9 @@ static void lpc_generate_sci(void)
 	SET_BIT(NPCX_HIPMIC(PMC_ACPI), NPCX_HIPMIC_SCIB);
 #endif
 
-	if (host_events & event_mask[LPC_HOST_EVENT_SCI])
-		CPRINTS("sci 0x%08x",
-			host_events & event_mask[LPC_HOST_EVENT_SCI]);
+	sci = lpc_get_host_events_by_type(LPC_HOST_EVENT_SCI);
+	if (sci)
+		HOST_EVENT_CPRINTS("sci", sci);
 }
 
 /**
@@ -230,7 +219,7 @@ static void lpc_generate_sci(void)
  *
  * @param wake_events	Currently asserted wake events
  */
-static void lpc_update_wake(uint32_t wake_events)
+static void lpc_update_wake(host_event_t wake_events)
 {
 	/*
 	 * Mask off power button event, since the AP gets that through a
@@ -314,7 +303,7 @@ int lpc_keyboard_input_pending(void)
 	return (NPCX_HIKMST&0x02) ? 1 : 0;
 }
 
-/* Put a char to host buffer and send IRQ if specified. */
+/* Put a char to host buffer by HIKDO and send IRQ if specified. */
 void lpc_keyboard_put_char(uint8_t chr, int send_irq)
 {
 	NPCX_HIKDO = chr;
@@ -328,81 +317,30 @@ void lpc_keyboard_put_char(uint8_t chr, int send_irq)
 	}
 }
 
-/*
- * Check host read is not in-progress and no timeout
- */
-static void lpc_sib_wait_host_read_done(void)
+/* Put an aux char to host buffer by HIMDO and assert status bit 5. */
+void lpc_aux_put_char(uint8_t chr, int send_irq)
 {
-	timestamp_t deadline;
+	if (send_irq)
+		SET_BIT(NPCX_HICTRL, NPCX_HICTRL_OBFMIE);
+	else
+		CLEAR_BIT(NPCX_HICTRL, NPCX_HICTRL_OBFMIE);
 
-	deadline.val = get_time().val + LPC_HOST_TRANSACTION_TIMEOUT_US;
-	while (IS_BIT_SET(NPCX_SIBCTRL, NPCX_SIBCTRL_CSRD)) {
-		if (timestamp_expired(deadline, NULL)) {
-			ccprintf("Unexpected time of host read transaction\n");
-			break;
-		}
-	}
-}
+	NPCX_HIKMST |= I8042_AUX_DATA;
+	NPCX_HIMDO = chr;
+	CPRINTS("AUX put %02x", chr);
 
-/*
- * Check host write is not in-progress and no timeout
- */
-static void lpc_sib_wait_host_write_done(void)
-{
-	timestamp_t deadline;
-
-	deadline.val = get_time().val + LPC_HOST_TRANSACTION_TIMEOUT_US;
-	while (IS_BIT_SET(NPCX_SIBCTRL, NPCX_SIBCTRL_CSWR)) {
-		if (timestamp_expired(deadline, NULL)) {
-			ccprintf("Unexpected time of host write transaction\n");
-			break;
-		}
-	}
-}
-
-/* Emulate host to read Keyboard I/O */
-uint8_t lpc_sib_read_kbc_reg(uint8_t io_offset)
-{
-	uint8_t data_value;
-
-	/* Disable interrupts */
-	interrupt_disable();
-
-	/* Lock host keyboard module */
-	SET_BIT(NPCX_LKSIOHA, NPCX_LKSIOHA_LKHIKBD);
-	/* Enable Core-to-Host Modules Access */
-	SET_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSAE);
-	/* Verify Core read/write to host modules is not in progress */
-	lpc_sib_wait_host_read_done();
-	lpc_sib_wait_host_write_done();
-	/* Enable Core access to keyboard module */
-	SET_BIT(NPCX_CRSMAE, NPCX_CRSMAE_HIKBDAE);
-
-	/* Specify the io_offset A0 = 0. the index register is accessed */
-	NPCX_IHIOA = io_offset;
-
-	/* Start a Core read from host module */
-	SET_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSRD);
-	/* Wait while Core read operation is in progress */
-	lpc_sib_wait_host_read_done();
-	/* Read the data */
-	data_value = NPCX_IHD;
-
-	/* Disable Core access to keyboard module */
-	CLEAR_BIT(NPCX_CRSMAE, NPCX_CRSMAE_HIKBDAE);
-	/* Disable Core-to-Host Modules Access */
-	CLEAR_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSAE);
-	/* unlock host keyboard module */
-	CLEAR_BIT(NPCX_LKSIOHA, NPCX_LKSIOHA_LKHIKBD);
-
-	/* Enable interrupts */
-	interrupt_enable();
-
-	return data_value;
+	/* Enable OBE interrupt to detect host read data out */
+	SET_BIT(NPCX_HICTRL, NPCX_HICTRL_OBECIE);
+	task_enable_irq(NPCX_IRQ_KBC_OBE);
 }
 
 void lpc_keyboard_clear_buffer(void)
 {
+	/*
+	 * Only npcx5 series need this bypass. The bug of FW_OBF is fixed in
+	 * npcx7 series and later npcx ec.
+	 */
+#ifdef CHIP_FAMILY_NPCX5
 	/* Clear OBF flag in host STATUS and HIKMST regs */
 	if (IS_BIT_SET(NPCX_HIKMST, NPCX_HIKMST_OBF)) {
 		/*
@@ -411,8 +349,16 @@ void lpc_keyboard_clear_buffer(void)
 		 * Emulate a host read to clear these two flags and also
 		 * deassert IRQ1
 		 */
-		lpc_sib_read_kbc_reg(0x0);
+		sib_read_kbc_reg(0x0);
 	}
+#else
+	/* Make sure the previous TOH and IRQ has been sent out. */
+	udelay(4);
+	/* Clear OBE flag in host STATUS  and HIKMST regs*/
+	SET_BIT(NPCX_HICTRL, NPCX_HICTRL_FW_OBF);
+	/* Ensure there is no TOH set in this period. */
+	udelay(4);
+#endif
 }
 
 void lpc_keyboard_resume_irq(void)
@@ -428,7 +374,7 @@ void lpc_keyboard_resume_irq(void)
  *   - SMI pulse via EC_SMI_L GPIO
  *   - SCI pulse via LPC0SCI
  */
-static void update_host_event_status(void)
+void lpc_update_host_event_status(void)
 {
 	int need_sci = 0;
 	int need_smi = 0;
@@ -438,7 +384,7 @@ static void update_host_event_status(void)
 
 	/* Disable LPC interrupt while updating status register */
 	lpc_task_disable_irq();
-	if (host_events & event_mask[LPC_HOST_EVENT_SMI]) {
+	if (lpc_get_host_events_by_type(LPC_HOST_EVENT_SMI)) {
 		/* Only generate SMI for first event */
 		if (!(NPCX_HIPMST(PMC_ACPI) & NPCX_HIPMST_ST2))
 			need_smi = 1;
@@ -446,7 +392,7 @@ static void update_host_event_status(void)
 	} else
 		CLEAR_BIT(NPCX_HIPMST(PMC_ACPI), NPCX_HIPMST_ST2);
 
-	if (host_events & event_mask[LPC_HOST_EVENT_SCI]) {
+	if (lpc_get_host_events_by_type(LPC_HOST_EVENT_SCI)) {
 		/* Generate SCI for every event */
 		need_sci = 1;
 		SET_BIT(NPCX_HIPMST(PMC_ACPI), NPCX_HIPMST_ST1);
@@ -454,12 +400,13 @@ static void update_host_event_status(void)
 		CLEAR_BIT(NPCX_HIPMST(PMC_ACPI), NPCX_HIPMST_ST1);
 
 	/* Copy host events to mapped memory */
-	*(uint32_t *)host_get_memmap(EC_MEMMAP_HOST_EVENTS) = host_events;
+	*(host_event_t *)host_get_memmap(EC_MEMMAP_HOST_EVENTS) =
+				lpc_get_host_events();
 
 	lpc_task_enable_irq();
 
 	/* Process the wake events. */
-	lpc_update_wake(host_events & event_mask[LPC_HOST_EVENT_WAKE]);
+	lpc_update_wake(lpc_get_host_events_by_type(LPC_HOST_EVENT_WAKE));
 
 	/* Send pulse on SMI signal if needed */
 	if (need_smi)
@@ -470,54 +417,6 @@ static void update_host_event_status(void)
 		lpc_generate_sci();
 }
 
-void lpc_set_host_event_state(uint32_t mask)
-{
-	if (mask != host_events) {
-		host_events = mask;
-		update_host_event_status();
-	}
-}
-
-int lpc_query_host_event_state(void)
-{
-	const uint32_t any_mask = event_mask[0] | event_mask[1] | event_mask[2];
-	int evt_index = 0;
-	int i;
-
-	for (i = 0; i < 32; i++) {
-		const uint32_t e = (1 << i);
-
-		if (host_events & e) {
-			host_clear_events(e);
-
-			/*
-			 * If host hasn't unmasked this event, drop it.  We do
-			 * this at query time rather than event generation time
-			 * so that the host has a chance to unmask events
-			 * before they're dropped by a query.
-			 */
-			if (!(e & any_mask))
-				continue;
-
-			evt_index = i + 1;	/* Events are 1-based */
-			break;
-		}
-	}
-
-	return evt_index;
-}
-
-void lpc_set_host_event_mask(enum lpc_host_event_type type, uint32_t mask)
-{
-	event_mask[type] = mask;
-	update_host_event_status();
-}
-
-uint32_t lpc_get_host_event_mask(enum lpc_host_event_type type)
-{
-	return event_mask[type];
-}
-
 void lpc_set_acpi_status_mask(uint8_t mask)
 {
 	NPCX_HIPMST(PMC_ACPI) |= mask;
@@ -620,31 +519,47 @@ static void handle_host_write(int is_cmd)
 /* Interrupt handlers */
 #ifdef HAS_TASK_KEYPROTO
 /* KB controller input buffer full ISR */
-void lpc_kbc_ibf_interrupt(void)
+static void lpc_kbc_ibf_interrupt(void)
 {
+	uint8_t status;
+	uint8_t ibf;
 	/* If "command" input 0, else 1*/
-	if (lpc_keyboard_input_pending())
-		keyboard_host_write(NPCX_HIKMDI, (NPCX_HIKMST & 0x08) ? 1 : 0);
-	CPRINTS("ibf isr %02x", NPCX_HIKMDI);
-	task_wake(TASK_ID_KEYPROTO);
+	if (lpc_keyboard_input_pending()) {
+		/*
+		 * Reading HIKMDI causes the IBF flag to deassert and allows
+		 * the host to write a new byte into the input buffer. So if we
+		 * don't capture the status before reading HIKMDI we will race
+		 * with the host and get an invalid value for HIKMST.A20.
+		 */
+		status = NPCX_HIKMST;
+		ibf = NPCX_HIKMDI;
+		keyboard_host_write(ibf, (status & 0x08) ? 1 : 0);
+		CPRINTS("ibf isr %02x", ibf);
+		task_wake(TASK_ID_KEYPROTO);
+	} else {
+		CPRINTS("ibf isr spurious");
+	}
 }
-DECLARE_IRQ(NPCX_IRQ_KBC_IBF, lpc_kbc_ibf_interrupt, 3);
+DECLARE_IRQ(NPCX_IRQ_KBC_IBF, lpc_kbc_ibf_interrupt, 4);
 
 /* KB controller output buffer empty ISR */
-void lpc_kbc_obe_interrupt(void)
+static void lpc_kbc_obe_interrupt(void)
 {
 	/* Disable KBC OBE interrupt */
 	CLEAR_BIT(NPCX_HICTRL, NPCX_HICTRL_OBECIE);
 	task_disable_irq(NPCX_IRQ_KBC_OBE);
 
 	CPRINTS("obe isr %02x", NPCX_HIKMST);
+
+	NPCX_HIKMST &= ~I8042_AUX_DATA;
+
 	task_wake(TASK_ID_KEYPROTO);
 }
-DECLARE_IRQ(NPCX_IRQ_KBC_OBE, lpc_kbc_obe_interrupt, 3);
+DECLARE_IRQ(NPCX_IRQ_KBC_OBE, lpc_kbc_obe_interrupt, 4);
 #endif
 
 /* PM channel input buffer full ISR */
-void lpc_pmc_ibf_interrupt(void)
+static void lpc_pmc_ibf_interrupt(void)
 {
 	/* Channel-1 for ACPI usage*/
 	/* Channel-2 for Host Command usage , so the argument data had been
@@ -654,30 +569,62 @@ void lpc_pmc_ibf_interrupt(void)
 	else if (NPCX_HIPMST(PMC_HOST_CMD) & 0x02)
 		handle_host_write((NPCX_HIPMST(PMC_HOST_CMD)&0x08) ? 1 : 0);
 }
-DECLARE_IRQ(NPCX_IRQ_PM_CHAN_IBF, lpc_pmc_ibf_interrupt, 3);
+DECLARE_IRQ(NPCX_IRQ_PM_CHAN_IBF, lpc_pmc_ibf_interrupt, 4);
 
 /* PM channel output buffer empty ISR */
-void lpc_pmc_obe_interrupt(void)
+static void lpc_pmc_obe_interrupt(void)
 {
 }
-DECLARE_IRQ(NPCX_IRQ_PM_CHAN_OBE, lpc_pmc_obe_interrupt, 3);
+DECLARE_IRQ(NPCX_IRQ_PM_CHAN_OBE, lpc_pmc_obe_interrupt, 4);
 
-void lpc_port80_interrupt(void)
+static void lpc_port80_interrupt(void)
 {
-	/* Send port 80 data to UART continuously if FIFO is not empty */
-	while (IS_BIT_SET(NPCX_DP80STS, 6))
-		port_80_write(NPCX_DP80BUF);
+	uint8_t i;
+	uint8_t count = 0;
+	uint32_t code = 0;
+
+	/* buffer Port80 data to the local buffer if FIFO is not empty */
+	while (IS_BIT_SET(NPCX_DP80STS, NPCX_DP80STS_FNE) &&
+		   (count < ARRAY_SIZE(port80_buf)))
+		port80_buf[count++] = NPCX_DP80BUF;
+
+	for (i = 0; i < count; i++) {
+		uint8_t offset;
+		uint32_t buf_data;
+
+		buf_data = port80_buf[i];
+		offset = GET_FIELD(buf_data, NPCX_DP80BUF_OFFS_FIELD);
+		code |= (buf_data & 0xFF) << (8 * offset);
+
+		if (i == count - 1) {
+			port_80_write(code);
+			break;
+		}
+
+		/* peek the offset of the next byte */
+		buf_data = port80_buf[i + 1];
+		offset = GET_FIELD(buf_data, NPCX_DP80BUF_OFFS_FIELD);
+		/*
+		 * If the peeked next byte's offset is 0 means it is the start
+		 * of the new code. Pass the current code to Port80
+		 * common layer.
+		 */
+		if (offset == 0) {
+			port_80_write(code);
+			code = 0;
+		}
+	}
 
 	/* If FIFO is overflow */
-	if (IS_BIT_SET(NPCX_DP80STS, 7)) {
-		SET_BIT(NPCX_DP80STS, 7);
+	if (IS_BIT_SET(NPCX_DP80STS, NPCX_DP80STS_FOR)) {
+		SET_BIT(NPCX_DP80STS, NPCX_DP80STS_FOR);
 		CPRINTS("DP80 FIFO Overflow!");
 	}
 
 	/* Clear pending bit of host writing */
-	SET_BIT(NPCX_DP80STS, 5);
+	SET_BIT(NPCX_DP80STS, NPCX_DP80STS_FWR);
 }
-DECLARE_IRQ(NPCX_IRQ_PORT80, lpc_port80_interrupt, 3);
+DECLARE_IRQ(NPCX_IRQ_PORT80, lpc_port80_interrupt, 4);
 
 /**
  * Preserve event masks across a sysjump.
@@ -695,158 +642,67 @@ static void lpc_sysjump(void)
 	/* Reset base address for Win 1 and 2. */
 	NPCX_WIN_BASE(0) = 0xfffffff8;
 	NPCX_WIN_BASE(1) = 0xfffffff8;
-
-	system_add_jump_tag(LPC_SYSJUMP_TAG, 1,
-			sizeof(event_mask), event_mask);
 }
 DECLARE_HOOK(HOOK_SYSJUMP, lpc_sysjump, HOOK_PRIO_DEFAULT);
 
-/**
- * Restore event masks after a sysjump.
- */
-static void lpc_post_sysjump(void)
-{
-	const uint32_t *prev_mask;
-	int size, version;
-
-	prev_mask = (const uint32_t *)system_get_jump_tag(LPC_SYSJUMP_TAG,
-			&version, &size);
-	if (!prev_mask || version != 1 || size != sizeof(event_mask))
-		return;
-
-	memcpy(event_mask, prev_mask, sizeof(event_mask));
-}
-
-/* Super-IO read/write function */
-void lpc_sib_write_reg(uint8_t io_offset, uint8_t index_value,
-		uint8_t io_data)
-{
-	/* Disable interrupts */
-	interrupt_disable();
-
-	/* Lock host CFG module */
-	SET_BIT(NPCX_LKSIOHA, NPCX_LKSIOHA_LKCFG);
-	/* Enable Core-to-Host Modules Access */
-	SET_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSAE);
-	/* Enable Core access to CFG module */
-	SET_BIT(NPCX_CRSMAE, NPCX_CRSMAE_CFGAE);
-	/* Verify Core read/write to host modules is not in progress */
-	lpc_sib_wait_host_read_done();
-	lpc_sib_wait_host_write_done();
-
-	/* Specify the io_offset A0 = 0. the index register is accessed */
-	NPCX_IHIOA = io_offset;
-	/* Write the data. This starts the write access to the host module */
-	NPCX_IHD = index_value;
-	/* Wait while Core write operation is in progress */
-	lpc_sib_wait_host_write_done();
-
-	/* Specify the io_offset A0 = 1. the data register is accessed */
-	NPCX_IHIOA = io_offset+1;
-	/* Write the data. This starts the write access to the host module */
-	NPCX_IHD = io_data;
-	/* Wait while Core write operation is in progress */
-	lpc_sib_wait_host_write_done();
-
-	/* Disable Core access to CFG module */
-	CLEAR_BIT(NPCX_CRSMAE, NPCX_CRSMAE_CFGAE);
-	/* Disable Core-to-Host Modules Access */
-	CLEAR_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSAE);
-	/* unlock host CFG  module */
-	CLEAR_BIT(NPCX_LKSIOHA, NPCX_LKSIOHA_LKCFG);
-
-	/* Enable interrupts */
-	interrupt_enable();
-}
-
-uint8_t lpc_sib_read_reg(uint8_t io_offset, uint8_t index_value)
-{
-	uint8_t data_value;
-
-	/* Disable interrupts */
-	interrupt_disable();
-
-	/* Lock host CFG module */
-	SET_BIT(NPCX_LKSIOHA, NPCX_LKSIOHA_LKCFG);
-	/* Enable Core-to-Host Modules Access */
-	SET_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSAE);
-	/* Enable Core access to CFG module */
-	SET_BIT(NPCX_CRSMAE, NPCX_CRSMAE_CFGAE);
-	/* Verify Core read/write to host modules is not in progress */
-	lpc_sib_wait_host_read_done();
-	lpc_sib_wait_host_write_done();
-
-	/* Specify the io_offset A0 = 0. the index register is accessed */
-	NPCX_IHIOA = io_offset;
-	/* Write the data. This starts the write access to the host module */
-	NPCX_IHD = index_value;
-	/* Wait while Core write operation is in progress */
-	lpc_sib_wait_host_write_done();
-
-	/* Specify the io_offset A0 = 1. the data register is accessed */
-	NPCX_IHIOA = io_offset+1;
-	/* Start a Core read from host module */
-	SET_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSRD);
-	/* Wait while Core read operation is in progress */
-	lpc_sib_wait_host_read_done();
-	/* Read the data */
-	data_value = NPCX_IHD;
-
-	/* Disable Core access to CFG module */
-	CLEAR_BIT(NPCX_CRSMAE, NPCX_CRSMAE_CFGAE);
-	/* Disable Core-to-Host Modules Access */
-	CLEAR_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSAE);
-	/* unlock host CFG  module */
-	CLEAR_BIT(NPCX_LKSIOHA, NPCX_LKSIOHA_LKCFG);
-
-	/* Enable interrupts */
-	interrupt_enable();
-
-	return data_value;
-}
-
 /* For LPC host register initial via SIB module */
 void host_register_init(void)
 {
-	/* enable ACPI*/
-	lpc_sib_write_reg(SIO_OFFSET, 0x07, 0x11);
-	lpc_sib_write_reg(SIO_OFFSET, 0x30, 0x01);
+	/* Enable Core-to-Host Modules Access */
+	SET_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSAE);
 
-	/* enable KBC*/
-	lpc_sib_write_reg(SIO_OFFSET, 0x07, 0x06);
-	lpc_sib_write_reg(SIO_OFFSET, 0x30, 0x01);
+	/* enable ACPI*/
+	sib_write_reg(SIO_OFFSET, 0x07, 0x11);
+	sib_write_reg(SIO_OFFSET, 0x30, 0x01);
+
+	/* Enable kbc and mouse */
+#ifdef HAS_TASK_KEYPROTO
+	/* LDN = 0x06 : keyboard */
+	sib_write_reg(SIO_OFFSET, 0x07, 0x06);
+	sib_write_reg(SIO_OFFSET, 0x30, 0x01);
+
+	/* LDN = 0x05 : mouse */
+	if (IS_ENABLED(CONFIG_PS2)) {
+		sib_write_reg(SIO_OFFSET, 0x07, 0x05);
+		sib_write_reg(SIO_OFFSET, 0x30, 0x01);
+	}
+#endif
 
 	/* Setting PMC2 */
 	/* LDN register = 0x12(PMC2) */
-	lpc_sib_write_reg(SIO_OFFSET, 0x07, 0x12);
+	sib_write_reg(SIO_OFFSET, 0x07, 0x12);
 	/* CMD port is 0x200 */
-	lpc_sib_write_reg(SIO_OFFSET, 0x60, 0x02);
-	lpc_sib_write_reg(SIO_OFFSET, 0x61, 0x00);
+	sib_write_reg(SIO_OFFSET, 0x60, 0x02);
+	sib_write_reg(SIO_OFFSET, 0x61, 0x00);
 	/* Data port is 0x204 */
-	lpc_sib_write_reg(SIO_OFFSET, 0x62, 0x02);
-	lpc_sib_write_reg(SIO_OFFSET, 0x63, 0x04);
+	sib_write_reg(SIO_OFFSET, 0x62, 0x02);
+	sib_write_reg(SIO_OFFSET, 0x63, 0x04);
 	/* enable PMC2 */
-	lpc_sib_write_reg(SIO_OFFSET, 0x30, 0x01);
+	sib_write_reg(SIO_OFFSET, 0x30, 0x01);
 
 	/* Setting SHM */
 	/* LDN register = 0x0F(SHM) */
-	lpc_sib_write_reg(SIO_OFFSET, 0x07, 0x0F);
+	sib_write_reg(SIO_OFFSET, 0x07, 0x0F);
 	/* WIN1&2 mapping to IO */
-	lpc_sib_write_reg(SIO_OFFSET, 0xF1,
-			lpc_sib_read_reg(SIO_OFFSET, 0xF1) | 0x30);
-	/* Host Command on the IO:0x0800 */
-	lpc_sib_write_reg(SIO_OFFSET, 0xF7, 0x00);
-	lpc_sib_write_reg(SIO_OFFSET, 0xF6, 0x00);
-	lpc_sib_write_reg(SIO_OFFSET, 0xF5, 0x08);
-	lpc_sib_write_reg(SIO_OFFSET, 0xF4, 0x00);
+	sib_write_reg(SIO_OFFSET, 0xF1,
+			sib_read_reg(SIO_OFFSET, 0xF1) | 0x30);
 	/* WIN1 as Host Command on the IO:0x0800 */
-	lpc_sib_write_reg(SIO_OFFSET, 0xFB, 0x00);
-	lpc_sib_write_reg(SIO_OFFSET, 0xFA, 0x00);
+	sib_write_reg(SIO_OFFSET, 0xF5, 0x08);
+	sib_write_reg(SIO_OFFSET, 0xF4, 0x00);
 	/* WIN2 as MEMMAP on the IO:0x900 */
-	lpc_sib_write_reg(SIO_OFFSET, 0xF9, 0x09);
-	lpc_sib_write_reg(SIO_OFFSET, 0xF8, 0x00);
+	sib_write_reg(SIO_OFFSET, 0xF9, 0x09);
+	sib_write_reg(SIO_OFFSET, 0xF8, 0x00);
+
+	/*
+	 * eSPI allows sending 4 bytes of Port80 code in a single PUT_IOWR_SHORT
+	 * transaction. When setting OFS0_SEL~OFS3_SEL in DPAR1 register to 1,
+	 * EC hardware will put those 4 bytes of Port80 code to DP80BUF FIFO.
+	 * This is only supported when CHIP_FAMILY >= NPCX9.
+	 */
+	if (IS_ENABLED(CONFIG_HOST_INTERFACE_ESPI))
+		sib_write_reg(SIO_OFFSET, 0xFD, 0x0F);
 	/* enable SHM */
-	lpc_sib_write_reg(SIO_OFFSET, 0x30, 0x01);
+	sib_write_reg(SIO_OFFSET, 0x30, 0x01);
 
 	CPRINTS("Host settings are done!");
 
@@ -866,34 +722,7 @@ int lpc_get_pltrst_asserted(void)
 	return IS_BIT_SET(NPCX_MSWCTL1, NPCX_MSWCTL1_PLTRST_ACT);
 }
 
-void lpc_host_reset(void)
-{
-	/* Host Reset Control will assert KBRST# (LPC) or RCIN# VW (eSPI) */
-#ifdef CONFIG_ESPI_VW_SIGNALS
-	int timeout = 100; /* 100 * 10us = 1ms */
-
-	/* Assert RCIN# VW to host */
-	SET_BIT(NPCX_MSWCTL1, NPCX_MSWCTL1_HRSTOB);
-
-	/* Poll for dirty bit to clear to indicate VW read by host */
-	while ((NPCX_VWEVSM(2) & VWEVSM_DIRTY(1))) {
-		if (!timeout--) {
-			CPRINTS("RCIN# VW Timeout");
-			break;
-		}
-		udelay(10);
-	}
-
-	/* Deassert RCIN# VW to host */
-	CLEAR_BIT(NPCX_MSWCTL1, NPCX_MSWCTL1_HRSTOB);
-#else
-	SET_BIT(NPCX_MSWCTL1, NPCX_MSWCTL1_HRSTOB);
-	udelay(10);
-	CLEAR_BIT(NPCX_MSWCTL1, NPCX_MSWCTL1_HRSTOB);
-#endif
-}
-
-#ifndef CONFIG_ESPI
+#ifndef CONFIG_HOST_INTERFACE_ESPI
 /* Initialize host settings by interrupt */
 void lpc_lreset_pltrst_handler(void)
 {
@@ -908,8 +737,7 @@ void lpc_lreset_pltrst_handler(void)
 
 	pltrst_asserted = lpc_get_pltrst_asserted();
 
-	ccprintf("LPC RESET# %sasserted",
-		pltrst_asserted ? "" : "de");
+	CPRINTS("LPC RESET# %sasserted", pltrst_asserted ? "" : "de");
 
 	/*
 	 * Once LRESET is de-asserted (low -> high), we need to initialize lpc
@@ -937,11 +765,16 @@ static void lpc_init(void)
 	/* Enable clock for LPC peripheral */
 	clock_enable_peripheral(CGC_OFFSET_LPC, CGC_LPC_MASK,
 			CGC_MODE_RUN | CGC_MODE_SLEEP);
-#ifdef CONFIG_ESPI
-	/* Enable clock for eSPI peripheral */
-	clock_enable_peripheral(CGC_OFFSET_ESPI, CGC_ESPI_MASK,
-		CGC_MODE_RUN | CGC_MODE_SLEEP);
-	/* Initialize eSPI IP */
+	/*
+	 * In npcx5/7, the host interface type (HIF_TYP_SEL in the DEVCNT
+	 * register) is updated by booter after VCC1 Power-Up reset according to
+	 * VHIF voltage.
+	 * In npcx9, the booter will not do this anymore. The HIF_TYP_SEL
+	 * field should be set by firmware.
+	 */
+#ifdef CONFIG_HOST_INTERFACE_ESPI
+	/* Initialize eSPI module */
+	NPCX_DEVCNT |= 0x08;
 	espi_init();
 #else
 	/* Switching to LPC interface */
@@ -955,7 +788,7 @@ static void lpc_init(void)
 	/* Clear Host Access Hold state */
 	NPCX_SMC_CTL = 0xC0;
 
-#ifndef CONFIG_ESPI
+#ifndef CONFIG_HOST_INTERFACE_ESPI
 	/*
 	 * Set alternative pin from GPIO to CLKRUN no matter SERIRQ is under
 	 * continuous or quiet mode.
@@ -968,13 +801,13 @@ static void lpc_init(void)
 	 * valid if CONFIG_SCI_GPIO isn't defined. eSPI sends SMI/SCI through VW
 	 * automatically by toggling them, too. It's unnecessary to set pin mux.
 	 */
-#if !defined(CONFIG_SCI_GPIO) && !defined(CONFIG_ESPI)
+#if !defined(CONFIG_SCI_GPIO) && !defined(CONFIG_HOST_INTERFACE_ESPI)
 	SET_BIT(NPCX_DEVALT(1), NPCX_DEVALT1_EC_SCI_SL);
 	SET_BIT(NPCX_DEVALT(1), NPCX_DEVALT1_SMI_SL);
 #endif
 
 	/* Initialize Hardware for UART Host */
-#if CONFIG_UART_HOST
+#ifdef CONFIG_UART_HOST
 	/* Init COMx LPC UART */
 	/* FMCLK have to using 50MHz */
 	NPCX_DEVALT(0xB) = 0xFF;
@@ -986,8 +819,6 @@ static void lpc_init(void)
 
 	/* Don't stall SHM transactions */
 	NPCX_SHM_CTL = NPCX_SHM_CTL & ~0x40;
-	/* Semaphore and Indirect access disable */
-	NPCX_SHCFG = 0xE0;
 	/* Disable Protect Win1&2*/
 	NPCX_WIN_WR_PROT(0) = 0;
 	NPCX_WIN_WR_PROT(1) = 0;
@@ -1013,7 +844,6 @@ static void lpc_init(void)
 
 	/* Turn on PMC2 for Host Command usage */
 	SET_BIT(NPCX_HIPMCTL(PMC_HOST_CMD), 0);
-	SET_BIT(NPCX_HIPMCTL(PMC_HOST_CMD), 1);
 
 	/*
 	 * Set required control value (avoid setting HOSTWAIT bit at this stage)
@@ -1022,25 +852,24 @@ static void lpc_init(void)
 	/* Clear status */
 	NPCX_SMC_STS = NPCX_SMC_STS;
 
-	/* Restore event masks if needed */
-	lpc_post_sysjump();
-
 	/* Create mailbox */
 
 	/*
 	 * Init KBC
 	 * Clear OBF status flag,
-	 * IBF(K&M) INT enable, OBE(K&M) empty INT enable ,
+	 * IBF(K&M) INT enable,
 	 * OBF Mouse Full INT enable and OBF KB Full INT enable
 	 */
+#ifdef HAS_TASK_KEYPROTO
 	lpc_keyboard_clear_buffer();
-	NPCX_HICTRL = 0x0F;
+	NPCX_HICTRL = 0x0B;
+#endif
 
 	/*
 	 * Turn on enhance mode on PM channel-1,
-	 * enable OBE/IBF core interrupt
+	 * enable IBF core interrupt
 	 */
-	NPCX_HIPMCTL(PMC_ACPI) |= 0x83;
+	NPCX_HIPMCTL(PMC_ACPI) |= 0x81;
 	/* Normally Polarity IRQ1,12 type (level + high) setting */
 	NPCX_HIIRQC = 0x00;
 
@@ -1048,7 +877,7 @@ static void lpc_init(void)
 	 * Init PORT80
 	 * Enable Port80, Enable Port80 function & Interrupt & Read auto
 	 */
-#ifdef CONFIG_ESPI
+#ifdef CONFIG_HOST_INTERFACE_ESPI
 	NPCX_DP80CTL = 0x2b;
 #else
 	NPCX_DP80CTL = 0x29;
@@ -1069,7 +898,7 @@ static void lpc_init(void)
 	CLEAR_BIT(NPCX_HIPMIC(PMC_ACPI), NPCX_HIPMIC_SMIPOL);
 	/* Set SMIB/SCIB to make sure SMI/SCI are high at init */
 	NPCX_HIPMIC(PMC_ACPI) = NPCX_HIPMIC(PMC_ACPI)
-			| (1 << NPCX_HIPMIC_SMIB) | (1 << NPCX_HIPMIC_SCIB);
+			| BIT(NPCX_HIPMIC_SMIB) | BIT(NPCX_HIPMIC_SCIB);
 #ifndef CONFIG_SCI_GPIO
 	/*
 	 * Allow SMI/SCI generated from PM module.
@@ -1085,7 +914,7 @@ static void lpc_init(void)
 	init_done = 1;
 
 	/* Update host events now that we can copy them to memmap */
-	update_host_event_status();
+	lpc_update_host_event_status();
 
 	/*
 	 * TODO: For testing LPC with Chromebox, please make sure LPC_CLK is
@@ -1098,7 +927,7 @@ static void lpc_init(void)
 	/* initial IO port address via SIB-write modules */
 	host_register_init();
 #else
-#ifndef CONFIG_ESPI
+#ifndef CONFIG_HOST_INTERFACE_ESPI
 	/*
 	 * Initialize LRESET# interrupt only in case of LPC. For eSPI, there is
 	 * no dedicated GPIO pin for LRESET/PLTRST. PLTRST is indicated as a VW
@@ -1123,38 +952,41 @@ static void lpc_init(void)
  */
 DECLARE_HOOK(HOOK_INIT, lpc_init, HOOK_PRIO_INIT_LPC);
 
-static void lpc_resume(void)
-{
-#ifdef CONFIG_POWER_S0IX
-	if (chipset_in_state(CHIPSET_STATE_SUSPEND | CHIPSET_STATE_ON))
-#endif
-	{
-		/* Mask all host events until the host unmasks them itself.  */
-		lpc_set_host_event_mask(LPC_HOST_EVENT_SMI, 0);
-		lpc_set_host_event_mask(LPC_HOST_EVENT_SCI, 0);
-		lpc_set_host_event_mask(LPC_HOST_EVENT_WAKE, 0);
-	}
-
-	/* Store port 80 event so we know where resume happened */
-	port_80_write(PORT_80_EVENT_RESUME);
-}
-DECLARE_HOOK(HOOK_CHIPSET_RESUME, lpc_resume, HOOK_PRIO_DEFAULT);
-
 /* Get protocol information */
-static int lpc_get_protocol_info(struct host_cmd_handler_args *args)
+static enum ec_status lpc_get_protocol_info(struct host_cmd_handler_args *args)
 {
 	struct ec_response_get_protocol_info *r = args->response;
 
 	memset(r, 0, sizeof(*r));
-	r->protocol_versions = (1 << 3);
+	r->protocol_versions = BIT(3);
 	r->max_request_packet_size = EC_LPC_HOST_PACKET_SIZE;
 	r->max_response_packet_size = EC_LPC_HOST_PACKET_SIZE;
 	r->flags = 0;
 
 	args->response_size = sizeof(*r);
 
-	return EC_SUCCESS;
+	return EC_RES_SUCCESS;
 }
 DECLARE_HOST_COMMAND(EC_CMD_GET_PROTOCOL_INFO,
 		lpc_get_protocol_info,
 		EC_VER_MASK(0));
+
+#if DEBUG_LPC
+static int command_lpc(int argc, char **argv)
+{
+	if (argc == 1)
+		return EC_ERROR_PARAM1;
+
+	if (!strcasecmp(argv[1], "sci"))
+		lpc_generate_sci();
+	else if (!strcasecmp(argv[1], "smi"))
+		lpc_generate_smi();
+	else if (!strcasecmp(argv[1], "wake"))
+		lpc_update_wake(-1);
+	else
+		return EC_ERROR_PARAM1;
+	return EC_SUCCESS;
+}
+DECLARE_CONSOLE_COMMAND(lpc, command_lpc, "[sci|smi|wake]", "Trigger SCI/SMI");
+
+#endif
