@@ -5,10 +5,9 @@
 
 /* NPCX-specific SHI module for Chrome EC */
 
-#ifndef SHI_CHIP_H_
-#define SHI_CHIP_H_
+#ifndef __CROS_EC_SHI_CHIP_H_
+#define __CROS_EC_SHI_CHIP_H_
 
-#ifdef CONFIG_HOSTCMD_SPS
 /**
  * Called when the NSS level changes, signalling the start of a SHI
  * transaction.
@@ -16,6 +15,8 @@
  * @param signal	GPIO signal that changed
  */
 void shi_cs_event(enum gpio_signal signal);
+#ifdef NPCX_SHI_V2
+void shi_cs_gpio_int(enum gpio_signal signal);
 #endif
 
-#endif /* SHI_CHIP_H_ */
+#endif /* __CROS_EC_SHI_CHIP_H_ */
