@@ -1,4 +1,4 @@
-/* Copyright (c) 2014 The Chromium OS Authors. All rights reserved.
+/* Copyright 2014 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -14,37 +14,52 @@
 #include "hooks.h"
 #include "host_command.h"
 #include "hwtimer_chip.h"
+#include "lct_chip.h"
 #include "registers.h"
 #include "rom_chip.h"
+#include "sib_chip.h"
 #include "system.h"
 #include "system_chip.h"
 #include "task.h"
 #include "timer.h"
 #include "util.h"
+#include "watchdog.h"
 
 /* Delay after writing TTC for value to latch */
 #define MTC_TTC_LOAD_DELAY_US 250
-#define MTC_ALARM_MASK     ((1 << 25) - 1)
+#define MTC_ALARM_MASK     (BIT(25) - 1)
 #define MTC_WUI_GROUP      MIWU_GROUP_4
 #define MTC_WUI_MASK       MASK_PIN7
 
 /* ROM address of chip revision */
-#define CHIP_REV_ADDR 0x00007FFC
+#if NPCX_FAMILY_VERSION >= NPCX_FAMILY_NPCX9
+#define CHIP_REV_ADDR        0x0000FFFC
+#define CHIP_REV_STR_SIZE    12
+#else
+#define CHIP_REV_ADDR        0x00007FFC
+#define CHIP_REV_STR_SIZE    6
+#endif
+
+/*  Legacy SuperI/O Configuration D register offset */
+#define SIOCFD_REG_OFFSET    0x2D
 
 /* Console output macros */
 #define CPUTS(outstr) cputs(CC_SYSTEM, outstr)
 #define CPRINTS(format, args...) cprints(CC_SYSTEM, format, ## args)
 #define CPRINTF(format, args...) cprintf(CC_SYSTEM, format, ## args)
 
+#if defined(NPCX_LCT_SUPPORT)
+/* A flag for waking up from hibernate mode by RTC overflow event */
+static int is_rtc_overflow_event;
+#endif
+
 /*****************************************************************************/
 /* Internal functions */
 
 void system_watchdog_reset(void)
 {
 	/* Unlock & stop watchdog registers */
-	NPCX_WDSDM = 0x87;
-	NPCX_WDSDM = 0x61;
-	NPCX_WDSDM = 0x63;
+	watchdog_stop_and_unlock();
 
 	/* Reset TWCFG */
 	NPCX_TWCFG = 0;
@@ -75,34 +90,31 @@ void system_watchdog_reset(void)
 /* Return true if index is stored as a single byte in bbram */
 static int bbram_is_byte_access(enum bbram_data_index index)
 {
-	return (index >= BBRM_DATA_INDEX_VBNVCNTXT &&
-		index <  BBRM_DATA_INDEX_RAMLOG)
-#ifdef CONFIG_USB_PD_DUAL_ROLE
-		|| index == BBRM_DATA_INDEX_PD0
-		|| index == BBRM_DATA_INDEX_PD1
-#endif
-		|| index == BBRM_DATA_INDEX_PANIC_FLAGS
-	;
+	return index == BBRM_DATA_INDEX_PD0 ||
+	       index == BBRM_DATA_INDEX_PD1 ||
+	       index == BBRM_DATA_INDEX_PD2 ||
+	       index == BBRM_DATA_INDEX_PANIC_FLAGS;
 }
 
-/* Check and clear BBRAM status on power-on reset */
+/* Check and clear BBRAM status on any reset */
 void system_check_bbram_on_reset(void)
 {
-	/* Check if power on reset */
-	if ((!IS_BIT_SET(NPCX_RSTCTL, NPCX_RSTCTL_VCC1_RST_SCRATCH)) &&
-	    (!IS_BIT_SET(NPCX_RSTCTL, NPCX_RSTCTL_VCC1_RST_STS))) {
+	if (IS_BIT_SET(NPCX_BKUP_STS, NPCX_BKUP_STS_IBBR)) {
 		/*
-		 * Clear IBBR bit because it's default value is 1
-		 * on reset whenever the VBAT supply is powered up.
+		 * If the reset cause is not power-on reset and VBAT has ever
+		 * dropped, print a warning message.
 		 */
-		SET_BIT(NPCX_BKUP_STS, NPCX_BKUP_STS_IBBR);
-	} else {
-		/*
-		 * When the reset cause is other than power on reset,
-		 * it is illegal if IBBR is set.
-		 */
-		if (IS_BIT_SET(NPCX_BKUP_STS, NPCX_BKUP_STS_IBBR))
+		if (IS_BIT_SET(NPCX_RSTCTL, NPCX_RSTCTL_VCC1_RST_SCRATCH) ||
+			IS_BIT_SET(NPCX_RSTCTL, NPCX_RSTCTL_VCC1_RST_STS))
 			CPRINTF("VBAT drop!\n");
+
+		/*
+		 * npcx5/npcx7m6g/npcx7m6f:
+		 *   Clear IBBR bit
+		 * npcx7m6fb/npcx7m6fc/npcx7m7fc/npcx7m7wb/npcx7m7wc:
+		 *   Clear IBBR/VSBY_STS/VCC1_STS bit
+		 */
+		NPCX_BKUP_STS = NPCX_BKUP_STS_ALL_MASK;
 	}
 }
 
@@ -115,7 +127,8 @@ static int bbram_valid(enum bbram_data_index index, int bytes)
 
 	/* Check BBRAM is valid */
 	if (IS_BIT_SET(NPCX_BKUP_STS, NPCX_BKUP_STS_IBBR)) {
-		CPRINTF("IBBR set: BBRAM corrupted!\n");
+		NPCX_BKUP_STS = BIT(NPCX_BKUP_STS_IBBR);
+		panic_printf("IBBR set: BBRAM corrupted!\n");
 		return 0;
 	}
 	return 1;
@@ -175,16 +188,14 @@ static int bbram_data_write(enum bbram_data_index index, uint32_t value)
 /* Map idx to a returned BBRM_DATA_INDEX_*, or return -1 on invalid idx */
 static int bbram_idx_lookup(enum system_bbram_idx idx)
 {
-	if (idx >= SYSTEM_BBRAM_IDX_VBNVBLOCK0 &&
-	    idx <= SYSTEM_BBRAM_IDX_VBNVBLOCK15)
-		return BBRM_DATA_INDEX_VBNVCNTXT +
-		       idx - SYSTEM_BBRAM_IDX_VBNVBLOCK0;
-#ifdef CONFIG_USB_PD_DUAL_ROLE
 	if (idx == SYSTEM_BBRAM_IDX_PD0)
 		return BBRM_DATA_INDEX_PD0;
 	if (idx == SYSTEM_BBRAM_IDX_PD1)
 		return BBRM_DATA_INDEX_PD1;
-#endif
+	if (idx == SYSTEM_BBRAM_IDX_PD2)
+		return BBRM_DATA_INDEX_PD2;
+	if (idx == SYSTEM_BBRAM_IDX_TRY_SLOT)
+		return BBRM_DATA_INDEX_TRY_SLOT;
 	return -1;
 }
 
@@ -235,7 +246,7 @@ void system_set_rtc(uint32_t seconds)
  *
  * index     |       data
  * ==========|=============
- *   36      |       MMFS
+ *   36      |       CFSR
  *   40      |       HFSR
  *   44      |       BFAR
  *   48      |      LREG1
@@ -244,18 +255,18 @@ void system_set_rtc(uint32_t seconds)
  *   60      |     reserved
  *
  * Above registers are chosen to be saved in case of panic because:
- * 1. MMFS, HFSR and BFAR seem to provide more information about the fault.
+ * 1. CFSR, HFSR and BFAR seem to provide more information about the fault.
  * 2. LREG1, LREG3 and LREG4 store exception, reason and info in case of
  * software panic.
  */
-#define BKUP_MMFS		(BBRM_DATA_INDEX_PANIC_BKUP + 0)
+#define BKUP_CFSR		(BBRM_DATA_INDEX_PANIC_BKUP + 0)
 #define BKUP_HFSR		(BBRM_DATA_INDEX_PANIC_BKUP + 4)
 #define BKUP_BFAR		(BBRM_DATA_INDEX_PANIC_BKUP + 8)
 #define BKUP_LREG1		(BBRM_DATA_INDEX_PANIC_BKUP + 12)
 #define BKUP_LREG3		(BBRM_DATA_INDEX_PANIC_BKUP + 16)
 #define BKUP_LREG4		(BBRM_DATA_INDEX_PANIC_BKUP + 20)
 
-#define BKUP_PANIC_DATA_VALID	(1 << 0)
+#define BKUP_PANIC_DATA_VALID	BIT(0)
 
 void chip_panic_data_backup(void)
 {
@@ -264,7 +275,7 @@ void chip_panic_data_backup(void)
 	if (!d)
 		return;
 
-	bbram_data_write(BKUP_MMFS, d->cm.mmfs);
+	bbram_data_write(BKUP_CFSR, d->cm.cfsr);
 	bbram_data_write(BKUP_HFSR, d->cm.hfsr);
 	bbram_data_write(BKUP_BFAR, d->cm.dfsr);
 	bbram_data_write(BKUP_LREG1, d->cm.regs[1]);
@@ -275,10 +286,10 @@ void chip_panic_data_backup(void)
 
 static void chip_panic_data_restore(void)
 {
-	struct panic_data *d = PANIC_DATA_PTR;
+	struct panic_data *d;
 
 	/* Ensure BBRAM is valid. */
-	if (!bbram_valid(BKUP_MMFS, 4))
+	if (!bbram_valid(BKUP_CFSR, 4))
 		return;
 
 	/* Ensure Panic data in BBRAM is valid. */
@@ -286,13 +297,15 @@ static void chip_panic_data_restore(void)
 	      BKUP_PANIC_DATA_VALID))
 		return;
 
-	memset(d, 0, sizeof(*d));
+	d = get_panic_data_write();
+
+	memset(d, 0, CONFIG_PANIC_DATA_SIZE);
 	d->magic = PANIC_DATA_MAGIC;
-	d->struct_size = sizeof(*d);
+	d->struct_size = CONFIG_PANIC_DATA_SIZE;
 	d->struct_version = 2;
 	d->arch = PANIC_ARCH_CORTEX_M;
 
-	d->cm.mmfs = bbram_data_read(BKUP_MMFS);
+	d->cm.cfsr = bbram_data_read(BKUP_CFSR);
 	d->cm.hfsr = bbram_data_read(BKUP_HFSR);
 	d->cm.dfsr = bbram_data_read(BKUP_BFAR);
 
@@ -305,35 +318,144 @@ static void chip_panic_data_restore(void)
 }
 #endif /* CONFIG_CHIP_PANIC_BACKUP */
 
-void chip_save_reset_flags(int flags)
+void chip_save_reset_flags(uint32_t flags)
 {
 	bbram_data_write(BBRM_DATA_INDEX_SAVED_RESET_FLAGS, flags);
 }
 
-/* Check reset cause */
-void system_check_reset_cause(void)
+uint32_t chip_read_reset_flags(void)
+{
+	return bbram_data_read(BBRM_DATA_INDEX_SAVED_RESET_FLAGS);
+}
+
+static void chip_set_hib_flag(uint32_t *flags, uint32_t hib_wake_flags)
+{
+	/* Hibernate via PSL */
+	if (hib_wake_flags & HIBERNATE_WAKE_PSL) {
+#ifdef NPCX_LCT_SUPPORT
+		if (npcx_lct_is_event_set()) {
+			*flags |= EC_RESET_FLAG_RTC_ALARM |
+					  EC_RESET_FLAG_HIBERNATE;
+			/* Is RTC overflow event? */
+			if (bbram_data_read(BBRM_DATA_INDEX_LCT_TIME) ==
+				NPCX_LCT_MAX) {
+				/*
+				 * Mark it as RTC overflow event and handle it
+				 * in hook init function later for logging info.
+				 */
+				is_rtc_overflow_event = 1;
+			}
+			npcx_lct_clear_event();
+			return;
+		}
+#endif
+		*flags |= EC_RESET_FLAG_WAKE_PIN |
+				  EC_RESET_FLAG_HIBERNATE;
+	} else { /* Hibernate via non-PSL */
+#ifdef NPCX_LCT_SUPPORT
+		if (hib_wake_flags & HIBERNATE_WAKE_LCT) {
+			*flags |= EC_RESET_FLAG_RTC_ALARM |
+					  EC_RESET_FLAG_HIBERNATE;
+			return;
+		}
+#endif
+		if (hib_wake_flags & HIBERNATE_WAKE_PIN) {
+			*flags |= EC_RESET_FLAG_WAKE_PIN |
+					  EC_RESET_FLAG_HIBERNATE;
+		} else if (hib_wake_flags & HIBERNATE_WAKE_MTC) {
+			*flags |= EC_RESET_FLAG_RTC_ALARM |
+					  EC_RESET_FLAG_HIBERNATE;
+		}
+	}
+}
+
+static void check_reset_cause(void)
 {
 	uint32_t hib_wake_flags = bbram_data_read(BBRM_DATA_INDEX_WAKE);
-	uint32_t flags = bbram_data_read(BBRM_DATA_INDEX_SAVED_RESET_FLAGS);
+	uint32_t chip_flags = chip_read_reset_flags();
+	uint32_t flags = chip_flags;
 
 	/* Clear saved reset flags in bbram */
-	chip_save_reset_flags(0);
+#ifdef CONFIG_POWER_BUTTON_INIT_IDLE
+	/*
+	 * We're not sure whether we're booting or not. AP_IDLE will be cleared
+	 * on S5->S3 transition.
+	 */
+	chip_flags &= EC_RESET_FLAG_AP_IDLE;
+#else
+	chip_flags = 0;
+#endif
 	/* Clear saved hibernate wake flag in bbram , too */
 	bbram_data_write(BBRM_DATA_INDEX_WAKE, 0);
 
+	chip_set_hib_flag(&flags, hib_wake_flags);
+
 	/* Use scratch bit to check power on reset or VCC1_RST reset */
 	if (!IS_BIT_SET(NPCX_RSTCTL, NPCX_RSTCTL_VCC1_RST_SCRATCH)) {
-#if defined(BOARD_WHEATLEY) || defined(BOARD_EVE) || defined(BOARD_POPPY) || defined(BOARD_SORAKA)
-		/* TODO(crosbug.com/p/61028): Remove workaround for Eve */
-		flags |= RESET_FLAG_RESET_PIN;
+#ifdef CONFIG_BOARD_FORCE_RESET_PIN
+		/* Treat all resets as RESET_PIN */
+		flags |= EC_RESET_FLAG_RESET_PIN;
 #else
 		/* Check for VCC1 reset */
-		if (IS_BIT_SET(NPCX_RSTCTL, NPCX_RSTCTL_VCC1_RST_STS))
-			flags |= RESET_FLAG_RESET_PIN;
-		else
-			flags |= RESET_FLAG_POWER_ON;
+		int reset = IS_BIT_SET(NPCX_RSTCTL, NPCX_RSTCTL_VCC1_RST_STS);
+
+		/*
+		 * If configured, check the saved flags to see whether
+		 * the previous restart was a power-on, in which case
+		 * treat this restart as a power-on as well.
+		 * This is to workaround the fact that the H1 will
+		 * reset the EC at power up.
+		 */
+		if (IS_ENABLED(CONFIG_BOARD_RESET_AFTER_POWER_ON)) {
+			/*
+			 * Reset pin restart rather than power-on, so check
+			 * for any flag set from a previous power-on.
+			 */
+			if (reset) {
+				if (flags & EC_RESET_FLAG_INITIAL_PWR)
+					/*
+					 * The previous restart was a power-on
+					 * so treat this restart as that, and
+					 * clear the flag so later code will
+					 * not wait for the second reset.
+					 */
+					flags =
+					  (flags & ~EC_RESET_FLAG_INITIAL_PWR)
+					   | EC_RESET_FLAG_POWER_ON;
+				else
+					/*
+					 * No previous power-on flag,
+					 * so this is a subsequent restart
+					 * i.e any restarts after the
+					 * second restart caused by the H1.
+					 */
+					flags |= EC_RESET_FLAG_RESET_PIN;
+			} else {
+				flags |= EC_RESET_FLAG_POWER_ON;
+
+				/*
+				 * Power-on restart, so set a flag and save it
+				 * for the next imminent reset. Later code
+				 * will check for this flag and wait for the
+				 * second reset. Waking from PSL hibernate is
+				 * power-on for EC but not for H1, so do not
+				 * wait for the second reset.
+				 */
+				if (!(flags & EC_RESET_FLAG_HIBERNATE)) {
+					flags |= EC_RESET_FLAG_INITIAL_PWR;
+					chip_flags |= EC_RESET_FLAG_INITIAL_PWR;
+				}
+			}
+		} else
+			/*
+			 * No second reset after power-on, so
+			 * set the flags according to the restart reason.
+			 */
+			flags |= reset ? EC_RESET_FLAG_RESET_PIN
+				       : EC_RESET_FLAG_POWER_ON;
 #endif
 	}
+	chip_save_reset_flags(chip_flags);
 
 	/*
 	 * Set scratch bit to distinguish VCC1RST# is asserted again
@@ -344,27 +466,21 @@ void system_check_reset_cause(void)
 
 	/* Software debugger reset */
 	if (IS_BIT_SET(NPCX_RSTCTL, NPCX_RSTCTL_DBGRST_STS)) {
-		flags |= RESET_FLAG_SOFT;
+		flags |= EC_RESET_FLAG_SOFT;
 		/* Clear debugger reset status initially*/
 		SET_BIT(NPCX_RSTCTL, NPCX_RSTCTL_DBGRST_STS);
 	}
 
-	/* Reset by hibernate */
-	if (hib_wake_flags & HIBERNATE_WAKE_PIN)
-		flags |= RESET_FLAG_WAKE_PIN | RESET_FLAG_HIBERNATE;
-	else if (hib_wake_flags & HIBERNATE_WAKE_MTC)
-		flags |= RESET_FLAG_RTC_ALARM | RESET_FLAG_HIBERNATE;
-
 	/* Watchdog Reset */
 	if (IS_BIT_SET(NPCX_T0CSR, NPCX_T0CSR_WDRST_STS)) {
 		/*
-		 * Don't set RESET_FLAG_WATCHDOG flag if watchdog is issued by
-		 * system_reset or hibernate in order to distinguish reset cause
-		 * is panic reason or not.
+		 * Don't set EC_RESET_FLAG_WATCHDOG flag if watchdog is issued
+		 * by system_reset or hibernate in order to distinguish reset
+		 * cause is panic reason or not.
 		 */
-		if (!(flags & (RESET_FLAG_SOFT | RESET_FLAG_HARD |
-				RESET_FLAG_HIBERNATE)))
-			flags |= RESET_FLAG_WATCHDOG;
+		if (!(flags & (EC_RESET_FLAG_SOFT | EC_RESET_FLAG_HARD |
+				EC_RESET_FLAG_HIBERNATE)))
+			flags |= EC_RESET_FLAG_WATCHDOG;
 
 		/* Clear watchdog reset status initially*/
 		SET_BIT(NPCX_T0CSR, NPCX_T0CSR_WDRST_STS);
@@ -376,7 +492,8 @@ void system_check_reset_cause(void)
 /**
  * Chip-level function to set GPIOs and wake-up inputs for hibernate.
  */
-void system_set_gpios_and_wakeup_inputs_hibernate(void)
+#ifdef CONFIG_SUPPORT_CHIP_HIBERNATION
+static void system_set_gpios_and_wakeup_inputs_hibernate(void)
 {
 	int table, i;
 
@@ -395,7 +512,7 @@ void system_set_gpios_and_wakeup_inputs_hibernate(void)
 		}
 	}
 
-#if defined(CHIP_FAMILY_NPCX7)
+#if NPCX_FAMILY_VERSION >= NPCX_FAMILY_NPCX7
 	/* Disable MIWU 2 group 6 inputs which used for the additional GPIOs */
 	NPCX_WKEN(MIWU_TABLE_2, MIWU_GROUP_6)  = 0x00;
 	NPCX_WKPCL(MIWU_TABLE_2, MIWU_GROUP_6) = 0xFF;
@@ -415,6 +532,35 @@ void system_set_gpios_and_wakeup_inputs_hibernate(void)
 	}
 }
 
+#ifdef NPCX_LCT_SUPPORT
+static void system_set_lct_alarm(uint32_t seconds, uint32_t microseconds)
+{
+	/* The min resolution of LCT is 1 seconds */
+	if ((seconds == 0) && (microseconds != 0))
+		seconds = 1;
+
+	npcx_lct_enable(0);
+	npcx_lct_sel_power_src(NPCX_LCT_PWR_SRC_VSBY);
+#ifdef CONFIG_HIBERNATE_PSL
+	/* Enable LCT event to PSL */
+	npcx_lct_config(seconds, 1, 0);
+	/* save the start time of LCT */
+	if (IS_ENABLED(CONFIG_HOSTCMD_RTC) || IS_ENABLED(CONFIG_CMD_RTC))
+		bbram_data_write(BBRM_DATA_INDEX_LCT_TIME, seconds);
+#else
+	/* Enable LCT event interrupt and MIWU */
+	npcx_lct_config(seconds, 0, 1);
+	task_disable_irq(NPCX_IRQ_LCT_WKINTF_2);
+	/* Enable wake-up input sources & clear pending bit */
+	NPCX_WKPCL(MIWU_TABLE_2, LCT_WUI_GROUP)  |= LCT_WUI_MASK;
+	NPCX_WKINEN(MIWU_TABLE_2, LCT_WUI_GROUP) |= LCT_WUI_MASK;
+	NPCX_WKEN(MIWU_TABLE_2, LCT_WUI_GROUP)   |= LCT_WUI_MASK;
+	task_enable_irq(NPCX_IRQ_LCT_WKINTF_2);
+#endif
+	npcx_lct_enable(1);
+}
+#endif
+
 /**
  * hibernate function for npcx ec.
  *
@@ -429,6 +575,14 @@ void __enter_hibernate(uint32_t seconds, uint32_t microseconds)
 	NPCX_ADCCNF = 0;
 	usleep(1000);
 
+#ifdef NPCX_LCT_SUPPORT
+	/*
+	 * This function must be called before the ITIM (system tick)
+	 * is disabled because it calls udelay inside this function
+	 */
+	npcx_lct_enable_clk(1);
+#endif
+
 	/* Set SPI pins to be in Tri-State */
 	SET_BIT(NPCX_DEVCNT, NPCX_DEVCNT_F_SPI_TRIS);
 
@@ -438,18 +592,16 @@ void __enter_hibernate(uint32_t seconds, uint32_t microseconds)
 	/* Disable interrupt */
 	interrupt_disable();
 
+	/* Unlock & stop watchdog */
+	watchdog_stop_and_unlock();
+
 	/* ITIM event module disable */
 	CLEAR_BIT(NPCX_ITCTS(ITIM_EVENT_NO), NPCX_ITCTS_ITEN);
 	/* ITIM time module disable */
-	CLEAR_BIT(NPCX_ITCTS(ITIM32), NPCX_ITCTS_ITEN);
+	CLEAR_BIT(NPCX_ITCTS(ITIM_SYSTEM_NO), NPCX_ITCTS_ITEN);
 	/* ITIM watchdog warn module disable */
 	CLEAR_BIT(NPCX_ITCTS(ITIM_WDG_NO), NPCX_ITCTS_ITEN);
 
-	/* Unlock & stop watchdog */
-	NPCX_WDSDM = 0x87;
-	NPCX_WDSDM = 0x61;
-	NPCX_WDSDM = 0x63;
-
 	/* Initialize watchdog */
 	NPCX_TWCFG = 0; /* Select T0IN clock as watchdog prescaler clock */
 	SET_BIT(NPCX_TWCFG, NPCX_TWCFG_WDCT0I);
@@ -466,8 +618,10 @@ void __enter_hibernate(uint32_t seconds, uint32_t microseconds)
 	system_set_gpios_and_wakeup_inputs_hibernate();
 
 	/*
-	 * Give the board a chance to do any late stage hibernation work.
-	 * This is likely going to configure GPIOs for hibernation.
+	 * Give the board a chance to do any late stage hibernation work.  This
+	 * is likely going to configure GPIOs for hibernation.  On some boards,
+	 * it's possible that this may not return at all.  On those boards,
+	 * power to the EC is likely being turn off entirely.
 	 */
 	if (board_hibernate_late)
 		board_hibernate_late();
@@ -476,22 +630,55 @@ void __enter_hibernate(uint32_t seconds, uint32_t microseconds)
 	for (i = NPCX_IRQ_0 ; i < NPCX_IRQ_COUNT ; i++)
 		task_clear_pending_irq(i);
 
-	/*
-	 * Set RTC interrupt in time to wake up before
-	 * next event.
-	 */
+	/* Set the timer interrupt for wake up.  */
+#ifdef NPCX_LCT_SUPPORT
+	if (seconds || microseconds) {
+		system_set_lct_alarm(seconds, microseconds);
+	} else if (IS_ENABLED(CONFIG_HIBERNATE_PSL_COMPENSATE_RTC)) {
+		system_set_lct_alarm(NPCX_LCT_MAX, 0);
+	}
+#else
 	if (seconds || microseconds)
 		system_set_rtc_alarm(seconds, microseconds);
-
+#endif
 
 	/* execute hibernate func depend on chip series */
 	__hibernate_npcx_series();
-
 }
 
-static char system_to_hex(uint8_t x)
+#ifdef CONFIG_HIBERNATE_PSL_COMPENSATE_RTC
+#ifndef NPCX_LCT_SUPPORT
+#error "Do not enable CONFIG_HIBERNATE_PSL_COMPENSATE_RTC if npcx ec doesn't \
+support LCT!"
+#endif
+/*
+ * The function uses the LCT counter value to compensate for RTC after hibernate
+ * wake-up. Because system_set_rtc() will invoke udelay(), the function should
+ * execute after timer_init(). The function also should execute before
+ * npcx_lct_init() which will clear all LCT register.
+ */
+void system_compensate_rtc(void)
 {
-	if (x >= 0 && x <= 9)
+	uint32_t rtc_time, ltc_start_time;
+
+	ltc_start_time = bbram_data_read(BBRM_DATA_INDEX_LCT_TIME);
+	if (ltc_start_time == 0)
+		return;
+
+	rtc_time = system_get_rtc_sec();
+	rtc_time += ltc_start_time - npcx_lct_get_time();
+	system_set_rtc(rtc_time);
+	/* Clear BBRAM data to avoid compensating again. */
+	bbram_data_write(BBRM_DATA_INDEX_LCT_TIME, 0);
+}
+#endif
+#endif /* CONFIG_SUPPORT_CHIP_HIBERNATION */
+
+static char system_to_hex(uint8_t val)
+{
+	uint8_t x = val & 0x0F;
+
+	if (x <= 9)
 		return '0' + x;
 	return 'a' + x - 10;
 }
@@ -508,7 +695,7 @@ void system_set_rtc_alarm(uint32_t seconds, uint32_t microseconds)
 {
 	uint32_t cur_secs, alarm_secs;
 
-	if (seconds == EC_RTC_ALARM_CLEAR) {
+	if (seconds == EC_RTC_ALARM_CLEAR && !microseconds) {
 		CLEAR_BIT(NPCX_WTC, NPCX_WTC_WIE);
 		SET_BIT(NPCX_WTC, NPCX_WTC_PTO);
 
@@ -535,7 +722,7 @@ void system_set_rtc_alarm(uint32_t seconds, uint32_t microseconds)
 	SET_BIT(NPCX_WTC, NPCX_WTC_WIE);
 
 	/* Enable MTC interrupt */
-	task_enable_irq(NPCX_IRQ_MTC_WKINTAD_0);
+	task_enable_irq(NPCX_IRQ_MTC);
 
 	/* Enable wake-up input sources & clear pending bit */
 	NPCX_WKPCL(MIWU_TABLE_0, MTC_WUI_GROUP)  |= MTC_WUI_MASK;
@@ -553,7 +740,7 @@ void system_reset_rtc_alarm(void)
 	SET_BIT(NPCX_WTC, NPCX_WTC_PTO);
 
 	/* Disable MTC interrupt */
-	task_disable_irq(NPCX_IRQ_MTC_WKINTAD_0);
+	task_disable_irq(NPCX_IRQ_MTC);
 }
 
 /*
@@ -580,7 +767,7 @@ uint32_t system_get_rtc_alarm(void)
  */
 void system_enable_hib_interrupt(void)
 {
-	task_enable_irq(NPCX_IRQ_MTC_WKINTAD_0);
+	task_enable_irq(NPCX_IRQ_MTC);
 }
 
 void system_hibernate(uint32_t seconds, uint32_t microseconds)
@@ -591,14 +778,75 @@ void system_hibernate(uint32_t seconds, uint32_t microseconds)
 	if (board_hibernate)
 		board_hibernate();
 
-#if SUPPORT_HIB
+#ifdef CONFIG_SUPPORT_CHIP_HIBERNATION
 	/* Add additional hibernate operations here */
 	__enter_hibernate(seconds, microseconds);
 #endif
 }
 
+#ifndef CONFIG_ENABLE_JTAG_SELECTION
+static void system_disable_host_sel_jtag(void)
+{
+	int data;
+
+	/* Enable Core-to-Host Modules Access */
+	SET_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSAE);
+	/* Clear SIOCFD.JEN0_HSL to disable JTAG0 */
+	data = sib_read_reg(SIO_OFFSET, SIOCFD_REG_OFFSET);
+	data &= ~0x80;
+	sib_write_reg(SIO_OFFSET, SIOCFD_REG_OFFSET, data);
+	/* Disable Core-to-Host Modules Access */
+	CLEAR_BIT(NPCX_SIBCTRL, NPCX_SIBCTRL_CSAE);
+}
+#endif
+
+void chip_pre_init(void)
+{
+	/* Setting for fixing JTAG issue */
+	NPCX_DBGCTRL = 0x04;
+	/* Enable automatic freeze mode */
+	CLEAR_BIT(NPCX_DBGFRZEN3, NPCX_DBGFRZEN3_GLBL_FRZ_DIS);
+
+	/*
+	 * Enable JTAG functionality by SW without pulling down strap-pin
+	 * nJEN0 or nJEN1 during ec POWERON or VCCRST reset occurs.
+	 * Please notice it will change pinmux to JTAG directly.
+	 */
+#ifdef NPCX_ENABLE_JTAG
+#if NPCX_JTAG_MODULE2
+	CLEAR_BIT(NPCX_DEVALT(ALT_GROUP_5), NPCX_DEVALT5_NJEN1_EN);
+#else
+	CLEAR_BIT(NPCX_DEVALT(ALT_GROUP_5), NPCX_DEVALT5_NJEN0_EN);
+#endif
+#endif
+
+#ifndef CONFIG_ENABLE_JTAG_SELECTION
+	/*
+	 * (b/129908668)
+	 * This is the workaround to disable the JTAG0 which is enabled
+	 * accidentally by a special key combination.
+	 */
+#if NPCX_FAMILY_VERSION < NPCX_FAMILY_NPCX9
+	if (!IS_BIT_SET(NPCX_DEVALT(5), NPCX_DEVALT5_NJEN0_EN)) {
+		/* Set DEVALT5.nJEN0_EN to disable JTAG0 */
+		SET_BIT(NPCX_DEVALT(5), NPCX_DEVALT5_NJEN0_EN);
+		system_disable_host_sel_jtag();
+	}
+#else
+	if (GET_FIELD(NPCX_JEN_CTL1, NPCX_JEN_CTL1_JEN_EN_FIELD) ==
+		      NPCX_JEN_CTL1_JEN_EN_ENA) {
+		SET_FIELD(NPCX_JEN_CTL1, NPCX_JEN_CTL1_JEN_EN_FIELD,
+			  NPCX_JEN_CTL1_JEN_EN_DIS);
+		system_disable_host_sel_jtag();
+	}
+#endif
+#endif
+}
+
 void system_pre_init(void)
 {
+	uint8_t pwdwn6;
+
 	/*
 	 * Add additional initialization here
 	 * EC should be initialized in Booter
@@ -609,15 +857,38 @@ void system_pre_init(void)
 	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_2) = 0xFF;
 #if defined(CHIP_FAMILY_NPCX5)
 	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_3) = 0x0F; /* Skip GDMA */
-#elif defined(CHIP_FAMILY_NPCX7)
-	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_3) = 0x1F; /* Skip GDMA */
+#elif NPCX_FAMILY_VERSION >= NPCX_FAMILY_NPCX7
+	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_3) = 0x3F; /* Skip GDMA */
 #endif
 	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_4) = 0xF4; /* Skip ITIM2/1_PD */
 	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_5) = 0xF8;
-	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_6) = 0xF5; /* Skip ITIM5_PD */
+
+	pwdwn6 = 0x70 |
+#if NPCX_FAMILY_VERSION <= NPCX_FAMILY_NPCX7
+		/*
+		 * Don't set PD of ITIM6 for NPCX9 and later chips because
+		 * they use it as the system timer.
+		 */
+		BIT(NPCX_PWDWN_CTL6_ITIM6_PD) |
+#endif
+		BIT(NPCX_PWDWN_CTL6_ITIM4_PD); /* Skip ITIM5_PD */
+#if !defined(CONFIG_HOST_INTERFACE_ESPI)
+	pwdwn6 |= 1 << NPCX_PWDWN_CTL6_ESPI_PD;
+#endif
+	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_6) = pwdwn6;
+
 #if defined(CHIP_FAMILY_NPCX7)
+#if defined(CHIP_VARIANT_NPCX7M6FB) || defined(CHIP_VARIANT_NPCX7M6FC) || \
+	defined(CHIP_VARIANT_NPCX7M7FC) || defined(CHIP_VARIANT_NPCX7M7WB) || \
+	defined(CHIP_VARIANT_NPCX7M7WC)
+	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_7) = 0xE7;
+#else
 	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_7) = 0x07;
 #endif
+#endif
+#if NPCX_FAMILY_VERSION >= NPCX_FAMILY_NPCX9
+	NPCX_PWDWN_CTL(NPCX_PMC_PWDWN_7) = 0xFF;
+#endif
 
 	/* Following modules can be powered down automatically in npcx7 */
 #if defined(CHIP_FAMILY_NPCX5)
@@ -637,35 +908,92 @@ void system_pre_init(void)
 	 */
 	system_mpu_config();
 
+	/*
+	 * Change FMUL_WIN_DLY from 0x8A to 0x81 for better WoV
+	 * audio quality.
+	 */
+#ifdef CHIP_FAMILY_NPCX7
+	NPCX_FMUL_WIN_DLY = 0x81;
+#endif
+
 #ifdef CONFIG_CHIP_PANIC_BACKUP
 	chip_panic_data_restore();
 #endif
+
+#if NPCX_FAMILY_VERSION >= NPCX_FAMILY_NPCX9
+	if (IS_ENABLED(CONFIG_HIBERNATE_PSL)) {
+		uint8_t opt_flag = CONFIG_HIBERNATE_PSL_OUT_FLAGS;
+
+		/* PSL Glitch Protection */
+		SET_BIT(NPCX_GLUE_PSL_MCTL2, NPCX_GLUE_PSL_MCTL2_PSL_GP_EN);
+
+		/*
+		 * TODO: Remove this when NPCX9 A2 chip is available because A2
+		 * chip will enable VCC1_RST to PSL wakeup source and lock it in
+		 * the booter.
+		 */
+		if (IS_ENABLED(CONFIG_HIBERNATE_PSL_VCC1_RST_WAKEUP)) {
+			/*
+			 * Enable VCC1_RST as the wake-up source from
+			 * hibernation.
+			 */
+			SET_BIT(NPCX_GLUE_PSL_MCTL1,
+					NPCX_GLUE_PSL_MCTL1_VCC1_RST_PSL);
+			/* Disable VCC_RST Pull-Up */
+			SET_BIT(NPCX_DEVALT(ALT_GROUP_G),
+					NPCX_DEVALTG_VCC1_RST_PUD);
+			/*
+			 * Lock this bit itself and VCC1_RST_PSL in the
+			 * PSL_MCTL1 register to read-only.
+			 */
+			SET_BIT(NPCX_GLUE_PSL_MCTL2,
+					NPCX_GLUE_PSL_MCTL2_VCC1_RST_PSL_LK);
+		}
+
+		/* Don't set PSL_OUT to open-drain if it is the level mode */
+		ASSERT((opt_flag & NPCX_PSL_CFG_PSL_OUT_PULSE) ||
+			 !(opt_flag & NPCX_PSL_CFG_PSL_OUT_OD));
+
+		if (opt_flag & NPCX_PSL_CFG_PSL_OUT_OD)
+			SET_BIT(NPCX_GLUE_PSL_MCTL1, NPCX_GLUE_PSL_MCTL1_OD_EN);
+		else
+			CLEAR_BIT(NPCX_GLUE_PSL_MCTL1,
+					NPCX_GLUE_PSL_MCTL1_OD_EN);
+
+		if (opt_flag & NPCX_PSL_CFG_PSL_OUT_PULSE)
+			SET_BIT(NPCX_GLUE_PSL_MCTL1,
+					NPCX_GLUE_PSL_MCTL1_PLS_EN);
+		else
+			CLEAR_BIT(NPCX_GLUE_PSL_MCTL1,
+					NPCX_GLUE_PSL_MCTL1_PLS_EN);
+	}
+#endif
 }
 
 void system_reset(int flags)
 {
-	uint32_t save_flags = 0;
+	uint32_t save_flags;
 
 	/* Disable interrupts to avoid task swaps during reboot */
 	interrupt_disable();
 
-	/* Save current reset reasons if necessary */
-	if (flags & SYSTEM_RESET_PRESERVE_FLAGS)
-		save_flags = system_get_reset_flags() | RESET_FLAG_PRESERVED;
-
-	/* Add in AP off flag into saved flags. */
-	if (flags & SYSTEM_RESET_LEAVE_AP_OFF)
-		save_flags |= RESET_FLAG_AP_OFF;
-
-	/* Save reset flag */
-	if (flags & SYSTEM_RESET_HARD)
-		save_flags |= RESET_FLAG_HARD;
-	else
-		save_flags |= RESET_FLAG_SOFT;
+	/*  Get flags to be saved in BBRAM */
+	system_encode_save_flags(flags, &save_flags);
 
 	/* Store flags to battery backed RAM. */
 	chip_save_reset_flags(save_flags);
 
+	/* If WAIT_EXT is set, then allow 10 seconds for external reset */
+	if (flags & SYSTEM_RESET_WAIT_EXT) {
+		int i;
+
+		/* Wait 10 seconds for external reset */
+		for (i = 0; i < 1000; i++) {
+			watchdog_reload();
+			udelay(10000);
+		}
+	}
+
 	/* Ask the watchdog to trigger a hard reboot */
 	system_watchdog_reset();
 
@@ -688,8 +1016,8 @@ const char *system_get_chip_vendor(void)
 	case 0x20:
 		return "Nuvoton";
 	default:
-		*p       = system_to_hex((fam_id & 0xF0) >> 4);
-		*(p + 1) = system_to_hex(fam_id & 0x0F);
+		*p       = system_to_hex(fam_id >> 4);
+		*(p + 1) = system_to_hex(fam_id);
 		*(p + 2) = '\0';
 		return str;
 	}
@@ -702,23 +1030,37 @@ const char *system_get_chip_name(void)
 
 	/* Read Chip ID in core register */
 	uint8_t chip_id = NPCX_DEVICE_ID_CR;
+
 	switch (chip_id) {
 #if defined(CHIP_FAMILY_NPCX5)
-	case 0x12:
+	case NPCX585G_CHIP_ID:
 		return "NPCX585G";
-	case 0x13:
+	case NPCX575G_CHIP_ID:
 		return "NPCX575G";
-	case 0x16:
+	case NPCX586G_CHIP_ID:
 		return "NPCX586G";
-	case 0x17:
+	case NPCX576G_CHIP_ID:
 		return "NPCX576G";
 #elif defined(CHIP_FAMILY_NPCX7)
-	case 0x21:
+	case NPCX787G_CHIP_ID:
+		return "NPCX787G";
+	case NPCX797F_C_CHIP_ID:
+		return "NPCX797F";
+	case NPCX796F_A_B_CHIP_ID:
+	case NPCX796F_C_CHIP_ID:
 		return "NPCX796F";
+	case NPCX797W_B_CHIP_ID:
+	case NPCX797W_C_CHIP_ID:
+		return "NPCX797W";
+#elif defined(CHIP_FAMILY_NPCX9)
+	case NPCX996F_CHIP_ID:
+		return "NPCX996F";
+	case NPCX993F_CHIP_ID:
+		return "NPCX993F";
 #endif
 	default:
-		*p       = system_to_hex((chip_id & 0xF0) >> 4);
-		*(p + 1) = system_to_hex(chip_id & 0x0F);
+		*p       = system_to_hex(chip_id >> 4);
+		*(p + 1) = system_to_hex(chip_id);
 		*(p + 2) = '\0';
 		return str;
 	}
@@ -726,21 +1068,65 @@ const char *system_get_chip_name(void)
 
 const char *system_get_chip_revision(void)
 {
-	static char rev[5];
+	static char rev[CHIP_REV_STR_SIZE];
+	char *p = rev;
+	/* Read chip generation from SRID_CR */
+	uint8_t chip_gen = NPCX_SRID_CR;
 	/* Read ROM data for chip revision directly */
+#if NPCX_FAMILY_VERSION >= NPCX_FAMILY_NPCX9
+	uint32_t rev_num = *((uint32_t *)CHIP_REV_ADDR);
+#else
 	uint8_t rev_num = *((uint8_t *)CHIP_REV_ADDR);
+#endif
 
-	*(rev) = 'A';
-	*(rev + 1) = '.';
-	*(rev + 2) = system_to_hex((rev_num & 0xF0) >> 4);
-	*(rev + 3) = system_to_hex(rev_num & 0x0F);
-	*(rev + 4) = '\0';
+#ifdef CHIP_FAMILY_NPCX7
+	uint8_t chip_id = NPCX_DEVICE_ID_CR;
+#endif
+	int s;
+
+	switch (chip_gen) {
+#if defined(CHIP_FAMILY_NPCX5)
+	case 0x05:
+		*p++ = 'A';
+		break;
+#elif defined(CHIP_FAMILY_NPCX7)
+	case 0x06:
+		*p++ = 'A';
+		break;
+	case 0x07:
+		if (chip_id == NPCX796F_A_B_CHIP_ID ||
+			chip_id == NPCX797W_B_CHIP_ID)
+			*p++ = 'B';
+		else
+			*p++ = 'C';
+		break;
+#elif defined(CHIP_FAMILY_NPCX9)
+	case 0x09:
+		*p++ = 'A';
+		break;
+#endif
+	default:
+		*p++ = system_to_hex(chip_gen >> 4);
+		*p++ = system_to_hex(chip_gen);
+		break;
+	}
+
+	*p++ = '.';
+	/*
+	 * For npcx5/npcx7, the revision number is 1 byte.
+	 * For NPCX9 and later chips, the revision number is 4 bytes.
+	 */
+	for (s = sizeof(rev_num) - 1; s >= 0; s--) {
+		uint8_t r = rev_num >> (s * 8);
+
+		*p++ = system_to_hex(r >> 4);
+		*p++ = system_to_hex(r);
+	}
+	*p++ = '\0';
 
 	return rev;
 }
 
-BUILD_ASSERT(BBRM_DATA_INDEX_VBNVCNTXT + EC_VBNV_BLOCK_SIZE <= NPCX_BBRAM_SIZE);
-
 /**
  * Set a scratchpad register to the specified value.
  *
@@ -755,9 +1141,10 @@ int system_set_scratchpad(uint32_t value)
 	return bbram_data_write(BBRM_DATA_INDEX_SCRATCHPAD, value);
 }
 
-uint32_t system_get_scratchpad(void)
+int system_get_scratchpad(uint32_t *value)
 {
-	return bbram_data_read(BBRM_DATA_INDEX_SCRATCHPAD);
+	*value = bbram_data_read(BBRM_DATA_INDEX_SCRATCHPAD);
+	return EC_SUCCESS;
 }
 
 int system_is_reboot_warm(void)
@@ -768,23 +1155,39 @@ int system_is_reboot_warm(void)
 	 * Check reset cause here,
 	 * gpio_pre_init is executed faster than system_pre_init
 	 */
-	system_check_reset_cause();
+	check_reset_cause();
 	reset_flags = system_get_reset_flags();
 
-	if ((reset_flags & RESET_FLAG_RESET_PIN) ||
-	    (reset_flags & RESET_FLAG_POWER_ON) ||
-	    (reset_flags & RESET_FLAG_WATCHDOG) ||
-	    (reset_flags & RESET_FLAG_HARD) ||
-	    (reset_flags & RESET_FLAG_SOFT) ||
-	    (reset_flags & RESET_FLAG_HIBERNATE))
+	if ((reset_flags & EC_RESET_FLAG_RESET_PIN) ||
+	    (reset_flags & EC_RESET_FLAG_POWER_ON) ||
+	    (reset_flags & EC_RESET_FLAG_WATCHDOG) ||
+	    (reset_flags & EC_RESET_FLAG_HARD) ||
+	    (reset_flags & EC_RESET_FLAG_SOFT) ||
+	    (reset_flags & EC_RESET_FLAG_HIBERNATE))
 		return 0;
 	else
 		return 1;
 }
 
+#if defined(CONFIG_HIBERNATE_PSL) && defined(NPCX_LCT_SUPPORT)
+static void system_init_check_rtc_wakeup_event(void)
+{
+	/*
+	 * If platform uses PSL (Power Switch Logic) for hibernating and RTC is
+	 * also supported, determine whether ec is woken up by RTC with overflow
+	 * event (16 weeks). If so, let it go to hibernate mode immediately.
+	 */
+	if (is_rtc_overflow_event){
+		CPRINTS("Hibernate due to RTC overflow event");
+		system_hibernate(0, 0);
+	}
+}
+DECLARE_HOOK(HOOK_INIT, system_init_check_rtc_wakeup_event,
+	     HOOK_PRIO_DEFAULT - 1);
+#endif
+
 /*****************************************************************************/
 /* Console commands */
-#ifdef CONFIG_CMD_RTC
 void print_system_rtc(enum console_channel ch)
 {
 	uint32_t sec = system_get_rtc_sec();
@@ -792,6 +1195,7 @@ void print_system_rtc(enum console_channel ch)
 	cprintf(ch, "RTC: 0x%08x (%d.00 s)\n", sec, sec);
 }
 
+#ifdef CONFIG_CMD_RTC
 static int command_system_rtc(int argc, char **argv)
 {
 	if (argc == 3 && !strcasecmp(argv[1], "set")) {
@@ -852,7 +1256,7 @@ DECLARE_CONSOLE_COMMAND(rtc_alarm, command_rtc_alarm_test,
 /* Host commands */
 
 #ifdef CONFIG_HOSTCMD_RTC
-static int system_rtc_get_value(struct host_cmd_handler_args *args)
+static enum ec_status system_rtc_get_value(struct host_cmd_handler_args *args)
 {
 	struct ec_response_rtc *r = args->response;
 
@@ -865,7 +1269,7 @@ DECLARE_HOST_COMMAND(EC_CMD_RTC_GET_VALUE,
 		system_rtc_get_value,
 		EC_VER_MASK(0));
 
-static int system_rtc_set_value(struct host_cmd_handler_args *args)
+static enum ec_status system_rtc_set_value(struct host_cmd_handler_args *args)
 {
 	const struct ec_params_rtc *p = args->params;
 
@@ -876,7 +1280,7 @@ DECLARE_HOST_COMMAND(EC_CMD_RTC_SET_VALUE,
 		system_rtc_set_value,
 		EC_VER_MASK(0));
 
-static int system_rtc_set_alarm(struct host_cmd_handler_args *args)
+static enum ec_status system_rtc_set_alarm(struct host_cmd_handler_args *args)
 {
 	const struct ec_params_rtc *p = args->params;
 
@@ -887,7 +1291,7 @@ DECLARE_HOST_COMMAND(EC_CMD_RTC_SET_ALARM,
 		system_rtc_set_alarm,
 		EC_VER_MASK(0));
 
-static int system_rtc_get_alarm(struct host_cmd_handler_args *args)
+static enum ec_status system_rtc_get_alarm(struct host_cmd_handler_args *args)
 {
 	struct ec_response_rtc *r = args->response;
 
@@ -913,14 +1317,25 @@ void system_jump_to_booter(void)
 	 * Get memory offset and size for RO/RW regions.
 	 * Both of them need 16-bytes alignment since GDMA burst mode.
 	 */
-	if (IS_BIT_SET(NPCX_FWCTRL, NPCX_FWCTRL_RO_REGION)) {
-		flash_offset = CONFIG_EC_PROTECTED_STORAGE_OFF +
-				CONFIG_RO_STORAGE_OFF;
-		flash_used = CONFIG_RO_SIZE;
-	} else {
+	switch (system_get_shrspi_image_copy()) {
+	case EC_IMAGE_RW:
 		flash_offset = CONFIG_EC_WRITABLE_STORAGE_OFF +
 				CONFIG_RW_STORAGE_OFF;
 		flash_used = CONFIG_RW_SIZE;
+		break;
+#ifdef CONFIG_RW_B
+	case EC_IMAGE_RW_B:
+		flash_offset = CONFIG_EC_WRITABLE_STORAGE_OFF +
+				CONFIG_RW_B_STORAGE_OFF;
+		flash_used = CONFIG_RW_SIZE;
+		break;
+#endif
+	case EC_IMAGE_RO:
+	default: /* Jump to RO by default */
+		flash_offset = CONFIG_EC_PROTECTED_STORAGE_OFF +
+				CONFIG_RO_STORAGE_OFF;
+		flash_used = CONFIG_RO_SIZE;
+		break;
 	}
 
 	/* Make sure the reset vector is inside the destination image */
@@ -933,8 +1348,14 @@ void system_jump_to_booter(void)
 	 */
 	clock_turbo();
 
+/*
+ * npcx9 Rev.1 has the problem for download_from_flash API.
+ * Workwaroud it by executing the system_download_from_flash function
+ * in the suspend RAM like npcx5.
+ * TODO: Removing npcx9 when Rev.2 is available.
+ */
 	/* Bypass for GMDA issue of ROM api utilities */
-#if defined(CHIP_FAMILY_NPCX5)
+#if defined(CHIP_FAMILY_NPCX5) || defined(CONFIG_WORKAROUND_FLASH_DOWNLOAD_API)
 	system_download_from_flash(
 		flash_offset,      /* The offset of the data in spi flash */
 		CONFIG_PROGRAM_MEMORY_BASE, /* RAM Addr of downloaded data */
@@ -963,21 +1384,54 @@ uint32_t system_get_lfw_address()
 	return jump_addr;
 }
 
-void system_set_image_copy(enum system_image_copy_t copy)
+/*
+ * Set and clear image copy flags in MDC register.
+ *
+ * NPCX_FWCTRL_RO_REGION: 1 - RO, 0 - RW
+ * NPCX_FWCTRL_FW_SLOT: 1 - SLOT_A, 0 - SLOT_B
+ */
+void system_set_image_copy(enum ec_image copy)
 {
-	/* Jump to RW region -- clear flag */
-	if (copy == SYSTEM_IMAGE_RW)
+	switch (copy) {
+	case EC_IMAGE_RW:
 		CLEAR_BIT(NPCX_FWCTRL, NPCX_FWCTRL_RO_REGION);
-	else /* Jump to RO region -- set flag */
+		SET_BIT(NPCX_FWCTRL, NPCX_FWCTRL_FW_SLOT);
+		break;
+#ifdef CONFIG_RW_B
+	case EC_IMAGE_RW_B:
+		CLEAR_BIT(NPCX_FWCTRL, NPCX_FWCTRL_RO_REGION);
+		CLEAR_BIT(NPCX_FWCTRL, NPCX_FWCTRL_FW_SLOT);
+		break;
+#endif
+	default:
+		CPRINTS("Invalid copy (%d) is requested as a jump destination. "
+			"Change it to %d.", copy, EC_IMAGE_RO);
+		/* Fall through to EC_IMAGE_RO */
+	case EC_IMAGE_RO:
 		SET_BIT(NPCX_FWCTRL, NPCX_FWCTRL_RO_REGION);
+		SET_BIT(NPCX_FWCTRL, NPCX_FWCTRL_FW_SLOT);
+		break;
+	}
 }
 
-enum system_image_copy_t system_get_shrspi_image_copy(void)
+enum ec_image system_get_shrspi_image_copy(void)
 {
-	/* RO region FW */
-	if (IS_BIT_SET(NPCX_FWCTRL, NPCX_FWCTRL_RO_REGION))
-		return SYSTEM_IMAGE_RO;
-	else/* RW region FW */
-		return SYSTEM_IMAGE_RW;
+	if (IS_BIT_SET(NPCX_FWCTRL, NPCX_FWCTRL_RO_REGION)) {
+		/* RO image */
+#ifdef CHIP_HAS_RO_B
+		if (!IS_BIT_SET(NPCX_FWCTRL, NPCX_FWCTRL_FW_SLOT))
+			return EC_IMAGE_RO_B;
+#endif
+		return EC_IMAGE_RO;
+	} else {
+#ifdef CONFIG_RW_B
+		/* RW image */
+		if (!IS_BIT_SET(NPCX_FWCTRL, NPCX_FWCTRL_FW_SLOT))
+			/* Slot A */
+			return EC_IMAGE_RW_B;
+#endif
+		return EC_IMAGE_RW;
+	}
 }
+
 #endif
