@@ -23,15 +23,15 @@
 #define I2C_ERROR_FAILED_START EC_ERROR_INTERNAL_FIRST
 
 /* Transmit timeout in microseconds */
-#define I2C_TX_TIMEOUT_MASTER   (10 * MSEC)
+#define I2C_TX_TIMEOUT_CONTROLLER   (10 * MSEC)
 
-#ifdef CONFIG_HOSTCMD_I2C_SLAVE_ADDR
+#ifdef CONFIG_HOSTCMD_I2C_ADDR_FLAGS
 #if (I2C_PORT_EC == STM32_I2C1_PORT)
-#define IRQ_SLAVE_EV STM32_IRQ_I2C1_EV
-#define IRQ_SLAVE_ER STM32_IRQ_I2C1_ER
+#define IRQ_PERIPHERAL_EV STM32_IRQ_I2C1_EV
+#define IRQ_PERIPHERAL_ER STM32_IRQ_I2C1_ER
 #else
-#define IRQ_SLAVE_EV STM32_IRQ_I2C2_EV
-#define IRQ_SLAVE_ER STM32_IRQ_I2C2_ER
+#define IRQ_PERIPHERAL_EV STM32_IRQ_I2C2_EV
+#define IRQ_PERIPHERAL_ER STM32_IRQ_I2C2_ER
 #endif
 #endif
 
@@ -43,8 +43,7 @@
  */
 #define STM32F4_FMPI2C_PORT	3
 
-
-static const struct dma_option dma_tx_option[I2C_PORT_COUNT] = {
+static const __unused struct dma_option dma_tx_option[I2C_PORT_COUNT] = {
 	{STM32_DMAC_I2C1_TX, (void *)&STM32_I2C_DR(STM32_I2C1_PORT),
 	 STM32_DMA_CCR_MSIZE_8_BIT | STM32_DMA_CCR_PSIZE_8_BIT |
 	 STM32_DMA_CCR_CHANNEL(STM32_I2C1_TX_REQ_CH)},
@@ -74,13 +73,13 @@ static const struct dma_option dma_rx_option[I2C_PORT_COUNT] = {
 	 STM32_DMA_CCR_CHANNEL(STM32_FMPI2C4_RX_REQ_CH)},
 };
 
-/* Callabck for ISR to wake task on DMA complete. */
+/* Callback for ISR to wake task on DMA complete. */
 static inline void _i2c_dma_wake_callback(void *cb_data, int port)
 {
 	task_id_t id = (task_id_t)(int)cb_data;
 
 	if (id != TASK_ID_INVALID)
-		task_set_event(id, TASK_EVENT_I2C_COMPLETION(port), 0);
+		task_set_event(id, TASK_EVENT_I2C_COMPLETION(port));
 }
 
 /* Each callback is hardcoded to an I2C channel. */
@@ -134,7 +133,7 @@ void i2c_dma_enable_tc_interrupt(enum dma_channel stream, int port)
 #define UNSET 0
 static int wait_sr1_poll(int port, int mask, int val, int poll)
 {
-	uint64_t timeout = get_time().val + I2C_TX_TIMEOUT_MASTER;
+	uint64_t timeout = get_time().val + I2C_TX_TIMEOUT_CONTROLLER;
 
 	while (get_time().val < timeout) {
 		int sr1 = STM32_I2C_SR1(port);
@@ -165,14 +164,14 @@ static int wait_sr1(int port, int mask)
 
 
 /**
- * Send a start condition and slave address on the specified port.
+ * Send a start condition and peripheral address on the specified port.
  *
  * @param port		I2C port
- * @param slave_addr	Slave address, with LSB set for receive-mode
+ * @param addr_8bit	I2C address, with LSB set for receive-mode
  *
  * @return Non-zero if error.
  */
-static int send_start(int port, int slave_addr)
+static int send_start(const int port, const uint16_t addr_8bit)
 {
 	int rv;
 
@@ -182,8 +181,8 @@ static int send_start(int port, int slave_addr)
 	if (rv)
 		return I2C_ERROR_FAILED_START;
 
-	/* Write slave address */
-	STM32_I2C_DR(port) = slave_addr & 0xff;
+	/* Write peripheral address */
+	STM32_I2C_DR(port) = addr_8bit;
 	rv = wait_sr1_poll(port, STM32_I2C_SR1_ADDR, SET, 1);
 	if (rv)
 		return rv;
@@ -225,7 +224,7 @@ static const struct i2c_port_t *find_port(int port)
  */
 static int wait_fmpi2c_isr_poll(int port, int mask, int val, int poll)
 {
-	uint64_t timeout = get_time().val + I2C_TX_TIMEOUT_MASTER;
+	uint64_t timeout = get_time().val + I2C_TX_TIMEOUT_CONTROLLER;
 
 	while (get_time().val < timeout) {
 		int isr = STM32_FMPI2C_ISR(port);
@@ -255,16 +254,17 @@ static int wait_fmpi2c_isr(int port, int mask)
 }
 
 /**
- * Send a start condition and slave address on the specified port.
+ * Send a start condition and peripheral address on the specified port.
  *
  * @param port		I2C port
- * @param slave_addr	Slave address
+ * @param addr_8bit	I2C address
  * @param size		bytes to transfer
  * @param is_read	read, or write?
  *
  * @return Non-zero if error.
  */
-static int send_fmpi2c_start(int port, int slave_addr, int size, int is_read)
+static int send_fmpi2c_start(const int port, const uint16_t addr_8bit,
+			     int size, int is_read)
 {
 	uint32_t reg;
 
@@ -274,7 +274,7 @@ static int send_fmpi2c_start(int port, int slave_addr, int size, int is_read)
 		FMPI2C_CR2_RELOAD | FMPI2C_CR2_AUTOEND |
 		FMPI2C_CR2_RD_WRN | FMPI2C_CR2_START | FMPI2C_CR2_STOP);
 	reg |= FMPI2C_CR2_START | FMPI2C_CR2_AUTOEND |
-		FMPI2C_CR2_SADD(slave_addr) | FMPI2C_CR2_SIZE(size) |
+		addr_8bit | FMPI2C_CR2_SIZE(size) |
 		(is_read ? FMPI2C_CR2_RD_WRN : 0);
 	STM32_FMPI2C_CR2(port) = reg;
 
@@ -392,7 +392,7 @@ static void fmpi2c_clear_regs(int port)
  * Perform an i2c transaction
  *
  * @param port		i2c port to use
- * @param slave_addr	the i2c slave addr
+ * @param addr_8bit	the i2c address
  * @param out		source buffer for data
  * @param out_bytes	bytes of data to write
  * @param in		destination buffer for data
@@ -401,8 +401,9 @@ static void fmpi2c_clear_regs(int port)
  *
  * @return		EC_SUCCESS on success.
  */
-static int chip_fmpi2c_xfer(int port, int slave_addr, const uint8_t *out,
-		     int out_bytes, uint8_t *in, int in_bytes, int flags)
+static int chip_fmpi2c_xfer(const int port, const uint16_t addr_8bit,
+			    const uint8_t *out, int out_bytes,
+			    uint8_t *in, int in_bytes, int flags)
 {
 	int started = (flags & I2C_XFER_START) ? 0 : 1;
 	int rv = EC_SUCCESS;
@@ -422,7 +423,7 @@ static int chip_fmpi2c_xfer(int port, int slave_addr, const uint8_t *out,
 	/* No out bytes and no in bytes means just check for active */
 	if (out_bytes || !in_bytes) {
 		rv = send_fmpi2c_start(
-			port, slave_addr, out_bytes, FMPI2C_WRITE);
+			port, addr_8bit, out_bytes, FMPI2C_WRITE);
 		if (rv)
 			goto xfer_exit;
 
@@ -448,7 +449,7 @@ static int chip_fmpi2c_xfer(int port, int slave_addr, const uint8_t *out,
 		i2c_dma_enable_tc_interrupt(dma->channel, port);
 
 		rv_start = send_fmpi2c_start(
-				port, slave_addr, in_bytes, FMPI2C_READ);
+				port, addr_8bit, in_bytes, FMPI2C_READ);
 		if (rv_start)
 			goto xfer_exit;
 
@@ -457,15 +458,13 @@ static int chip_fmpi2c_xfer(int port, int slave_addr, const uint8_t *out,
 			goto xfer_exit;
 		STM32_FMPI2C_CR1(port) |= FMPI2C_CR1_RXDMAEN;
 
-		if (!rv_start) {
-			rv = task_wait_event_mask(
-					TASK_EVENT_I2C_COMPLETION(port),
-					DMA_TRANSFER_TIMEOUT_US);
-			if (rv & TASK_EVENT_I2C_COMPLETION(port))
-				rv = EC_SUCCESS;
-			else
-				rv = EC_ERROR_TIMEOUT;
-		}
+		rv = task_wait_event_mask(
+				TASK_EVENT_I2C_COMPLETION(port),
+				DMA_TRANSFER_TIMEOUT_US);
+		if (rv & TASK_EVENT_I2C_COMPLETION(port))
+			rv = EC_SUCCESS;
+		else
+			rv = EC_ERROR_TIMEOUT;
 
 		dma_disable(dma->channel);
 		dma_disable_tc_interrupt(dma->channel);
@@ -475,9 +474,6 @@ static int chip_fmpi2c_xfer(int port, int slave_addr, const uint8_t *out,
 			rv = wait_fmpi2c_isr(port, FMPI2C_ISR_STOPF);
 
 		STM32_FMPI2C_CR1(port) &= ~FMPI2C_CR1_RXDMAEN;
-
-		if (rv_start)
-			rv = rv_start;
 	}
 
  xfer_exit:
@@ -513,7 +509,7 @@ static int chip_fmpi2c_xfer(int port, int slave_addr, const uint8_t *out,
 
 		/*
 		 * Allow bus to idle for at least one 100KHz clock = 10 us.
-		 * This allows slaves on the bus to detect bus-idle before
+		 * This allows peripherals on the bus to detect bus-idle before
 		 * the next start condition.
 		 */
 		STM32_FMPI2C_CR1(port) &= ~FMPI2C_CR1_PE;
@@ -552,9 +548,11 @@ static void i2c_clear_regs(int port)
  */
 
 /* Perform an i2c transaction. */
-int chip_i2c_xfer(int port, int slave_addr, const uint8_t *out, int out_bytes,
+int chip_i2c_xfer(const int port, const uint16_t addr_flags,
+		  const uint8_t *out, int out_bytes,
 		  uint8_t *in, int in_bytes, int flags)
 {
+	int addr_8bit = I2C_STRIP_FLAGS(addr_flags) << 1;
 	int started = (flags & I2C_XFER_START) ? 0 : 1;
 	int rv = EC_SUCCESS;
 	int i;
@@ -565,15 +563,16 @@ int chip_i2c_xfer(int port, int slave_addr, const uint8_t *out, int out_bytes,
 	ASSERT(!started);
 
 	if (p->port == STM32F4_FMPI2C_PORT) {
-		return chip_fmpi2c_xfer(port, slave_addr, out, out_bytes,
-			in, in_bytes, flags);
+		return chip_fmpi2c_xfer(port, addr_8bit,
+					out, out_bytes,
+					in, in_bytes, flags);
 	}
 
 	i2c_clear_regs(port);
 
 	/* No out bytes and no in bytes means just check for active */
 	if (out_bytes || !in_bytes) {
-		rv = send_start(port, slave_addr);
+		rv = send_start(port, addr_8bit);
 		if (rv)
 			goto xfer_exit;
 
@@ -612,7 +611,7 @@ int chip_i2c_xfer(int port, int slave_addr, const uint8_t *out, int out_bytes,
 		STM32_I2C_CR2(port) |= STM32_I2C_CR2_LAST;
 		STM32_I2C_CR2(port) |= STM32_I2C_CR2_DMAEN;
 
-		rv_start = send_start(port, slave_addr | 0x01);
+		rv_start = send_start(port, addr_8bit | 0x01);
 
 		if ((in_bytes == 1) && (flags & I2C_XFER_STOP))
 			STM32_I2C_CR1(port) |= STM32_I2C_CR1_STOP;
@@ -676,7 +675,7 @@ int chip_i2c_xfer(int port, int slave_addr, const uint8_t *out, int out_bytes,
 
 		/*
 		 * Allow bus to idle for at least one 100KHz clock = 10 us.
-		 * This allows slaves on the bus to detect bus-idle before
+		 * This allows peripherals on the bus to detect bus-idle before
 		 * the next start condition.
 		 */
 		usleep(10);
@@ -716,7 +715,7 @@ int i2c_get_line_levels(int port)
 /*****************************************************************************/
 /* Hooks */
 
-#ifdef CONFIG_I2C_MASTER
+#ifdef CONFIG_I2C_CONTROLLER
 /* Handle CPU clock changing frequency */
 static void i2c_freq_change(void)
 {
@@ -755,9 +754,9 @@ DECLARE_HOOK(HOOK_FREQ_CHANGE, i2c_freq_change_hook, HOOK_PRIO_DEFAULT);
 #endif
 
 /*****************************************************************************/
-/* Slave */
-#ifdef CONFIG_HOSTCMD_I2C_SLAVE_ADDR
-/* Host command slave */
+/* Peripheral */
+#ifdef CONFIG_HOSTCMD_I2C_ADDR_FLAGS
+/* Host command peripheral */
 /*
  * Buffer for received host command packets (including prefix byte on request,
  * and result/size on response).  After any protocol-specific headers, the
@@ -791,7 +790,7 @@ static void i2c_send_response_packet(struct host_packet *pkt)
 
 	/*
 	 * Set the transmitter to be in 'not full' state to keep sending
-	 * '0xec' in the event loop. Because of this, the master i2c
+	 * '0xec' in the event loop. Because of this, the controller i2c
 	 * doesn't need to snoop the response stream to abort transaction.
 	 */
 	STM32_I2C_CR2(host_i2c_resp_port) |= STM32_I2C_CR2_ITBUFEN;
@@ -835,7 +834,7 @@ static void i2c_process_command(void)
 	host_packet_receive(&i2c_packet);
 }
 
-#ifdef CONFIG_BOARD_I2C_SLAVE_ADDR
+#ifdef CONFIG_BOARD_I2C_ADDR_FLAGS
 static void i2c_send_board_response(int len)
 {
 	/* host_buffer data range, beyond this length, will return 0xec */
@@ -859,9 +858,9 @@ static void i2c_event_handler(int port)
 	volatile uint32_t i2c_sr2;
 	volatile uint32_t i2c_sr1;
 	static int rx_pending, buf_idx;
-	static uint16_t addr;
+	static uint16_t addr_8bit;
 
-	volatile uint32_t dummy __attribute__((unused));
+	volatile uint32_t unused __attribute__((unused));
 
 	i2c_cr1 = STM32_I2C_CR1(port);
 	i2c_sr2 = STM32_I2C_SR2(port);
@@ -869,7 +868,7 @@ static void i2c_event_handler(int port)
 
 	/*
 	 * Check for error conditions. Note, arbitration loss and bus error
-	 * are the only two errors we can get as a slave allowing clock
+	 * are the only two errors we can get as a peripheral allowing clock
 	 * stretching and in non-SMBus mode.
 	 */
 	if (i2c_sr1 & (STM32_I2C_SR1_ARLO | STM32_I2C_SR1_BERR)) {
@@ -882,21 +881,21 @@ static void i2c_event_handler(int port)
 					 STM32_I2C_SR1_BERR);
 	}
 
-	/* Transfer matched our slave address */
+	/* Transfer matched our peripheral address */
 	if (i2c_sr1 & STM32_I2C_SR1_ADDR) {
-		addr = ((i2c_sr2 & STM32_I2C_SR2_DUALF) ?
+		addr_8bit = ((i2c_sr2 & STM32_I2C_SR2_DUALF) ?
 			STM32_I2C_OAR2(port) : STM32_I2C_OAR1(port)) & 0xfe;
 		if (i2c_sr2 & STM32_I2C_SR2_TRA) {
-			/* Transmitter slave */
+			/* Transmitter peripheral */
 			i2c_sr1 |= STM32_I2C_SR1_TXE;
-#ifdef CONFIG_BOARD_I2C_SLAVE_ADDR
+#ifdef CONFIG_BOARD_I2C_ADDR_FLAGS
 			if (!rx_pending && !tx_pending) {
 				tx_pending = 1;
-				i2c_process_board_command(1, addr, 0);
+				i2c_process_board_command(1, addr_8bit, 0);
 			}
 #endif
 		} else {
-			/* Receiver slave */
+			/* Receiver peripheral */
 			buf_idx = 0;
 			rx_pending = 1;
 		}
@@ -904,13 +903,13 @@ static void i2c_event_handler(int port)
 		/* Enable buffer interrupt to start receive/response */
 		STM32_I2C_CR2(port) |= STM32_I2C_CR2_ITBUFEN;
 		/* Clear ADDR bit */
-		dummy = STM32_I2C_SR1(port);
-		dummy = STM32_I2C_SR2(port);
+		unused = STM32_I2C_SR1(port);
+		unused = STM32_I2C_SR2(port);
 		/* Inhibit stop mode when addressed until STOPF flag is set */
-		disable_sleep(SLEEP_MASK_I2C_SLAVE);
+		disable_sleep(SLEEP_MASK_I2C_PERIPHERAL);
 	}
 
-	/* I2C in slave transmitter */
+	/* I2C in peripheral transmitter */
 	if (i2c_sr2 & STM32_I2C_SR2_TRA) {
 		if (i2c_sr1 & (STM32_I2C_SR1_BTF | STM32_I2C_SR1_TXE)) {
 			if (tx_pending) {
@@ -927,9 +926,11 @@ static void i2c_event_handler(int port)
 				host_i2c_resp_port = port;
 				/* Disable buffer interrupt */
 				STM32_I2C_CR2(port) &= ~STM32_I2C_CR2_ITBUFEN;
-#ifdef CONFIG_BOARD_I2C_SLAVE_ADDR
-				if (addr == CONFIG_BOARD_I2C_SLAVE_ADDR)
-					i2c_process_board_command(1, addr,
+#ifdef CONFIG_BOARD_I2C_ADDR_FLAGS
+				if ((addr_8bit >> 1) ==
+				    I2C_STRIP_FLAGS(
+					    CONFIG_BOARD_I2C_ADDR_FLAGS))
+					i2c_process_board_command(1, addr_8bit,
 								  buf_idx);
 				else
 #endif
@@ -941,7 +942,7 @@ static void i2c_event_handler(int port)
 				STM32_I2C_DR(port) = 0xec;
 			}
 		}
-	} else { /* I2C in slave receiver */
+	} else { /* I2C in peripheral receiver */
 		if (i2c_sr1 & (STM32_I2C_SR1_BTF | STM32_I2C_SR1_RXNE))
 			host_buffer[buf_idx++] = STM32_I2C_DR(port);
 	}
@@ -951,9 +952,11 @@ static void i2c_event_handler(int port)
 		/* Disable buffer interrupt */
 		STM32_I2C_CR2(port) &= ~STM32_I2C_CR2_ITBUFEN;
 
-#ifdef CONFIG_BOARD_I2C_SLAVE_ADDR
-		if (rx_pending && addr == CONFIG_BOARD_I2C_SLAVE_ADDR)
-			i2c_process_board_command(0, addr, buf_idx);
+#ifdef CONFIG_BOARD_I2C_ADDR_FLAGS
+		if (rx_pending &&
+		    (addr_8b >> 1) ==
+		    I2C_STRIP_FLAGS(CONFIG_BOARD_I2C_ADDR_FLAGS))
+			i2c_process_board_command(0, addr_8bit, buf_idx);
 #endif
 		rx_pending = 0;
 		tx_pending = 0;
@@ -961,25 +964,25 @@ static void i2c_event_handler(int port)
 		/* Clear AF */
 		STM32_I2C_SR1(port) &= ~STM32_I2C_SR1_AF;
 		/* Clear STOPF: read SR1 and write CR1 */
-		dummy = STM32_I2C_SR1(port);
+		unused = STM32_I2C_SR1(port);
 		STM32_I2C_CR1(port) = i2c_cr1 | STM32_I2C_CR1_PE;
 
 		/* No longer inhibit deep sleep after stop condition */
-		enable_sleep(SLEEP_MASK_I2C_SLAVE);
+		enable_sleep(SLEEP_MASK_I2C_PERIPHERAL);
 	}
 
 	/* Enable again */
 	if (!(i2c_cr1 & STM32_I2C_CR1_PE))
 		STM32_I2C_CR1(port) |= STM32_I2C_CR1_PE;
 }
-void i2c_event_interrupt(void) { i2c_event_handler(I2C_PORT_EC); }
-DECLARE_IRQ(IRQ_SLAVE_EV, i2c_event_interrupt, 2);
-DECLARE_IRQ(IRQ_SLAVE_ER, i2c_event_interrupt, 2);
+static void i2c_event_interrupt(void) { i2c_event_handler(I2C_PORT_EC); }
+DECLARE_IRQ(IRQ_PERIPHERAL_EV, i2c_event_interrupt, 2);
+DECLARE_IRQ(IRQ_PERIPHERAL_ER, i2c_event_interrupt, 2);
 #endif
 
 
 /* Init all available i2c ports */
-static void i2c_init(void)
+void i2c_init(void)
 {
 	const struct i2c_port_t *p = i2c_ports;
 	int i;
@@ -988,21 +991,20 @@ static void i2c_init(void)
 		i2c_init_port(p);
 
 
-#ifdef CONFIG_HOSTCMD_I2C_SLAVE_ADDR
+#ifdef CONFIG_HOSTCMD_I2C_ADDR_FLAGS
 	/* Enable ACK */
 	STM32_I2C_CR1(I2C_PORT_EC) |= STM32_I2C_CR1_ACK;
 	/* Enable interrupts */
 	STM32_I2C_CR2(I2C_PORT_EC) |= STM32_I2C_CR2_ITEVTEN
 			| STM32_I2C_CR2_ITERREN;
-	/* Setup host command slave */
+	/* Setup host command peripheral */
 	STM32_I2C_OAR1(I2C_PORT_EC) = STM32_I2C_OAR1_B14
-			| CONFIG_HOSTCMD_I2C_SLAVE_ADDR;
-#ifdef CONFIG_BOARD_I2C_SLAVE_ADDR
+		| (I2C_STRIP_ADDR(CONFIG_HOSTCMD_I2C_ADDR_FLAGS) << 1);
+#ifdef CONFIG_BOARD_I2C_ADDR_FLAGS
 	STM32_I2C_OAR2(I2C_PORT_EC) = STM32_I2C_OAR2_ENDUAL
-			| CONFIG_BOARD_I2C_SLAVE_ADDR;
+		| (I2C_STRIP_FLAGS(CONFIG_BOARD_I2C_ADDR_FLAGS) << 1);
 #endif
-	task_enable_irq(IRQ_SLAVE_EV);
-	task_enable_irq(IRQ_SLAVE_ER);
+	task_enable_irq(IRQ_PERIPHERAL_EV);
+	task_enable_irq(IRQ_PERIPHERAL_ER);
 #endif
 }
-DECLARE_HOOK(HOOK_INIT, i2c_init, HOOK_PRIO_INIT_I2C);
