@@ -1,4 +1,4 @@
-/* Copyright (c) 2013 The Chromium OS Authors. All rights reserved.
+/* Copyright 2013 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -9,17 +9,21 @@
 #include "clock.h"
 #include "common.h"
 #include "console.h"
+#include "ec_commands.h"
 #include "hooks.h"
 #include "host_command.h"
 #include "keyboard_config.h"
 #include "keyboard_protocol.h"
 #include "keyboard_raw.h"
 #include "keyboard_scan.h"
+#include "keyboard_test.h"
 #include "lid_switch.h"
 #include "switch.h"
 #include "system.h"
+#include "tablet_mode.h"
 #include "task.h"
 #include "timer.h"
+#include "usb_api.h"
 #include "util.h"
 
 /* Console output macros */
@@ -49,21 +53,29 @@
 #define CONFIG_KEYBOARD_POST_SCAN_CLOCKS 16000
 #endif
 
-#ifndef CONFIG_KEYBOARD_BOARD_CONFIG
-/* Use default keyboard scan config, because board didn't supply one */
-struct keyboard_scan_config keyscan_config = {
+__overridable struct keyboard_scan_config keyscan_config = {
+#ifdef CONFIG_KEYBOARD_COL2_INVERTED
+	/*
+	 * CONFIG_KEYBOARD_COL2_INVERTED is defined for passing the column 2
+	 * to H1 which inverts the signal. The signal passing through H1
+	 * adds more delay. Need a larger delay value. Otherwise, pressing
+	 * Refresh key will also trigger T key, which is in the next scanning
+	 * column line. See http://b/156007029.
+	 */
+	.output_settle_us = 80,
+#else
 	.output_settle_us = 50,
+#endif /* CONFIG_KEYBOARD_COL2_INVERTED */
 	.debounce_down_us = 9 * MSEC,
 	.debounce_up_us = 30 * MSEC,
 	.scan_period_us = 3 * MSEC,
 	.min_post_scan_delay_us = 1000,
 	.poll_timeout_us = 100 * MSEC,
 	.actual_key_mask = {
-		0x14, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xff,
+		0x1c, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xff,
 		0xa4, 0xff, 0xfe, 0x55, 0xfa, 0xca  /* full set */
 	},
 };
-#endif
 
 /* Boot key list.  Must be in same order as enum boot_key. */
 struct boot_key_entry {
@@ -80,34 +92,37 @@ static const struct boot_key_entry boot_key_list[] = {
 static uint32_t boot_key_value = BOOT_KEY_NONE;
 #endif
 
+uint8_t keyboard_cols = KEYBOARD_COLS_MAX;
+
 /* Debounced key matrix */
-static uint8_t __bss_slow debounced_state[KEYBOARD_COLS];
-/* Matrix from previous scan */
-static uint8_t __bss_slow prev_state[KEYBOARD_COLS];
+static uint8_t debounced_state[KEYBOARD_COLS_MAX];
 /* Mask of keys being debounced */
-static uint8_t __bss_slow debouncing[KEYBOARD_COLS];
+static uint8_t debouncing[KEYBOARD_COLS_MAX];
 /* Keys simulated-pressed */
-static uint8_t __bss_slow simulated_key[KEYBOARD_COLS];
+static uint8_t simulated_key[KEYBOARD_COLS_MAX];
+#ifdef CONFIG_KEYBOARD_LANGUAGE_ID
+static uint8_t keyboard_id[KEYBOARD_IDS];
+#endif
 
 /* Times of last scans */
-static uint32_t __bss_slow scan_time[SCAN_TIME_COUNT];
+static uint32_t scan_time[SCAN_TIME_COUNT];
 /* Current scan_time[] index */
-static int __bss_slow scan_time_index;
+static int scan_time_index;
 
 /* Index into scan_time[] when each key started debouncing */
-static uint8_t __bss_slow scan_edge_index[KEYBOARD_COLS][KEYBOARD_ROWS];
+static uint8_t scan_edge_index[KEYBOARD_COLS_MAX][KEYBOARD_ROWS];
 
 /* Minimum delay between keyboard scans based on current clock frequency */
-static uint32_t __bss_slow post_scan_clock_us;
+static uint32_t post_scan_clock_us;
 
 /*
  * Print all keyboard scan state changes?  Off by default because it generates
  * a lot of debug output, which makes the saved EC console data less useful.
  */
-static int __bss_slow print_state_changes;
+static int print_state_changes;
 
 /* Must init to 0 for scanning at boot */
-static volatile uint32_t __bss_slow disable_scanning_mask;
+static volatile uint32_t disable_scanning_mask;
 
 /* Constantly incrementing counter of the number of times we polled */
 static volatile int kbd_polls;
@@ -125,9 +140,9 @@ void keyboard_scan_enable(int enable, enum kb_scan_disable_masks mask)
 {
 	/* Access atomically */
 	if (enable) {
-		atomic_clear((uint32_t *)&disable_scanning_mask, mask);
+		atomic_clear_bits((atomic_t *)&disable_scanning_mask, mask);
 	} else {
-		atomic_or((uint32_t *)&disable_scanning_mask, mask);
+		atomic_or((atomic_t *)&disable_scanning_mask, mask);
 		clear_typematic_key();
 	}
 
@@ -145,8 +160,8 @@ static void print_state(const uint8_t *state, const char *msg)
 {
 	int c;
 
-	CPRINTF("[%T KB %s:", msg);
-	for (c = 0; c < KEYBOARD_COLS; c++) {
+	CPRINTF("[%pT KB %s:", PRINTF_TIMESTAMP_NOW, msg);
+	for (c = 0; c < keyboard_cols; c++) {
 		if (state[c])
 			CPRINTF(" %02x", state[c]);
 		else
@@ -190,10 +205,10 @@ static void simulate_key(int row, int col, int pressed)
 {
 	int old_polls;
 
-	if ((simulated_key[col] & (1 << row)) == ((pressed ? 1 : 0) << row))
+	if ((simulated_key[col] & BIT(row)) == ((pressed ? 1 : 0) << row))
 		return;  /* No change */
 
-	simulated_key[col] ^= (1 << row);
+	simulated_key[col] ^= BIT(row);
 
 	/* Keep track of polls now that we've got keys simulated */
 	old_polls = kbd_polls;
@@ -222,51 +237,81 @@ static void simulate_key(int row, int col, int pressed)
  * Used in pre-init, so must not make task-switching-dependent calls; udelay()
  * is ok because it's a spin-loop.
  *
- * @param state		Destination for new state (must be KEYBOARD_COLS long).
+ * @param state		Destination for new state (must be KEYBOARD_COLS_MAX
+ *			long).
  *
  * @return 1 if at least one key is pressed, else zero.
  */
 static int read_matrix(uint8_t *state)
 {
 	int c;
-	uint8_t r;
 	int pressed = 0;
 
-	for (c = 0; c < KEYBOARD_COLS; c++) {
+	/* 1. Read input pins */
+	for (c = 0; c < keyboard_cols; c++) {
 		/*
-		 * Stop if scanning becomes disabled. Note, scanning is enabled
-		 * on boot by default.
+		 * Skip if scanning becomes disabled. Clear the state
+		 * to make sure we don't mix new and old states in the
+		 * same array.
+		 *
+		 * Note, scanning is enabled on boot by default.
 		 */
-		if (!keyboard_scan_is_enabled())
-			break;
+		if (!keyboard_scan_is_enabled()) {
+			state[c] = 0;
+			continue;
+		}
 
 		/* Select column, then wait a bit for it to settle */
 		keyboard_raw_drive_column(c);
 		udelay(keyscan_config.output_settle_us);
 
 		/* Read the row state */
-		r = keyboard_raw_read_rows();
+		state[c] = keyboard_raw_read_rows();
 
+		/* Use simulated keyscan sequence instead if testing active */
+		if (IS_ENABLED(CONFIG_KEYBOARD_TEST))
+			state[c] = keyscan_seq_get_scan(c, state[c]);
+	}
+
+	/* 2. Detect transitional ghost */
+	for (c = 0; c < keyboard_cols; c++) {
+		int c2;
+
+		for (c2 = 0; c2 < c; c2++) {
+			/*
+			 * If two columns shares at least one key but their
+			 * states are different, maybe the state changed between
+			 * two "keyboard_raw_read_rows"s. If this happened,
+			 * update both columns to the union of them.
+			 *
+			 * Note that in theory we need to rescan from col 0 if
+			 * anything is updated, to make sure the newly added
+			 * bits does not introduce more inconsistency.
+			 * Let's ignore this rare case for now.
+			 */
+			if ((state[c] & state[c2]) && (state[c] != state[c2])) {
+				uint8_t merged = state[c] | state[c2];
+
+				state[c] = state[c2] = merged;
+			}
+		}
+	}
+
+	/* 3. Fix result */
+	for (c = 0; c < keyboard_cols; c++) {
 		/* Add in simulated keypresses */
-		r |= simulated_key[c];
+		state[c] |= simulated_key[c];
 
 		/*
 		 * Keep track of what keys appear to be pressed.  Even if they
 		 * don't exist in the matrix, they'll keep triggering
 		 * interrupts, so we can't leave scanning mode.
 		 */
-		pressed |= r;
+		pressed |= state[c];
 
 		/* Mask off keys that don't exist on the actual keyboard */
-		r &= keyscan_config.actual_key_mask[c];
+		state[c] &= keyscan_config.actual_key_mask[c];
 
-#ifdef CONFIG_KEYBOARD_TEST
-		/* Use simulated keyscan sequence instead if testing active */
-		r = keyscan_seq_get_scan(c, r);
-#endif
-
-		/* Store the masked state */
-		state[c] = r;
 	}
 
 	keyboard_raw_drive_column(KEYBOARD_COLUMN_NONE);
@@ -274,7 +319,51 @@ static int read_matrix(uint8_t *state)
 	return pressed ? 1 : 0;
 }
 
+#ifdef CONFIG_KEYBOARD_LANGUAGE_ID
+/**
+ * Read the raw keyboard IDs state.
+ *
+ * Used in pre-init, so must not make task-switching-dependent calls; udelay()
+ * is ok because it's a spin-loop.
+ *
+ * @param id		Destination for keyboard id (must be KEYBOARD_IDS long).
+ *
+ */
+static void read_matrix_id(uint8_t *id)
+{
+	int c;
+
+	for (c = 0; c < KEYBOARD_IDS; c++) {
+		/* Select the ID pin, then wait a bit for it to settle.
+		 * Caveat: If a keyboard maker puts ID pins right after scan
+		 * columns, we can't support variable column size with a single
+		 * image. */
+		keyboard_raw_drive_column(KEYBOARD_COLS_MAX + c);
+		udelay(keyscan_config.output_settle_us);
+
+		/* Read the row state */
+		id[c] = keyboard_raw_read_rows();
+
+		CPRINTS("Keyboard ID%u: 0x%02x", c, id[c]);
+	}
+
+	keyboard_raw_drive_column(KEYBOARD_COLUMN_NONE);
+}
+#endif
+
 #ifdef CONFIG_KEYBOARD_RUNTIME_KEYS
+
+static uint8_t key_vol_up_row = KEYBOARD_DEFAULT_ROW_VOL_UP;
+static uint8_t key_vol_up_col = KEYBOARD_DEFAULT_COL_VOL_UP;
+
+void set_vol_up_key(uint8_t row, uint8_t col)
+{
+	if (col < KEYBOARD_COLS_MAX && row < KEYBOARD_ROWS) {
+		key_vol_up_row = row;
+		key_vol_up_col = col;
+	}
+}
+
 /**
  * Check special runtime key combinations.
  *
@@ -287,52 +376,11 @@ static int check_runtime_keys(const uint8_t *state)
 	int num_press = 0;
 	int c;
 
-#ifdef BOARD_SAMUS
-	int16_t chg_override;
-
-	/*
-	 * TODO(crosbug.com/p/34850): remove these hot-keys for samus, should
-	 * be done at higher level than this.
-	 */
-	/*
-	 * On samus, ctrl + search + 0|1|2 sets the active charge port
-	 * by sending the charge override host command. Should only be sent
-	 * when chipset is in S0. Note that 'search' and '1' keys are on
-	 * the same column.
-	 */
-	if ((state[KEYBOARD_COL_LEFT_CTRL] == KEYBOARD_MASK_LEFT_CTRL ||
-	     state[KEYBOARD_COL_RIGHT_CTRL] == KEYBOARD_MASK_RIGHT_CTRL) &&
-	    ((state[KEYBOARD_COL_SEARCH] & KEYBOARD_MASK_SEARCH) ==
-						KEYBOARD_MASK_SEARCH) &&
-	    chipset_in_state(CHIPSET_STATE_ON)) {
-		if (state[KEYBOARD_COL_KEY_0] == KEYBOARD_MASK_KEY_0) {
-			/* Charge from neither port */
-			chg_override = -2;
-			pd_host_command(EC_CMD_PD_CHARGE_PORT_OVERRIDE, 0,
-					&chg_override, 2, NULL, 0);
-			return 0;
-		} else if (state[KEYBOARD_COL_KEY_1] ==
-			   (KEYBOARD_MASK_KEY_1 | KEYBOARD_MASK_SEARCH)) {
-			/* Charge from port 0 (left side) */
-			chg_override = 0;
-			pd_host_command(EC_CMD_PD_CHARGE_PORT_OVERRIDE, 0,
-					&chg_override, 2, NULL, 0);
-			return 0;
-		} else if (state[KEYBOARD_COL_KEY_2] == KEYBOARD_MASK_KEY_2) {
-			/* Charge from port 1 (right side) */
-			chg_override = 1;
-			pd_host_command(EC_CMD_PD_CHARGE_PORT_OVERRIDE, 0,
-					&chg_override, 2, NULL, 0);
-			return 0;
-		}
-	}
-#endif
-
 	/*
 	 * All runtime key combos are (right or left ) alt + volume up + (some
 	 * key NOT on the same col as alt or volume up )
 	 */
-	if (state[KEYBOARD_COL_VOL_UP] != KEYBOARD_MASK_VOL_UP)
+	if (state[key_vol_up_col] != KEYBOARD_ROW_TO_MASK(key_vol_up_row))
 		return 0;
 
 	if (state[KEYBOARD_COL_RIGHT_ALT] != KEYBOARD_MASK_RIGHT_ALT &&
@@ -344,7 +392,7 @@ static int check_runtime_keys(const uint8_t *state)
 	 * pressed for volume up and alt, so if only one more key is pressed
 	 * there will be exactly 3 non-zero columns.
 	 */
-	for (c = 0; c < KEYBOARD_COLS; c++) {
+	for (c = 0; c < keyboard_cols; c++) {
 		if (state[c])
 			num_press++;
 	}
@@ -357,12 +405,12 @@ static int check_runtime_keys(const uint8_t *state)
 		/* R = reboot */
 		CPRINTS("KB warm reboot");
 		keyboard_clear_buffer();
-		chipset_reset(0);
+		chipset_reset(CHIPSET_RESET_KB_WARM_REBOOT);
 		return 1;
 	} else if (state[KEYBOARD_COL_KEY_H] == KEYBOARD_MASK_KEY_H) {
 		/* H = hibernate */
 		CPRINTS("KB hibernate");
-		system_hibernate(0, 0);
+		system_enter_hibernate(0, 0);
 		return 1;
 	}
 
@@ -385,11 +433,11 @@ static int has_ghosting(const uint8_t *state)
 {
 	int c, c2;
 
-	for (c = 0; c < KEYBOARD_COLS; c++) {
+	for (c = 0; c < keyboard_cols; c++) {
 		if (!state[c])
 			continue;
 
-		for (c2 = c + 1; c2 < KEYBOARD_COLS; c2++) {
+		for (c2 = c + 1; c2 < keyboard_cols; c2++) {
 			/*
 			 * A little bit of cleverness here.  Ghosting happens
 			 * if 2 columns share at least 2 keys.  So we OR the
@@ -407,6 +455,16 @@ static int has_ghosting(const uint8_t *state)
 	return 0;
 }
 
+/* Inform keyboard module if scanning is enabled */
+static void key_state_changed(int row, int col, uint8_t state)
+{
+	if (!keyboard_scan_is_enabled())
+		return;
+
+	/* No-op for protocols that require full keyboard matrix (e.g. MKBP). */
+	keyboard_state_changed(row, col, !!(state & BIT(row)));
+}
+
 /**
  * Update keyboard state using low-level interface to read keyboard.
  *
@@ -419,7 +477,7 @@ static int check_keys_changed(uint8_t *state)
 	int any_pressed = 0;
 	int c, i;
 	int any_change = 0;
-	static uint8_t __bss_slow new_state[KEYBOARD_COLS];
+	static uint8_t new_state[KEYBOARD_COLS_MAX];
 	uint32_t tnow = get_time().le.lo;
 
 	/* Save the current scan time */
@@ -435,57 +493,58 @@ static int check_keys_changed(uint8_t *state)
 		return any_pressed;
 
 	/* Check for changes between previous scan and this one */
-	for (c = 0; c < KEYBOARD_COLS; c++) {
-		int diff = new_state[c] ^ prev_state[c];
+	for (c = 0; c < keyboard_cols; c++) {
+		int diff = new_state[c] ^ state[c];
 
-		if (!diff)
-			continue;
-
-		for (i = 0; i < KEYBOARD_ROWS; i++) {
-			if (diff & (1 << i))
-				scan_edge_index[c][i] = scan_time_index;
-		}
-
-		debouncing[c] |= diff;
-		prev_state[c] = new_state[c];
-	}
-
-	/* Check for keys which are done debouncing */
-	for (c = 0; c < KEYBOARD_COLS; c++) {
-		int debc = debouncing[c];
-
-		if (!debc)
-			continue;
-
-		for (i = 0; i < KEYBOARD_ROWS; i++) {
-			int mask = 1 << i;
-			int new_mask = new_state[c] & mask;
-
-			/* Are we done debouncing this key? */
-			if (!(debc & mask))
-				continue;  /* Not debouncing this key */
+		/* Clear debouncing flag, if sufficient time has elapsed. */
+		for (i = 0; i < KEYBOARD_ROWS && debouncing[c]; i++) {
+			if (!(debouncing[c] & BIT(i)))
+				continue;
 			if (tnow - scan_time[scan_edge_index[c][i]] <
-			    (new_mask ? keyscan_config.debounce_down_us :
+			    (state[c] ? keyscan_config.debounce_down_us :
 					keyscan_config.debounce_up_us))
 				continue;  /* Not done debouncing */
+			debouncing[c] &= ~BIT(i);
 
-			debouncing[c] &= ~mask;
-
-			/* Did the key change from its previous state? */
-			if ((state[c] & mask) == new_mask)
-				continue;  /* No */
-
-			state[c] ^= mask;
+			if (!IS_ENABLED(CONFIG_KEYBOARD_STRICT_DEBOUNCE))
+				continue;
+			if (!(diff & BIT(i)))
+				/* Debounced but no difference. */
+				continue;
 			any_change = 1;
+			key_state_changed(i, c, new_state[c]);
+			/*
+			 * This makes state[c] == new_state[c] for row i.
+			 * Thus, when diff is calculated below, it won't
+			 * be asserted (for row i).
+			 */
+			state[c] ^= diff & BIT(i);
+		}
 
-			/* Inform keyboard module if scanning is enabled */
-			if (keyboard_scan_is_enabled()) {
-				/* This is no-op for protocols that require a
-				 * full keyboard matrix (e.g., MKBP).
-				 */
-				keyboard_state_changed(i, c, new_mask ? 1 : 0);
+		/* Recognize change in state, unless debounce in effect. */
+		diff = (new_state[c] ^ state[c]) & ~debouncing[c];
+		if (!diff)
+			continue;
+		for (i = 0; i < KEYBOARD_ROWS; i++) {
+			if (!(diff & BIT(i)))
+				continue;
+			scan_edge_index[c][i] = scan_time_index;
+
+			if (!IS_ENABLED(CONFIG_KEYBOARD_STRICT_DEBOUNCE)) {
+				any_change = 1;
+				key_state_changed(i, c, new_state[c]);
 			}
 		}
+
+		/* For any keyboard events just sent, turn on debouncing. */
+		debouncing[c] |= diff;
+		/*
+		 * Note: In order to "remember" what was last reported
+		 * (up or down), the state bits are only updated if the
+		 * edge was not suppressed due to debouncing.
+		 */
+		if (!IS_ENABLED(CONFIG_KEYBOARD_STRICT_DEBOUNCE))
+			state[c] ^= diff;
 	}
 
 	if (any_change) {
@@ -500,7 +559,7 @@ static int check_keys_changed(uint8_t *state)
 
 #ifdef CONFIG_KEYBOARD_PRINT_SCAN_TIMES
 		/* Print delta times from now back to each previous scan */
-		CPRINTF("[%T kb deltaT");
+		CPRINTF("[%pT kb deltaT", PRINTF_TIMESTAMP_NOW);
 		for (i = 0; i < SCAN_TIME_COUNT; i++) {
 			int tnew = scan_time[
 				(SCAN_TIME_COUNT + scan_time_index - i) %
@@ -517,7 +576,7 @@ static int check_keys_changed(uint8_t *state)
 #endif
 
 #ifdef CONFIG_KEYBOARD_PROTOCOL_MKBP
-		keyboard_fifo_add(state);
+		mkbp_keyboard_add(state);
 #endif
 	}
 
@@ -526,6 +585,15 @@ static int check_keys_changed(uint8_t *state)
 	return any_pressed;
 }
 
+static uint8_t keyboard_mask_refresh;
+__overridable uint8_t board_keyboard_row_refresh(void)
+{
+	if (IS_ENABLED(CONFIG_KEYBOARD_REFRESH_ROW3))
+		return 3;
+	else
+		return 2;
+}
+
 #ifdef CONFIG_KEYBOARD_BOOT_KEYS
 /*
  * Returns mask of the boot keys that are pressed, with at most the keys used
@@ -533,7 +601,7 @@ static int check_keys_changed(uint8_t *state)
  */
 static uint32_t check_key_list(const uint8_t *state)
 {
-	uint8_t curr_state[KEYBOARD_COLS];
+	uint8_t curr_state[KEYBOARD_COLS_MAX];
 	int c;
 	uint32_t boot_key_mask = BOOT_KEY_NONE;
 	const struct boot_key_entry *k;
@@ -541,32 +609,34 @@ static uint32_t check_key_list(const uint8_t *state)
 	/* Make copy of current debounced state. */
 	memcpy(curr_state, state, sizeof(curr_state));
 
-#ifdef CONFIG_KEYBOARD_PWRBTN_ASSERTS_KSI2
+#ifdef KEYBOARD_MASK_PWRBTN
 	/*
-	 * Check if KSI2 is asserted for all columns due to power button hold,
-	 * and ignore it if so.
+	 * Check if KSI2 or KSI3 is asserted for all columns due to power
+	 * button hold, and ignore it if so.
 	 */
-	for (c = 0; c < KEYBOARD_COLS; c++)
-		if ((keyscan_config.actual_key_mask[c] & KEYBOARD_MASK_KSI2) &&
-		   !(curr_state[c] & KEYBOARD_MASK_KSI2))
+	for (c = 0; c < keyboard_cols; c++)
+		if ((keyscan_config.actual_key_mask[c] & KEYBOARD_MASK_PWRBTN)
+		    && !(curr_state[c] & KEYBOARD_MASK_PWRBTN))
 			break;
 
-	if (c == KEYBOARD_COLS)
-		for (c = 0; c < KEYBOARD_COLS; c++)
-			curr_state[c] &= ~KEYBOARD_MASK_KSI2;
+	if (c == keyboard_cols)
+		for (c = 0; c < keyboard_cols; c++)
+			curr_state[c] &= ~KEYBOARD_MASK_PWRBTN;
 #endif
 
+	curr_state[KEYBOARD_COL_REFRESH] &= ~keyboard_mask_refresh;
+
 	/* Update mask with all boot keys that were pressed. */
 	k = boot_key_list;
 	for (c = 0; c < ARRAY_SIZE(boot_key_list); c++, k++) {
 		if (curr_state[k->mask_index] & k->mask_value) {
-			boot_key_mask |= (1 << c);
+			boot_key_mask |= BIT(c);
 			curr_state[k->mask_index] &= ~k->mask_value;
 		}
 	}
 
 	/* If any other key was pressed, ignore all boot keys. */
-	for (c = 0; c < KEYBOARD_COLS; c++) {
+	for (c = 0; c < keyboard_cols; c++) {
 		if (curr_state[c])
 			return BOOT_KEY_NONE;
 	}
@@ -591,12 +661,12 @@ static uint32_t check_boot_key(const uint8_t *state)
 	 * re-triggering events in RW firmware that were already processed by
 	 * RO firmware.
 	 */
-	if (system_jumped_to_this_image())
+	if (system_jumped_late())
 		return BOOT_KEY_NONE;
 
 	/* If reset was not caused by reset pin, refresh must be held down */
-	if (!(system_get_reset_flags() & RESET_FLAG_RESET_PIN) &&
-	    !(state[KEYBOARD_COL_REFRESH] & KEYBOARD_MASK_REFRESH))
+	if (!(system_get_reset_flags() & EC_RESET_FLAG_RESET_PIN) &&
+	    !(state[KEYBOARD_COL_REFRESH] & keyboard_mask_refresh))
 		return BOOT_KEY_NONE;
 
 	return check_key_list(state);
@@ -632,6 +702,19 @@ const uint8_t *keyboard_scan_get_state(void)
 
 void keyboard_scan_init(void)
 {
+	if (IS_ENABLED(CONFIG_KEYBOARD_STRICT_DEBOUNCE) &&
+	    keyscan_config.debounce_down_us != keyscan_config.debounce_up_us) {
+		/*
+		 * Strict debouncer is prone to keypress reordering if debounce
+		 * durations for down and up are not equal. crbug.com/547131
+		 */
+		CPRINTS("KB WARN: Debounce durations not equal");
+	}
+
+	/* Configure refresh key matrix */
+	keyboard_mask_refresh = KEYBOARD_ROW_TO_MASK(
+		board_keyboard_row_refresh());
+
 	/* Configure GPIO */
 	keyboard_raw_init();
 
@@ -640,7 +723,11 @@ void keyboard_scan_init(void)
 
 	/* Initialize raw state */
 	read_matrix(debounced_state);
-	memcpy(prev_state, debounced_state, sizeof(prev_state));
+
+#ifdef CONFIG_KEYBOARD_LANGUAGE_ID
+	/* Check keyboard ID state */
+	read_matrix_id(keyboard_id);
+#endif
 
 #ifdef CONFIG_KEYBOARD_BOOT_KEYS
 	/* Check for keys held down at boot */
@@ -656,6 +743,16 @@ void keyboard_scan_init(void)
 #ifdef CONFIG_HOSTCMD_EVENTS
 	if (boot_key_value & BOOT_KEY_ESC) {
 		host_set_single_event(EC_HOST_EVENT_KEYBOARD_RECOVERY);
+		/*
+		 * In recovery mode, we should force clamshell mode in order to
+		 * prevent the keyboard from being disabled unintentionally due
+		 * to unstable accel readings.
+		 *
+		 * You get the same effect if motion sensors or a motion sense
+		 * task are disabled in RO.
+		 */
+		if (IS_ENABLED(CONFIG_TABLET_MODE))
+			tablet_disable();
 		if (boot_key_value & BOOT_KEY_LEFT_SHIFT)
 			host_set_single_event(
 				EC_HOST_EVENT_KEYBOARD_RECOVERY_HW_REINIT);
@@ -664,7 +761,7 @@ void keyboard_scan_init(void)
 #endif /* CONFIG_KEYBOARD_BOOT_KEYS */
 }
 
-void keyboard_scan_task(void)
+void keyboard_scan_task(void *u)
 {
 	timestamp_t poll_deadline, start;
 	int wait_time;
@@ -777,10 +874,26 @@ DECLARE_HOOK(HOOK_INIT, keyboard_lid_change, HOOK_PRIO_INIT_LID + 1);
 
 #endif
 
+#ifdef CONFIG_USB_SUSPEND
+static void keyboard_usb_pm_change(void)
+{
+	/*
+	 * If USB interface is suspended, and host is not asking us to do remote
+	 * wakeup, we can turn off the key scanning.
+	 */
+	if (usb_is_suspended() && !usb_is_remote_wakeup_enabled())
+		keyboard_scan_enable(0, KB_SCAN_DISABLE_USB_SUSPENDED);
+	else
+		keyboard_scan_enable(1, KB_SCAN_DISABLE_USB_SUSPENDED);
+}
+DECLARE_HOOK(HOOK_USB_PM_CHANGE, keyboard_usb_pm_change, HOOK_PRIO_DEFAULT);
+#endif
+
 /*****************************************************************************/
 /* Host commands */
 
-static int mkbp_command_simulate_key(struct host_cmd_handler_args *args)
+static enum ec_status
+mkbp_command_simulate_key(struct host_cmd_handler_args *args)
 {
 	const struct ec_params_mkbp_simulate_key *p = args->params;
 
@@ -788,7 +901,7 @@ static int mkbp_command_simulate_key(struct host_cmd_handler_args *args)
 	if (system_is_locked())
 		return EC_RES_ACCESS_DENIED;
 
-	if (p->col >= KEYBOARD_COLS || p->row >= KEYBOARD_ROWS)
+	if (p->col >= keyboard_cols || p->row >= KEYBOARD_ROWS)
 		return EC_RES_INVALID_PARAM;
 
 	simulate_key(p->row, p->col, p->pressed);
@@ -800,7 +913,71 @@ DECLARE_HOST_COMMAND(EC_CMD_MKBP_SIMULATE_KEY,
 		     EC_VER_MASK(0));
 
 #ifdef CONFIG_KEYBOARD_FACTORY_TEST
-static int keyboard_factory_test(struct host_cmd_handler_args *args)
+
+/* Run keyboard factory testing, scan out KSO/KSI if any shorted. */
+int keyboard_factory_test_scan(void)
+{
+	int i, j, flags;
+	uint16_t shorted = 0;
+	int port, id;
+
+	/* Disable keyboard scan while testing */
+	keyboard_scan_enable(0, KB_SCAN_DISABLE_LID_CLOSED);
+	flags = gpio_get_default_flags(GPIO_KBD_KSO2);
+
+	/* Set all of KSO/KSI pins to internal pull-up and input */
+	for (i = 0; i < keyboard_factory_scan_pins_used; i++) {
+
+		if (keyboard_factory_scan_pins[i][0] < 0)
+			continue;
+
+		port = keyboard_factory_scan_pins[i][0];
+		id = keyboard_factory_scan_pins[i][1];
+
+		gpio_set_alternate_function(port, 1 << id,
+					GPIO_ALT_FUNC_NONE);
+		gpio_set_flags_by_mask(port, 1 << id,
+			GPIO_INPUT | GPIO_PULL_UP);
+	}
+
+	/*
+	 * Set start pin to output low, then check other pins
+	 * going to low level, it indicate the two pins are shorted.
+	 */
+	for (i = 0; i < keyboard_factory_scan_pins_used; i++) {
+
+		if (keyboard_factory_scan_pins[i][0] < 0)
+			continue;
+
+		port = keyboard_factory_scan_pins[i][0];
+		id = keyboard_factory_scan_pins[i][1];
+
+		gpio_set_flags_by_mask(port, 1 << id, GPIO_OUT_LOW);
+
+		for (j = 0; j < keyboard_factory_scan_pins_used; j++) {
+
+			if (keyboard_factory_scan_pins[j][0] < 0 || i == j)
+				continue;
+
+			if (keyboard_raw_is_input_low(
+					keyboard_factory_scan_pins[j][0],
+					keyboard_factory_scan_pins[j][1])) {
+				shorted = i << 8 | j;
+				goto done;
+			}
+		}
+		gpio_set_flags_by_mask(port, 1 << id,
+			GPIO_INPUT | GPIO_PULL_UP);
+	}
+done:
+	gpio_config_module(MODULE_KEYBOARD_SCAN, 1);
+	gpio_set_flags(GPIO_KBD_KSO2, flags);
+	keyboard_scan_enable(1, KB_SCAN_DISABLE_LID_CLOSED);
+
+	return shorted;
+}
+
+static enum ec_status keyboard_factory_test(struct host_cmd_handler_args *args)
 {
 	struct ec_response_keyboard_factory_test *r = args->response;
 
@@ -823,16 +1000,41 @@ DECLARE_HOST_COMMAND(EC_CMD_KEYBOARD_FACTORY_TEST,
 		     EC_VER_MASK(0));
 #endif
 
+#ifdef CONFIG_KEYBOARD_LANGUAGE_ID
+int keyboard_get_keyboard_id(void)
+{
+	int c;
+	uint32_t id = 0;
+
+	BUILD_ASSERT(sizeof(id) >= KEYBOARD_IDS);
+
+	for (c = 0; c < KEYBOARD_IDS; c++) {
+		/* Check ID ghosting if more than one bit in any KSIs was set */
+		if (keyboard_id[c] & (keyboard_id[c] - 1))
+			/* ID ghosting is found */
+			return KEYBOARD_ID_UNREADABLE;
+		else
+			id |= keyboard_id[c] << (c * 8);
+	}
+	return id;
+}
+#endif
+
 /*****************************************************************************/
 /* Console commands */
 #ifdef CONFIG_CMD_KEYBOARD
 static int command_ksstate(int argc, char **argv)
 {
-	if (argc > 1 && !parse_bool(argv[1], &print_state_changes))
-		return EC_ERROR_PARAM1;
+	if (argc > 1) {
+		if (!strcasecmp(argv[1], "force")) {
+			print_state_changes = 1;
+			keyboard_scan_enable(1, -1);
+		} else if (!parse_bool(argv[1], &print_state_changes)) {
+			return EC_ERROR_PARAM1;
+		}
+	}
 
 	print_state(debounced_state, "debounced ");
-	print_state(prev_state, "prev      ");
 	print_state(debouncing, "debouncing");
 
 	ccprintf("Keyboard scan disable mask: 0x%08x\n",
@@ -842,7 +1044,7 @@ static int command_ksstate(int argc, char **argv)
 	return EC_SUCCESS;
 }
 DECLARE_CONSOLE_COMMAND(ksstate, command_ksstate,
-			"ksstate [on | off]",
+			"ksstate [on | off | force]",
 			"Show or toggle printing keyboard scan state");
 
 static int command_keyboard_press(int argc, char **argv)
@@ -851,11 +1053,11 @@ static int command_keyboard_press(int argc, char **argv)
 		int i, j;
 
 		ccputs("Simulated keys:\n");
-		for (i = 0; i < KEYBOARD_COLS; ++i) {
+		for (i = 0; i < keyboard_cols; ++i) {
 			if (simulated_key[i] == 0)
 				continue;
 			for (j = 0; j < KEYBOARD_ROWS; ++j)
-				if (simulated_key[i] & (1 << j))
+				if (simulated_key[i] & BIT(j))
 					ccprintf("\t%d %d\n", i, j);
 		}
 
@@ -864,7 +1066,7 @@ static int command_keyboard_press(int argc, char **argv)
 		char *e;
 
 		c = strtoi(argv[1], &e, 0);
-		if (*e || c < 0 || c >= KEYBOARD_COLS)
+		if (*e || c < 0 || c >= keyboard_cols)
 			return EC_ERROR_PARAM1;
 
 		r = strtoi(argv[2], &e, 0);
