@@ -17,10 +17,10 @@
 #include "watchdog.h"
 
 #ifdef CONFIG_SPI_NOR_DEBUG
-#define DEBUG_CPRINTS_DEVICE(spi_nor_device, string, args...) \
-	cprints(CC_SPI, "SPI NOR %s: " string, spi_nor_device->name, ## args)
+#define CPRINTS(dev, string, args...) \
+	cprints(CC_SPI, "SPI NOR %s: " string, (dev)->name, ## args)
 #else
-#define DEBUG_CPRINTS_DEVICE(spi_nor_device, string, args...)
+#define CPRINTS(dev, string, args...)
 #endif
 
 /* Time to sleep while serial NOR flash write is in progress. */
@@ -60,7 +60,7 @@ static int spi_nor_read_status(const struct spi_nor_device_t *spi_nor_device,
 {
 	uint8_t cmd = SPI_NOR_OPCODE_READ_STATUS;
 
-	return spi_transaction(&spi_devices[spi_nor_device->spi_master],
+	return spi_transaction(&spi_devices[spi_nor_device->spi_controller],
 			       &cmd, 1, status_register_value, 1);
 }
 
@@ -74,7 +74,7 @@ static int spi_nor_write_enable(const struct spi_nor_device_t *spi_nor_device)
 	int rv = EC_SUCCESS;
 
 	/* Set the write enable latch. */
-	rv = spi_transaction(&spi_devices[spi_nor_device->spi_master],
+	rv = spi_transaction(&spi_devices[spi_nor_device->spi_controller],
 			     &cmd, 1, NULL, 0);
 	if (rv)
 		return rv;
@@ -89,6 +89,62 @@ static int spi_nor_write_enable(const struct spi_nor_device_t *spi_nor_device)
 	return rv;
 }
 
+/**
+ * Read from the extended address register.
+ * @param  spi_nor_device The Serial NOR Flash device to use.
+ * @param  value          The value to read to.
+ * @return                ec_error_list (non-zero on error and timeout).
+ */
+static int spi_nor_read_ear(const struct spi_nor_device_t *spi_nor_device,
+			    uint8_t *value)
+{
+	uint8_t command = SPI_NOR_OPCODE_RDEAR;
+
+	return spi_transaction(&spi_devices[spi_nor_device->spi_controller],
+			&command, sizeof(command), value, 1);
+}
+
+int spi_nor_write_ear(const struct spi_nor_device_t *spi_nor_device,
+		      const uint8_t value)
+{
+	uint8_t buf[2];
+	int rv;
+	uint8_t ear;
+
+	mutex_lock(&driver_mutex);
+
+	rv = spi_nor_write_enable(spi_nor_device);
+	if (rv) {
+		CPRINTS(spi_nor_device, "Failed to write enable");
+		goto err_free;
+	}
+
+	buf[0] = SPI_NOR_OPCODE_WREAR;
+	buf[1] = value;
+
+	rv = spi_transaction(&spi_devices[spi_nor_device->spi_controller],
+			     buf, sizeof(buf), NULL, 0);
+	if (rv) {
+		CPRINTS(spi_nor_device, "Failed to write EAR, rv=%d", rv);
+		goto err_free;
+	}
+
+	rv = spi_nor_read_ear(spi_nor_device, &ear);
+	if (rv)
+		goto err_free;
+
+	if (ear != value) {
+		CPRINTS(spi_nor_device,
+			"Write EAR error: write=%d, read=%d", value, ear);
+		rv = EC_ERROR_UNKNOWN;  /* WEL not set but should be. */
+		goto err_free;
+	}
+
+err_free:
+	mutex_unlock(&driver_mutex);
+	return rv;
+}
+
 /**
  * Block until the Serial NOR Flash clears the BUSY/WIP bit in its status reg.
  */
@@ -136,7 +192,7 @@ static int spi_nor_read_jedec_mfn_id(
 	uint8_t cmd = SPI_NOR_OPCODE_JEDEC_ID;
 
 	/* Read the standardized part of the JEDEC ID. */
-	rv = spi_transaction(&spi_devices[spi_nor_device->spi_master],
+	rv = spi_transaction(&spi_devices[spi_nor_device->spi_controller],
 			     &cmd, 1, jedec_id, SPI_NOR_JEDEC_ID_BANKS);
 	if (rv)
 		return rv;
@@ -172,8 +228,8 @@ static int spi_nor_read_sfdp_dword(
 	sfdp_cmd[1] = (sfdp_offset & 0xFF0000) >> 16;
 	sfdp_cmd[2] = (sfdp_offset & 0xFF00) >> 8;
 	sfdp_cmd[3] = (sfdp_offset & 0xFF);
-	sfdp_cmd[4] = 0;  /* Required dummy cycle. */
-	return spi_transaction(&spi_devices[spi_nor_device->spi_master],
+	sfdp_cmd[4] = 0;  /* Required extra cycle. */
+	return spi_transaction(&spi_devices[spi_nor_device->spi_controller],
 			       sfdp_cmd, 5, (uint8_t *)out_dw, 4);
 }
 
@@ -232,7 +288,7 @@ static int locate_sfdp_basic_parameter_table(
 	 * compatible, older basic parameter tables which are compatible with
 	 * this driver in the parameter headers. */
 	if (!SFDP_HEADER_DW1_SFDP_SIGNATURE_VALID(dw1)) {
-		DEBUG_CPRINTS_DEVICE(spi_nor_device, "SFDP signature invalid");
+		CPRINTS(spi_nor_device, "SFDP signature invalid");
 		return EC_ERROR_UNKNOWN;
 	}
 
@@ -240,15 +296,14 @@ static int locate_sfdp_basic_parameter_table(
 		SFDP_GET_BITFIELD(SFDP_HEADER_DW2_SFDP_MAJOR, dw2);
 	*out_sfdp_minor_rev =
 		SFDP_GET_BITFIELD(SFDP_HEADER_DW2_SFDP_MINOR, dw2);
-	DEBUG_CPRINTS_DEVICE(spi_nor_device, "SFDP v%d.%d discovered",
-			     *out_sfdp_major_rev, *out_sfdp_minor_rev);
+	CPRINTS(spi_nor_device, "SFDP v%d.%d discovered",
+		*out_sfdp_major_rev, *out_sfdp_minor_rev);
 
 	/* NPH is 0-based, so add 1. */
 	number_parameter_headers =
 		SFDP_GET_BITFIELD(SFDP_HEADER_DW2_NPH, dw2) + 1;
-	DEBUG_CPRINTS_DEVICE(spi_nor_device,
-			     "There are %d SFDP parameter headers",
-			     number_parameter_headers);
+	CPRINTS(spi_nor_device, "There are %d SFDP parameter headers",
+		number_parameter_headers);
 
 	/* Search for the newest, compatible basic flash parameter table. */
 	*out_table_major_rev = 0;
@@ -302,15 +357,13 @@ static int locate_sfdp_basic_parameter_table(
 	}
 
 	if (!table_found) {
-		DEBUG_CPRINTS_DEVICE(
-			spi_nor_device,
+		CPRINTS(spi_nor_device,
 			"No compatible Basic Flash Parameter Table found");
 		return EC_ERROR_UNKNOWN;
 	}
 
-	DEBUG_CPRINTS_DEVICE(spi_nor_device,
-			     "Using Basic Flash Parameter Table v%d.%d",
-			     *out_sfdp_major_rev, *out_sfdp_minor_rev);
+	CPRINTS(spi_nor_device, "Using Basic Flash Parameter Table v%d.%d",
+		*out_sfdp_major_rev, *out_sfdp_minor_rev);
 
 	return EC_SUCCESS;
 }
@@ -422,8 +475,9 @@ static int spi_nor_read_internal(const struct spi_nor_device_t *spi_nor_device,
 			read_command_size = 4;
 		}
 
-		rv = spi_transaction(&spi_devices[spi_nor_device->spi_master],
-				     buf, read_command_size, data, read_size);
+		rv = spi_transaction(
+			&spi_devices[spi_nor_device->spi_controller],
+			buf, read_command_size, data, read_size);
 		if (rv)
 			return rv;
 
@@ -489,8 +543,7 @@ int spi_nor_init(void)
 				mutex_lock(&driver_mutex);
 				spi_nor_device->capacity = capacity;
 				spi_nor_device->page_size = page_size;
-				DEBUG_CPRINTS_DEVICE(
-					spi_nor_device,
+				CPRINTS(spi_nor_device,
 					"Updated to SFDP params: %dKiB w/ %dB pages",
 					spi_nor_device->capacity >> 10,
 					spi_nor_device->page_size);
@@ -514,9 +567,9 @@ int spi_nor_init(void)
  *
  * WARNING:
  * 1) In 3 Byte addressing mode only 16MiB of Serial NOR Flash is accessible.
- * 2) If there's a second SPI master communicating with this Serial NOR Flash
- *    part on the board, the user is responsible for ensuring addressing mode
- *    compatibility and cooperation.
+ * 2) If there's a second SPI controller communicating with this Serial
+ *    NOR Flash part on the board, the user is responsible for ensuring
+ *    addressing mode compatibility and cooperation.
  * 3) The user must ensure that multiple users do not trample on each other
  *    by having multiple parties changing the device's addressing mode.
  *
@@ -542,16 +595,15 @@ int spi_nor_set_4b_mode(struct spi_nor_device_t *spi_nor_device,
 	/* Claim the driver mutex to modify the device state. */
 	mutex_lock(&driver_mutex);
 
-	rv = spi_transaction(&spi_devices[spi_nor_device->spi_master],
+	rv = spi_transaction(&spi_devices[spi_nor_device->spi_controller],
 			     &cmd, 1, NULL, 0);
 	if (rv == EC_SUCCESS) {
 		spi_nor_device->in_4b_addressing_mode =
 			enter_4b_addressing_mode;
 	}
 
-	DEBUG_CPRINTS_DEVICE(spi_nor_device,
-			     "Entered %s Addressing Mode",
-			     enter_4b_addressing_mode ? "4-Byte" : "3-Byte");
+	CPRINTS(spi_nor_device, "Entered %s Addressing Mode",
+		enter_4b_addressing_mode ? "4-Byte" : "3-Byte");
 
 	/* Release the driver mutex. */
 	mutex_unlock(&driver_mutex);
@@ -576,7 +628,7 @@ int spi_nor_read_jedec_id(const struct spi_nor_device_t *spi_nor_device,
 	/* Claim the driver mutex. */
 	mutex_lock(&driver_mutex);
 	/* Read the JEDEC ID. */
-	rv = spi_transaction(&spi_devices[spi_nor_device->spi_master],
+	rv = spi_transaction(&spi_devices[spi_nor_device->spi_controller],
 			     &cmd, 1, data, size);
 	/* Release the driver mutex. */
 	mutex_unlock(&driver_mutex);
@@ -640,6 +692,11 @@ int spi_nor_erase(const struct spi_nor_device_t *spi_nor_device,
 		erase_opcode = SPI_NOR_DRIVER_SPECIFIED_OPCODE_4KIB_ERASE;
 		erase_size = 4096;
 
+		/* Wait for the previous operation to finish. */
+		rv = spi_nor_wait(spi_nor_device);
+		if (rv)
+			goto err_free;
+
 #ifdef CONFIG_SPI_NOR_BLOCK_ERASE
 		if (!(offset % 65536) && size >= 65536) {
 			erase_opcode =
@@ -659,6 +716,13 @@ int spi_nor_erase(const struct spi_nor_device_t *spi_nor_device,
 			assert(read_size >= 4 && (read_size % 4) == 0);
 			rv = spi_nor_read_internal(spi_nor_device, read_offset,
 						   read_size, buffer);
+
+			/* Note: the return value here is lost below
+			 * at the write enable, this is not a problem,
+			 * as this code is only an optimisation, if it
+			 * fails, the full erase functionality still
+			 * gets done, and the error from that returned
+			 */
 			if (rv != EC_SUCCESS)
 				break;
 			/* Aligned word verify reduced the overall (read +
@@ -681,20 +745,15 @@ int spi_nor_erase(const struct spi_nor_device_t *spi_nor_device,
 		}
 		if (!read_left) {
 			/* Sector/block already erased. */
-			DEBUG_CPRINTS_DEVICE(spi_nor_device,
-					     "Skipping erase [%x:%x] "
-					     "(already erased)",
-					     offset, erase_size);
+			CPRINTS(spi_nor_device,
+				"Skipping erase [%x:%x] "
+				"(already erased)",
+				offset, erase_size);
 			offset += erase_size;
 			size -= erase_size;
 			continue;
 		}
 #endif
-		/* Wait for the previous operation to finish. */
-		rv = spi_nor_wait(spi_nor_device);
-		if (rv)
-			goto err_free;
-
 		/* Enable writing to serial NOR flash. */
 		rv = spi_nor_write_enable(spi_nor_device);
 		if (rv)
@@ -716,7 +775,7 @@ int spi_nor_erase(const struct spi_nor_device_t *spi_nor_device,
 		}
 
 		rv = spi_transaction(
-			&spi_devices[spi_nor_device->spi_master],
+			&spi_devices[spi_nor_device->spi_controller],
 			buf, erase_command_size, NULL, 0);
 		if (rv)
 			goto err_free;
@@ -793,8 +852,9 @@ int spi_nor_write(const struct spi_nor_device_t *spi_nor_device,
 		/* Copy data to write into the buffer after the prefix. */
 		memmove(buf + prefix_size, data, write_size);
 
-		rv = spi_transaction(&spi_devices[spi_nor_device->spi_master],
-				     buf, prefix_size + write_size, NULL, 0);
+		rv = spi_transaction(
+			&spi_devices[spi_nor_device->spi_controller],
+			buf, prefix_size + write_size, NULL, 0);
 		if (rv)
 			goto err_free;
 
@@ -846,8 +906,8 @@ static int command_spi_nor_info(int argc, char **argv)
 
 		ccprintf("Serial NOR Flash Device %d:\n", spi_nor_device_index);
 		ccprintf("\tName: %s\n", spi_nor_device->name);
-		ccprintf("\tSPI master index: %d\n",
-			 spi_nor_device->spi_master);
+		ccprintf("\tSPI controller index: %d\n",
+			 spi_nor_device->spi_controller);
 		ccprintf("\tTimeout: %d uSec\n",
 			 spi_nor_device->timeout_usec);
 		ccprintf("\tCapacity: %d KiB\n",
