@@ -1,4 +1,4 @@
-/* Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+/* Copyright 2012 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
@@ -18,7 +18,8 @@ static void cutoff(void)
 }
 DECLARE_DEFERRED(cutoff);
 
-int battery_command_cut_off(struct host_cmd_handler_args *args)
+static enum ec_status
+battery_command_cut_off(struct host_cmd_handler_args *args)
 {
 	/*
 	 * Queue battery cutoff.  This must be deferred so we can send the
