@@ -1,4 +1,4 @@
-/* Copyright (c) 2014 The Chromium OS Authors. All rights reserved.
+/* Copyright 2014 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
@@ -13,7 +13,7 @@
 #include "util.h"
 #include "timer.h"
 
-#define BQ27621_ADDR                        0xaa
+#define BQ27621_ADDR_FLAGS                  0x55
 #define BQ27621_TYPE_ID                     0x0621
 
 #define REG_CTRL                            0x00
@@ -474,11 +474,6 @@ int battery_time_at_rate(int rate, int *minutes)
 	return EC_ERROR_UNIMPLEMENTED;
 }
 
-int battery_manufacturer_name(char *dest, int size)
-{
-	return EC_ERROR_UNIMPLEMENTED;
-}
-
 int battery_device_chemistry(char *dest, int size)
 {
 	uint32_t rv;
@@ -502,6 +497,11 @@ int battery_serial_number(int *serial)
 	return EC_ERROR_UNIMPLEMENTED;
 }
 
+int battery_manufacture_date(int *year, int *month, int *day)
+{
+	return EC_ERROR_UNIMPLEMENTED;
+}
+
 int battery_design_voltage(int *voltage)
 {
 	*voltage = BATTERY_VOLTAGE_NORMAL;
@@ -744,4 +744,3 @@ DECLARE_CONSOLE_COMMAND(fgcmdrd, command_fgcmdrd,
 			"Send a 2-byte cmd to the fg, read back the 2-byte result");
 
 #endif /* CONFIG_CMD_BATDEBUG */
-
