@@ -7,15 +7,17 @@
 
 /* Type-C port manager for Analogix's anx74xx chips */
 
+#include "console.h"
 #include "anx74xx.h"
 #include "task.h"
-#include "tcpci.h"
-#include "tcpm.h"
+#include "tcpm/tcpci.h"
+#include "tcpm/tcpm.h"
 #include "timer.h"
 #include "usb_charge.h"
 #include "usb_mux.h"
 #include "usb_pd.h"
 #include "usb_pd_tcpc.h"
+#include "usb_pd_tcpm.h"
 #include "util.h"
 
 #if !defined(CONFIG_USB_PD_TCPM_TCPCI)
@@ -23,6 +25,14 @@
 #error "Please upgrade your board configuration"
 #endif
 
+#if defined(CONFIG_USB_PD_REV30)
+#error "ANX74xx chips were developed before PD 3.0 and aren't PD 3.0 compliant"
+#error "Please undefine PD 3.0.  See b/159253723 for details"
+#endif
+
+#define CPRINTF(format, args...) cprintf(CC_USBPD, format, ## args)
+#define CPRINTS(format, args...) cprints(CC_USBPD, format, ## args)
+
 struct anx_state {
 	int	polarity;
 	int	vconn_en;
@@ -39,17 +49,34 @@ struct anx_state {
 			reg | 0x01); \
 	} while (0)
 
-static struct anx_state anx[CONFIG_USB_PD_PORT_COUNT];
+static struct anx_state anx[CONFIG_USB_PD_PORT_MAX_COUNT];
 
-/* Save the selected rp value */
-static int selected_rp[CONFIG_USB_PD_PORT_COUNT];
+#ifdef CONFIG_USB_PD_DECODE_SOP
+/* Save the message address */
+static int msg_sop[CONFIG_USB_PD_PORT_MAX_COUNT];
+#endif
 
 static int anx74xx_tcpm_init(int port);
 
 static void anx74xx_tcpm_set_auto_good_crc(int port, int enable)
 {
-	tcpc_write(port, ANX74XX_REG_TX_AUTO_GOODCRC_2,
-		   enable ? ANX74XX_REG_REPLY_SOP_EN : 0);
+	int reply_sop_en = 0;
+
+	if (enable) {
+		reply_sop_en = ANX74XX_REG_REPLY_SOP_EN;
+#ifdef CONFIG_USB_PD_DECODE_SOP
+		/*
+		 * Only the VCONN Source is allowed to communicate
+		 * with the Cable Plugs.
+		 */
+		if (anx[port].vconn_en) {
+			reply_sop_en |= ANX74XX_REG_REPLY_SOP_1_EN |
+					ANX74XX_REG_REPLY_SOP_2_EN;
+		}
+#endif
+	}
+
+	tcpc_write(port, ANX74XX_REG_TX_AUTO_GOODCRC_2, reply_sop_en);
 }
 
 static void anx74xx_update_cable_det(int port, int mode)
@@ -150,6 +177,29 @@ static void anx74xx_set_power_mode(int port, int mode)
 	}
 }
 
+#if defined(CONFIG_USB_PD_DUAL_ROLE_AUTO_TOGGLE) && \
+	defined(CONFIG_USB_PD_TCPC_LOW_POWER)
+
+static int anx74xx_tcpc_drp_toggle(int port)
+{
+	/*
+	 * The ANX3429 always auto-toggles when in low power mode. Since this is
+	 * not configurable, there is nothing to do here. DRP auto-toggle will
+	 * happen once low power mode is set via anx74xx_enter_low_power_mode().
+	 * Note: this means the ANX3429 auto-toggles in PD_DRP_FORCE_SINK mode,
+	 * which is undesirable (b/72007056).
+	 */
+	return EC_SUCCESS;
+}
+
+static int anx74xx_enter_low_power_mode(int port)
+{
+	anx74xx_set_power_mode(port, ANX74XX_STANDBY_MODE);
+	return EC_SUCCESS;
+}
+
+#endif
+
 void anx74xx_tcpc_set_vbus(int port, int enable)
 {
 	int reg;
@@ -180,18 +230,26 @@ static void anx74xx_tcpc_discharge_vbus(int port, int enable)
  * timestamp of the next possible toggle to ensure the 2-ms spacing
  * between IRQ_HPD.
  */
-static uint64_t hpd_deadline[CONFIG_USB_PD_PORT_COUNT];
+static uint64_t hpd_deadline[CONFIG_USB_PD_PORT_MAX_COUNT];
 
-void anx74xx_tcpc_update_hpd_status(int port, int hpd_lvl, int hpd_irq)
+void anx74xx_tcpc_update_hpd_status(const struct usb_mux *me,
+				    mux_state_t mux_state,
+				    bool *ack_required)
 {
 	int reg;
+	int port = me->usb_port;
+	int hpd_lvl = (mux_state & USB_PD_MUX_HPD_LVL) ? 1 : 0;
+	int hpd_irq = (mux_state & USB_PD_MUX_HPD_IRQ) ? 1 : 0;
 
-	tcpc_read(port, ANX74XX_REG_HPD_CTRL_0, &reg);
+	/* This driver does not use host command ACKs */
+	*ack_required = false;
+
+	mux_read(me, ANX74XX_REG_HPD_CTRL_0, &reg);
 	if (hpd_lvl)
 		reg |= ANX74XX_REG_HPD_OUT_DATA;
 	else
 		reg &= ~ANX74XX_REG_HPD_OUT_DATA;
-	tcpc_write(port, ANX74XX_REG_HPD_CTRL_0, reg);
+	mux_write(me, ANX74XX_REG_HPD_CTRL_0, reg);
 
 	if (hpd_irq) {
 		uint64_t now = get_time().val;
@@ -199,12 +257,12 @@ void anx74xx_tcpc_update_hpd_status(int port, int hpd_lvl, int hpd_irq)
 		if (now < hpd_deadline[port])
 			usleep(hpd_deadline[port] - now);
 
-		tcpc_read(port, ANX74XX_REG_HPD_CTRL_0, &reg);
+		mux_read(me, ANX74XX_REG_HPD_CTRL_0, &reg);
 		reg &= ~ANX74XX_REG_HPD_OUT_DATA;
-		tcpc_write(port, ANX74XX_REG_HPD_CTRL_0, reg);
+		mux_write(me, ANX74XX_REG_HPD_CTRL_0, reg);
 		usleep(HPD_DSTREAM_DEBOUNCE_IRQ);
 		reg |= ANX74XX_REG_HPD_OUT_DATA;
-		tcpc_write(port, ANX74XX_REG_HPD_CTRL_0, reg);
+		mux_write(me, ANX74XX_REG_HPD_CTRL_0, reg);
 	}
 	/* enforce 2-ms delay between HPD pulses */
 	hpd_deadline[port] = get_time().val + HPD_USTREAM_DEBOUNCE_LVL;
@@ -220,14 +278,13 @@ void anx74xx_tcpc_clear_hpd_status(int port)
 }
 
 #ifdef CONFIG_USB_PD_TCPM_MUX
-static int anx74xx_tcpm_mux_init(int i2c_addr)
+static int anx74xx_tcpm_mux_init(const struct usb_mux *me)
 {
-	int port = i2c_addr;
-
 	/* Nothing to do here, ANX initializes its muxes
-	 * as (MUX_USB_ENABLED | MUX_DP_ENABLED)
+	 * as (USB_PD_MUX_USB_ENABLED | USB_PD_MUX_DP_ENABLED)
 	 */
-	anx[port].mux_state = MUX_USB_ENABLED | MUX_DP_ENABLED;
+	anx[me->usb_port].mux_state = USB_PD_MUX_USB_ENABLED |
+				      USB_PD_MUX_DP_ENABLED;
 
 	return EC_SUCCESS;
 }
@@ -235,10 +292,11 @@ static int anx74xx_tcpm_mux_init(int i2c_addr)
 static int anx74xx_tcpm_mux_enter_safe_mode(int port)
 {
 	int reg;
+	const struct usb_mux *me = &usb_muxes[port];
 
-	if (tcpc_read(port, ANX74XX_REG_ANALOG_CTRL_2, &reg))
+	if (mux_read(me, ANX74XX_REG_ANALOG_CTRL_2, &reg))
 		return EC_ERROR_UNKNOWN;
-	if (tcpc_write(port, ANX74XX_REG_ANALOG_CTRL_2, reg |
+	if (mux_write(me, ANX74XX_REG_ANALOG_CTRL_2, reg |
 		       ANX74XX_REG_MODE_TRANS))
 		return EC_ERROR_UNKNOWN;
 
@@ -249,10 +307,11 @@ static int anx74xx_tcpm_mux_enter_safe_mode(int port)
 static int anx74xx_tcpm_mux_exit_safe_mode(int port)
 {
 	int reg;
+	const struct usb_mux *me = &usb_muxes[port];
 
-	if (tcpc_read(port, ANX74XX_REG_ANALOG_CTRL_2, &reg))
+	if (mux_read(me, ANX74XX_REG_ANALOG_CTRL_2, &reg))
 		return EC_ERROR_UNKNOWN;
-	if (tcpc_write(port, ANX74XX_REG_ANALOG_CTRL_2, reg &
+	if (mux_write(me, ANX74XX_REG_ANALOG_CTRL_2, reg &
 		       ~ANX74XX_REG_MODE_TRANS))
 		return EC_ERROR_UNKNOWN;
 
@@ -263,12 +322,13 @@ static int anx74xx_tcpm_mux_exit_safe_mode(int port)
 static int anx74xx_tcpm_mux_exit(int port)
 {
 	int reg;
+	const struct usb_mux *me = &usb_muxes[port];
 
 	/*
 	 * Safe mode must be entered before any changes are made to the mux
-	 * settings used to enable ALT_DP mode. This funciton is called either
-	 * from anx74xx_tcpm_mux_set when TYPEC_MUX_NONE is selected as the new
-	 * mux state, or when both cc lines are determined to be
+	 * settings used to enable ALT_DP mode. This function is called either
+	 * from anx74xx_tcpm_mux_set when USB_PD_MUX_NONE is selected as the
+	 * new mux state, or when both cc lines are determined to be
 	 * TYPEC_CC_VOLT_OPEN. Therefore, safe mode must be entered and exited
 	 * here so that both entry paths are handled.
 	 */
@@ -276,18 +336,18 @@ static int anx74xx_tcpm_mux_exit(int port)
 		return EC_ERROR_UNKNOWN;
 
 	/* Disconnect aux from sbu */
-	if (tcpc_read(port, ANX74XX_REG_ANALOG_CTRL_2, &reg))
+	if (mux_read(me, ANX74XX_REG_ANALOG_CTRL_2, &reg))
 		return EC_ERROR_UNKNOWN;
-	if (tcpc_write(port, ANX74XX_REG_ANALOG_CTRL_2, reg & 0xf))
+	if (mux_write(me, ANX74XX_REG_ANALOG_CTRL_2, reg & 0xf))
 		return EC_ERROR_UNKNOWN;
 
 	/* Clear Bit[7:0] R_SWITCH */
-	if (tcpc_write(port, ANX74XX_REG_ANALOG_CTRL_1, 0x0))
+	if (mux_write(me, ANX74XX_REG_ANALOG_CTRL_1, 0x0))
 		return EC_ERROR_UNKNOWN;
 	/* Clear Bit[7:4] R_SWITCH_H */
-	if (tcpc_read(port, ANX74XX_REG_ANALOG_CTRL_5, &reg))
+	if (mux_read(me, ANX74XX_REG_ANALOG_CTRL_5, &reg))
 		return EC_ERROR_UNKNOWN;
-	if (tcpc_write(port, ANX74XX_REG_ANALOG_CTRL_5, reg & 0x0f))
+	if (mux_write(me, ANX74XX_REG_ANALOG_CTRL_5, reg & 0x0f))
 		return EC_ERROR_UNKNOWN;
 
 	/* Exit safe mode */
@@ -303,6 +363,7 @@ static int anx74xx_mux_aux_to_sbu(int port, int polarity, int enabled)
 	int reg;
 	const int aux_mask = ANX74XX_REG_AUX_SWAP_SET_CC2 |
 		ANX74XX_REG_AUX_SWAP_SET_CC1;
+	const struct usb_mux *me = &usb_muxes[port];
 
 	/*
 	 * Get the current value of analog_ctrl_2 register. Note, that safe mode
@@ -310,7 +371,7 @@ static int anx74xx_mux_aux_to_sbu(int port, int polarity, int enabled)
 	 * about setting the correct value for the upper 4 bits of analog_ctrl_2
 	 * here.
 	 */
-	if (tcpc_read(port, ANX74XX_REG_ANALOG_CTRL_2, &reg))
+	if (mux_read(me, ANX74XX_REG_ANALOG_CTRL_2, &reg))
 		return EC_ERROR_UNKNOWN;
 
 	/* Assume aux_p/n lines are not connected */
@@ -324,46 +385,65 @@ static int anx74xx_mux_aux_to_sbu(int port, int polarity, int enabled)
 			reg |= ANX74XX_REG_AUX_SWAP_SET_CC1;
 	}
 	/* Write new aux <-> sbu settings */
-	if (tcpc_write(port, ANX74XX_REG_ANALOG_CTRL_2, reg))
+	if (mux_write(me, ANX74XX_REG_ANALOG_CTRL_2, reg))
 		return EC_ERROR_UNKNOWN;
 
 	return EC_SUCCESS;
 }
 
-static int anx74xx_tcpm_mux_set(int i2c_addr, mux_state_t mux_state)
+static int anx74xx_tcpm_mux_set(const struct usb_mux *me,
+				mux_state_t mux_state,
+				bool *ack_required)
 {
-	int reg;
-	int pin_cfg = 0;
+	int ctrl5;
+	int ctrl1 = 0;
 	int rv;
-	int port = i2c_addr;
+	int port = me->usb_port;
 
-	if (!(mux_state & ~MUX_POLARITY_INVERTED)) {
+	/* This driver does not use host command ACKs */
+	*ack_required = false;
+
+	if (!(mux_state & ~USB_PD_MUX_POLARITY_INVERTED)) {
 		anx[port].mux_state = mux_state;
 		return anx74xx_tcpm_mux_exit(port);
 	}
 
-	rv = tcpc_read(port, ANX74XX_REG_ANALOG_CTRL_5, &reg);
+	rv = mux_read(me, ANX74XX_REG_ANALOG_CTRL_5, &ctrl5);
 	if (rv)
 		return EC_ERROR_UNKNOWN;
-	reg &= 0x0f;
+	ctrl5 &= 0x0f;
 
-	if (mux_state & MUX_USB_ENABLED) {
-		/* Set pin assignment D */
-		if (mux_state & MUX_POLARITY_INVERTED) {
-			pin_cfg = ANX74XX_REG_MUX_DP_MODE_BDF_CC2;
-			reg |= ANX74XX_REG_MUX_SSTX_B;
+	if (mux_state & USB_PD_MUX_USB_ENABLED) {
+		/* Connect USB SS switches */
+		if (mux_state & USB_PD_MUX_POLARITY_INVERTED) {
+			ctrl1 = ANX74XX_REG_MUX_SSRX_RX2;
+			ctrl5 |= ANX74XX_REG_MUX_SSTX_TX2;
 		} else {
-			pin_cfg = ANX74XX_REG_MUX_DP_MODE_BDF_CC1;
-			reg |= ANX74XX_REG_MUX_SSTX_A;
+			ctrl1 = ANX74XX_REG_MUX_SSRX_RX1;
+			ctrl5 |= ANX74XX_REG_MUX_SSTX_TX1;
 		}
-	} else if (mux_state & MUX_DP_ENABLED) {
+		if (mux_state & USB_PD_MUX_DP_ENABLED) {
+			/* Set pin assignment D */
+			if (mux_state & USB_PD_MUX_POLARITY_INVERTED)
+				ctrl1 |= (ANX74XX_REG_MUX_ML0_RX1 |
+					  ANX74XX_REG_MUX_ML1_TX1);
+			else
+				ctrl1 |= (ANX74XX_REG_MUX_ML0_RX2 |
+					  ANX74XX_REG_MUX_ML1_TX2);
+		}
+		/* Keep ML0/ML1 unconnected if DP is not enabled */
+	} else if (mux_state & USB_PD_MUX_DP_ENABLED) {
 		/* Set pin assignment C */
-		if (mux_state & MUX_POLARITY_INVERTED) {
-			pin_cfg = ANX74XX_REG_MUX_DP_MODE_ACE_CC2;
-			reg |= ANX74XX_REG_MUX_ML2_B;
+		if (mux_state & USB_PD_MUX_POLARITY_INVERTED) {
+			ctrl1 = (ANX74XX_REG_MUX_ML0_RX1 |
+				 ANX74XX_REG_MUX_ML1_TX1 |
+				 ANX74XX_REG_MUX_ML3_RX2);
+			ctrl5 |= ANX74XX_REG_MUX_ML2_TX2;
 		} else {
-			pin_cfg = ANX74XX_REG_MUX_DP_MODE_ACE_CC1;
-			reg |= ANX74XX_REG_MUX_ML2_A;
+			ctrl1 = (ANX74XX_REG_MUX_ML0_RX2 |
+				 ANX74XX_REG_MUX_ML1_TX2 |
+				 ANX74XX_REG_MUX_ML3_RX1);
+			ctrl5 |= ANX74XX_REG_MUX_ML2_TX1;
 		}
 	} else if (!mux_state) {
 		return anx74xx_tcpm_mux_exit(port);
@@ -380,15 +460,16 @@ static int anx74xx_tcpm_mux_set(int i2c_addr, mux_state_t mux_state)
 		return EC_ERROR_UNKNOWN;
 
 	/* Write updated pin assignment */
-	rv = tcpc_write(port, ANX74XX_REG_ANALOG_CTRL_1, pin_cfg);
+	rv = mux_write(me, ANX74XX_REG_ANALOG_CTRL_1, ctrl1);
 	/* Write Rswitch config bits */
-	rv |= tcpc_write(port, ANX74XX_REG_ANALOG_CTRL_5, reg);
+	rv |= mux_write(me, ANX74XX_REG_ANALOG_CTRL_5, ctrl5);
 	if (rv)
 		return EC_ERROR_UNKNOWN;
 
 	/* Configure DP aux to sbu settings */
-	if (anx74xx_mux_aux_to_sbu(port, mux_state & MUX_POLARITY_INVERTED,
-				   mux_state & MUX_DP_ENABLED))
+	if (anx74xx_mux_aux_to_sbu(port,
+				   mux_state & USB_PD_MUX_POLARITY_INVERTED,
+				   mux_state & USB_PD_MUX_DP_ENABLED))
 		return EC_ERROR_UNKNOWN;
 
 	/* Exit safe mode */
@@ -401,19 +482,18 @@ static int anx74xx_tcpm_mux_set(int i2c_addr, mux_state_t mux_state)
 }
 
 /* current mux state  */
-static int anx74xx_tcpm_mux_get(int i2c_addr, mux_state_t *mux_state)
+static int anx74xx_tcpm_mux_get(const struct usb_mux *me,
+				mux_state_t *mux_state)
 {
-	int port = i2c_addr;
-
-	*mux_state = anx[port].mux_state;
+	*mux_state = anx[me->usb_port].mux_state;
 
 	return EC_SUCCESS;
 }
 
 const struct usb_mux_driver anx74xx_tcpm_usb_mux_driver = {
-	.init = anx74xx_tcpm_mux_init,
-	.set = anx74xx_tcpm_mux_set,
-	.get = anx74xx_tcpm_mux_get,
+	.init = &anx74xx_tcpm_mux_init,
+	.set = &anx74xx_tcpm_mux_set,
+	.get = &anx74xx_tcpm_mux_get,
 };
 #endif /* CONFIG_USB_PD_TCPM_MUX */
 
@@ -564,15 +644,15 @@ static int anx74xx_check_cc_type(int cc_reg)
 		break;
 
 	case BIT_VALUE_OF_SNK_CC_DEFAULT:
-		cc = TYPEC_CC_VOLT_SNK_DEF;
+		cc = TYPEC_CC_VOLT_RP_DEF;
 		break;
 
 	case BIT_VALUE_OF_SNK_CC_1_P_5:
-		cc = TYPEC_CC_VOLT_SNK_1_5;
+		cc = TYPEC_CC_VOLT_RP_1_5;
 		break;
 
 	case BIT_VALUE_OF_SNK_CC_3_P_0:
-		cc = TYPEC_CC_VOLT_SNK_3_0;
+		cc = TYPEC_CC_VOLT_RP_3_0;
 		break;
 
 	default:
@@ -583,7 +663,8 @@ static int anx74xx_check_cc_type(int cc_reg)
 	return cc;
 }
 
-static int anx74xx_tcpm_get_cc(int port, int *cc1, int *cc2)
+static int anx74xx_tcpm_get_cc(int port, enum tcpc_cc_voltage_status *cc1,
+	enum tcpc_cc_voltage_status *cc2)
 {
 	int rv = EC_SUCCESS;
 	int reg = 0;
@@ -641,8 +722,10 @@ static int anx74xx_rp_control(int port, int rp)
 
 static int anx74xx_tcpm_select_rp_value(int port, int rp)
 {
+	/* Keep track of current RP value */
+	tcpci_set_cached_rp(port, rp);
+
 	/* For ANX3429 cannot get cc correctly when Rp != USB_Default */
-	selected_rp[port] = rp;
 	return EC_SUCCESS;
 }
 
@@ -700,41 +783,14 @@ static int anx74xx_tcpm_set_cc(int port, int pull)
 	return rv;
 }
 
-#if defined(CONFIG_USB_PD_DUAL_ROLE_AUTO_TOGGLE) && \
-	defined(CONFIG_USB_PD_TCPC_LOW_POWER)
-void anx74xx_handle_power_mode(int port, int mode)
-{
-	if (mode == ANX74XX_STANDBY_MODE) {
-		anx74xx_set_power_mode(port, mode);
-	} else if (anx[port].prev_mode != ANX74XX_NORMAL_MODE) {
-		/*
-		 * TODO: Interrupt high follows CC line hence ignore multiple
-		 * interrupts.
-		 */
-		anx74xx_tcpm_init(port);
-	}
-}
-
-static int anx74xx_tcpc_drp_toggle(int port, int enable)
-{
-	/*
-	 * When using low power mode, this function is an entry to point to
-	 * bring the ANX3429 in to or out of standby mode. DRP toggle is
-	 * associated with the chip being in standby mode.
-	 */
-	anx74xx_handle_power_mode(port, enable ? ANX74XX_STANDBY_MODE :
-				  ANX74XX_NORMAL_MODE);
-
-	return EC_SUCCESS;
-}
-#endif /* CONFIG_USB_PD_DUAL_ROLE_AUTO_TOGGLE && CONFIG_USB_PD_TCPC_LOW_POWER */
-
-static int anx74xx_tcpm_set_polarity(int port, int polarity)
+static int anx74xx_tcpm_set_polarity(int port, enum tcpc_cc_polarity polarity)
 {
 	int reg, mux_state, rv = EC_SUCCESS;
+	const struct usb_mux *me = &usb_muxes[port];
+	bool unused;
 
 	rv |= tcpc_read(port, ANX74XX_REG_CC_SOFTWARE_CTRL, &reg);
-	if (polarity) /* Inform ANX to use CC2 */
+	if (polarity_rm_dts(polarity)) /* Inform ANX to use CC2 */
 		reg &= ~ANX74XX_REG_SELECT_CC1;
 	else /* Inform ANX to use CC1 */
 		reg |= ANX74XX_REG_SELECT_CC1;
@@ -744,19 +800,14 @@ static int anx74xx_tcpm_set_polarity(int port, int polarity)
 
 	/* Update mux polarity */
 #ifdef CONFIG_USB_PD_TCPM_MUX
-	mux_state = anx[port].mux_state & ~MUX_POLARITY_INVERTED;
-	if (polarity)
-		mux_state |= MUX_POLARITY_INVERTED;
-	anx74xx_tcpm_mux_set(port, mux_state);
+	mux_state = anx[port].mux_state & ~USB_PD_MUX_POLARITY_INVERTED;
+	if (polarity_rm_dts(polarity))
+		mux_state |= USB_PD_MUX_POLARITY_INVERTED;
+	anx74xx_tcpm_mux_set(me, mux_state, &unused);
 #endif
 	return rv;
 }
 
-int anx74xx_tcpc_get_fw_version(int port, int *version)
-{
-	return tcpc_read(port, ANX74XX_REG_FW_VERSION, version);
-}
-
 static int anx74xx_tcpm_set_vconn(int port, int enable)
 {
 	int reg, rv = EC_SUCCESS;
@@ -776,6 +827,23 @@ static int anx74xx_tcpm_set_vconn(int port, int enable)
 	rv |= tcpc_write(port, ANX74XX_REG_INTP_VCONN_CTRL, reg);
 	anx[port].vconn_en = enable;
 
+#ifdef CONFIG_USB_PD_DECODE_SOP
+	rv |= tcpc_read(port, ANX74XX_REG_TX_AUTO_GOODCRC_2, &reg);
+	if (rv)
+		return EC_ERROR_UNKNOWN;
+
+	if (reg & ANX74XX_REG_REPLY_SOP_EN) {
+		if (enable) {
+			reg |= ANX74XX_REG_REPLY_SOP_1_EN |
+				ANX74XX_REG_REPLY_SOP_2_EN;
+		} else {
+			reg &= ~(ANX74XX_REG_REPLY_SOP_1_EN |
+				 ANX74XX_REG_REPLY_SOP_2_EN);
+		}
+
+		tcpc_write(port, ANX74XX_REG_TX_AUTO_GOODCRC_2, reg);
+	}
+#endif
 	return rv;
 }
 
@@ -785,55 +853,6 @@ static int anx74xx_tcpm_set_msg_header(int port, int power_role, int data_role)
 		  ANX74XX_REG_AUTO_GOODCRC_SET(!!data_role, !!power_role));
 }
 
-static int anx74xx_alert_status(int port, int *alert)
-{
-	int reg, rv = EC_SUCCESS;
-
-	/* Clear soft irq bit */
-	rv |= tcpc_write(port, ANX74XX_REG_IRQ_EXT_SOURCE_3,
-			 ANX74XX_REG_CLEAR_SOFT_IRQ);
-	*alert = 0;
-	rv = tcpc_read(port, ANX74XX_REG_IRQ_SOURCE_RECV_MSG, &reg);
-	if (rv)
-		return EC_ERROR_UNKNOWN;
-
-	/*Clear msg received bit, until read it by TCPM*/
-	rv |= tcpc_write(port, ANX74XX_REG_RECVD_MSG_INT, (reg & 0xFE));
-
-	if (reg & ANX74XX_REG_IRQ_CC_MSG_INT)
-		*alert |= ANX74XX_REG_ALERT_MSG_RECV;
-
-	if (reg & ANX74XX_REG_IRQ_CC_STATUS_INT)
-		*alert |= ANX74XX_REG_ALERT_CC_CHANGE;
-
-	if (reg & ANX74XX_REG_IRQ_GOOD_CRC_INT)
-		*alert |= ANX74XX_REG_ALERT_TX_ACK_RECV;
-
-	if (reg & ANX74XX_REG_IRQ_TX_FAIL_INT)
-		*alert |= ANX74XX_REG_ALERT_TX_MSG_ERROR;
-
-	rv |= tcpc_read(port, ANX74XX_REG_IRQ_EXT_SOURCE_1, &reg);
-	if (rv)
-		return EC_ERROR_UNKNOWN;
-	/* Clears interrupt bits */
-	rv |= tcpc_write(port, ANX74XX_REG_IRQ_EXT_SOURCE_1, reg);
-
-	/* Check for Hard Reset done bit */
-	if (reg & ANX74XX_REG_ALERT_TX_HARD_RESETOK)
-		*alert |= ANX74XX_REG_ALERT_TX_HARD_RESETOK;
-
-	/* Read TCPC Alert register2 */
-	rv |= tcpc_read(port, ANX74XX_REG_IRQ_EXT_SOURCE_2, &reg);
-
-	/* Clears interrupt bits */
-	rv |= tcpc_write(port, ANX74XX_REG_IRQ_EXT_SOURCE_2, reg);
-
-	if (reg & ANX74XX_REG_EXT_HARD_RST)
-		*alert |= ANX74XX_REG_ALERT_HARD_RST_RECV;
-
-	return rv;
-}
-
 static int anx74xx_tcpm_set_rx_enable(int port, int enable)
 {
 	int reg, rv;
@@ -844,7 +863,7 @@ static int anx74xx_tcpm_set_rx_enable(int port, int enable)
 	if (enable) {
 		reg &= ~(ANX74XX_REG_IRQ_CC_MSG_INT);
 		anx74xx_tcpm_set_auto_good_crc(port, 1);
-		anx74xx_rp_control(port, selected_rp[port]);
+		anx74xx_rp_control(port, tcpci_get_cached_rp(port));
 	} else {
 		/* Disable RX message by masking interrupt */
 		reg |= (ANX74XX_REG_IRQ_CC_MSG_INT);
@@ -858,27 +877,32 @@ static int anx74xx_tcpm_set_rx_enable(int port, int enable)
 }
 
 #ifdef CONFIG_USB_PD_VBUS_DETECT_TCPC
-static int anx74xx_tcpm_get_vbus_level(int port)
+static bool anx74xx_tcpm_check_vbus_level(int port, enum vbus_level level)
 {
 	int reg = 0;
 
 	tcpc_read(port, ANX74XX_REG_ANALOG_STATUS, &reg);
-	return ((reg & ANX74XX_REG_VBUS_STATUS) ? 1 : 0);
+	if (level == VBUS_PRESENT)
+		return ((reg & ANX74XX_REG_VBUS_STATUS) ? 1 : 0);
+	else
+		return ((reg & ANX74XX_REG_VBUS_STATUS) ? 0 : 1);
 }
 #endif
 
-static int anx74xx_tcpm_get_message(int port, uint32_t *payload, int *head)
+static int anx74xx_tcpm_get_message_raw(int port, uint32_t *payload, int *head)
 {
-	int reg = 0, rv = EC_SUCCESS;
-	int len = 0;
+	int reg;
+	int len;
 
 	/* Fetch the header */
-	rv |= tcpc_read16(port, ANX74XX_REG_PD_HEADER, &reg);
-	if (rv) {
+	if (tcpc_read16(port, ANX74XX_REG_PD_HEADER, &reg)) {
 		clear_recvd_msg_int(port);
 		return EC_ERROR_UNKNOWN;
 	}
 	*head = reg;
+#ifdef CONFIG_USB_PD_DECODE_SOP
+	*head |= PD_HEADER_SOP(msg_sop[port]);
+#endif
 
 	len = PD_HEADER_CNT(*head) * 4;
 	if (!len) {
@@ -889,14 +913,10 @@ static int anx74xx_tcpm_get_message(int port, uint32_t *payload, int *head)
 	/* Receive message : assuming payload have enough
 	 * memory allocated
 	 */
-	rv |= anx74xx_read_pd_obj(port, (uint8_t *)payload, len);
-	if (rv)
-		return EC_ERROR_UNKNOWN;
-
-	return rv;
+	return anx74xx_read_pd_obj(port, (uint8_t *)payload, len);
 }
 
-static int anx74xx_tcpm_transmit(int port, enum tcpm_transmit_type type,
+static int anx74xx_tcpm_transmit(int port, enum tcpci_msg_type type,
 		  uint16_t header,
 		  const uint32_t *data)
 {
@@ -905,14 +925,14 @@ static int anx74xx_tcpm_transmit(int port, enum tcpm_transmit_type type,
 
 	switch (type) {
 	/* ANX is aware of type */
-	case TCPC_TX_SOP:
-	case TCPC_TX_SOP_PRIME:
-	case TCPC_TX_SOP_PRIME_PRIME:
+	case TCPCI_MSG_SOP:
+	case TCPCI_MSG_SOP_PRIME:
+	case TCPCI_MSG_SOP_PRIME_PRIME:
 		len = PD_HEADER_CNT(header) * 4 + 2;
 		ret = anx74xx_send_message(port, header,
 						  data, type, len);
 		break;
-	case TCPC_TX_HARD_RESET:
+	case TCPCI_MSG_TX_HARD_RESET:
 	/* Request HARD RESET */
 		tcpc_read(port, ANX74XX_REG_TX_CTRL_1, &reg);
 		reg |= ANX74XX_REG_TX_HARD_RESET_REQ;
@@ -920,13 +940,13 @@ static int anx74xx_tcpm_transmit(int port, enum tcpm_transmit_type type,
 	/*After Hard Reset, TCPM shall disable goodCRC*/
 		anx74xx_tcpm_set_auto_good_crc(port, 0);
 		break;
-	case TCPC_TX_CABLE_RESET:
+	case TCPCI_MSG_CABLE_RESET:
 	/* Request CABLE RESET */
 		tcpc_read(port, ANX74XX_REG_TX_CTRL_1, &reg);
 		reg |= ANX74XX_REG_TX_CABLE_RESET_REQ;
 		ret = tcpc_write(port, ANX74XX_REG_TX_CTRL_1, reg);
 		break;
-	case TCPC_TX_BIST_MODE_2:
+	case TCPCI_MSG_TX_BIST_MODE_2:
 	/* Request BIST MODE 2 */
 		reg = ANX74XX_REG_TX_BIST_START
 				| ANX74XX_REG_TX_BIXT_FOREVER | (0x02 << 4);
@@ -949,53 +969,94 @@ static int anx74xx_tcpm_transmit(int port, enum tcpm_transmit_type type,
 	return ret;
 }
 
+/*
+ * Don't let the TCPC try to pull from the RX buffer forever. We typical only
+ * have 1 or 2 messages waiting.
+ */
+#define MAX_ALLOW_FAILED_RX_READS 10
+
 void anx74xx_tcpc_alert(int port)
 {
-	int status;
+	int reg;
+	int failed_attempts;
 
-	/* Check the alert status from anx74xx */
-	if (anx74xx_alert_status(port, &status))
-		status = 0;
-	if (status) {
+	/* Clear soft irq bit */
+	tcpc_write(port, ANX74XX_REG_IRQ_EXT_SOURCE_3,
+		   ANX74XX_REG_CLEAR_SOFT_IRQ);
 
-		if (status & ANX74XX_REG_ALERT_CC_CHANGE) {
-			/* CC status changed, wake task */
-			task_set_event(PD_PORT_TO_TASK_ID(port),
-					PD_EVENT_CC, 0);
-		}
+	/* Read main alert register for pending alerts */
+	reg = 0;
+	tcpc_read(port, ANX74XX_REG_IRQ_SOURCE_RECV_MSG, &reg);
 
-		/* If alert is to receive a message */
-		if (status & ANX74XX_REG_ALERT_MSG_RECV) {
-			/* Set a PD_EVENT_RX */
-			task_set_event(PD_PORT_TO_TASK_ID(port),
-						PD_EVENT_RX, 0);
-		}
-		if (status & ANX74XX_REG_ALERT_TX_ACK_RECV) {
-			/* Inform PD about this TX success */
-			pd_transmit_complete(port,
-						TCPC_TX_COMPLETE_SUCCESS);
-		}
-		if (status & ANX74XX_REG_ALERT_TX_MSG_ERROR) {
-			/* let PD does not wait for this */
-			pd_transmit_complete(port,
-					      TCPC_TX_COMPLETE_FAILED);
-		}
-		if (status & ANX74XX_REG_ALERT_TX_CABLE_RESETOK) {
-			/* ANX hardware clears the request bit */
-			pd_transmit_complete(port,
-					      TCPC_TX_COMPLETE_SUCCESS);
-		}
-		if (status & ANX74XX_REG_ALERT_TX_HARD_RESETOK) {
-			/* ANX hardware clears the request bit */
-			pd_transmit_complete(port,
-					     TCPC_TX_COMPLETE_SUCCESS);
-		}
-		if (status & ANX74XX_REG_ALERT_HARD_RST_RECV) {
-			/* hard reset received */
-			pd_execute_hard_reset(port);
-			task_wake(PD_PORT_TO_TASK_ID(port));
+	/* Prioritize TX completion because PD state machine is waiting */
+	if (reg & ANX74XX_REG_IRQ_GOOD_CRC_INT)
+		pd_transmit_complete(port, TCPC_TX_COMPLETE_SUCCESS);
+
+	if (reg & ANX74XX_REG_IRQ_TX_FAIL_INT)
+		pd_transmit_complete(port, TCPC_TX_COMPLETE_FAILED);
+
+	/* Pull all RX messages from TCPC into EC memory */
+	failed_attempts = 0;
+	while (reg & ANX74XX_REG_IRQ_CC_MSG_INT) {
+		if (tcpm_enqueue_message(port))
+			++failed_attempts;
+		if (tcpc_read(port, ANX74XX_REG_IRQ_SOURCE_RECV_MSG, &reg))
+			++failed_attempts;
+
+		/* Ensure we don't loop endlessly */
+		if (failed_attempts >= MAX_ALLOW_FAILED_RX_READS) {
+			CPRINTF("C%d Cannot consume RX buffer after %d failed "
+				"attempts!", port, failed_attempts);
+			/*
+			 * The port is in a bad state, we don't want to consume
+			 * all EC resources so suspend the port for a little
+			 * while.
+			 */
+			pd_set_suspend(port, 1);
+			pd_deferred_resume(port);
+			return;
 		}
 	}
+
+	/* Clear all pending alerts */
+	tcpc_write(port, ANX74XX_REG_RECVD_MSG_INT, reg);
+
+	if (reg & ANX74XX_REG_IRQ_CC_STATUS_INT)
+		/* CC status changed, wake task */
+		task_set_event(PD_PORT_TO_TASK_ID(port), PD_EVENT_CC);
+
+	/* Read and clear extended alert register 1 */
+	reg = 0;
+	tcpc_read(port, ANX74XX_REG_IRQ_EXT_SOURCE_1, &reg);
+	tcpc_write(port, ANX74XX_REG_IRQ_EXT_SOURCE_1, reg);
+
+#ifdef CONFIG_USB_PD_DECODE_SOP
+	if (reg & ANX74XX_REG_EXT_SOP)
+		msg_sop[port] = TCPCI_MSG_SOP;
+	else if (reg & ANX74XX_REG_EXT_SOP_PRIME)
+		msg_sop[port] = TCPCI_MSG_SOP_PRIME;
+#endif
+
+	/* Check for Hard Reset done bit */
+	if (reg & ANX74XX_REG_ALERT_TX_HARD_RESETOK)
+		/* ANX hardware clears the request bit */
+		pd_transmit_complete(port, TCPC_TX_COMPLETE_SUCCESS);
+
+	/* Read and clear TCPC extended alert register 2 */
+	reg = 0;
+	tcpc_read(port, ANX74XX_REG_IRQ_EXT_SOURCE_2, &reg);
+	tcpc_write(port, ANX74XX_REG_IRQ_EXT_SOURCE_2, reg);
+
+#ifdef CONFIG_USB_PD_DECODE_SOP
+	if (reg & ANX74XX_REG_EXT_SOP_PRIME_PRIME)
+		msg_sop[port] = TCPCI_MSG_SOP_PRIME_PRIME;
+#endif
+
+	if (reg & ANX74XX_REG_EXT_HARD_RST) {
+		/* hard reset received */
+		task_set_event(PD_PORT_TO_TASK_ID(port),
+			       PD_EVENT_RX_HARD_RESET);
+	}
 }
 
 static int anx74xx_tcpm_init(int port)
@@ -1013,11 +1074,18 @@ static int anx74xx_tcpm_init(int port)
 	rv |= tcpc_write(port, ANX74XX_REG_IRQ_EXT_MASK_1,
 			 ANX74XX_REG_CLEAR_SET_BITS);
 
+	/* Initialize interrupt open-drain */
+	rv |= tcpc_read(port, ANX74XX_REG_INTP_VCONN_CTRL, &reg);
+	if (tcpc_config[port].flags & TCPC_FLAGS_ALERT_OD)
+		reg |= ANX74XX_REG_R_INTERRUPT_OPEN_DRAIN;
+	else
+		reg &= ~ANX74XX_REG_R_INTERRUPT_OPEN_DRAIN;
+	rv |= tcpc_write(port, ANX74XX_REG_INTP_VCONN_CTRL, reg);
+
 	/* Initialize interrupt polarity */
-	rv |= tcpc_write(port, ANX74XX_REG_IRQ_STATUS,
-			tcpc_config[port].pol == TCPC_ALERT_ACTIVE_LOW ?
-			ANX74XX_REG_IRQ_POL_LOW :
-			ANX74XX_REG_IRQ_POL_HIGH);
+	reg = tcpc_config[port].flags & TCPC_FLAGS_ALERT_ACTIVE_HIGH ?
+		ANX74XX_REG_IRQ_POL_HIGH : ANX74XX_REG_IRQ_POL_LOW;
+	rv |= tcpc_write(port, ANX74XX_REG_IRQ_STATUS, reg);
 
 	/* unmask interrupts */
 	rv |= tcpc_read(port, ANX74XX_REG_IRQ_EXT_MASK_1, &reg);
@@ -1057,32 +1125,80 @@ static int anx74xx_tcpm_init(int port)
 	if (rv)
 		return EC_ERROR_UNKNOWN;
 
+	tcpm_get_chip_info(port, 1, NULL);
+
+	return EC_SUCCESS;
+}
+
+static int anx74xx_get_chip_info(int port, int live,
+			struct ec_response_pd_chip_info_v1 *chip_info)
+{
+	int rv = tcpci_get_chip_info(port, live, chip_info);
+	int val;
+
+	if (rv)
+		return rv;
+
+	if (chip_info->fw_version_number == 0 ||
+	    chip_info->fw_version_number == -1 || live) {
+		rv = tcpc_read(port, ANX74XX_REG_FW_VERSION, &val);
+
+		if (rv)
+			return rv;
+
+		chip_info->fw_version_number = val;
+	}
+
+#ifdef CONFIG_USB_PD_TCPM_ANX3429
+	/*
+	 * Min firmware version of ANX3429 to ensure that false SOP' detection
+	 * doesn't occur for e-marked cables. See b/116255749#comment8 and
+	 * b/64752060#comment11
+	 */
+	chip_info->min_req_fw_version_number = 0x16;
+#endif
+
+	return rv;
+}
+
+/*
+ * Dissociate from the TCPC.
+ */
+
+static int anx74xx_tcpm_release(int port)
+{
 	return EC_SUCCESS;
 }
 
 const struct tcpm_drv anx74xx_tcpm_drv = {
 	.init			= &anx74xx_tcpm_init,
+	.release		= &anx74xx_tcpm_release,
 	.get_cc			= &anx74xx_tcpm_get_cc,
 #ifdef CONFIG_USB_PD_VBUS_DETECT_TCPC
-	.get_vbus_level		= &anx74xx_tcpm_get_vbus_level,
+	.check_vbus_level	= &anx74xx_tcpm_check_vbus_level,
 #endif
 	.select_rp_value	= &anx74xx_tcpm_select_rp_value,
 	.set_cc			= &anx74xx_tcpm_set_cc,
 	.set_polarity		= &anx74xx_tcpm_set_polarity,
+#ifdef CONFIG_USB_PD_DECODE_SOP
+	.sop_prime_enable	= &tcpci_tcpm_sop_prime_enable,
+#endif
 	.set_vconn		= &anx74xx_tcpm_set_vconn,
 	.set_msg_header		= &anx74xx_tcpm_set_msg_header,
 	.set_rx_enable		= &anx74xx_tcpm_set_rx_enable,
-	.get_message		= &anx74xx_tcpm_get_message,
+	.get_message_raw	= &anx74xx_tcpm_get_message_raw,
 	.transmit		= &anx74xx_tcpm_transmit,
 	.tcpc_alert		= &anx74xx_tcpc_alert,
 #ifdef CONFIG_USB_PD_DISCHARGE_TCPC
 	.tcpc_discharge_vbus	= &anx74xx_tcpc_discharge_vbus,
 #endif
-	.get_chip_info		= &tcpci_get_chip_info,
+	.get_chip_info		= &anx74xx_get_chip_info,
 #if defined(CONFIG_USB_PD_DUAL_ROLE_AUTO_TOGGLE) && \
 		defined(CONFIG_USB_PD_TCPC_LOW_POWER)
 	.drp_toggle		= &anx74xx_tcpc_drp_toggle,
+	.enter_low_power_mode	= &anx74xx_enter_low_power_mode,
 #endif
+	.set_bist_test_mode	= &tcpci_set_bist_test_mode,
 };
 
 #ifdef CONFIG_CMD_I2C_STRESS_TEST_TCPC
