@@ -7,7 +7,9 @@
 
 #define __CROS_EC_VEC_4_H
 
-typedef float vec4_t[4];
+#include "math_util.h"
+
+typedef float floatv4_t[4];
+typedef fp_t fpv4_t[4];
 
 #endif  /* __CROS_EC_VEC_4_H */
-
