@@ -12,15 +12,20 @@
 #include "util.h"
 
 #ifdef TEST_RSA3
+#if CONFIG_RSA_KEY_SIZE == 3072
+#include "rsa3072-3.h"
+#else
 #include "rsa2048-3.h"
+#endif
 #else
 #include "rsa2048-F4.h"
 #endif
 
-void run_test(void)
+static uint32_t rsa_workbuf[3 * RSANUMBYTES/4];
+
+void run_test(int argc, char **argv)
 {
 	int good;
-	uint32_t rsa_workbuf[3 * RSANUMBYTES/4];
 
 	good = rsa_verify(rsa_key, sig, hash, rsa_workbuf);
 	if (!good) {
@@ -50,4 +55,3 @@ void run_test(void)
 
 	test_pass();
 }
-
