@@ -1,8 +1,8 @@
-/* Copyright (c) 2014 The Chromium OS Authors. All rights reserved.
+/* Copyright 2014 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
- * Test lid switch.
+ * Test charge_state_v2 behavior
  */
 
 #include "battery_smart.h"
@@ -12,6 +12,8 @@
 #include "gpio.h"
 #include "hooks.h"
 #include "host_command.h"
+#include "math_util.h"
+#include "power.h"
 #include "task.h"
 #include "test_util.h"
 #include "util.h"
@@ -19,24 +21,34 @@
 #define WAIT_CHARGER_TASK 600
 #define BATTERY_DETACH_DELAY 35000
 
+enum ec_charge_control_mode get_chg_ctrl_mode(void);
+
 static int mock_chipset_state = CHIPSET_STATE_ON;
 static int is_shutdown;
 static int is_force_discharge;
 static int is_hibernated;
 static int override_voltage, override_current, override_usec;
+static int display_soc;
+static int is_full;
 
 /* The simulation doesn't really hibernate, so we must reset this ourselves */
-extern timestamp_t shutdown_warning_time;
+extern timestamp_t shutdown_target_time;
 
 static void reset_mocks(void)
 {
 	mock_chipset_state = CHIPSET_STATE_ON;
 	is_shutdown = is_force_discharge = is_hibernated = 0;
 	override_voltage = override_current = override_usec = 0;
-	shutdown_warning_time.val = 0ULL;
+	shutdown_target_time.val = 0ULL;
+	is_full = 0;
 }
 
-void chipset_force_shutdown(void)
+int board_cut_off_battery(void)
+{
+	return EC_SUCCESS;
+}
+
+void chipset_force_shutdown(enum chipset_shutdown_reason reason)
 {
 	is_shutdown = 1;
 	mock_chipset_state = CHIPSET_STATE_HARD_OFF;
@@ -47,6 +59,21 @@ int chipset_in_state(int state_mask)
 	return state_mask & mock_chipset_state;
 }
 
+int chipset_in_or_transitioning_to_state(int state_mask)
+{
+	return state_mask & mock_chipset_state;
+}
+
+enum power_state power_get_state(void)
+{
+	if (is_shutdown)
+		return POWER_S5;
+	else if (is_hibernated)
+		return POWER_G3;
+	else
+		return POWER_S0;
+}
+
 int board_discharge_on_ac(int enabled)
 {
 	is_force_discharge = enabled;
@@ -107,10 +134,24 @@ static int wait_charging_state(void)
 
 static int charge_control(enum ec_charge_control_mode mode)
 {
-	struct ec_params_charge_control params;
-	params.mode = mode;
-	return test_send_host_command(EC_CMD_CHARGE_CONTROL, 1, &params,
-				      sizeof(params), NULL, 0);
+	struct ec_params_charge_control p;
+
+	p.cmd = EC_CHARGE_CONTROL_CMD_SET;
+	p.mode = mode;
+	p.sustain_soc.lower = -1;
+	p.sustain_soc.upper = -1;
+	return test_send_host_command(EC_CMD_CHARGE_CONTROL, 2, &p, sizeof(p),
+				      NULL, 0);
+}
+
+__override int charge_get_display_charge(void)
+{
+	return display_soc;
+}
+
+__override int calc_is_full(void)
+{
+	return is_full;
 }
 
 /* Setup init condition */
@@ -175,7 +216,7 @@ static int test_charge_state(void)
 
 	/* Detach battery, charging error */
 	ccprintf("[CHARGING TEST] Detach battery\n");
-	TEST_ASSERT(test_detach_i2c(I2C_PORT_BATTERY, BATTERY_ADDR) ==
+	TEST_ASSERT(test_detach_i2c(I2C_PORT_BATTERY, BATTERY_ADDR_FLAGS) ==
 		    EC_SUCCESS);
 	msleep(BATTERY_DETACH_DELAY);
 	state = wait_charging_state();
@@ -183,7 +224,7 @@ static int test_charge_state(void)
 
 	/* Attach battery again, charging */
 	ccprintf("[CHARGING TEST] Attach battery\n");
-	test_attach_i2c(I2C_PORT_BATTERY, BATTERY_ADDR);
+	test_attach_i2c(I2C_PORT_BATTERY, BATTERY_ADDR_FLAGS);
 	/* And changing full capacity should trigger a host event */
 	ev_clear(EC_HOST_EVENT_BATTERY);
 	sb_write(SB_FULL_CHARGE_CAPACITY, 0xeff0);
@@ -324,6 +365,35 @@ static int test_low_battery(void)
 	return EC_SUCCESS;
 }
 
+static int test_deep_charge_battery(void)
+{
+	enum charge_state_v2 state_v2;
+	const struct battery_info *bat_info = battery_get_info();
+
+	test_setup(1);
+
+	/* battery pack voltage bellow voltage_min */
+	sb_write(SB_VOLTAGE, (bat_info->voltage_min - 200));
+	wait_charging_state();
+	state_v2 = charge_get_state_v2();
+	TEST_ASSERT(state_v2 == ST_PRECHARGE);
+
+	/*
+	 * Battery voltage keep bellow voltage_min,
+	 * precharge over time CONFIG_BATTERY_LOW_VOLTAGE_TIMEOUT
+	 */
+	usleep(CONFIG_BATTERY_LOW_VOLTAGE_TIMEOUT);
+	state_v2 = charge_get_state_v2();
+	TEST_ASSERT(state_v2 == ST_IDLE);
+
+        /* recovery from a low voltage. */
+	sb_write(SB_VOLTAGE, (bat_info->voltage_normal));
+	wait_charging_state();
+	state_v2 = charge_get_state_v2();
+	TEST_ASSERT(state_v2 == ST_CHARGE);
+
+	return EC_SUCCESS;
+}
 static int test_high_temp_battery(void)
 {
 	test_setup(1);
@@ -345,6 +415,36 @@ static int test_high_temp_battery(void)
 	return EC_SUCCESS;
 }
 
+static int test_cold_battery_with_ac(void)
+{
+	test_setup(1);
+
+	ccprintf("[CHARGING TEST] Cold battery no shutdown with AC\n");
+	ev_clear(EC_HOST_EVENT_BATTERY_SHUTDOWN);
+	sb_write(SB_TEMPERATURE, CELSIUS_TO_DECI_KELVIN(-90));
+	wait_charging_state();
+	sleep(CONFIG_BATTERY_CRITICAL_SHUTDOWN_TIMEOUT);
+	TEST_ASSERT(!is_shutdown);
+
+	return EC_SUCCESS;
+}
+
+static int test_cold_battery_no_ac(void)
+{
+	test_setup(0);
+
+	ccprintf("[CHARGING TEST] Cold battery shutdown when discharging\n");
+	ev_clear(EC_HOST_EVENT_BATTERY_SHUTDOWN);
+	sb_write(SB_TEMPERATURE, CELSIUS_TO_DECI_KELVIN(-90));
+	wait_charging_state();
+	TEST_ASSERT(ev_is_set(EC_HOST_EVENT_BATTERY_SHUTDOWN));
+	TEST_ASSERT(!is_shutdown);
+	sleep(CONFIG_BATTERY_CRITICAL_SHUTDOWN_TIMEOUT);
+	TEST_ASSERT(is_shutdown);
+
+	return EC_SUCCESS;
+}
+
 static int test_external_funcs(void)
 {
 	int rv, temp;
@@ -676,17 +776,195 @@ static int test_low_battery_hostevents(void)
 	return EC_SUCCESS;
 }
 
+static int test_battery_sustainer(void)
+{
+	struct ec_params_charge_control p;
+	struct ec_response_charge_control r;
+	int rv;
 
+	test_setup(1);
 
-void run_test(void)
+	/* Enable sustainer */
+	p.cmd = EC_CHARGE_CONTROL_CMD_SET;
+	p.mode = CHARGE_CONTROL_NORMAL;
+	p.sustain_soc.lower = 79;
+	p.sustain_soc.upper = 80;
+	rv = test_send_host_command(EC_CMD_CHARGE_CONTROL, 2,
+				    &p, sizeof(p), NULL, 0);
+	TEST_ASSERT(rv == EC_RES_SUCCESS);
+
+	p.cmd = EC_CHARGE_CONTROL_CMD_GET;
+	rv = test_send_host_command(EC_CMD_CHARGE_CONTROL, 2,
+				    &p, sizeof(p), &r, sizeof(r));
+	TEST_ASSERT(rv == EC_RES_SUCCESS);
+	TEST_ASSERT(r.sustain_soc.lower == 79);
+	TEST_ASSERT(r.sustain_soc.upper == 80);
+
+	/* Check mode transition as the SoC changes. */
+
+	ccprintf("Test SoC < lower < upper.\n");
+	display_soc = 780;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_NORMAL);
+	ccprintf("Pass.\n");
+
+	ccprintf("Test lower < upper < SoC.\n");
+	display_soc = 810;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_DISCHARGE);
+	ccprintf("Pass.\n");
+
+	ccprintf("Test unplug AC.\n");
+	gpio_set_level(GPIO_AC_PRESENT, 0);
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_NORMAL);
+	ccprintf("Pass.\n");
+
+	ccprintf("Test replug AC.\n");
+	gpio_set_level(GPIO_AC_PRESENT, 1);
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_DISCHARGE);
+	ccprintf("Pass.\n");
+
+	ccprintf("Test lower < SoC < upper.\n");
+	display_soc = 799;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_DISCHARGE);
+	ccprintf("Pass.\n");
+
+	ccprintf("Test SoC < lower < upper.\n");
+	display_soc = 789;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_NORMAL);
+	ccprintf("Pass.\n");
+
+	ccprintf("Test disable sustainer.\n");
+	charge_control(CHARGE_CONTROL_NORMAL);
+	display_soc = 810;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_NORMAL);
+	ccprintf("Pass.\n");
+
+	ccprintf("Test enable sustainer when battery is full.\n");
+	display_soc = 1000;
+	is_full = 1;
+	wait_charging_state();
+	/* Enable sustainer. */
+	p.cmd = EC_CHARGE_CONTROL_CMD_SET;
+	p.mode = CHARGE_CONTROL_NORMAL;
+	p.sustain_soc.lower = 79;
+	p.sustain_soc.upper = 80;
+	rv = test_send_host_command(EC_CMD_CHARGE_CONTROL, 2,
+				    &p, sizeof(p), NULL, 0);
+	TEST_ASSERT(rv == EC_RES_SUCCESS);
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_DISCHARGE);
+	ccprintf("Pass.\n");
+
+	/* Disable sustainer, unplug AC, upper < SoC < 100. */
+	charge_control(CHARGE_CONTROL_NORMAL);
+	display_soc = 810;
+	is_full = 0;
+	gpio_set_level(GPIO_AC_PRESENT, 0);
+	wait_charging_state();
+
+	ccprintf("Test enable sustainer when AC is present.\n");
+	gpio_set_level(GPIO_AC_PRESENT, 1);
+	wait_charging_state();
+	/* Enable sustainer. */
+	p.cmd = EC_CHARGE_CONTROL_CMD_SET;
+	p.mode = CHARGE_CONTROL_NORMAL;
+	p.sustain_soc.lower = 79;
+	p.sustain_soc.upper = 80;
+	rv = test_send_host_command(EC_CMD_CHARGE_CONTROL, 2,
+				    &p, sizeof(p), NULL, 0);
+	TEST_ASSERT(rv == EC_RES_SUCCESS);
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_DISCHARGE);
+	ccprintf("Pass.\n");
+
+	return EC_SUCCESS;
+}
+
+static int test_battery_sustainer_discharge_idle(void)
+{
+	struct ec_params_charge_control p;
+	int rv;
+
+	test_setup(1);
+
+	/* Enable sustainer */
+	p.cmd = EC_CHARGE_CONTROL_CMD_SET;
+	p.mode = CHARGE_CONTROL_NORMAL;
+	p.sustain_soc.lower = 80;
+	p.sustain_soc.upper = 80;
+	rv = test_send_host_command(EC_CMD_CHARGE_CONTROL, 2,
+				    &p, sizeof(p), NULL, 0);
+	TEST_ASSERT(rv == EC_RES_SUCCESS);
+
+	/* Check mode transition as the SoC changes. */
+
+	/* SoC < lower (= upper) */
+	display_soc = 780;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_NORMAL);
+
+	/* (lower =) upper < SoC */
+	display_soc = 810;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_IDLE);
+
+	/* Unplug AC. Sustainer gets deactivated. */
+	gpio_set_level(GPIO_AC_PRESENT, 0);
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_NORMAL);
+
+	/* Replug AC. Sustainer gets re-activated. */
+	gpio_set_level(GPIO_AC_PRESENT, 1);
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_IDLE);
+
+	/* lower = SoC = upper */
+	display_soc = 800;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_IDLE);
+
+	/* SoC < lower (= upper) */
+	display_soc = 789;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_NORMAL);
+
+	/* Disable sustainer */
+	p.cmd = EC_CHARGE_CONTROL_CMD_SET;
+	p.mode = CHARGE_CONTROL_NORMAL;
+	p.sustain_soc.lower = -1;
+	p.sustain_soc.upper = -1;
+	rv = test_send_host_command(EC_CMD_CHARGE_CONTROL, 2,
+				    &p, sizeof(p), NULL, 0);
+	TEST_ASSERT(rv == EC_RES_SUCCESS);
+
+	/* This time, mode will stay in NORMAL even when upper < SoC. */
+	display_soc = 810;
+	wait_charging_state();
+	TEST_ASSERT(get_chg_ctrl_mode() == CHARGE_CONTROL_NORMAL);
+
+	return EC_SUCCESS;
+}
+
+void run_test(int argc, char **argv)
 {
 	RUN_TEST(test_charge_state);
 	RUN_TEST(test_low_battery);
 	RUN_TEST(test_high_temp_battery);
+	RUN_TEST(test_cold_battery_with_ac);
+	RUN_TEST(test_cold_battery_no_ac);
 	RUN_TEST(test_external_funcs);
 	RUN_TEST(test_hc_charge_state);
 	RUN_TEST(test_hc_current_limit);
 	RUN_TEST(test_low_battery_hostevents);
+	RUN_TEST(test_battery_sustainer);
+	RUN_TEST(test_battery_sustainer_discharge_idle);
+	RUN_TEST(test_deep_charge_battery);
 
 	test_print_result();
 }
