@@ -1,8 +1,24 @@
-/* Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+/* Copyright 2012 The Chromium OS Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
 
+/** @brief A handler for an `ectool` command.  */
+struct command {
+	/** The name of the command. */
+	const char *name;
+
+	/**
+	 * The function to handle the command.
+	 *
+	 * @param argc The length of `argv`
+	 * @param argv The arguments passed, including the command itself but
+	 *             not 'ectool'.
+	 * @return 0 if successful, or a negative `enum ec_status` value.
+	 */
+	int (*handler)(int argc, char *argv[]);
+};
+
 /**
  * Test low-level key scanning
  *
@@ -22,9 +38,14 @@
  * <keys_pressed> is a (possibly empty) list of ASCII keys
  *
  * The key matrix is read from the fdt.
- *
- * @param argc	Number of arguments (excluding 'ectool')
- * @param argv	List of arguments
- * @return 0 if ok, -1 on error
  */
 int cmd_keyscan(int argc, char *argv[]);
+
+/* ASCII mode for printing, default off */
+extern int ascii_mode;
+
+int cmd_i2c_protect(int argc, char *argv[]);
+int cmd_i2c_read(int argc, char *argv[]);
+int cmd_i2c_speed(int argc, char *argv[]);
+int cmd_i2c_write(int argc, char *argv[]);
+int cmd_i2c_xfer(int argc, char *argv[]);
