@@ -9,6 +9,7 @@
 #include <limits.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <string.h>
 #include <unistd.h>
 #if defined (__FreeBSD__) || defined(__OpenBSD__)
 #include <sys/wait.h>
@@ -21,20 +22,9 @@
 #include "updater.h"
 
 #define COMMAND_BUFFER_SIZE 256
-#define FLASHROM_OUTPUT_WP_PATTERN "write protect is "
-
-enum flashrom_ops {
-	FLASHROM_READ,
-	FLASHROM_WRITE,
-	FLASHROM_WP_STATUS,
-};
 
 /* System environment values. */
-static const char * const STR_REV = "rev",
-		  * const FLASHROM_OUTPUT_WP_ENABLED =
-			  FLASHROM_OUTPUT_WP_PATTERN "enabled",
-		  * const FLASHROM_OUTPUT_WP_DISABLED =
-			  FLASHROM_OUTPUT_WP_PATTERN "disabled";
+static const char * const STR_REV = "rev";
 
 /*
  * Strips a string (usually from shell execution output) by removing all the
@@ -172,6 +162,45 @@ static int load_firmware_version(struct firmware_image *image,
 	return 0;
 }
 
+static int parse_firmware_image(struct firmware_image *image)
+{
+	int ret = IMAGE_LOAD_SUCCESS;
+	const char *section_a = NULL, *section_b = NULL;
+
+	VB2_DEBUG("Image size: %d\n", image->size);
+	assert(image->data);
+
+	image->fmap_header = fmap_find(image->data, image->size);
+
+	if (!image->fmap_header) {
+		ERROR("Invalid image file (missing FMAP): %s\n", image->file_name);
+		ret = IMAGE_PARSE_FAILURE;
+	}
+
+	if (load_firmware_version(image, FMAP_RO_FRID, &image->ro_version))
+		ret = IMAGE_PARSE_FAILURE;
+
+	if (firmware_section_exists(image, FMAP_RW_FWID_A)) {
+		section_a = FMAP_RW_FWID_A;
+		section_b = FMAP_RW_FWID_B;
+	} else if (firmware_section_exists(image, FMAP_RW_FWID)) {
+		section_a = FMAP_RW_FWID;
+		section_b = FMAP_RW_FWID;
+	} else if (!ret) {
+		ERROR("Unsupported VBoot firmware (no RW ID): %s\n", image->file_name);
+		ret = IMAGE_PARSE_FAILURE;
+	}
+
+	/*
+	 * Load and initialize both RW A and B sections.
+	 * Note some unit tests will create only RW A.
+	 */
+	load_firmware_version(image, section_a, &image->rw_version_a);
+	load_firmware_version(image, section_b, &image->rw_version_b);
+
+	return ret;
+}
+
 /*
  * Loads a firmware image from file.
  * If archive is provided and file_name is a relative path, read the file from
@@ -180,11 +209,8 @@ static int load_firmware_version(struct firmware_image *image,
  * failure, or IMAGE_PARSE_FAILURE for non-vboot images.
  */
 int load_firmware_image(struct firmware_image *image, const char *file_name,
-			struct archive *archive)
+			struct u_archive *archive)
 {
-	int ret = IMAGE_LOAD_SUCCESS;
-	const char *section_a = NULL, *section_b = NULL;
-
 	if (!file_name) {
 		ERROR("No file name given\n");
 		return IMAGE_READ_FAILURE;
@@ -202,38 +228,9 @@ int load_firmware_image(struct firmware_image *image, const char *file_name,
 		return IMAGE_READ_FAILURE;
 	}
 
-	VB2_DEBUG("Image size: %d\n", image->size);
-	assert(image->data);
 	image->file_name = strdup(file_name);
-	image->fmap_header = fmap_find(image->data, image->size);
 
-	if (!image->fmap_header) {
-		ERROR("Invalid image file (missing FMAP): %s\n", file_name);
-		ret = IMAGE_PARSE_FAILURE;
-	}
-
-	if (load_firmware_version(image, FMAP_RO_FRID, &image->ro_version))
-		ret = IMAGE_PARSE_FAILURE;
-
-	if (firmware_section_exists(image, FMAP_RW_FWID_A)) {
-		section_a = FMAP_RW_FWID_A;
-		section_b = FMAP_RW_FWID_B;
-	} else if (firmware_section_exists(image, FMAP_RW_FWID)) {
-		section_a = FMAP_RW_FWID;
-		section_b = FMAP_RW_FWID;
-	} else if (!ret) {
-		ERROR("Unsupported VBoot firmware (no RW ID): %s\n", file_name);
-		ret = IMAGE_PARSE_FAILURE;
-	}
-
-	/*
-	 * Load and initialize both RW A and B sections.
-	 * Note some unit tests will create only RW A.
-	 */
-	load_firmware_version(image, section_a, &image->rw_version_a);
-	load_firmware_version(image, section_b, &image->rw_version_b);
-
-	return ret;
+	return parse_firmware_image(image);
 }
 
 /*
@@ -422,7 +419,7 @@ static int host_get_tpm_fwver(void)
 static int host_get_wp_hw(void)
 {
 	/* wpsw refers to write protection 'switch', not 'software'. */
-	return VbGetSystemPropertyInt("wpsw_cur");
+	return VbGetSystemPropertyInt("wpsw_cur") ? WP_ENABLED : WP_DISABLED;
 }
 
 /* A helper function to return "fw_vboot2" system property. */
@@ -459,23 +456,33 @@ static int host_get_platform_version(void)
  * Helper function to detect type of Servo board attached to host.
  * Returns a string as programmer parameter on success, otherwise NULL.
  */
-char *host_detect_servo(int *need_prepare_ptr)
+char *host_detect_servo(const char **prepare_ctrl_name)
 {
 	const char *servo_port = getenv(ENV_SERVOD_PORT);
+	const char *servo_name = getenv(ENV_SERVOD_NAME);
 	char *servo_type = host_shell("dut-control -o servo_type 2>/dev/null");
 	const char *programmer = NULL;
 	char *ret = NULL;
-	int need_prepare = 0;  /* To prepare by dut-control cpu_fw_spi:on */
 	char *servo_serial = NULL;
 
+	static const char * const cpu_fw_spi = "cpu_fw_spi";
+	static const char * const ccd_cpu_fw_spi = "ccd_cpu_fw_spi";
+
+	/* By default, no control is needed. */
+	*prepare_ctrl_name = NULL;
+
 	/* Get serial name if servo port is provided. */
-	if (servo_port && *servo_port) {
+	if ((servo_port && *servo_port) || (servo_name && *servo_name)) {
 		const char *cmd = "dut-control -o serialname 2>/dev/null";
 
-		VB2_DEBUG("Select servod using port: %s\n", servo_port);
+		VB2_DEBUG("Select servod using port: %s or name: %s\n",
+			  servo_port, servo_name);
 		if (strstr(servo_type, "with_servo_micro"))
 			cmd = ("dut-control -o servo_micro_serialname"
 			       " 2>/dev/null");
+		else if (strstr(servo_type, "with_c2d2"))
+			cmd = ("dut-control -o c2d2_serialname"
+			       " 2>/dev/null");
 		else if (strstr(servo_type, "with_ccd"))
 			cmd = "dut-control -o ccd_serialname 2>/dev/null";
 
@@ -490,14 +497,20 @@ char *host_detect_servo(int *need_prepare_ptr)
 	} else if (strstr(servo_type, "servo_micro")) {
 		VB2_DEBUG("Selected Servo Micro.\n");
 		programmer = "raiden_debug_spi";
-		need_prepare = 1;
-	} else if (strstr(servo_type, "ccd_cr50")) {
-		VB2_DEBUG("Selected CCD CR50.\n");
-		programmer = "raiden_debug_spi:target=AP";
+		*prepare_ctrl_name = cpu_fw_spi;
+	} else if (strstr(servo_type, "c2d2")) {
+		VB2_DEBUG("Selected C2D2.\n");
+		programmer = "raiden_debug_spi";
+		*prepare_ctrl_name = cpu_fw_spi;
+	} else if (strstr(servo_type, "ccd_cr50") ||
+		   strstr(servo_type, "ccd_gsc")) {
+		VB2_DEBUG("Selected CCD.\n");
+		programmer = "raiden_debug_spi:target=AP,custom_rst=true";
+		*prepare_ctrl_name = ccd_cpu_fw_spi;
 	} else {
 		VB2_DEBUG("Selected Servo V2.\n");
 		programmer = "ft2232_spi:type=google-servo-v2";
-		need_prepare = 1;
+		*prepare_ctrl_name = cpu_fw_spi;
 	}
 
 	if (programmer) {
@@ -513,183 +526,304 @@ char *host_detect_servo(int *need_prepare_ptr)
 
 	free(servo_type);
 	free(servo_serial);
-	*need_prepare_ptr = need_prepare;
 
 	return ret;
 }
+/*
+ * Returns 1 if the programmers in image1 and image2 are the same.
+ */
+static int is_the_same_programmer(const struct firmware_image *image1,
+				  const struct firmware_image *image2)
+{
+	assert(image1 && image2);
+
+	/* Including if both are NULL. */
+	if (image1->programmer == image2->programmer)
+		return 1;
+
+	/* Not the same if either one is NULL. */
+	if (!image1->programmer || !image2->programmer)
+		return 0;
+
+	return strcmp(image1->programmer, image2->programmer) == 0;
+}
+
+enum flash_command {
+	FLASH_READ = 0,
+	FLASH_WRITE,
+};
+
+/* Converts the flashrom_params to an equivalent flashrom command. */
+static char *get_flashrom_command(enum flash_command flash_cmd,
+				  struct flashrom_params *params,
+				  const char *image_name,
+				  const char *contents_name)
+{
+	int i, len = 0;
+	char *partial = NULL;
+	char *cmd = NULL;
+
+	if (!image_name)
+		image_name = "<IMAGE>";
+	if (!contents_name)
+		contents_name = "<OLD-IMAGE>";
+
+	for (i = 0; params->regions && params->regions[i]; i++)
+		len += strlen(params->regions[i]) + strlen(" -i ");
+
+	if (len) {
+		partial = (char *)malloc(len + 1);
+		if (!partial) {
+			ERROR("Failed to allocate a string buffer.\n");
+			return NULL;
+		}
+
+		partial[0] = '\0';
+		for (i = 0; params->regions[i]; i++) {
+			strcat(partial, " -i ");
+			strcat(partial, params->regions[i]);
+		}
+		assert(strlen(partial) == len);
+	}
+
+	switch (flash_cmd) {
+	case FLASH_READ:
+		ASPRINTF(&cmd, "flashrom -r %s -p %s%s%s",
+			 image_name,
+			 params->image->programmer,
+			 params->verbose > 1 ? " -V" : "",
+			 partial ? partial : "");
+		break;
+
+	case FLASH_WRITE:
+		ASPRINTF(&cmd, "flashrom -w %s -p %s%s%s%s%s%s",
+			 image_name,
+			 params->image->programmer,
+			 params->flash_contents ? " --flash-contents " : "",
+			 params->flash_contents ? contents_name : "",
+			 params->noverify ? " --noverify" : "",
+			 params->verbose > 1 ? " -V" : "",
+			 partial ? partial : "");
+		break;
+
+	default:
+		ERROR("Unknown command: %d.\n", flash_cmd);
+		break;
+	}
+	free(partial);
+	return cmd;
+}
 
 /*
- * A helper function to invoke flashrom(8) command.
+ * Emulates writing a firmware image to the system.
  * Returns 0 if success, non-zero if error.
  */
-static int host_flashrom(enum flashrom_ops op, const char *image_path,
-			 const char *programmer, int verbose,
-			 const char *section_name, const char *extra)
+static int emulate_write_firmware(const char *filename,
+				  const struct firmware_image *image,
+				  const char * const sections[])
 {
-	char *command, *result;
-	const char *op_cmd, *dash_i = "-i", *postfix = "";
-	int r;
+	int i, errorcnt = 0;
+	struct firmware_image to_image = {0};
 
-	switch (verbose) {
-	case 0:
-		postfix = " >/dev/null 2>&1";
-		break;
-	case 1:
-		break;
-	case 2:
-		postfix = "-V";
-		break;
-	case 3:
-		postfix = "-V -V";
-		break;
-	default:
-		postfix = "-V -V -V";
-		break;
-	}
+	INFO("Writing from %s to %s (emu=%s).\n",
+	     image->file_name, image->programmer, filename);
 
-	if (!section_name || !*section_name) {
-		dash_i = "";
-		section_name = "";
-	}
-
-	switch (op) {
-	case FLASHROM_READ:
-		op_cmd = "-r";
-		assert(image_path);
-		break;
-
-	case FLASHROM_WRITE:
-		op_cmd = "-w";
-		assert(image_path);
-		break;
-
-	case FLASHROM_WP_STATUS:
-		op_cmd = "--wp-status";
-		assert(image_path == NULL);
-		image_path = "";
-		/* grep is needed because host_shell only returns 1 line. */
-		postfix = " 2>/dev/null | grep \"" \
-			   FLASHROM_OUTPUT_WP_PATTERN "\"";
-		break;
-
-	default:
-		assert(0);
+	if (load_firmware_image(&to_image, filename, NULL)) {
+		ERROR("Cannot load image from %s.\n", filename);
 		return -1;
 	}
 
-	if (!extra)
-		extra = "";
-
-	/* TODO(hungte) In future we should link with flashrom directly. */
-	ASPRINTF(&command, "flashrom %s %s -p %s %s %s %s %s", op_cmd,
-		 image_path, programmer, dash_i, section_name, extra,
-		 postfix);
-
-	if (verbose)
-		INFO("Executing: %s\n", command);
-
-	if (op != FLASHROM_WP_STATUS) {
-		r = system(command);
-		free(command);
-		if (r)
-			ERROR("Error code: %d\n", r);
-		return r;
+	if (image->size != to_image.size) {
+		ERROR("Image size is different (%s:%d != %s:%d)\n",
+		      image->file_name, image->size, to_image.file_name,
+		      to_image.size);
+		errorcnt++;
+		goto exit;
 	}
 
-	result = host_shell(command);
-	strip_string(result, NULL);
-	free(command);
-	VB2_DEBUG("wp-status: %s\n", result);
+	if (!sections) {
+		VB2_DEBUG(" - write the whole image.\n");
+		memmove(to_image.data, image->data, image->size);
+	}
+	for (i = 0; sections && sections[i]; i++) {
+		VB2_DEBUG(" - write the section: %s.\n", sections[i]);
+		if (preserve_firmware_section(image, &to_image, sections[i])) {
+			ERROR("Failed to write the section: %s\n", sections[i]);
+			errorcnt++;
+			/*
+			 * Exit the loop, but still write the file to reflect
+			 * the partial changes - same as real flashrom behavior.
+			 */
+			break;
+		}
+	}
+
+	if (vb2_write_file(filename, to_image.data, to_image.size)) {
+		ERROR("Failed writing to file: %s\n", filename);
+		errorcnt++;
+		goto exit;
+	}
+
+exit:
+	free_firmware_image(&to_image);
+	return errorcnt;
+}
+
+static int external_flashrom(enum flash_command flash_cmd,
+			     struct flashrom_params *params,
+			     struct tempfile *tempfiles)
+{
+	int r;
+	char *cmd;
+	const char *image_name = NULL, *contents_name = NULL;
+
+	switch (flash_cmd) {
+	case FLASH_READ:
+		image_name = create_temp_file(tempfiles);
+		break;
+
+	case FLASH_WRITE:
+		image_name = get_firmware_image_temp_file(
+				params->image, tempfiles);
+		if (params->flash_contents)
+			contents_name = get_firmware_image_temp_file(
+					params->flash_contents, tempfiles);
+		break;
+
+	default:
+		ERROR("Unknown command: %d\n", flash_cmd);
+		return -1;
+	}
+
+	cmd = get_flashrom_command(flash_cmd, params, image_name,
+				   contents_name);
+	if (!cmd)
+		return -1;
+
+	VB2_DEBUG(cmd);
+	r = system(cmd);
+	free(cmd);
+	if (r)
+		return r;
+
+	switch (flash_cmd) {
+	case FLASH_READ:
+		r = load_firmware_image(params->image, image_name, NULL);
+		break;
+	default:
+		break;
+	}
 
-	if (strstr(result, FLASHROM_OUTPUT_WP_ENABLED))
-		r = WP_ENABLED;
-	else if (strstr(result, FLASHROM_OUTPUT_WP_DISABLED))
-		r = WP_DISABLED;
-	else
-		r = WP_ERROR;
-	free(result);
 	return r;
 }
 
-/* Helper function to return write protection status via given programmer. */
-enum wp_state host_get_wp(const char *programmer)
+static int read_flash(struct flashrom_params *params,
+		      struct updater_config *cfg)
 {
-	return host_flashrom(FLASHROM_WP_STATUS, NULL, programmer, 0, NULL,
-			     NULL);
+	if (get_config_quirk(QUIRK_EXTERNAL_FLASHROM, cfg))
+		return external_flashrom(FLASH_READ, params, &cfg->tempfiles);
+
+	return flashrom_read_image(params->image, NULL, params->verbose);
 }
 
-/* Helper function to return host software write protection status. */
-static int host_get_wp_sw(void)
+static int write_flash(struct flashrom_params *params,
+		       struct updater_config *cfg)
 {
-	return host_get_wp(PROG_HOST);
+	int r;
+
+	if (get_config_quirk(QUIRK_EXTERNAL_FLASHROM, cfg))
+		return external_flashrom(FLASH_WRITE, params, &cfg->tempfiles);
+
+	r = flashrom_write_image(params->image,
+				 params->regions,
+				 params->flash_contents,
+				 !params->noverify,
+				 params->verbose);
+	/*
+	 * Force a newline to flush stdout in case if
+	 * flashrom_write_image left some messages in the buffer.
+	 */
+	fprintf(stdout, "\n");
+	return r;
 }
 
 /*
  * Loads the active system firmware image (usually from SPI flash chip).
  * Returns 0 if success, non-zero if error.
  */
-int load_system_firmware(struct firmware_image *image,
-			 struct tempfile *tempfiles, int verbosity)
+int load_system_firmware(struct updater_config *cfg,
+			 struct firmware_image *image)
 {
-	int r;
-	const char *tmp_path = create_temp_file(tempfiles);
+	int r, i;
+	char *cmd;
+	const int tries = 1 + get_config_quirk(QUIRK_EXTRA_RETRIES, cfg);
+	struct flashrom_params params = {0};
 
-	if (!tmp_path)
-		return -1;
+	params.image = image;
+	params.verbose = cfg->verbosity + 1; /* libflashrom verbose 1 = WARN. */
 
-	r = host_flashrom(FLASHROM_READ, tmp_path, image->programmer,
-			  verbosity, NULL, NULL);
-	/*
-	 * The verbosity for host_flashrom will be translated to
-	 * (verbosity-1)*'-V', and usually 3*'-V' is enough for debugging.
-	 */
-	const int debug_verbosity = 4;
-	if (r && verbosity < debug_verbosity) {
-		/* Read again, with verbose messages for debugging. */
-		WARN("Failed reading system firmware (%d), try again...\n", r);
-		r = host_flashrom(FLASHROM_READ, tmp_path, image->programmer,
-				  debug_verbosity, NULL, NULL);
+	cmd = get_flashrom_command(FLASH_READ, &params, NULL, NULL);
+	INFO("%s\n", cmd);
+	free(cmd);
+
+	for (i = 1, r = -1; i <= tries && r != 0; i++, params.verbose++) {
+		if (i > 1)
+			WARN("Retry reading firmware (%d/%d)...\n", i, tries);
+		r = read_flash(&params, cfg);
 	}
 	if (!r)
-		r = load_firmware_image(image, tmp_path, NULL);
+		r = parse_firmware_image(image);
 	return r;
 }
 
 /*
- * Writes a section from given firmware image to system firmware.
- * If section_name is NULL, write whole image.
+ * Writes sections from a given firmware image to the system firmware.
+ * Regions should be NULL for writing the whole image, or a list of
+ * FMAP section names (and ended with a NULL).
  * Returns 0 if success, non-zero if error.
  */
-int write_system_firmware(const struct firmware_image *image,
-			  const struct firmware_image *diff_image,
-			  const char *section_name,
-			  struct tempfile *tempfiles,
-			  int verbosity)
+int write_system_firmware(struct updater_config *cfg,
+			  const struct firmware_image *image,
+			  const char * const sections[])
 {
-	const char *tmp_path = get_firmware_image_temp_file(image, tempfiles);
-	const char *tmp_diff = NULL;
+	int r = 0, i;
+	char *cmd;
+	const int tries = 1 + get_config_quirk(QUIRK_EXTRA_RETRIES, cfg);
+	struct flashrom_params params = {0};
+	struct firmware_image *flash_contents = NULL;
 
-	const char *programmer = image->programmer;
-	char *extra = NULL;
-	int r;
+	if (cfg->emulation)
+		return emulate_write_firmware(cfg->emulation, image, sections);
 
-	if (!tmp_path)
-		return -1;
+	if (cfg->use_diff_image && cfg->image_current.data &&
+	    is_the_same_programmer(&cfg->image_current, image))
+		flash_contents = &cfg->image_current;
 
-	if (diff_image) {
-		tmp_diff = get_firmware_image_temp_file(
-				diff_image, tempfiles);
-		if (!tmp_diff)
-			return -1;
-		ASPRINTF(&extra, "--noverify --flash-contents=%s", tmp_diff);
+	params.image = (struct firmware_image *)image;
+	params.flash_contents = flash_contents;
+	params.regions = sections;
+	params.noverify = !cfg->do_verify;
+	params.noverify_all = true;
+	params.verbose = cfg->verbosity + 1; /* libflashrom verbose 1 = WARN. */
+
+	cmd = get_flashrom_command(FLASH_WRITE, &params, NULL, NULL);
+	INFO("%s\n", cmd);
+	free(cmd);
+
+	for (i = 1, r = -1; i <= tries && r != 0; i++, params.verbose++) {
+		if (i > 1)
+			WARN("Retry writing firmware (%d/%d)...\n", i, tries);
+		r = write_flash(&params, cfg);
 	}
-
-	r = host_flashrom(FLASHROM_WRITE, tmp_path, programmer, verbosity,
-			  section_name, extra);
-	free(extra);
 	return r;
 }
 
+/* Helper function to return host software write protection status. */
+static int host_get_wp_sw(void)
+{
+	return flashrom_get_wp(PROG_HOST, -1);
+}
+
 /* Helper function to configure all properties. */
 void init_system_properties(struct system_property *props, int num)
 {
