@@ -14,13 +14,20 @@ load_shflags || exit 1
 # Constants used by DEFINE_*
 VBOOT_BASE='/usr/share/vboot'
 DEFAULT_KEYS_FOLDER="$VBOOT_BASE/devkeys"
-DEFAULT_BACKUP_FOLDER='/mnt/stateful_partition/backups'
 DEFAULT_PARTITIONS='2 4'
 
+# Only store backup in stateful partition if available.
+if [ -d /mnt/stateful_partition ]; then
+  DEFAULT_BACKUP_FOLDER="/mnt/stateful_partition"
+else
+  DEFAULT_BACKUP_FOLDER="$(pwd)"
+fi
+DEFAULT_BACKUP_FOLDER="${DEFAULT_BACKUP_FOLDER}/cros_sign_backups"
+
 # TODO(hungte) The default image selection is no longer a SSD, so the script
 # works more like "make_dev_image".  We may change the file name in future.
 ROOTDEV="$(rootdev -s 2>/dev/null)"
-ROOTDEV_PARTITION="$(echo $ROOTDEV | sed -n 's/.*\([0-9][0-9]*\)$/\1/p')"
+ROOTDEV_PARTITION="$(echo $ROOTDEV | sed -n 's/.*[^0-9]\([0-9][0-9]*\)$/\1/p')"
 ROOTDEV_DISK="$(rootdev -s -d 2>/dev/null)"
 ROOTDEV_KERNEL="$((ROOTDEV_PARTITION - 1))"
 
@@ -28,7 +35,16 @@ ROOTDEV_KERNEL="$((ROOTDEV_PARTITION - 1))"
 DEFINE_string image "$ROOTDEV_DISK" "Path to device or image file" "i"
 DEFINE_string keys "$DEFAULT_KEYS_FOLDER" "Path to folder of dev keys" "k"
 DEFINE_boolean remove_rootfs_verification \
-  $FLAGS_FALSE "Modify kernel boot config to disable rootfs verification" ""
+  "${FLAGS_FALSE}" "Modify kernel boot config to disable rootfs verification" \
+  "r"
+DEFINE_boolean enable_earlycon "${FLAGS_FALSE}" \
+  "Enable earlycon from stdout-path (ARM/ARM64) or SPCR (x86)." ""
+DEFINE_boolean disable_earlycon "${FLAGS_FALSE}" \
+  "Disable earlycon." ""
+DEFINE_boolean enable_console "${FLAGS_FALSE}" \
+  "Enable serial console." ""
+DEFINE_boolean disable_console "${FLAGS_FALSE}" \
+  "Disable serial console." ""
 DEFINE_string backup_dir \
   "$DEFAULT_BACKUP_FOLDER" "Path of directory to store kernel backups" ""
 DEFINE_string save_config "" \
@@ -40,8 +56,11 @@ DEFINE_boolean edit_config "${FLAGS_FALSE}" \
 DEFINE_string partitions "" \
   "List of partitions to examine (default: $DEFAULT_PARTITIONS)" ""
 DEFINE_boolean recovery_key "$FLAGS_FALSE" \
- "Use recovery key to sign image (to boot from USB" ""
-DEFINE_boolean force "$FLAGS_FALSE" "Skip sanity checks and make the change" "f"
+  "Use recovery key to sign image (to boot from USB)" ""
+DEFINE_boolean force "$FLAGS_FALSE" \
+  "Skip validity checks and make the change" "f"
+DEFINE_boolean default_rw_root "${FLAGS_TRUE}" \
+  "When --remove_rootfs_verification is set, change root mount option to RW." ""
 
 # Parse command line
 FLAGS "$@" || exit 1
@@ -66,13 +85,18 @@ EXEC_LOG="$(make_temp_file)"
 remove_rootfs_verification() {
   local new_root="PARTUUID=%U/PARTNROFF=1"
   # the first line in sed is to strip out bootcache details
+  local rw_root_opt="s| ro | rw |"
+  if [ "${FLAGS_default_rw_root}" = "${FLAGS_FALSE}" ]; then
+    rw_root_opt="s| rw | ro |"
+  fi
+
   echo "$*" | sed '
     s| dm=\"2 [^"]*bootcache[^"]* vroot | dm=\"1 vroot |
     s| root=/dev/dm-[0-9] | root='"$new_root"' |
     s| dm_verity.dev_wait=1 | dm_verity.dev_wait=0 |
     s| payload=PARTUUID=%U/PARTNROFF=1 | payload=ROOT_DEV |
     s| hashtree=PARTUUID=%U/PARTNROFF=1 | hashtree=HASH_DEV |
-    s| ro | rw |'
+    '"${rw_root_opt}"
 }
 
 remove_legacy_boot_rootfs_verification() {
@@ -83,14 +107,37 @@ remove_legacy_boot_rootfs_verification() {
   debug_msg "Removing rootfs verification for legacy boot configuration."
   mount_image_partition "$image" 12 "$mount_point" || return $FLAGS_FALSE
   config_file="$mount_point/efi/boot/grub.cfg"
-  [ ! -f "$config_file" ] ||
-    sudo sed -i 's/^ *set default=2 *$/set default=0/g' "$config_file"
+  [ ! -f "${config_file}" ] ||
+    sudo sed -i -e 's/^ *defaultA=2 *$/defaultA=0/g' \
+                -e 's/^ *defaultB=3 *$/defaultB=1/g' "${config_file}"
   config_file="$mount_point/syslinux/default.cfg"
   [ ! -f "$config_file" ] ||
     sudo sed -i 's/-vusb/-usb/g; s/-vhd/-hd/g' "$config_file"
   sudo umount "$mount_point"
 }
 
+# Enable/Disable earlycon or serial console
+insert_parameter() {
+  local cmdline="$1"
+  local param="$2"
+
+  if [ -n "${cmdline##*${param}*}" ]; then
+    cmdline="${param} ${cmdline}"
+  fi
+
+  echo "${cmdline}"
+}
+
+remove_parameter() {
+  local cmdline="$1"
+  local param="$2"
+
+  cmdline=$(echo "${cmdline}" | sed '
+    s/'"${param} "'//g')
+
+  echo "${cmdline}"
+}
+
 # Wrapped version of dd
 mydd() {
   # oflag=sync is safer, but since we need bs=512, syncing every block would be
@@ -137,13 +184,11 @@ find_valid_kernel_partitions() {
 
 # Resigns a kernel on SSD or image.
 resign_ssd_kernel() {
-  # bs=512 is the fixed block size for dd and cgpt
-  local bs=512
   local ssd_device="$1"
+  local bs="$(blocksize "${ssd_device}")"
 
   # reasonable size for current kernel partition
-  local min_kernel_size=16000
-  local max_kernel_size=65536
+  local min_kernel_size=$((8000 * 1024 / bs))
   local resigned_kernels=0
 
   for kernel_index in $FLAGS_partitions; do
@@ -162,10 +207,6 @@ resign_ssd_kernel() {
       info "${name} seems too small (${size}), ignored."
       continue
     fi
-    if [ ! $size -le $max_kernel_size ]; then
-      info "${name} seems too large (${size}), ignored."
-      continue
-    fi
 
     debug_msg "Reading $name from partition $kernel_index"
     mydd if="$ssd_device" of="$old_blob" bs=$bs skip=$offset count=$size
@@ -235,6 +276,26 @@ resign_ssd_kernel() {
       remove_legacy_boot_rootfs_verification "$ssd_device"
     fi
 
+    if [ "${FLAGS_enable_earlycon}" = "${FLAGS_TRUE}" ]; then
+      debug_msg "Enabling earlycon"
+      kernel_config="$(insert_parameter "${kernel_config}" "earlycon")"
+      debug_msg "New kernel config: ${kernel_config}"
+    elif [ "${FLAGS_disable_earlycon}" = "${FLAGS_TRUE}" ]; then
+      debug_msg "Disabling earlycon"
+      kernel_config="$(remove_parameter "${kernel_config}" "earlycon")"
+      debug_msg "New kernel config: ${kernel_config}"
+    fi
+
+    if [ "${FLAGS_enable_console}" = "${FLAGS_TRUE}" ]; then
+      debug_msg "Enabling serial console"
+      kernel_config="$(remove_parameter "${kernel_config}" "console=")"
+      debug_msg "New kernel config: ${kernel_config}"
+    elif [ "${FLAGS_disable_console}" = "${FLAGS_TRUE}" ]; then
+      debug_msg "Disabling serial console"
+      kernel_config="$(insert_parameter "${kernel_config}" "console=")"
+      debug_msg "New kernel config: ${kernel_config}"
+    fi
+
     local new_kernel_config_file="$(make_temp_file)"
     echo -n "$kernel_config"  >"$new_kernel_config_file"
 
@@ -275,7 +336,7 @@ resign_ssd_kernel() {
       cp -f "$old_blob" "$backup_file_path"; then
       info "Backup of ${name} is stored in: ${backup_file_path}"
     else
-      warning "Cannot create file in ${FLAGS_backup_dir} ... Ignore backups."
+      warn "Cannot create file in ${FLAGS_backup_dir} ... Ignore backups."
     fi
 
     debug_msg "Writing $name to partition $kernel_index"
@@ -295,7 +356,7 @@ resign_ssd_kernel() {
     # (3) change kernel config to rw
     if [ ${FLAGS_remove_rootfs_verification} = $FLAGS_TRUE ]; then
       local root_offset_sector=$(partoffset "$ssd_device" $rootfs_index)
-      local root_offset_bytes=$((root_offset_sector * 512))
+      local root_offset_bytes=$((root_offset_sector * bs))
       if ! is_ext2 "$ssd_device" "$root_offset_bytes"; then
         debug_msg "Non-ext2 partition: $ssd_device$rootfs_index, skip."
       elif ! rw_mount_disabled "$ssd_device" "$root_offset_bytes"; then
@@ -326,8 +387,8 @@ resign_ssd_kernel() {
   return $resigned_kernels
 }
 
-sanity_check_crossystem_flags() {
-  debug_msg "crossystem sanity check"
+validity_check_crossystem_flags() {
+  debug_msg "crossystem validity check"
   if [ -n "${FLAGS_save_config}" ]; then
     debug_msg "not resigning kernel."
     return
@@ -352,8 +413,8 @@ sanity_check_crossystem_flags() {
   return $FLAGS_FALSE
 }
 
-sanity_check_live_partitions() {
-  debug_msg "Partition sanity check"
+validity_check_live_partitions() {
+  debug_msg "Partition validity check"
   if [ "$FLAGS_partitions" = "$ROOTDEV_KERNEL" ]; then
     debug_msg "only for current active partition - safe."
     return
@@ -379,8 +440,8 @@ sanity_check_live_partitions() {
   return $FLAGS_FALSE
 }
 
-sanity_check_live_firmware() {
-  debug_msg "Firmware compatibility sanity check"
+validity_check_live_firmware() {
+  debug_msg "Firmware compatibility validity check"
   if [ "$(crossystem mainfw_type)" = "developer" ]; then
     debug_msg "developer type firmware in active."
     return
@@ -431,6 +492,20 @@ sanity_check_live_firmware() {
   return $FLAGS_FALSE
 }
 
+validity_check() {
+  validity_check_live_partitions || return $FLAGS_FALSE
+
+  # Remaining checks depend on firmware; skip if device is running in a VM.
+  if crossystem 'inside_vm?1'; then
+    debug_msg "Device is a VM, skipping firmware checks"
+    return $FLAGS_TRUE
+  fi
+
+  validity_check_live_firmware || return $FLAGS_FALSE
+  validity_check_crossystem_flags || return $FLAGS_FALSE
+  return $FLAGS_TRUE
+}
+
 # Main
 # ----------------------------------------------------------------------------
 main() {
@@ -463,12 +538,12 @@ main() {
       die "No valid kernel partitions on ${FLAGS_image} (${FLAGS_partitions})."
     FLAGS_partitions="$valid_partitions"
 
-    # Sanity checks
+    # Validity checks
     if [ "$FLAGS_force" = "$FLAGS_TRUE" ]; then
       echo "
-      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-      ! INFO: ALL SANITY CHECKS WERE BYPASSED. YOU ARE ON YOUR OWN. !
-      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+      ! INFO: ALL VALIDITY CHECKS WERE BYPASSED. YOU ARE ON YOUR OWN. !
+      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
       " >&2
       local i
       for i in $(seq 5 -1 1); do
@@ -476,9 +551,7 @@ main() {
         sleep 1
       done
       echo ""
-    elif ! sanity_check_live_firmware ||
-         ! sanity_check_live_partitions ||
-         ! sanity_check_crossystem_flags; then
+    elif ! validity_check; then
       die "IMAGE ${FLAGS_image} IS NOT MODIFIED."
     fi
   fi
@@ -490,6 +563,7 @@ main() {
     # signed something at least
     info "Successfully re-signed ${num_signed} of ${num_given} kernel(s)" \
       " on device ${FLAGS_image}."
+    info "Please remember to reboot before updating the kernel on this device."
   else
     die "Failed re-signing kernels."
   fi
