@@ -45,10 +45,14 @@
 #include <stdbool.h>
 #include <libpayload-config.h>
 #include <cbgfx.h>
+#include <commonlib/bsd/elog.h>
+#include <commonlib/bsd/fmap_serialized.h>
+#include <commonlib/bsd/helpers.h>
+#include <commonlib/bsd/mem_chip_info.h>
 #include <ctype.h>
 #include <die.h>
 #include <endian.h>
-#include <fmap_serialized.h>
+#include <fmap.h>
 #include <ipchksum.h>
 #include <kconfig.h>
 #include <stddef.h>
@@ -65,33 +69,8 @@
 #include <pci.h>
 #include <archive.h>
 
-/* Double-evaluation unsafe min/max, for bitfields and outside of functions */
-#define __CMP_UNSAFE(a, b, op) ((a) op (b) ? (a) : (b))
-#define MIN_UNSAFE(a, b) __CMP_UNSAFE(a, b, <)
-#define MAX_UNSAFE(a, b) __CMP_UNSAFE(a, b, >)
-
-#define __CMP_SAFE(a, b, op, var_a, var_b) ({ \
-	__TYPEOF_UNLESS_CONST(a, b) var_a = (a); \
-	__TYPEOF_UNLESS_CONST(b, a) var_b = (b); \
-	var_a op var_b ? var_a : var_b; \
-})
-
-#define __CMP(a, b, op) __builtin_choose_expr( \
-	__builtin_constant_p(a) && __builtin_constant_p(b), \
-	__CMP_UNSAFE(a, b, op), __CMP_SAFE(a, b, op, __TMPNAME, __TMPNAME))
-
-#define MIN(a, b) __CMP(a, b, <)
-#define MAX(a, b) __CMP(a, b, >)
-
-#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))
 #define BIT(x)	(1ul << (x))
 
-#define DIV_ROUND_UP(x, y) ({ \
-	typeof(x) _div_local_x = (x); \
-	typeof(y) _div_local_y = (y); \
-	(_div_local_x + _div_local_y - 1) / _div_local_y; \
-})
-
 static inline u32 div_round_up(u32 n, u32 d) { return (n + d - 1) / d; }
 
 #define LITTLE_ENDIAN	1234
@@ -457,6 +436,8 @@ static inline int clz(u32 x)
 static inline int log2(u32 x) { return (int)sizeof(x) * 8 - clz(x) - 1; }
 /* Find First Set: __ffs(0xf) == 0, __ffs(0) == -1, __ffs(1 << 31) == 31 */
 static inline int __ffs(u32 x) { return log2(x & (u32)(-(s32)x)); }
+/* Find Last Set: __fls(1) == 0, __fls(5) == 2, __fls(1 << 31) == 31 */
+static inline int __fls(u32 x) { return log2(x); }
 
 static inline int popcnt64(u64 x) { return __builtin_popcountll(x); }
 static inline int clz64(u64 x)
@@ -466,6 +447,7 @@ static inline int clz64(u64 x)
 
 static inline int log2_64(u64 x) { return sizeof(x) * 8 - clz64(x) - 1; }
 static inline int __ffs64(u64 x) { return log2_64(x & (u64)(-(s64)x)); }
+static inline int __fls64(u64 x) { return log2_64(x); }
 /** @} */
 
 /**
