@@ -2,6 +2,7 @@
 #define _STDDEF_H
 
 #include <arch/types.h>
+#include <commonlib/bsd/helpers.h>
 
 #ifndef __WCHAR_TYPE__
 #define __WCHAR_TYPE__ int
@@ -22,22 +23,6 @@ typedef __SIZE_TYPE__ size_t;
 typedef __SIZE_TYPE__ ssize_t;
 #undef unsigned
 
-#define offsetof(TYPE, MEMBER)	((size_t) &((TYPE *) 0)->MEMBER)
-#define member_size(TYPE, MEMBER)	(sizeof(((TYPE *) 0)->MEMBER))
-
-#define check_member(structure, member, offset) _Static_assert( \
-	offsetof(struct structure, member) == offset, \
-	"`struct " #structure "` offset for `" #member "` is not " #offset)
-
-/* Standard units. */
-#define KiB (1 << 10)
-#define MiB (1 << 20)
-#define GiB (1 << 30)
-
-#define KHz (1000)
-#define MHz (1000*KHz)
-#define GHz (1000*MHz)
-
 #define NSECS_PER_SEC 1000000000
 #define USECS_PER_SEC 1000000
 #define MSECS_PER_SEC 1000
