@@ -83,6 +83,8 @@ struct sysinfo_t {
 	uintptr_t compiler;
 	uintptr_t linker;
 	uintptr_t assembler;
+	uintptr_t mem_chip_base;
+	uintptr_t pcie_ctrl_base; /* Base address of PCIe controller */
 
 	uintptr_t cb_version;
 
@@ -110,6 +112,7 @@ struct sysinfo_t {
 	uintptr_t mrc_cache;
 	uintptr_t acpi_gnvs;
 	uintptr_t acpi_cnvs;
+	uintptr_t acpi_rsdp;
 
 #define UNDEFINED_STRAPPING_ID	(~0)
 #define UNDEFINED_FW_CONFIG	~((uint64_t)0)
@@ -150,6 +153,12 @@ struct sysinfo_t {
 #endif
 	/* USB Type-C Port Configuration Info */
 	uintptr_t type_c_info;
+
+	/* CBFS RW/RO Metadata Cache */
+	uintptr_t cbfs_ro_mcache_offset;
+	uint32_t cbfs_ro_mcache_size;
+	uintptr_t cbfs_rw_mcache_offset;
+	uint32_t cbfs_rw_mcache_size;
 };
 
 extern struct sysinfo_t lib_sysinfo;
