@@ -6,7 +6,6 @@
 #include <arch/ioapic.h>
 #include <arch/smp/mpspec.h>
 #include <string.h>
-#include <arch/cpu.h>
 #include <cpu/cpu.h>
 #include <cpu/x86/lapic.h>
 #include <drivers/generic/ioapic/chip.h>
@@ -161,12 +160,7 @@ void smp_write_processors(struct mp_config_table *mc)
 	for (order_id = 0; order_id < 256; order_id++) {
 		for (cpu = all_devices; cpu; cpu = cpu->next) {
 			unsigned long cpu_flag;
-			if ((cpu->path.type != DEVICE_PATH_APIC) ||
-				(cpu->bus->dev->path.type !=
-					DEVICE_PATH_CPU_CLUSTER))
-				continue;
-
-			if (!cpu->enabled)
+			if (!is_enabled_cpu(cpu))
 				continue;
 
 			cpu_flag = MPC_CPU_ENABLED;
@@ -581,7 +575,7 @@ unsigned long __weak write_smp_table(unsigned long addr)
 
 	for (dev = all_devices; dev; dev = dev->next) {
 
-		if (dev->path.type != DEVICE_PATH_PCI || !dev->enabled)
+		if (!is_enabled_pci(dev))
 			continue;
 
 		have_fixed_entries = 0;
