@@ -1,17 +1,17 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
+#include <acpi/acpi.h>
+#include <amdblocks/cpu.h>
+#include <amdblocks/smm.h>
 #include <console/console.h>
-#include <cpu/x86/msr.h>
 #include <cpu/amd/msr.h>
-#include <cpu/x86/mtrr.h>
 #include <cpu/amd/mtrr.h>
-#include <cpu/x86/smm.h>
-#include <device/device.h>
-#include <cpu/x86/pae.h>
-#include <cpu/x86/lapic.h>
 #include <cpu/cpu.h>
 #include <cpu/x86/cache.h>
-#include <acpi/acpi.h>
+#include <cpu/x86/msr.h>
+#include <cpu/x86/mtrr.h>
+#include <cpu/x86/smm.h>
+#include <device/device.h>
 #include <northbridge/amd/agesa/agesa_helper.h>
 
 static void model_15_init(struct device *dev)
@@ -59,11 +59,8 @@ static void model_15_init(struct device *dev)
 	/* zero the machine check error status registers */
 	mca_clear_status();
 
-	/* Enable the local CPU APICs */
-	setup_lapic();
-
 #if CONFIG(LOGICAL_CPUS)
-	siblings = cpuid_ecx(0x80000008) & 0xff;
+	siblings = get_cpu_count() - 1; // minus BSP
 
 	if (siblings > 0) {
 		msr = rdmsr_amd(CPU_ID_FEATURES_MSR);
@@ -98,9 +95,7 @@ static void model_15_init(struct device *dev)
 	}
 
 	/* Write protect SMM space with SMMLOCK. */
-	msr = rdmsr(HWCR_MSR);
-	msr.lo |= (1 << 0);
-	wrmsr(HWCR_MSR, msr);
+	lock_smm();
 }
 
 static struct device_operations cpu_dev_ops = {
