@@ -1,22 +1,20 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
-#include <arch/cpu.h>
 #include <arch/romstage.h>
-#include <console/console.h>
-#include <cpu/x86/smm.h>
 #include <arch/symbols.h>
 #include <commonlib/helpers.h>
+#include <console/console.h>
+#include <cpu/x86/smm.h>
 #include <program_loading.h>
-#include <timestamp.h>
+#include <romstage_common.h>
 #include <security/vboot/vboot_common.h>
+#include <types.h>
 
 /* If we do not have a constrained _car_stack region size, use the
    following as a guideline for acceptable stack usage. */
 #define DCACHE_RAM_ROMSTAGE_STACK_SIZE 0x2000
 
-static struct postcar_frame early_mtrrs;
-
-static void romstage_main(void)
+void __noreturn romstage_main(void)
 {
 	int i;
 	const int num_guards = 64;
@@ -54,16 +52,6 @@ static void romstage_main(void)
 	if (CONFIG(SMM_TSEG))
 		smm_list_regions();
 
-	prepare_and_run_postcar(&early_mtrrs);
+	prepare_and_run_postcar();
 	/* We do not return here. */
 }
-
-asmlinkage void car_stage_entry(void)
-{
-	timestamp_add_now(TS_START_ROMSTAGE);
-
-	/* Assumes the hardware was set up during the bootblock */
-	console_init();
-
-	romstage_main();
-}
