@@ -4,11 +4,13 @@
 #include <arch/cpu.h>
 #include <console/console.h>
 #include <cpu/intel/msr.h>
-#include <cpu/x86/lapic.h>
 #include <cpu/x86/msr.h>
+#include <cpu/intel/turbo.h>
 #include "common.h"
 
-#define  CPUID_6_ECX_EPB	(1 << 3)
+#define  CPUID_6_ECX_EPB		(1 << 3)
+#define  CPUID_6_ENGERY_PERF_PREF	(1 << 10)
+#define  CPUID_6_HWP			(1 << 7)
 
 void set_vmx_and_lock(void)
 {
@@ -16,11 +18,10 @@ void set_vmx_and_lock(void)
 	set_feature_ctrl_lock();
 }
 
-void set_feature_ctrl_vmx(void)
+void set_feature_ctrl_vmx_arg(bool enable)
 {
 	msr_t msr;
 	uint32_t feature_flag;
-	int enable = CONFIG(ENABLE_VMX);
 
 	feature_flag = cpu_get_feature_flags_ecx();
 	/* Check that the VMX is supported before reading or writing the MSR. */
@@ -62,6 +63,12 @@ void set_feature_ctrl_vmx(void)
 	printk(BIOS_DEBUG, "VMX status: %s\n",
 		enable ? "enabled" : "disabled");
 }
+
+void set_feature_ctrl_vmx(void)
+{
+	set_feature_ctrl_vmx_arg(CONFIG(ENABLE_VMX));
+}
+
 void set_feature_ctrl_lock(void)
 {
 	msr_t msr;
@@ -178,3 +185,43 @@ void set_energy_perf_bias(u8 policy)
 	msr_unset_and_set(IA32_ENERGY_PERF_BIAS, ENERGY_POLICY_MASK, epb);
 	printk(BIOS_DEBUG, "cpu: energy policy set to %u\n", epb);
 }
+
+/*
+ * Check energy performance preference and HWP capabilities from Thermal and
+ * Power Management Leaf CPUID
+ */
+bool check_energy_perf_cap(void)
+{
+	const u32 cap = cpuid_eax(CPUID_LEAF_PM);
+	if (!(cap & CPUID_6_ENGERY_PERF_PREF))
+		return false;
+	if (!(cap & CPUID_6_HWP))
+		return false;
+	return true;
+}
+
+/*
+ * Instructs the CPU to use EPP hints. This means that any energy policies set
+ * up in `set_energy_perf_bias` will be ignored afterwards.
+ */
+void enable_energy_perf_pref(void)
+{
+	msr_t msr = rdmsr(IA32_PM_ENABLE);
+	if (!(msr.lo & HWP_ENABLE)) {
+		/* Package-scoped MSR */
+		printk(BIOS_DEBUG, "HWP_ENABLE: energy-perf preference in favor of energy-perf bias\n");
+		msr_set(IA32_PM_ENABLE, HWP_ENABLE);
+	}
+}
+
+/*
+ * Set the IA32_HWP_REQUEST Energy-Performance Preference bits on the logical
+ * thread. 0 is a hint to the HWP to prefer performance, and 255 is a hint to
+ * prefer energy efficiency.
+ * This function needs to be called when HWP_ENABLE is set.
+*/
+void set_energy_perf_pref(u8 pref)
+{
+	msr_unset_and_set(IA32_HWP_REQUEST, IA32_HWP_REQUEST_EPP_MASK,
+		pref << IA32_HWP_REQUEST_EPP_SHIFT);
+}
