@@ -4,7 +4,6 @@
 #include <device/device.h>
 #include <cpu/cpu.h>
 #include <cpu/x86/msr.h>
-#include <cpu/x86/lapic.h>
 #include <cpu/intel/speedstep.h>
 #include <cpu/x86/cache.h>
 #include <cpu/x86/name.h>
@@ -90,7 +89,6 @@ static void configure_misc(void)
 	msr = rdmsr(IA32_PECI_CTL);
 	msr.lo |= 1;
 	wrmsr(IA32_PECI_CTL, msr);
-
 }
 
 #define PIC_SENS_CFG	0x1aa
@@ -120,9 +118,6 @@ static void model_6fx_init(struct device *cpu)
 	/* Setup Page Attribute Tables (PAT) */
 	// TODO set up PAT
 
-	/* Enable the local CPU APICs */
-	setup_lapic();
-
 	/* Configure C States */
 	configure_c_states();
 
