@@ -8,20 +8,21 @@
  * Programming
  */
 
-#include <stddef.h>
-#include <string.h>
+#include <assert.h>
 #include <bootstate.h>
 #include <commonlib/helpers.h>
 #include <console/console.h>
-#include <device/device.h>
-#include <device/pci_ids.h>
+#include <cpu/amd/mtrr.h>
 #include <cpu/cpu.h>
+#include <cpu/x86/cache.h>
 #include <cpu/x86/msr.h>
 #include <cpu/x86/mtrr.h>
-#include <cpu/x86/cache.h>
+#include <device/device.h>
+#include <device/pci_ids.h>
 #include <memrange.h>
-#include <cpu/amd/mtrr.h>
-#include <assert.h>
+#include <string.h>
+#include <types.h>
+
 #if CONFIG(X86_AMD_FIXED_MTRRS)
 #define MTRR_FIXED_WRBACK_BITS (MTRR_READ_MEM | MTRR_WRITE_MEM)
 #else
@@ -41,11 +42,7 @@ static int total_mtrrs;
 
 static void detect_var_mtrrs(void)
 {
-	msr_t msr;
-
-	msr = rdmsr(MTRR_CAP_MSR);
-
-	total_mtrrs = msr.lo & 0xff;
+	total_mtrrs = get_var_mtrr_count();
 
 	if (total_mtrrs > NUM_MTRR_STATIC_STORAGE) {
 		printk(BIOS_WARNING,
@@ -106,7 +103,6 @@ static void enable_var_mtrr(unsigned char deftype)
 	(((x) > RANGE_SHIFT) ? ((x) - RANGE_SHIFT) : RANGE_SHIFT)
 #define PHYS_TO_RANGE_ADDR(x) ((x) >> RANGE_SHIFT)
 #define RANGE_TO_PHYS_ADDR(x) (((resource_t)(x)) << RANGE_SHIFT)
-#define NUM_FIXED_MTRRS (NUM_FIXED_RANGES / RANGES_PER_FIXED_MTRR)
 
 /* Helpful constants. */
 #define RANGE_1MB PHYS_TO_RANGE_ADDR(1ULL << 20)
@@ -158,7 +154,7 @@ static void print_physical_address_space(const struct memranges *addr_space,
 	memranges_each_entry(r, addr_space)
 		printk(BIOS_DEBUG,
 		       "0x%016llx - 0x%016llx size 0x%08llx type %ld\n",
-		       range_entry_base(r), range_entry_end(r),
+		       range_entry_base(r), range_entry_end(r) - 1,
 		       range_entry_size(r), range_entry_tag(r));
 }
 
@@ -276,7 +272,7 @@ static void calc_fixed_mtrrs(void)
 			type = range_entry_tag(r);
 			printk(MTRR_VERBOSE_LEVEL,
 			       "MTRR addr 0x%x-0x%x set to %d type @ %d\n",
-			       begin, begin + desc->step, type, type_index);
+			       begin, begin + desc->step - 1, type, type_index);
 			if (type == MTRR_TYPE_WRBACK)
 				type |= MTRR_FIXED_WRBACK_BITS;
 			fixed_mtrr_types[type_index] = type;
@@ -346,7 +342,6 @@ static void commit_fixed_mtrrs(void)
 		wrmsr(msr_index[i], fixed_msrs[i]);
 	enable_cache();
 	fixed_mtrrs_hide_amd_rwdram();
-
 }
 
 void x86_setup_fixed_mtrrs_no_enable(void)
@@ -409,7 +404,7 @@ static void prep_var_mtrr(struct var_mtrr_state *var_state,
 	resource_t mask;
 
 	if (var_state->mtrr_index >= total_mtrrs) {
-		printk(BIOS_ERR, "ERROR: Not enough MTRRs available! MTRR index is %d with %d MTRRs in total.\n",
+		printk(BIOS_ERR, "Not enough MTRRs available! MTRR index is %d with %d MTRRs in total.\n",
 		       var_state->mtrr_index, total_mtrrs);
 		return;
 	}
@@ -872,6 +867,28 @@ void mtrr_use_temp_range(uintptr_t begin, size_t size, int type)
 	struct memranges addr_space;
 	const int above4gb = 1; /* Cover above 4GiB by default. */
 	int address_bits;
+	static struct temp_range {
+		uintptr_t begin;
+		size_t size;
+		int type;
+	} temp_ranges[10];
+
+	if (size == 0)
+		return;
+
+	int i;
+	for (i = 0; i < ARRAY_SIZE(temp_ranges); i++) {
+		if (temp_ranges[i].size == 0) {
+			temp_ranges[i].begin = begin;
+			temp_ranges[i].size = size;
+			temp_ranges[i].type = type;
+			break;
+		}
+	}
+	if (i == ARRAY_SIZE(temp_ranges)) {
+		printk(BIOS_ERR, "Out of temporary ranges for MTRR use\n");
+		return;
+	}
 
 	/* Make a copy of the original address space and tweak it with the
 	 * provided range. */
@@ -890,7 +907,11 @@ void mtrr_use_temp_range(uintptr_t begin, size_t size, int type)
 	}
 
 	/* Place new range into the address space. */
-	memranges_insert(&addr_space, begin, size, type);
+	for (i = 0; i < ARRAY_SIZE(temp_ranges); i++) {
+		if (temp_ranges[i].size != 0)
+			memranges_insert(&addr_space, temp_ranges[i].begin,
+					 temp_ranges[i].size, temp_ranges[i].type);
+	}
 
 	print_physical_address_space(&addr_space, "TEMPORARY");
 
@@ -904,7 +925,7 @@ void mtrr_use_temp_range(uintptr_t begin, size_t size, int type)
 
 	if (commit_var_mtrrs(&sol) < 0)
 		printk(BIOS_WARNING, "Unable to insert temporary MTRR range: 0x%016llx - 0x%016llx size 0x%08llx type %d\n",
-			(long long)begin, (long long)begin + size,
+			(long long)begin, (long long)begin + size - 1,
 			(long long)size, type);
 	else
 		put_back_original_solution = true;
@@ -919,4 +940,4 @@ static void remove_temp_solution(void *unused)
 }
 
 BOOT_STATE_INIT_ENTRY(BS_OS_RESUME, BS_ON_ENTRY, remove_temp_solution, NULL);
-BOOT_STATE_INIT_ENTRY(BS_PAYLOAD_LOAD, BS_ON_EXIT, remove_temp_solution, NULL);
+BOOT_STATE_INIT_ENTRY(BS_PAYLOAD_BOOT, BS_ON_ENTRY, remove_temp_solution, NULL);
