@@ -5,6 +5,7 @@
 #include <device/path.h>
 #include <device/pci_def.h>
 #include <device/resource.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -522,7 +523,7 @@ void report_resource_stored(struct device *dev, const struct resource *resource,
 		snprintf(buf, sizeof(buf),
 			 "bus %02x ", dev->link_list->secondary);
 	}
-	printk(BIOS_DEBUG, "%s %02lx <- [0x%010llx - 0x%010llx] size 0x%08llx "
+	printk(BIOS_DEBUG, "%s %02lx <- [0x%016llx - 0x%016llx] size 0x%08llx "
 	       "gran 0x%02x %s%s%s\n", dev_path(dev), resource->index,
 		base, end, resource->size, resource->gran, buf,
 		resource_type(resource), comment);
@@ -586,6 +587,10 @@ void search_global_resources(unsigned long type_mask, unsigned long type,
 			if (res->flags & IORESOURCE_SUBTRACTIVE)
 				continue;
 
+			/* If the resource is not assigned ignore it. */
+			if (!(res->flags & IORESOURCE_ASSIGNED))
+				continue;
+
 			search(gp, curdev, res);
 		}
 	}
@@ -800,7 +805,7 @@ void show_one_resource(int debug_level, struct device *dev,
 	end = resource_end(resource);
 	buf[0] = '\0';
 
-	printk(debug_level, "%s %02lx <- [0x%010llx - 0x%010llx] "
+	printk(debug_level, "%s %02lx <- [0x%016llx - 0x%016llx] "
 		  "size 0x%08llx gran 0x%02x %s%s%s\n", dev_path(dev),
 		  resource->index, base, end, resource->size, resource->gran,
 		  buf, resource_type(resource), comment);
@@ -822,43 +827,45 @@ void show_all_devs_resources(int debug_level, const char *msg)
 	}
 }
 
-void fixed_mem_resource(struct device *dev, unsigned long index,
-			unsigned long basek, unsigned long sizek,
-			unsigned long type)
+const struct resource *fixed_resource_range_idx(struct device *dev, unsigned long index,
+				uint64_t base, uint64_t size, unsigned long flags)
 {
 	struct resource *resource;
-
-	if (!sizek)
-		return;
+	if (!size)
+		return NULL;
 
 	resource = new_resource(dev, index);
-	resource->base = ((resource_t)basek) << 10;
-	resource->size = ((resource_t)sizek) << 10;
-	resource->flags = IORESOURCE_MEM | IORESOURCE_FIXED |
-		 IORESOURCE_STORED | IORESOURCE_ASSIGNED;
+	resource->base = base;
+	resource->size = size;
+	resource->flags = IORESOURCE_FIXED | IORESOURCE_ASSIGNED;
+	resource->flags |= flags;
 
-	resource->flags |= type;
+	printk(BIOS_SPEW, "dev: %s, index: 0x%lx, base: 0x%llx, size: 0x%llx\n",
+	       dev_path(dev), resource->index, resource->base, resource->size);
+
+	return resource;
 }
 
-void fixed_io_resource(struct device *dev, unsigned long index,
-			unsigned long base, unsigned long size)
+const struct resource *lower_ram_end(struct device *dev, unsigned long index, uint64_t end)
 {
-	struct resource *resource;
+	return ram_from_to(dev, index, 0, end);
+}
 
-	resource = new_resource(dev, index);
-	resource->base = (resource_t)base;
-	resource->size = (resource_t)size;
-	resource->limit = resource->base + resource->size - 1;
-	resource->flags = IORESOURCE_IO | IORESOURCE_FIXED |
-		 IORESOURCE_STORED | IORESOURCE_ASSIGNED |
-		 IORESOURCE_RESERVE;
+const struct resource *upper_ram_end(struct device *dev, unsigned long index, uint64_t end)
+{
+	if (end <= 4ull * GiB)
+		return NULL;
+
+	printk(BIOS_INFO, "Available memory above 4GB: %lluM\n", (end - 4ull * GiB) / MiB);
+
+	return ram_from_to(dev, index, 4ull * GiB, end);
 }
 
 void mmconf_resource(struct device *dev, unsigned long index)
 {
 	struct resource *resource = new_resource(dev, index);
-	resource->base = CONFIG_MMCONF_BASE_ADDRESS;
-	resource->size = CONFIG_MMCONF_LENGTH;
+	resource->base = CONFIG_ECAM_MMCONF_BASE_ADDRESS;
+	resource->size = CONFIG_ECAM_MMCONF_LENGTH;
 	resource->flags = IORESOURCE_MEM | IORESOURCE_RESERVE |
 		IORESOURCE_FIXED | IORESOURCE_STORED | IORESOURCE_ASSIGNED;
 
@@ -910,10 +917,7 @@ int dev_count_cpu(void)
 	int count = 0;
 
 	for (cpu = all_devices; cpu; cpu = cpu->next) {
-		if ((cpu->path.type != DEVICE_PATH_APIC) ||
-		    (cpu->bus->dev->path.type != DEVICE_PATH_CPU_CLUSTER))
-			continue;
-		if (!cpu->enabled)
+		if (!is_enabled_cpu(cpu))
 			continue;
 		count++;
 	}
@@ -932,3 +936,47 @@ const char *dev_path_name(enum device_path_type type)
 		type_name = type_names[type];
 	return type_name;
 }
+
+bool dev_path_hotplug(const struct device *dev)
+{
+	for (dev = dev->bus->dev; dev != dev->bus->dev; dev = dev->bus->dev) {
+		if (dev->hotplug_port)
+			return true;
+	}
+	return false;
+}
+
+void log_resource(const char *type, const struct device *dev, const struct resource *res,
+			const char *srcfile, const int line)
+{
+	printk(BIOS_SPEW, "%s:%d res: %s, dev: %s, index: 0x%lx, base: 0x%llx, "
+			  "end: 0x%llx, size_kb: 0x%llx\n",
+			  srcfile, line, type, dev_path(dev), res->index, res->base,
+			  resource_end(res), res->size / KiB);
+}
+
+bool is_cpu(const struct device *cpu)
+{
+	return cpu->path.type == DEVICE_PATH_APIC &&
+	       cpu->bus->dev->path.type == DEVICE_PATH_CPU_CLUSTER;
+}
+
+bool is_enabled_cpu(const struct device *cpu)
+{
+	return is_cpu(cpu) && cpu->enabled;
+}
+
+bool is_pci(const struct device *pci)
+{
+	return pci->path.type == DEVICE_PATH_PCI;
+}
+
+bool is_enabled_pci(const struct device *pci)
+{
+	return is_pci(pci) && pci->enabled;
+}
+
+bool is_pci_dev_on_bus(const struct device *pci, unsigned int bus)
+{
+	return is_pci(pci) && pci->bus->secondary == bus;
+}
