@@ -13,6 +13,7 @@
 #include <smp/node.h>
 #include <string.h>
 #include <timestamp.h>
+#include <romstage_common.h>
 #include <northbridge/amd/agesa/agesa_helper.h>
 #include <northbridge/amd/agesa/state_machine.h>
 
@@ -31,27 +32,18 @@ static void fill_sysinfo(struct sysinfo *cb)
  */
 static void ap_romstage_main(void);
 
-static void romstage_main(void)
+void __noreturn romstage_main(void)
 {
-	struct postcar_frame pcf;
 	struct sysinfo romstage_state;
 	struct sysinfo *cb = &romstage_state;
-	unsigned int initial_apic_id = initial_lapicid();
 	int cbmem_initted = 0;
 
+	printk(BIOS_DEBUG, "APIC %02u: CPU Family_Model = %08x\n",
+	       initial_lapicid(), cpuid_eax(1));
+
 	fill_sysinfo(cb);
 
-	if (initial_apic_id == 0) {
-
-		timestamp_add_now(TS_START_ROMSTAGE);
-
-		board_BeforeAgesa(cb);
-
-		console_init();
-	}
-
-	printk(BIOS_DEBUG, "APIC %02u: CPU Family_Model = %08x\n",
-		initial_apic_id, cpuid_eax(1));
+	board_BeforeAgesa(cb);
 
 	set_ap_entry_ptr(ap_romstage_main);
 
@@ -59,14 +51,14 @@ static void romstage_main(void)
 
 	agesa_execute_state(cb, AMD_INIT_EARLY);
 
-	timestamp_add_now(TS_BEFORE_INITRAM);
+	timestamp_add_now(TS_INITRAM_START);
 
 	if (!cb->s3resume)
 		agesa_execute_state(cb, AMD_INIT_POST);
 	else
 		agesa_execute_state(cb, AMD_INIT_RESUME);
 
-	timestamp_add_now(TS_AFTER_INITRAM);
+	timestamp_add_now(TS_INITRAM_END);
 
 	/* Work around AGESA setting all memory as WB on normal
 	 * boot path.
@@ -82,10 +74,7 @@ static void romstage_main(void)
 
 	romstage_handoff_init(cb->s3resume);
 
-	postcar_frame_init(&pcf, HIGH_ROMSTAGE_STACK_SIZE);
-	recover_postcar_frame(&pcf, cb->s3resume);
-
-	run_postcar_phase(&pcf);
+	prepare_and_run_postcar();
 	/* We do not return. */
 }
 
@@ -104,11 +93,6 @@ static void ap_romstage_main(void)
 	halt();
 }
 
-asmlinkage void car_stage_entry(void)
-{
-	romstage_main();
-}
-
 void *cbmem_top_chipset(void)
 {
 	/* Top of CBMEM is at highest usable DRAM address below 4GiB. */
