@@ -18,7 +18,7 @@
 #include "Dispatcher.h"
 #endif
 
-#if ENV_ROMSTAGE
+#if ENV_RAMINIT
 #include <PlatformMemoryConfiguration.h>
 CONST PSO_ENTRY ROMDATA DefaultPlatformMemoryConfiguration[] = {PSO_END};
 #endif
@@ -36,7 +36,7 @@ static void agesa_locate_image(AMD_CONFIG_PARAMS *StdHeader)
 
 	image = LibAmdLocateImage(agesa, agesa + file_size, 4096,
 		ModuleIdentifier);
-	StdHeader->ImageBasePtr = (void *) image;
+	StdHeader->ImageBasePtr = (void *)image;
 #endif
 }
 
@@ -159,7 +159,6 @@ static AGESA_STATUS romstage_dispatch(struct sysinfo *cb,
 		{
 			break;
 		}
-
 	}
 	return status;
 }
@@ -222,7 +221,6 @@ static AGESA_STATUS ramstage_dispatch(struct sysinfo *cb,
 		{
 			break;
 		}
-
 	}
 	return status;
 }
@@ -247,7 +245,7 @@ int agesa_execute_state(struct sysinfo *cb, AGESA_STRUCT_NAME func)
 
 	/* For these calls, heap is not available. */
 	if (func == AMD_INIT_RESET || func == AMD_S3LATE_RESTORE) {
-		buf = (void *) &agesa_params;
+		buf = (void *)&agesa_params;
 		len = sizeof(agesa_params);
 		memcpy(buf, &cb->StdHeader, sizeof(cb->StdHeader));
 	}
@@ -262,7 +260,7 @@ int agesa_execute_state(struct sysinfo *cb, AGESA_STRUCT_NAME func)
 	if (CONFIG(AGESA_EXTRA_TIMESTAMPS) && task.ts_entry_id)
 		timestamp_add_now(task.ts_entry_id);
 
-	if (ENV_ROMSTAGE)
+	if (ENV_RAMINIT)
 		final = romstage_dispatch(cb, func, StdHeader);
 
 	if (ENV_RAMSTAGE)
