@@ -17,14 +17,15 @@
 
 #include <commonlib/endian.h>
 #include <commonlib/helpers.h>
+#include <console/console.h>
+#include <delay.h>
+#include <device/i2c_simple.h>
+#include <drivers/tpm/cr50.h>
+#include <endian.h>
+#include <security/tpm/tis.h>
 #include <string.h>
 #include <types.h>
-#include <delay.h>
-#include <console/console.h>
-#include <device/i2c_simple.h>
-#include <endian.h>
 #include <timer.h>
-#include <security/tpm/tis.h>
 
 #include "tpm.h"
 
@@ -33,8 +34,8 @@
 #define CR50_TIMEOUT_LONG_MS	2000	/* Long timeout while waiting for TPM */
 #define CR50_TIMEOUT_SHORT_MS	2	/* Short timeout during transactions */
 #define CR50_TIMEOUT_NOIRQ_MS	20	/* Timeout for TPM ready without IRQ */
-#define CR50_TIMEOUT_IRQ_MS	100	/* Timeout for TPM ready with IRQ */
 #define CR50_DID_VID		0x00281ae0L
+#define TI50_DID_VID		0x504a6666L
 
 struct tpm_inf_dev {
 	int bus;
@@ -49,7 +50,7 @@ __weak int tis_plat_irq_status(void)
 	static int warning_displayed;
 
 	if (!warning_displayed) {
-		printk(BIOS_WARNING, "WARNING: %s() not implemented, wasting 20ms to wait on"
+		printk(BIOS_WARNING, "%s() not implemented, wasting 20ms to wait on"
 		       " Cr50!\n", __func__);
 		warning_displayed = 1;
 	}
@@ -58,25 +59,9 @@ __weak int tis_plat_irq_status(void)
 	return 1;
 }
 
-/* Wait for interrupt to indicate the TPM is ready */
-static int cr50_i2c_wait_tpm_ready(struct tpm_chip *chip)
-{
-	struct stopwatch sw;
-
-	stopwatch_init_msecs_expire(&sw, CR50_TIMEOUT_IRQ_MS);
-
-	while (!tis_plat_irq_status())
-		if (stopwatch_expired(&sw)) {
-			printk(BIOS_ERR, "Cr50 i2c TPM IRQ timeout!\n");
-			return -1;
-		}
-	return 0;
-}
-
 /*
  * cr50_i2c_read() - read from TPM register
  *
- * @chip: TPM chip information
  * @addr: register address to read from
  * @buffer: provided by caller
  * @len: number of bytes to read
@@ -87,8 +72,7 @@ static int cr50_i2c_wait_tpm_ready(struct tpm_chip *chip)
  *
  * Return -1 on error, 0 on success.
  */
-static int cr50_i2c_read(struct tpm_chip *chip, uint8_t addr,
-			 uint8_t *buffer, size_t len)
+static int cr50_i2c_read(uint8_t addr, uint8_t *buffer, size_t len)
 {
 	if (tpm_dev.addr == 0)
 		return -1;
@@ -103,7 +87,7 @@ static int cr50_i2c_read(struct tpm_chip *chip, uint8_t addr,
 	}
 
 	/* Wait for TPM to be ready with response data */
-	if (cr50_i2c_wait_tpm_ready(chip) < 0)
+	if (cr50_wait_tpm_ready() != CB_SUCCESS)
 		return -1;
 
 	/* Read response data from the TPM */
@@ -118,7 +102,6 @@ static int cr50_i2c_read(struct tpm_chip *chip, uint8_t addr,
 /*
  * cr50_i2c_write() - write to TPM register
  *
- * @chip: TPM chip information
  * @addr: register address to write to
  * @buffer: data to write
  * @len: number of bytes to write
@@ -129,8 +112,7 @@ static int cr50_i2c_read(struct tpm_chip *chip, uint8_t addr,
  *
  * Returns -1 on error, 0 on success.
  */
-static int cr50_i2c_write(struct tpm_chip *chip,
-			  uint8_t addr, uint8_t *buffer, size_t len)
+static int cr50_i2c_write(uint8_t addr, const uint8_t *buffer, size_t len)
 {
 	if (tpm_dev.addr == 0)
 		return -1;
@@ -151,7 +133,7 @@ static int cr50_i2c_write(struct tpm_chip *chip,
 	}
 
 	/* Wait for TPM to be ready */
-	return cr50_i2c_wait_tpm_ready(chip);
+	return cr50_wait_tpm_ready() == CB_SUCCESS ? 0 : -1;
 }
 
 /*
@@ -161,7 +143,7 @@ static int cr50_i2c_write(struct tpm_chip *chip,
  * This function will make sure that the AP does not proceed with boot until
  * TPM finished reset processing.
  */
-static int process_reset(struct tpm_chip *chip)
+static int process_reset(void)
 {
 	struct stopwatch sw;
 	int rv = 0;
@@ -180,7 +162,7 @@ static int process_reset(struct tpm_chip *chip)
 		const uint8_t mask =
 			TPM_ACCESS_VALID | TPM_ACCESS_ACTIVE_LOCALITY;
 
-		rv = cr50_i2c_read(chip, TPM_ACCESS(0),
+		rv = cr50_i2c_read(TPM_ACCESS(0),
 				   &access, sizeof(access));
 		if (rv || ((access & mask) == mask)) {
 			/*
@@ -191,7 +173,7 @@ static int process_reset(struct tpm_chip *chip)
 			continue;
 		}
 
-		printk(BIOS_INFO, "TPM ready after %ld ms\n",
+		printk(BIOS_INFO, "TPM ready after %lld ms\n",
 		       stopwatch_duration_msecs(&sw));
 
 		return 0;
@@ -201,7 +183,7 @@ static int process_reset(struct tpm_chip *chip)
 		printk(BIOS_ERR, "Failed to read TPM\n");
 	else
 		printk(BIOS_ERR,
-			"TPM failed to reset after %ld ms, status: %#x\n",
+			"TPM failed to reset after %lld ms, status: %#x\n",
 			stopwatch_duration_msecs(&sw), access);
 
 	return -1;
@@ -212,12 +194,12 @@ static int process_reset(struct tpm_chip *chip)
  * the RO did not release it), or not yet claimed, if this is verstage or the
  * older RO did release it.
  */
-static int claim_locality(struct tpm_chip *chip)
+static int claim_locality(void)
 {
 	uint8_t access;
 	const uint8_t mask = TPM_ACCESS_VALID | TPM_ACCESS_ACTIVE_LOCALITY;
 
-	if (cr50_i2c_read(chip, TPM_ACCESS(0), &access, sizeof(access)))
+	if (cr50_i2c_read(TPM_ACCESS(0), &access, sizeof(access)))
 		return -1;
 
 	if ((access & mask) == mask) {
@@ -226,11 +208,11 @@ static int claim_locality(struct tpm_chip *chip)
 	}
 
 	access = TPM_ACCESS_REQUEST_USE;
-	if (cr50_i2c_write(chip, TPM_ACCESS(0),
+	if (cr50_i2c_write(TPM_ACCESS(0),
 			   &access, sizeof(access)))
 		return -1;
 
-	if (cr50_i2c_read(chip, TPM_ACCESS(0), &access, sizeof(access)))
+	if (cr50_i2c_read(TPM_ACCESS(0), &access, sizeof(access)))
 		return -1;
 
 	if ((access & mask) != mask) {
@@ -245,7 +227,7 @@ static int claim_locality(struct tpm_chip *chip)
 static uint8_t cr50_i2c_tis_status(struct tpm_chip *chip)
 {
 	uint8_t buf[4];
-	if (cr50_i2c_read(chip, TPM_STS(chip->vendor.locality),
+	if (cr50_i2c_read(TPM_STS(chip->vendor.locality),
 			  buf, sizeof(buf)) < 0) {
 		printk(BIOS_ERR, "%s: Failed to read status\n", __func__);
 		return 0;
@@ -257,7 +239,7 @@ static uint8_t cr50_i2c_tis_status(struct tpm_chip *chip)
 static void cr50_i2c_tis_ready(struct tpm_chip *chip)
 {
 	uint8_t buf[4] = { TPM_STS_COMMAND_READY };
-	cr50_i2c_write(chip, TPM_STS(chip->vendor.locality), buf, sizeof(buf));
+	cr50_i2c_write(TPM_STS(chip->vendor.locality), buf, sizeof(buf));
 	mdelay(CR50_TIMEOUT_SHORT_MS);
 }
 
@@ -272,7 +254,7 @@ static int cr50_i2c_wait_burststs(struct tpm_chip *chip, uint8_t mask,
 	stopwatch_init_msecs_expire(&sw, CR50_TIMEOUT_LONG_MS);
 
 	while (!stopwatch_expired(&sw)) {
-		if (cr50_i2c_read(chip, TPM_STS(chip->vendor.locality),
+		if (cr50_i2c_read(TPM_STS(chip->vendor.locality),
 				  buf, sizeof(buf)) != 0) {
 			mdelay(CR50_TIMEOUT_SHORT_MS);
 			continue;
@@ -310,7 +292,7 @@ static int cr50_i2c_tis_recv(struct tpm_chip *chip, uint8_t *buf,
 	}
 
 	/* Read first chunk of burstcnt bytes */
-	if (cr50_i2c_read(chip, addr, buf, burstcnt) != 0) {
+	if (cr50_i2c_read(addr, buf, burstcnt) != 0) {
 		printk(BIOS_ERR, "%s: Read failed\n", __func__);
 		goto out_err;
 	}
@@ -331,7 +313,7 @@ static int cr50_i2c_tis_recv(struct tpm_chip *chip, uint8_t *buf,
 			goto out_err;
 
 		len = MIN(burstcnt, expected - current);
-		if (cr50_i2c_read(chip, addr, buf + current, len) != 0) {
+		if (cr50_i2c_read(addr, buf + current, len) != 0) {
 			printk(BIOS_ERR, "%s: Read failed\n", __func__);
 			goto out_err;
 		}
@@ -390,7 +372,7 @@ static int cr50_i2c_tis_send(struct tpm_chip *chip, uint8_t *buf, size_t len)
 		/* Use burstcnt - 1 to account for the address byte
 		 * that is inserted by cr50_i2c_write() */
 		limit = MIN(burstcnt - 1, len);
-		if (cr50_i2c_write(chip, TPM_DATA_FIFO(chip->vendor.locality),
+		if (cr50_i2c_write(TPM_DATA_FIFO(chip->vendor.locality),
 				   &buf[sent], limit) != 0) {
 			printk(BIOS_ERR, "%s: Write failed\n", __func__);
 			goto out_err;
@@ -409,7 +391,7 @@ static int cr50_i2c_tis_send(struct tpm_chip *chip, uint8_t *buf, size_t len)
 	}
 
 	/* Start the TPM command */
-	if (cr50_i2c_write(chip, TPM_STS(chip->vendor.locality), tpm_go,
+	if (cr50_i2c_write(TPM_STS(chip->vendor.locality), tpm_go,
 			   sizeof(tpm_go)) < 0) {
 		printk(BIOS_ERR, "%s: Start command failed\n", __func__);
 		goto out_err;
@@ -445,19 +427,20 @@ static int cr50_i2c_probe(struct tpm_chip *chip, uint32_t *did_vid)
 	int retries;
 
 	/*
-	 * 150 ms should be enough to synchronize with the TPM even under the
+	 * 1s should be enough to synchronize with the TPM even under the
 	 * worst nested reset request conditions. In vast majority of cases
-	 * there would be no wait at all.
+	 * there would be no wait at all. If this probe fails, boot likely
+	 * cannot proceed, so an extra long timeout is appropriate.
 	 */
 	printk(BIOS_INFO, "Probing TPM I2C: ");
 
-	for (retries = 15; retries > 0; retries--) {
+	for (retries = 100; retries > 0; retries--) {
 		int rc;
 
-		rc = cr50_i2c_read(chip, TPM_DID_VID(0), (uint8_t *)did_vid, 4);
+		rc = cr50_i2c_read(TPM_DID_VID(0), (uint8_t *)did_vid, 4);
 
 		/* Exit once DID and VID verified */
-		if (!rc && (*did_vid == CR50_DID_VID)) {
+		if (!rc && (*did_vid == CR50_DID_VID || *did_vid == TI50_DID_VID)) {
 			printk(BIOS_INFO, "done! DID_VID 0x%08x\n", *did_vid);
 			return 0;
 		}
@@ -492,15 +475,21 @@ int tpm_vendor_init(struct tpm_chip *chip, unsigned int bus, uint32_t dev_addr)
 		return -1;
 
 	if (ENV_SEPARATE_VERSTAGE || ENV_BOOTBLOCK)
-		if (process_reset(chip))
+		if (process_reset())
 			return -1;
 
-	if (claim_locality(chip))
+	if (claim_locality())
 		return -1;
 
 	printk(BIOS_DEBUG, "cr50 TPM 2.0 (i2c %u:0x%02x id 0x%x)\n",
 	       bus, dev_addr, did_vid >> 16);
 
+	if (tpm_first_access_this_boot()) {
+		/* This is called for the side-effect of printing the version string. */
+		cr50_get_firmware_version(NULL);
+                cr50_set_board_cfg();
+	}
+
 	chip->is_open = 1;
 	return 0;
 }
@@ -508,3 +497,13 @@ int tpm_vendor_init(struct tpm_chip *chip, unsigned int bus, uint32_t dev_addr)
 void tpm_vendor_cleanup(struct tpm_chip *chip)
 {
 }
+
+enum cb_err tis_vendor_write(unsigned int addr, const void *buffer, size_t bytes)
+{
+	return cr50_i2c_write(addr & 0xff, buffer, bytes) ? CB_ERR : CB_SUCCESS;
+}
+
+enum cb_err tis_vendor_read(unsigned int addr, void *buffer, size_t bytes)
+{
+	return cr50_i2c_read(addr & 0xff, buffer, bytes) ? CB_ERR : CB_SUCCESS;
+}
