@@ -22,6 +22,11 @@ const uint8_t fsp_reserved_memory_guid[16] = {
 	0xa6, 0xc4, 0xc7, 0xf5, 0x9e, 0xfd, 0x98, 0x6e,
 };
 
+const uint8_t fsp_nv_storage_guid_2[16] = {
+	0x8f, 0x78, 0x66, 0x48, 0xa8, 0x6b, 0xd8, 0x47,
+	0x83, 0x6, 0xac, 0xf7, 0x7f, 0x55, 0x10, 0x46
+};
+
 const uint8_t fsp_nv_storage_guid[16] = {
 	0x02, 0xcf, 0x1a, 0x72, 0x77, 0x4d, 0x2a, 0x4c,
 	0xb3, 0xdc, 0x27, 0x0b, 0x7b, 0xa9, 0xe4, 0xb0
@@ -104,13 +109,13 @@ static void save_hob_list(int is_recovery)
 	*cbmem_loc = (uintptr_t)hob_list;
 }
 
-ROMSTAGE_CBMEM_INIT_HOOK(save_hob_list);
+CBMEM_CREATION_HOOK(save_hob_list);
 
 const void *fsp_get_hob_list(void)
 {
 	uint32_t *list_loc;
 
-	if (ENV_ROMSTAGE)
+	if (ENV_RAMINIT)
 		return fsp_hob_list_ptr;
 	list_loc = cbmem_find(CBMEM_ID_FSP_RUNTIME);
 	return (list_loc) ? (void *)(uintptr_t)(*list_loc) : NULL;
@@ -129,7 +134,6 @@ struct hob_resource *find_resource_hob_by_guid(const struct hob_header *hob,
 
 	for (; hob->type != HOB_TYPE_END_OF_HOB_LIST;
 		hob = fsp_next_hob(hob)) {
-
 		if (hob->type != HOB_TYPE_RESOURCE_DESCRIPTOR)
 			continue;
 
@@ -193,7 +197,6 @@ const void *fsp_find_extension_hob_by_guid(const uint8_t *guid, size_t *size)
 
 	for (; hob->type != HOB_TYPE_END_OF_HOB_LIST;
 		hob = fsp_next_hob(hob)) {
-
 		if (hob->type != HOB_TYPE_GUID_EXTENSION)
 			continue;
 
@@ -286,7 +289,6 @@ void fsp_display_fvi_version_hob(void)
 {
 	const uint8_t *hob_uuid;
 	const struct hob_header *hob = fsp_get_hob_list();
-	size_t size;
 
 	if (!hob)
 		return;
@@ -300,7 +302,6 @@ void fsp_display_fvi_version_hob(void)
 		hob_uuid = hob_header_to_struct(hob);
 
 		if (fsp_guid_compare(hob_uuid, uuid_fv_info)) {
-			size = hob->length - (HOB_HEADER_LEN + 16);
 			display_fsp_version_info_hob(hob);
 		}
 	}
@@ -308,6 +309,16 @@ void fsp_display_fvi_version_hob(void)
 
 const void *fsp_find_nv_storage_data(size_t *size)
 {
+	if (CONFIG(PLATFORM_USES_FSP2_3)) {
+		const struct fsp_nvs_hob2_data_region_header *hob;
+
+		hob = (const struct fsp_nvs_hob2_data_region_header *)
+		      fsp_find_extension_hob_by_guid(fsp_nv_storage_guid_2, size);
+		if (hob != NULL) {
+			*size = hob->nvs_data_length;
+			return (void *)(uintptr_t)hob->nvs_data_ptr;
+		}
+	}
 	return fsp_find_extension_hob_by_guid(fsp_nv_storage_guid, size);
 }
 
