@@ -187,7 +187,7 @@ static void init(struct device *dev)
 	/*Check first whether there is a valid MAC address available */
 	status = mainboard_get_mac_address(dev, adr_to_set);
 	if (status != CB_SUCCESS) {
-		printk(BIOS_ERR, "I210: No valid MAC address found\n");
+		printk(BIOS_NOTICE, "I210: Mainboard has no address, keep the one in MAC.\n");
 		return;
 	}
 	/* Before we will write a new address, check the existing one */
@@ -203,7 +203,6 @@ static void init(struct device *dev)
 	} else {
 		printk(BIOS_INFO, "I210: MAC address is up to date.\n");
 	}
-	return;
 }
 
 static void enable_bus_master(struct device *dev)
@@ -224,6 +223,6 @@ static const unsigned short i210_device_ids[] = { 0x1537, 0x1538, 0x1533, 0 };
 
 static const struct pci_driver i210_driver __pci_driver = {
 	.ops    = &i210_ops,
-	.vendor = PCI_VENDOR_ID_INTEL,
+	.vendor = PCI_VID_INTEL,
 	.devices = i210_device_ids,
 };
