@@ -192,23 +192,29 @@ int spi_flash_cmd_poll_bit(const struct spi_flash *flash, unsigned long timeout,
 {
 	const struct spi_slave *spi = &flash->spi;
 	int ret;
+	int attempt = 0;
 	u8 status;
-	struct mono_time current, end;
-
-	timer_monotonic_get(&current);
-	end = current;
-	mono_time_add_msecs(&end, timeout);
+	struct stopwatch sw;
 
+	stopwatch_init_msecs_expire(&sw, timeout);
 	do {
+		attempt++;
+
 		ret = do_spi_flash_cmd(spi, &cmd, 1, &status, 1);
-		if (ret)
+		if (ret) {
+			printk(BIOS_WARNING,
+			       "SF: SPI command failed on attempt %d with rc %d\n", attempt,
+			       ret);
 			return -1;
+		}
+
 		if ((status & poll_bit) == 0)
 			return 0;
-		timer_monotonic_get(&current);
-	} while (!mono_time_after(&current, &end));
+	} while (!stopwatch_expired(&sw));
+
+	printk(BIOS_WARNING, "SF: timeout at %lld msec after %d attempts\n",
+	       stopwatch_duration_msecs(&sw), attempt);
 
-	printk(BIOS_DEBUG, "SF: timeout at %ld msec\n",timeout);
 	return -1;
 }
 
@@ -515,6 +521,10 @@ int spi_flash_probe(unsigned int bus, unsigned int cs, struct spi_flash *flash)
 			" CONFIG_ROM_SIZE 0x%x!!\n", flash->size,
 			CONFIG_ROM_SIZE);
 	}
+
+	if (CONFIG(SPI_FLASH_EXIT_4_BYTE_ADDR_MODE) && ENV_INITIAL_STAGE)
+		spi_flash_cmd(&flash->spi, CMD_EXIT_4BYTE_ADDR_MODE, NULL, 0);
+
 	return 0;
 }
 
