@@ -1,24 +1,11 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2006 Eric Biederman (ebiederm@xmission.com)
- * Copyright (C) 2007 AMD
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
-#include <stddef.h>
+#include <stdint.h>
 #include <console/console.h>
 #include <console/usb.h>
 #include <arch/io.h>
-#include <arch/early_variables.h>
+#include <device/mmio.h>
+#include <arch/symbols.h>
 #include <string.h>
 #include <cbmem.h>
 
@@ -27,21 +14,21 @@
 #include "ehci.h"
 
 struct ehci_debug_info {
-	void *ehci_base;
-	void *ehci_debug;
+	u64 ehci_base;
+	u64 ehci_debug;
 
 	struct dbgp_pipe ep_pipe[DBGP_MAX_ENDPOINTS];
-};
+} __packed;
 
-#if IS_ENABLED(CONFIG_DEBUG_USBDEBUG)
-static void dbgp_print_data(struct ehci_dbg_port *ehci_debug);
-static int dbgp_enabled(void);
-# define dprintk(LEVEL, args...) \
-	do { if (!dbgp_enabled()) printk(LEVEL, ##args); } while (0)
-#else
-# define dbgp_print_data(x)	do {} while (0)
-# define dprintk(LEVEL, args...)   do {} while (0)
-#endif
+/* With CONFIG(DEBUG_CONSOLE_INIT), you can debug the connection of
+ * usbdebug dongle. EHCI port register bits and USB packets are dumped
+ * on console, assuming some other console already works.
+ */
+#define dprintk(LEVEL, args...)						\
+	do {								\
+		if (CONFIG(DEBUG_CONSOLE_INIT) && !dbgp_enabled())	\
+			printk(LEVEL, ##args);				\
+	} while (0)
 
 #define DBGP_LEN_UPDATE(x, len) (((x) & ~0x0f) | ((len) & 0x0f))
 
@@ -56,15 +43,27 @@ static int dbgp_enabled(void);
 #define DBGP_MICROFRAME_RETRIES		10
 #define DBGP_MAX_PACKET		8
 
-static struct ehci_debug_info glob_dbg_info CAR_GLOBAL;
-static struct ehci_debug_info * glob_dbg_info_p CAR_GLOBAL;
+static int dbgp_enabled(void);
+static void dbgp_print_data(struct ehci_dbg_port *ehci_debug);
+
+static struct ehci_debug_info glob_dbg_info;
+static struct ehci_debug_info *glob_dbg_info_p;
 
 static inline struct ehci_debug_info *dbgp_ehci_info(void)
 {
-	if (car_get_var(glob_dbg_info_p) == NULL)
-		car_set_var(glob_dbg_info_p, &glob_dbg_info);
-
-	return car_get_var(glob_dbg_info_p);
+	if (glob_dbg_info_p == NULL) {
+		struct ehci_debug_info *info;
+		if (ENV_BOOTBLOCK || ENV_SEPARATE_VERSTAGE || ENV_ROMSTAGE) {
+			/* The message likely does not show if we hit this. */
+			if (sizeof(*info) > _car_ehci_dbg_info_size)
+				die("BUG: Increase ehci_dbg_info reserve in CAR");
+			info = (void *)_car_ehci_dbg_info;
+		} else {
+			info = &glob_dbg_info;
+		}
+		glob_dbg_info_p = info;
+	}
+	return glob_dbg_info_p;
 }
 
 static int dbgp_wait_until_complete(struct ehci_dbg_port *ehci_debug)
@@ -80,7 +79,7 @@ static int dbgp_wait_until_complete(struct ehci_dbg_port *ehci_debug)
 	} while (++loop < DBGP_MICROFRAME_TIMEOUT_LOOPS);
 
 	if (! (ctrl & DBGP_DONE)) {
-		dprintk(BIOS_ERR, "dbgp_wait_until_complete: retry timeout.\n");
+		dprintk(BIOS_ERR, "%s: retry timeout.\n", __func__);
 		return -DBGP_ERR_SIGNAL;
 	}
 
@@ -97,7 +96,7 @@ static void dbgp_breath(void)
 }
 
 static int dbgp_wait_until_done(struct ehci_dbg_port *ehci_debug, struct dbgp_pipe *pipe,
-	unsigned ctrl, const int timeout)
+	unsigned int ctrl, const int timeout)
 {
 	u32 rd_ctrl, rd_pids;
 	u32 ctrl_prev = 0, pids_prev = 0;
@@ -197,13 +196,19 @@ static void dbgp_get_data(struct ehci_dbg_port *ehci_debug, void *buf, int size)
 		bytes[i] = (hi >> (8*(i - 4))) & 0xff;
 }
 
-#if IS_ENABLED(CONFIG_DEBUG_USBDEBUG)
 static void dbgp_print_data(struct ehci_dbg_port *ehci_debug)
 {
-	u32 ctrl = read32(&ehci_debug->control);
-	u32	lo = read32(&ehci_debug->data03);
-	u32	hi = read32(&ehci_debug->data47);
-	int len = DBGP_LEN(ctrl);
+	int len;
+	u32 ctrl, lo, hi;
+
+	if (!CONFIG(DEBUG_CONSOLE_INIT) || dbgp_enabled())
+		return;
+
+	ctrl = read32(&ehci_debug->control);
+	lo = read32(&ehci_debug->data03);
+	hi = read32(&ehci_debug->data47);
+
+	len = DBGP_LEN(ctrl);
 	if (len) {
 		int i;
 		dprintk(BIOS_SPEW, "dbgp:    buf:");
@@ -214,7 +219,6 @@ static void dbgp_print_data(struct ehci_dbg_port *ehci_debug)
 		dprintk(BIOS_SPEW, "\n");
 	}
 }
-#endif
 
 static int dbgp_bulk_write(struct ehci_dbg_port *ehci_debug, struct dbgp_pipe *pipe,
 	const char *bytes, int size)
@@ -244,7 +248,9 @@ static int dbgp_bulk_write(struct ehci_dbg_port *ehci_debug, struct dbgp_pipe *p
 int dbgp_bulk_write_x(struct dbgp_pipe *pipe, const char *bytes, int size)
 {
 	struct ehci_debug_info *dbg_info = dbgp_ehci_info();
-	return dbgp_bulk_write(dbg_info->ehci_debug, pipe, bytes, size);
+	struct ehci_dbg_port *port;
+	port = (void *)(uintptr_t)dbg_info->ehci_debug;
+	return dbgp_bulk_write(port, pipe, bytes, size);
 }
 
 static int dbgp_bulk_read(struct ehci_dbg_port *ehci_debug, struct dbgp_pipe *pipe,
@@ -278,7 +284,9 @@ static int dbgp_bulk_read(struct ehci_dbg_port *ehci_debug, struct dbgp_pipe *pi
 int dbgp_bulk_read_x(struct dbgp_pipe *pipe, void *data, int size)
 {
 	struct ehci_debug_info *dbg_info = dbgp_ehci_info();
-	return dbgp_bulk_read(dbg_info->ehci_debug, pipe, data, size);
+	struct ehci_dbg_port *port;
+	port = (void *)(uintptr_t)dbg_info->ehci_debug;
+	return dbgp_bulk_read(port, pipe, data, size);
 }
 
 void dbgp_mdelay(int ms)
@@ -291,7 +299,7 @@ void dbgp_mdelay(int ms)
 	}
 }
 
-int dbgp_control_msg(struct ehci_dbg_port *ehci_debug, unsigned devnum, int requesttype,
+int dbgp_control_msg(struct ehci_dbg_port *ehci_debug, unsigned int devnum, int requesttype,
 		int request, int value, int index, void *data, int size)
 {
 	struct ehci_debug_info *info = dbgp_ehci_info();
@@ -362,7 +370,7 @@ static int ehci_reset_port(struct ehci_regs *ehci_regs, int port)
 	u32 portsc;
 	int loop;
 
-	/* Reset the usb debug port */
+	/* Reset the USB debug port */
 	portsc = read32(&ehci_regs->port_status[port - 1]);
 	portsc &= ~PORT_PE;
 	portsc |= PORT_RESET;
@@ -412,9 +420,7 @@ static int ehci_wait_for_port(struct ehci_regs *ehci_regs, int port)
 	return -1; //-ENOTCONN;
 }
 
-
-
-static int usbdebug_init_(unsigned ehci_bar, unsigned offset, struct ehci_debug_info *info)
+static int usbdebug_init_(uintptr_t ehci_bar, unsigned int offset, struct ehci_debug_info *info)
 {
 	struct ehci_caps *ehci_caps;
 	struct ehci_regs *ehci_regs;
@@ -427,17 +433,18 @@ static int usbdebug_init_(unsigned ehci_bar, unsigned offset, struct ehci_debug_
 	int playtimes = 3;
 
 	/* Keep all endpoints disabled before any printk() call. */
-	memset(info, 0, sizeof (*info));
-	info->ehci_base = (void *)ehci_bar;
-	info->ehci_debug = (void *)(ehci_bar + offset);
+	memset(info, 0, sizeof(*info));
+	info->ehci_base = ehci_bar;
+	info->ehci_debug = ehci_bar + offset;
+	info->ep_pipe[0].status	|= DBGP_EP_NOT_PRESENT;
 
-	dprintk(BIOS_INFO, "ehci_bar: 0x%x debug_offset 0x%x\n", ehci_bar, offset);
+	dprintk(BIOS_INFO, "ehci_bar: 0x%lx debug_offset 0x%x\n", ehci_bar, offset);
 
 	ehci_caps  = (struct ehci_caps *)ehci_bar;
 	ehci_regs  = (struct ehci_regs *)(ehci_bar +
 			HC_LENGTH(read32(&ehci_caps->hc_capbase)));
 
-	struct ehci_dbg_port *ehci_debug = info->ehci_debug;
+	struct ehci_dbg_port *ehci_debug = (void *)(uintptr_t)info->ehci_debug;
 
 	if (CONFIG_USBDEBUG_DEFAULT_PORT > 0)
 		ehci_debug_select_port(CONFIG_USBDEBUG_DEFAULT_PORT);
@@ -539,7 +546,6 @@ try_next_port:
 	}
 	dprintk(BIOS_INFO, "EHCI done waiting for port.\n");
 
-
 	/* Enable the debug port */
 	ctrl = read32(&ehci_debug->control);
 	ctrl |= DBGP_CLAIM;
@@ -553,22 +559,18 @@ try_next_port:
 	}
 	dprintk(BIOS_INFO, "EHCI debug port enabled.\n");
 
-#if 0
-	/* Completely transfer the debug device to the debug controller */
-	portsc = read32(&ehci_regs->port_status[debug_port - 1]);
-	portsc &= ~PORT_PE;
-	write32(&ehci_regs->port_status[debug_port - 1], portsc);
-#endif
-
 	dbgp_mdelay(100);
 
-	ret = dbgp_probe_gadget(info->ehci_debug, &info->ep_pipe[0]);
+	struct ehci_dbg_port *port = (void *)(uintptr_t)info->ehci_debug;
+	ret = dbgp_probe_gadget(port, &info->ep_pipe[0]);
 	if (ret < 0) {
 		dprintk(BIOS_INFO, "Could not probe gadget on debug port.\n");
 		ret = -6;
 		goto err;
 	}
 
+	info->ep_pipe[0].status	&= ~DBGP_EP_NOT_PRESENT;
+
 	return 0;
 err:
 	/* Things didn't work so remove my claim */
@@ -578,25 +580,23 @@ err:
 	//return ret;
 
 next_debug_port:
-#if CONFIG_USBDEBUG_DEFAULT_PORT == 0
-	port_map_tried |= (1 << (debug_port - 1));
-	new_debug_port = ((debug_port-1 + 1) % n_ports) + 1;
-	if (port_map_tried != ((1 << n_ports) - 1)) {
-		ehci_debug_select_port(new_debug_port);
-		goto try_next_port;
+	if (CONFIG_USBDEBUG_DEFAULT_PORT == 0) {
+		port_map_tried |= (1 << (debug_port - 1));
+		new_debug_port = ((debug_port-1 + 1) % n_ports) + 1;
+		if (port_map_tried != ((1 << n_ports) - 1)) {
+			ehci_debug_select_port(new_debug_port);
+			goto try_next_port;
+		}
+		if (--playtimes) {
+			ehci_debug_select_port(new_debug_port);
+			goto try_next_time;
+		}
+	} else {
+		if (--playtimes)
+			goto try_next_time;
 	}
-	if (--playtimes) {
-		ehci_debug_select_port(new_debug_port);
-		goto try_next_time;
-	}
-#else
-	if (0)
-		goto try_next_port;
-	if (--playtimes)
-		goto try_next_time;
-#endif
 
-	return -10;
+	return ret;
 }
 
 static int dbgp_enabled(void)
@@ -605,6 +605,12 @@ static int dbgp_enabled(void)
 	return (globals->status & DBGP_EP_ENABLED);
 }
 
+static int dbgp_not_present(void)
+{
+	struct dbgp_pipe *globals = &dbgp_ehci_info()->ep_pipe[DBGP_SETUP_EP0];
+	return (globals->status & DBGP_EP_NOT_PRESENT);
+}
+
 int dbgp_try_get(struct dbgp_pipe *pipe)
 {
 	struct dbgp_pipe *globals = &dbgp_ehci_info()->ep_pipe[DBGP_SETUP_EP0];
@@ -623,15 +629,15 @@ void dbgp_put(struct dbgp_pipe *pipe)
 }
 
 #if ENV_RAMSTAGE
-void usbdebug_re_enable(unsigned ehci_base)
+void usbdebug_re_enable(uintptr_t ehci_base)
 {
 	struct ehci_debug_info *dbg_info = dbgp_ehci_info();
-	unsigned diff;
+	u64 diff;
 	int i;
 
-	diff = (unsigned)dbg_info->ehci_base - ehci_base;
+	diff = dbg_info->ehci_base - ehci_base;
 	dbg_info->ehci_debug -= diff;
-	dbg_info->ehci_base = (void*)ehci_base;
+	dbg_info->ehci_base = ehci_base;
 
 	for (i=0; i<DBGP_MAX_ENDPOINTS; i++)
 		if (dbg_info->ep_pipe[i].status & DBGP_EP_VALID)
@@ -648,11 +654,22 @@ void usbdebug_disable(void)
 
 #endif
 
-static int usbdebug_hw_init(void)
+int usbdebug_hw_init(bool force)
 {
 	struct ehci_debug_info *dbg_info = dbgp_ehci_info();
-	unsigned int ehci_base, dbg_offset;
+	u32 ehci_base, dbg_offset;
 
+	if (dbgp_enabled() && !force)
+		return 0;
+
+	if (dbgp_not_present() && !force)
+		return -1;
+
+	/* Do not attempt slow gadget init in postcar. */
+	if (ENV_POSTCAR)
+		return -1;
+
+	/* Do full init if state claims we are still not enabled. */
 	if (ehci_debug_hw_enable(&ehci_base, &dbg_offset))
 		return -1;
 	return usbdebug_init_(ehci_base, dbg_offset, dbg_info);
@@ -663,7 +680,7 @@ static void migrate_ehci_debug(int is_recovery)
 	struct ehci_debug_info *dbg_info_cbmem;
 	int rv;
 
-	if (ENV_ROMSTAGE) {
+	if (ENV_CREATES_CBMEM) {
 		/* Move state from CAR to CBMEM. */
 		struct ehci_debug_info *dbg_info = dbgp_ehci_info();
 		dbg_info_cbmem = cbmem_add(CBMEM_ID_EHCI_DEBUG,
@@ -671,32 +688,25 @@ static void migrate_ehci_debug(int is_recovery)
 		if (dbg_info_cbmem == NULL)
 			return;
 		memcpy(dbg_info_cbmem, dbg_info, sizeof(*dbg_info));
-		car_set_var(glob_dbg_info_p, dbg_info_cbmem);
+		glob_dbg_info_p = dbg_info_cbmem;
 		return;
 	}
 
-	if (IS_ENABLED(CONFIG_USBDEBUG_IN_ROMSTAGE)) {
+	if (CONFIG(USBDEBUG_IN_PRE_RAM)) {
 		/* Use state in CBMEM. */
 		dbg_info_cbmem = cbmem_find(CBMEM_ID_EHCI_DEBUG);
 		if (dbg_info_cbmem)
-			car_set_var(glob_dbg_info_p, dbg_info_cbmem);
+			glob_dbg_info_p = dbg_info_cbmem;
 	}
 
-	/* Redo full init in ramstage if state claims we
-	 * are still not enabled. Should never happen. */
-	rv = dbgp_enabled() ? 0 : -1;
-	if (!ENV_POSTCAR && rv < 0)
-		rv = usbdebug_hw_init();
-
+	rv = usbdebug_hw_init(false);
 	if (rv < 0)
 		printk(BIOS_DEBUG, "usbdebug: Failed hardware init\n");
 	else
 		printk(BIOS_DEBUG, "usbdebug: " ENV_STRING " starting...\n");
 }
 
-ROMSTAGE_CBMEM_INIT_HOOK(migrate_ehci_debug);
-POSTCAR_CBMEM_INIT_HOOK(migrate_ehci_debug);
-RAMSTAGE_CBMEM_INIT_HOOK(migrate_ehci_debug);
+CBMEM_READY_HOOK(migrate_ehci_debug);
 
 int dbgp_ep_is_active(struct dbgp_pipe *pipe)
 {
@@ -716,15 +726,16 @@ struct dbgp_pipe *dbgp_console_input(void)
 void usbdebug_init(void)
 {
 	/* USB console init is done early in romstage, yet delayed to
-	 * CBMEM_INIT_HOOKs for postcar and ramstage as we recover state
+	 * CBMEM_READY_HOOKs for postcar and ramstage as we recover state
 	 * from CBMEM.
 	 */
-	if (IS_ENABLED(CONFIG_USBDEBUG_IN_ROMSTAGE) && ENV_ROMSTAGE)
-		usbdebug_hw_init();
+	if (CONFIG(USBDEBUG_IN_PRE_RAM)
+	    && (ENV_ROMSTAGE || ENV_BOOTBLOCK))
+		usbdebug_hw_init(false);
 
 	/* USB console init is done early in ramstage if it was
 	 * not done in romstage, this does not require CBMEM.
 	 */
-	if (!IS_ENABLED(CONFIG_USBDEBUG_IN_ROMSTAGE) && ENV_RAMSTAGE)
-		usbdebug_hw_init();
+	if (!CONFIG(USBDEBUG_IN_PRE_RAM) && ENV_RAMSTAGE)
+		usbdebug_hw_init(false);
 }
