@@ -10,11 +10,15 @@
 #include <wrdd.h>
 
 #include "chip.h"
+#include "wifi.h"
 #include "wifi_private.h"
 
 /* WIFI Domain type */
 #define DOMAIN_TYPE_WIFI 0x7
 
+/* Maximum number DSM UUID bifurcations in _DSM */
+#define MAX_DSM_FUNCS 2
+
 /*
  * WIFI ACPI NAME = "WF" + hex value of last 8 bits of dev_path_encode + '\0'
  * The above representation returns unique and consistent name every time
@@ -26,19 +30,17 @@
 /* Unique ID for the WIFI _DSM */
 #define ACPI_DSM_OEM_WIFI_UUID    "F21202BF-8F78-4DC6-A5B3-1F738E285ADE"
 
+/* ID for the Wifi DmaProperty _DSD */
+#define ACPI_DSD_DMA_PROPERTY_UUID	  "70D24161-6DD5-4C9E-8070-705531292865"
+
+/* Unique ID for CnviDdrRfim entry in WIFI _DSM */
+#define ACPI_DSM_RFIM_WIFI_UUID   "7266172C-220B-4B29-814F-75E4DD26B5FD"
+
 __weak int get_wifi_sar_limits(union wifi_sar_limits *sar_limits)
 {
 	return -1;
 }
 
-/*
- * Generate ACPI AML code for _DSM method.
- * This function takes as input uuid for the device, set of callbacks and
- * argument to pass into the callbacks. Callbacks should ensure that Local0 and
- * Local1 are left untouched. Use of Local2-Local7 is permitted in callbacks.
- */
-void wifi_emit_dsm(struct dsm_profile *dsm);
-
 /*
  * Function 1: Allow PC OEMs to set ETSI 5.8GHz SRD in Passive/Disabled ESTI SRD
  * Channels: 149, 153, 157, 161, 165
@@ -142,6 +144,12 @@ static void wifi_dsm_unii4_control_enable(void *args)
 	acpigen_write_return_integer(dsm_config->unii_4);
 }
 
+static void wifi_dsm_ddrrfim_func3_cb(void *ptr)
+{
+	const bool is_cnvi_ddr_rfim_enabled = *(bool *)ptr;
+	acpigen_write_return_integer(is_cnvi_ddr_rfim_enabled ? 1 : 0);
+}
+
 static void (*wifi_dsm_callbacks[])(void *) = {
 	NULL,					/* Function 0 */
 	wifi_dsm_srd_active_channels,		/* Function 1 */
@@ -153,20 +161,16 @@ static void (*wifi_dsm_callbacks[])(void *) = {
 	wifi_dsm_unii4_control_enable,		/* Function 7 */
 };
 
-void wifi_emit_dsm(struct dsm_profile *dsm)
-{
-	int i;
-	size_t count = ARRAY_SIZE(wifi_dsm_callbacks);
-
-	if (dsm == NULL)
-		return;
-
-	for (i = 1; i < count; i++)
-		if (!(dsm->supported_functions & (1 << i)))
-			wifi_dsm_callbacks[i] = NULL;
-
-	acpigen_write_dsm(ACPI_DSM_OEM_WIFI_UUID, wifi_dsm_callbacks, count, dsm);
-}
+/*
+ * The current DSM2 table is only exporting one function (function 3), some more
+ * functions are reserved so marking them NULL.
+*/
+static void (*wifi_dsm2_callbacks[])(void *) = {
+	NULL,				/* Function 0 */
+	NULL,				/* Function 1 */
+	NULL,				/* Function 2 */
+	wifi_dsm_ddrrfim_func3_cb,	/* Function 3 */
+};
 
 static const uint8_t *sar_fetch_set(const struct sar_profile *sar, size_t set_num)
 {
@@ -209,7 +213,7 @@ static void sar_emit_wrds(const struct sar_profile *sar)
 	 * })
 	 */
 	if (sar->revision > MAX_SAR_REVISION) {
-		printk(BIOS_ERR, "ERROR: Invalid SAR table revision: %d\n", sar->revision);
+		printk(BIOS_ERR, "Invalid SAR table revision: %d\n", sar->revision);
 		return;
 	}
 
@@ -255,12 +259,12 @@ static void sar_emit_ewrd(const struct sar_profile *sar)
 	 * })
 	 */
 	if (sar->revision > MAX_SAR_REVISION) {
-		printk(BIOS_ERR, "ERROR: Invalid SAR table revision: %d\n", sar->revision);
+		printk(BIOS_ERR, "Invalid SAR table revision: %d\n", sar->revision);
 		return;
 	}
 
 	if (sar->dsar_set_count == 0) {
-		printk(BIOS_WARNING, "WARNING: DSAR set count is 0\n");
+		printk(BIOS_WARNING, "DSAR set count is 0\n");
 		return;
 	}
 
@@ -273,7 +277,7 @@ static void sar_emit_ewrd(const struct sar_profile *sar)
 	 * Emit 'Domain Type' + 'Dynamic SAR Enable' + 'Extended SAR sets count'
 	 * + number of bytes for Set#2 & 3 & 4
 	 */
-	package_size = 1 + 1 + 1 + table_size * sar->dsar_set_count;
+	package_size = 1 + 1 + 1 + table_size * MAX_DSAR_SET_COUNT;
 	acpigen_write_package(package_size);
 	acpigen_write_dword(DOMAIN_TYPE_WIFI);
 	acpigen_write_dword(1);
@@ -338,7 +342,7 @@ static void sar_emit_wgds(struct geo_profile *wgds)
 	 * })
 	 */
 	if (wgds->revision > MAX_GEO_OFFSET_REVISION) {
-		printk(BIOS_ERR, "ERROR: Invalid WGDS revision: %d\n", wgds->revision);
+		printk(BIOS_ERR, "Invalid WGDS revision: %d\n", wgds->revision);
 		return;
 	}
 
@@ -459,20 +463,20 @@ static void sar_emit_wtas(struct avg_profile *wtas)
 	acpigen_write_package_end();
 }
 
-static void emit_sar_acpi_structures(const struct device *dev)
+static void emit_sar_acpi_structures(const struct device *dev, struct dsm_profile *dsm)
 {
-	union wifi_sar_limits sar_limits;
+	union wifi_sar_limits sar_limits = {{NULL, NULL, NULL, NULL, NULL} };
 
 	/*
 	 * If device type is PCI, ensure that the device has Intel vendor ID. CBFS SAR and SAR
 	 * ACPI tables are currently used only by Intel WiFi devices.
 	 */
-	if (dev->path.type == DEVICE_PATH_PCI && dev->vendor != PCI_VENDOR_ID_INTEL)
+	if (dev->path.type == DEVICE_PATH_PCI && dev->vendor != PCI_VID_INTEL)
 		return;
 
 	/* Retrieve the sar limits data */
 	if (get_wifi_sar_limits(&sar_limits) < 0) {
-		printk(BIOS_ERR, "ERROR: failed getting SAR limits!\n");
+		printk(BIOS_ERR, "failed getting SAR limits!\n");
 		return;
 	}
 
@@ -481,7 +485,10 @@ static void emit_sar_acpi_structures(const struct device *dev)
 	sar_emit_wgds(sar_limits.wgds);
 	sar_emit_ppag(sar_limits.ppag);
 	sar_emit_wtas(sar_limits.wtas);
-	wifi_emit_dsm(sar_limits.dsm);
+
+	/* copy the dsm data to be later used for creating _DSM function */
+	if (sar_limits.dsm != NULL)
+		memcpy(dsm, &sar_limits.dsm, sizeof(struct dsm_profile));
 
 	free(sar_limits.sar);
 }
@@ -505,13 +512,27 @@ static void wifi_ssdt_write_properties(const struct device *dev, const char *sco
 {
 	const struct drivers_wifi_generic_config *config = dev->chip_info;
 
+	bool is_cnvi_ddr_rfim_enabled = config && config->enable_cnvi_ddr_rfim;
+
 	/* Scope */
 	acpigen_write_scope(scope);
 
-	/* Wake capabilities */
-	if (config)
+	if (config) {
+		/* Wake capabilities */
 		acpigen_write_PRW(config->wake, ACPI_S3);
 
+		/* Add _DSD for DmaProperty property. */
+		if (config->is_untrusted) {
+			struct acpi_dp *dsd, *pkg;
+
+			dsd = acpi_dp_new_table("_DSD");
+			pkg = acpi_dp_new_table(ACPI_DSD_DMA_PROPERTY_UUID);
+			acpi_dp_add_integer(pkg, "DmaProperty", 1);
+			acpi_dp_add_package(dsd, pkg);
+			acpi_dp_write(dsd);
+		}
+	}
+
 	/* Fill regulatory domain structure */
 	if (CONFIG(HAVE_REGULATORY_DOMAIN)) {
 		/*
@@ -533,9 +554,37 @@ static void wifi_ssdt_write_properties(const struct device *dev, const char *sco
 		acpigen_pop_len();
 	}
 
+	struct dsm_uuid dsm_ids[MAX_DSM_FUNCS];
+	/* We will need a copy dsm data to be used later for creating _DSM function */
+	struct dsm_profile dsm = {0};
+	uint8_t dsm_count = 0;
+
 	/* Fill Wifi sar related ACPI structures */
-	if (CONFIG(USE_SAR))
-		emit_sar_acpi_structures(dev);
+	if (CONFIG(USE_SAR)) {
+		emit_sar_acpi_structures(dev, &dsm);
+
+		if (dsm.supported_functions != 0) {
+			for (int i = 1; i < ARRAY_SIZE(wifi_dsm_callbacks); i++)
+				if (!(dsm.supported_functions & (1 << i)))
+					wifi_dsm_callbacks[i] = NULL;
+
+			dsm_ids[dsm_count].uuid = ACPI_DSM_OEM_WIFI_UUID;
+			dsm_ids[dsm_count].callbacks = &wifi_dsm_callbacks[0];
+			dsm_ids[dsm_count].count = ARRAY_SIZE(wifi_dsm_callbacks);
+			dsm_ids[dsm_count].arg = NULL;
+			dsm_count++;
+		}
+	}
+
+	if (is_cnvi_ddr_rfim_enabled) {
+		dsm_ids[dsm_count].uuid = ACPI_DSM_RFIM_WIFI_UUID;
+		dsm_ids[dsm_count].callbacks = &wifi_dsm2_callbacks[0];
+		dsm_ids[dsm_count].count = ARRAY_SIZE(wifi_dsm2_callbacks);
+		dsm_ids[dsm_count].arg = &is_cnvi_ddr_rfim_enabled;
+		dsm_count++;
+	}
+
+	acpigen_write_dsm_uuid_arr(dsm_ids, dsm_count);
 
 	acpigen_pop_len(); /* Scope */
 
@@ -568,6 +617,8 @@ const char *wifi_pcie_acpi_name(const struct device *dev)
 void wifi_cnvi_fill_ssdt(const struct device *dev)
 {
 	const char *path;
+	if (!dev)
+		return;
 
 	path = acpi_device_path(dev->bus->dev);
 	if (!path)
