@@ -18,7 +18,6 @@ void h8_trackpoint_enable(int on)
 {
 	ec_write(H8_TRACKPOINT_CTRL,
 		 on ? H8_TRACKPOINT_ON : H8_TRACKPOINT_OFF);
-
 }
 
 /* Controls radio-off pin in WLAN MiniPCIe slot.  */
@@ -119,7 +118,6 @@ void h8_disable_event(int event)
 		return;
 
 	ec_clr_bit(0x10 + (event >> 3), event & 7);
-
 }
 
 void h8_usb_always_on_enable(enum usb_always_on on)
@@ -313,7 +311,7 @@ static void h8_enable(struct device *dev)
 	if (conf->has_uwb)
 		h8_uwb_enable(get_uint_option("uwb", 1));
 
-	h8_fn_ctrl_swap(get_uint_option("fn_ctrl_swap", 0));
+	h8_fn_ctrl_swap(get_uint_option("fn_ctrl_swap", CONFIG(H8_FN_CTRL_SWAP)));
 
 	h8_sticky_fn(get_uint_option("sticky_fn", 0));
 
