@@ -1,3 +1,5 @@
+/* SPDX-License-Identifier: GPL-2.0-only */
+
 #ifndef DEVICE_RESOURCE_H
 #define DEVICE_RESOURCE_H
 
@@ -36,8 +38,9 @@
 #define IORESOURCE_FIXED	0x80000000
 
 /* PCI specific resource bits (IORESOURCE_BITS) */
-#define IORESOURCE_PCI64	(1<<0)	/* 64bit long pci resource */
-#define IORESOURCE_PCI_BRIDGE	(1<<1)  /* A bridge pci resource */
+#define IORESOURCE_PCI64		(1<<0)	/* 64bit long pci resource */
+#define IORESOURCE_PCI_BRIDGE		(1<<1)  /* A bridge pci resource */
+#define IORESOURCE_PCIE_RESIZABLE_BAR	(1<<2)  /* A Resizable BAR */
 
 typedef u64 resource_t;
 struct resource {
@@ -89,6 +92,15 @@ static inline void *res2mmio(const struct resource *res, unsigned long offset,
 	return (void *)(uintptr_t)((res->base + offset) & ~mask);
 }
 
+void log_resource(const char *type, const struct device *dev, const struct resource *res,
+		  const char *srcfile, const int line);
+
+#define LOG_RESOURCE(type, dev, res)						\
+	do {									\
+		if (CONFIG(DEBUG_RESOURCES) && (dev) && (res))			\
+			log_resource(type, (dev), (res), __func__, __LINE__);	\
+	} while (0)
+
 /*
  * Pick largest resource on the bus using the given mask and type.
  * Params:
