@@ -268,15 +268,15 @@
 /* Indicates memory layout is determined with arch/x86/car.ld. */
 #define ENV_CACHE_AS_RAM		(ENV_ROMSTAGE_OR_BEFORE && !CONFIG(RESET_VECTOR_IN_RAM))
 /* No .data sections with execute-in-place from ROM.  */
-#define ENV_STAGE_HAS_DATA_SECTION	!ENV_CACHE_AS_RAM
+#define ENV_HAS_DATA_SECTION	!ENV_CACHE_AS_RAM
 #else
 /* Both .data and .bss, sometimes SRAM not DRAM. */
-#define ENV_STAGE_HAS_DATA_SECTION	1
+#define ENV_HAS_DATA_SECTION	1
 #define ENV_CACHE_AS_RAM		0
 #endif
 
 /* Currently rmodules, ramstage and smm have heap. */
-#define ENV_STAGE_HAS_HEAP_SECTION	(ENV_RMODULE || ENV_RAMSTAGE || ENV_SMM)
+#define ENV_HAS_HEAP_SECTION	(ENV_RMODULE || ENV_RAMSTAGE || ENV_SMM)
 
 /* Set USER_SPACE in the makefile for the rare code that runs in userspace */
 #if defined(__USER_SPACE__)
@@ -292,6 +292,28 @@
 #define ENV_INITIAL_STAGE		ENV_BOOTBLOCK
 #endif
 
+#define ENV_CREATES_CBMEM	ENV_ROMSTAGE
+#define ENV_HAS_CBMEM		(ENV_ROMSTAGE | ENV_POSTCAR | ENV_RAMSTAGE)
+#define ENV_RAMINIT		ENV_ROMSTAGE
+
+#if ENV_X86
+#define ENV_HAS_SPINLOCKS		!ENV_ROMSTAGE_OR_BEFORE
+#elif ENV_RISCV
+#define ENV_HAS_SPINLOCKS		1
+#else
+#define ENV_HAS_SPINLOCKS		0
+#endif
+
+/* When set <arch/smp/spinlock.h> is included for the spinlock implementation. */
+#define ENV_SUPPORTS_SMP		(CONFIG(SMP) && ENV_HAS_SPINLOCKS)
+
+#if ENV_X86 && CONFIG(COOP_MULTITASKING) && (ENV_RAMSTAGE || ENV_ROMSTAGE)
+/* TODO: Enable in all x86 stages */
+#define ENV_SUPPORTS_COOP         1
+#else
+#define ENV_SUPPORTS_COOP         0
+#endif
+
 /**
  * For pre-DRAM stages and post-CAR always build with simple device model, ie.
  * PCI, PNP and CPU functions operate without use of devicetree. The reason
