@@ -34,6 +34,22 @@ int smbios_write_type38(unsigned long *current, int *handle,
 int smbios_write_type41(unsigned long *current, int *handle,
 			const char *name, u8 instance, u16 segment,
 			u8 bus, u8 device, u8 function, u8 device_type);
+enum smbios_temp_location;
+enum smbios_temp_status;
+int smbios_write_type28(unsigned long *current, int *handle,
+			const char *name,
+			const enum smbios_temp_location location,
+			const enum smbios_temp_status status,
+			u16 max_value, u16 min_value,
+			u16 resolution, u16 tolerance,
+			u16 accuracy,
+			u32 oem,
+			u16 nominal_value);
+
+int smbios_write_type43(unsigned long *current, int *handle, const u32 vendor_id,
+			const u8 major_spec_ver, const u8 minor_spec_ver,
+			const u32 fw_ver1, const u32 fw_ver2, const char *description,
+			const u64 characteristics, const u32 oem_defined);
 
 struct device;
 int get_smbios_data(struct device *dev, int *handle, unsigned long *current);
@@ -250,9 +266,11 @@ typedef enum {
 	SMBIOS_MEMORY_DEVICE = 17,
 	SMBIOS_MEMORY_ARRAY_MAPPED_ADDRESS = 19,
 	SMBIOS_MEMORY_DEVICE_MAPPED_ADDRESS = 20,
+	SMBIOS_TEMPERATURE_PROBE = 28,
 	SMBIOS_SYSTEM_BOOT_INFORMATION = 32,
 	SMBIOS_IPMI_DEVICE_INFORMATION = 38,
 	SMBIOS_ONBOARD_DEVICES_EXTENDED_INFORMATION = 41,
+	SMBIOS_TPM_DEVICE = 43,
 	SMBIOS_END_OF_TABLE = 127,
 } smbios_struct_type_t;
 
@@ -730,7 +748,13 @@ enum misc_slot_type {
 	SlotTypePciExpressGen4x2 = 0xBA,
 	SlotTypePciExpressGen4x4 = 0xBB,
 	SlotTypePciExpressGen4x8 = 0xBC,
-	SlotTypePciExpressGen4x16 = 0xBD
+	SlotTypePciExpressGen4x16 = 0xBD,
+	SlotTypePciExpressGen5 = 0xBE,
+	SlotTypePciExpressGen5x1 = 0xBF,
+	SlotTypePciExpressGen5x2 = 0xC0,
+	SlotTypePciExpressGen5x4 = 0xC1,
+	SlotTypePciExpressGen5x8 = 0xC2,
+	SlotTypePciExpressGen5x16 = 0xC3
 };
 
 /* System Slots - Slot Data Bus Width. */
@@ -909,6 +933,46 @@ struct smbios_type20 {
 	u8 eos[2];
 } __packed;
 
+/* Bit[7..5] = Temp status */
+enum smbios_temp_status {
+	SMBIOS_TEMP_STATUS_OTHER = 0x01,
+	SMBIOS_TEMP_STATUS_UNKNOWN,
+	SMBIOS_TEMP_STATUS_OK,
+	SMBIOS_TEMP_STATUS_NONCRITICAL,
+	SMBIOS_TEMP_STATUS_CRITICAL,
+	SMBIOS_TEMP_STATUS_NONREC,			// Non-Recoverable.
+};
+
+/* Bit[4..0] = Temp location */
+enum smbios_temp_location {
+	SMBIOS_TEMP_LOCATION_OTHER = 0x01,
+	SMBIOS_TEMP_LOCATION_UNKNOWN,
+	SMBIOS_TEMP_LOCATION_PROCESSOR,
+	SMBIOS_TEMP_LOCATION_DISK,
+	SMBIOS_TEMP_LOCATION_BAY,			// Peripheral Bay.
+	SMBIOS_TEMP_LOCATION_SMM,			// System Management Module.
+	SMBIOS_TEMP_LOCATION_BOARD,			// Motherboard.
+	SMBIOS_TEMP_LOCATION_MM,			// Memory.
+	SMBIOS_TEMP_LOCATION_PM,			// Processor Module.
+	SMBIOS_TEMP_LOCATION_POW,			// Power Unit.
+	SMBIOS_TEMP_LOCATION_ADDCARD,
+};
+
+struct smbios_type28 {
+	struct smbios_header header;
+	u8 description;
+	u8 location_and_status;
+	u16 maximum_value;
+	u16 minimum_value;
+	u16 resolution;
+	u16 tolerance;
+	u16 accuracy;
+	u32 oem_defined;
+	u16 nominal_value;
+	u8 eos[2];
+} __packed;
+
+
 struct smbios_type32 {
 	struct smbios_header header;
 	u8 reserved[6];
@@ -947,6 +1011,12 @@ typedef enum {
 	SMBIOS_DEVICE_TYPE_PATA,
 	SMBIOS_DEVICE_TYPE_SATA,
 	SMBIOS_DEVICE_TYPE_SAS,
+	SMBIOS_DEVICE_TYPE_WIRELESS_LAN,
+	SMBIOS_DEVICE_TYPE_BLUETOOTH,
+	SMBIOS_DEVICE_TYPE_WWAN,
+	SMBIOS_DEVICE_TYPE_EMMC,
+	SMBIOS_DEVICE_TYPE_NVME,
+	SMBIOS_DEVICE_TYPE_UFS,
 } smbios_onboard_device_type;
 
 #define SMBIOS_DEVICE_TYPE_COUNT 10
@@ -964,6 +1034,25 @@ struct smbios_type41 {
 	u8 eos[2];
 } __packed;
 
+
+#define SMBIOS_TPM_DEVICE_CHARACTERISTICS_NOT_SUPPORTED (1ULL << 2)
+#define SMBIOS_TPM_DEVICE_FAMILY_CONFIGURABLE_VIA_FW_UPD (1ULL << 3)
+#define SMBIOS_TPM_DEVICE_FAMILY_CONFIGURABLE_VIA_PLATFORM_SW_SUPPORT (1ULL << 4)
+#define SMBIOS_TPM_DEVICE_FAMILY_CONFIGURABLE_VIA_OEM_PROPRIETARY (1ULL << 5)
+
+struct smbios_type43 {
+	struct smbios_header header;
+	u32 vendor_id;
+	u8 major_spec_ver;
+	u8 minor_spec_ver;
+	u32 fw_ver1;
+	u32 fw_ver2;
+	u8 description;
+	u64 characteristics;
+	u32 oem_defined;
+	u8 eos[2];
+} __packed;
+
 struct smbios_type127 {
 	struct smbios_header header;
 	u8 eos[2];
