@@ -1,3 +1,5 @@
+/* SPDX-License-Identifier: GPL-2.0-only */
+
 #ifndef STDDEF_H
 #define STDDEF_H
 
@@ -32,7 +34,7 @@ typedef __WINT_TYPE__ wint_t;
 #define DEVTREE_CONST
 #endif
 
-#if ENV_STAGE_HAS_DATA_SECTION
+#if ENV_HAS_DATA_SECTION
 #define MAYBE_STATIC_NONZERO static
 #else
 #define MAYBE_STATIC_NONZERO
