@@ -52,7 +52,6 @@ DECLARE_REGION(asan_shadow)
 
 /* Regions for execution units. */
 
-DECLARE_REGION(payload_preload_cache)
 DECLARE_REGION(payload)
 /* "program" always refers to the current execution unit. */
 DECLARE_REGION(program)
@@ -77,6 +76,7 @@ DECLARE_REGION(pdpt)
 DECLARE_OPTIONAL_REGION(opensbi)
 DECLARE_OPTIONAL_REGION(bl31)
 DECLARE_REGION(transfer_buffer)
+DECLARE_OPTIONAL_REGION(watchdog_tombstone)
 
 /* Returns true when pre-RAM symbols are known to the linker.
  * (Does not necessarily mean that the memory is accessible.) */
