@@ -1,9 +1,11 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <console/cbmem_console.h>
+#include <console/console.h>
 #include <console/uart.h>
 #include <cbmem.h>
 #include <symbols.h>
+#include <types.h>
 
 /*
  * Structure describing console buffer. It is overlaid on a flat memory area,
@@ -37,6 +39,8 @@ _Static_assert(CONFIG_CONSOLE_CBMEM_BUFFER_SIZE <= MAX_SIZE,
 
 static struct cbmem_console *current_console;
 
+static bool console_paused;
+
 /*
  * While running from ROM, before DRAM is initialized, some area in cache as
  * RAM space is used for the console buffer storage. The size and location of
@@ -80,6 +84,13 @@ void cbmemc_init(void)
 	if (ENV_ROMSTAGE_OR_BEFORE) {
 		/* Pre-RAM environments use special buffer placed by linker script. */
 		init_console_ptr(_preram_cbmem_console, REGION_SIZE(preram_cbmem_console));
+	} else if (ENV_SMM) {
+		void *cbmemc = NULL;
+		size_t cbmemc_size = 0;
+
+		smm_get_cbmemc_buffer(&cbmemc, &cbmemc_size);
+
+		init_console_ptr(cbmemc, cbmemc_size);
 	} else {
 		/* Post-RAM uses static (BSS) buffer before CBMEM is reinitialized. */
 		init_console_ptr(static_console, sizeof(static_console));
@@ -88,7 +99,7 @@ void cbmemc_init(void)
 
 void cbmemc_tx_byte(unsigned char data)
 {
-	if (!current_console || !current_console->size)
+	if (!current_console || !current_console->size || console_paused)
 		return;
 
 	u32 flags = current_console->cursor & ~CURSOR_MASK;
@@ -135,6 +146,16 @@ static void copy_console_buffer(struct cbmem_console *src_cons_p)
 	src_cons_p->size = 0;
 }
 
+void cbmemc_copy_in(void *buffer, size_t size)
+{
+	struct cbmem_console *previous = (void *)buffer;
+
+	if (!buffer_valid(previous, size))
+		return;
+
+	copy_console_buffer(previous);
+}
+
 static void cbmemc_reinit(int is_recovery)
 {
 	const size_t size = CONFIG_CONSOLE_CBMEM_BUFFER_SIZE;
@@ -146,26 +167,59 @@ static void cbmemc_reinit(int is_recovery)
 	copy_console_buffer(previous_cons_p);
 }
 
-/* Run the romstage hook early so that the console region is one of the earliest created, and
+/* Run this hook early so that the console region is one of the earliest created, and
    therefore more likely to stay in the same place even across different boot modes where some
    other regions may sometimes not get created (e.g. RW_MCACHE in vboot recovery mode). */
-ROMSTAGE_CBMEM_INIT_HOOK_EARLY(cbmemc_reinit)
-RAMSTAGE_CBMEM_INIT_HOOK(cbmemc_reinit)
-POSTCAR_CBMEM_INIT_HOOK(cbmemc_reinit)
+CBMEM_READY_HOOK_EARLY(cbmemc_reinit);
 
 #if CONFIG(CONSOLE_CBMEM_DUMP_TO_UART)
+void cbmem_dump_console_to_uart(void)
+{
+	u32 cursor;
+	unsigned int console_index;
+
+	if (!current_console)
+		return;
+
+	console_index = get_uart_for_console();
+
+	uart_init(console_index);
+	if (current_console->cursor & OVERFLOW) {
+		for (cursor = current_console->cursor & CURSOR_MASK;
+		     cursor < current_console->size; cursor++) {
+			if (BIOS_LOG_IS_MARKER(current_console->body[cursor]))
+				continue;
+			if (current_console->body[cursor] == '\n')
+				uart_tx_byte(console_index, '\r');
+			uart_tx_byte(console_index, current_console->body[cursor]);
+		}
+	}
+	for (cursor = 0; cursor < (current_console->cursor & CURSOR_MASK); cursor++) {
+		if (BIOS_LOG_IS_MARKER(current_console->body[cursor]))
+			continue;
+		if (current_console->body[cursor] == '\n')
+			uart_tx_byte(console_index, '\r');
+		uart_tx_byte(console_index, current_console->body[cursor]);
+	}
+}
+#endif
+
 void cbmem_dump_console(void)
 {
 	u32 cursor;
 	if (!current_console)
 		return;
 
-	uart_init(0);
+	console_paused = true;
+
 	if (current_console->cursor & OVERFLOW)
 		for (cursor = current_console->cursor & CURSOR_MASK;
 		     cursor < current_console->size; cursor++)
-			uart_tx_byte(0, current_console->body[cursor]);
+			if (!BIOS_LOG_IS_MARKER(current_console->body[cursor]))
+				do_putchar(current_console->body[cursor]);
 	for (cursor = 0; cursor < (current_console->cursor & CURSOR_MASK); cursor++)
-		uart_tx_byte(0, current_console->body[cursor]);
+		if (!BIOS_LOG_IS_MARKER(current_console->body[cursor]))
+			do_putchar(current_console->body[cursor]);
+
+	console_paused = false;
 }
-#endif
