@@ -80,7 +80,7 @@ static void setup_preram_cache(struct region_device *cache_rdev)
 		if (!verify_fmap(fmap))
 			goto register_cache;
 
-		printk(BIOS_ERR, "ERROR: FMAP cache corrupted?!\n");
+		printk(BIOS_ERR, "FMAP cache corrupted?!\n");
 		if (CONFIG(TOCTOU_SAFETY))
 			die("TOCTOU safety relies on FMAP cache");
 	}
@@ -273,7 +273,7 @@ ssize_t fmap_overwrite_area(const char *name, const void *buffer, size_t size)
 	return rdev_writeat(&rdev, buffer, 0, size);
 }
 
-static void fmap_register_cbmem_cache(int unused)
+static void fmap_register_cbmem_cache(void)
 {
 	const struct cbmem_entry *e;
 
@@ -290,7 +290,7 @@ static void fmap_register_cbmem_cache(int unused)
  * The main reason to copy the FMAP into CBMEM is to make it available to the
  * OS on every architecture. As side effect use the CBMEM copy as cache.
  */
-static void fmap_setup_cbmem_cache(int unused)
+static void fmap_add_cbmem_cache(void)
 {
 	struct region_device fmrd;
 
@@ -301,21 +301,25 @@ static void fmap_setup_cbmem_cache(int unused)
 	const size_t s = region_device_sz(&fmrd);
 	struct fmap *fmap = cbmem_add(CBMEM_ID_FMAP, s);
 	if (!fmap) {
-		printk(BIOS_ERR, "ERROR: Failed to allocate CBMEM\n");
+		printk(BIOS_ERR, "Failed to allocate CBMEM\n");
 		return;
 	}
 
 	const ssize_t ret = rdev_readat(&fmrd, fmap, 0, s);
 	if (ret != s) {
-		printk(BIOS_ERR, "ERROR: Failed to read FMAP into CBMEM\n");
+		printk(BIOS_ERR, "Failed to read FMAP into CBMEM\n");
 		cbmem_entry_remove(cbmem_entry_find(CBMEM_ID_FMAP));
 		return;
 	}
-
-	/* Finally advertise the cache for the current stage */
-	fmap_register_cbmem_cache(unused);
 }
 
-ROMSTAGE_CBMEM_INIT_HOOK(fmap_setup_cbmem_cache)
-RAMSTAGE_CBMEM_INIT_HOOK(fmap_register_cbmem_cache)
-POSTCAR_CBMEM_INIT_HOOK(fmap_register_cbmem_cache)
+static void fmap_setup_cbmem_cache(int unused)
+{
+	if (ENV_CREATES_CBMEM)
+		fmap_add_cbmem_cache();
+
+	/* Finally advertise the cache for the current stage */
+	fmap_register_cbmem_cache();
+}
+
+CBMEM_READY_HOOK(fmap_setup_cbmem_cache);
