@@ -19,12 +19,10 @@
 #include <device/device.h>
 #include <device/pci.h>
 #include <program_loading.h>
-#include <stdlib.h>
 #include <thread.h>
 #include <timer.h>
 #include <timestamp.h>
 #include <types.h>
-#include <vendorcode/google/chromeos/gnvs.h>
 #include <version.h>
 
 static boot_state_t bs_pre_device(void *arg);
@@ -285,11 +283,14 @@ static void bs_call_callbacks(struct boot_state *state,
 			bscb->callback(bscb->arg);
 			if (CONFIG(DEBUG_BOOT_STATE)) {
 				timer_monotonic_get(&mt_stop);
-				printk(BIOS_DEBUG, "BS: callback (%p) @ %s (%ld ms).\n", bscb,
+				printk(BIOS_DEBUG, "BS: callback (%p) @ %s (%lld ms).\n", bscb,
 				       bscb_location(bscb),
 				       mono_time_diff_microseconds(&mt_start, &mt_stop)
 					       / USECS_PER_MSEC);
 			}
+
+			bs_run_timers(0);
+
 			continue;
 		}
 
@@ -355,6 +356,8 @@ static void bs_walk_state_machine(void)
 
 		bs_sample_time(state);
 
+		bs_run_timers(0);
+
 		bs_call_callbacks(state, current_phase.seq);
 
 		if (CONFIG(DEBUG_BOOT_STATE))
@@ -452,19 +455,12 @@ void main(void)
 	 */
 	cbmem_initialize();
 
-	timestamp_add_now(TS_START_RAMSTAGE);
+	timestamp_add_now(TS_RAMSTAGE_START);
 	post_code(POST_ENTRY_HARDWAREMAIN);
 
 	/* Handoff sleep type from romstage. */
 	acpi_is_wakeup_s3();
 
-	/* Initialise GNVS early. */
-	if (CONFIG(ACPI_SOC_NVS))
-		acpi_create_gnvs();
-
-	if (CONFIG(CHROMEOS_NVS))
-		chromeos_init_chromeos_acpi();
-
 	/* Schedule the static boot state entries. */
 	boot_state_schedule_static_entries();
 
