@@ -1,7 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
-
-#include <stdlib.h>
 #include <cbfs.h>
 #include <cbmem.h>
 #include <console/console.h>
@@ -11,11 +9,10 @@
 #include <program_loading.h>
 #include <reset.h>
 #include <rmodule.h>
+#include <security/vboot/vboot_common.h>
 #include <stage_cache.h>
 #include <symbols.h>
-#include <thread.h>
 #include <timestamp.h>
-#include <security/vboot/vboot_common.h>
 
 void run_romstage(void)
 {
@@ -24,17 +21,17 @@ void run_romstage(void)
 
 	vboot_run_logic();
 
-	timestamp_add_now(TS_START_COPYROM);
+	timestamp_add_now(TS_COPYROM_START);
 
 	if (ENV_X86 && CONFIG(BOOTBLOCK_NORMAL)) {
-		if (legacy_romstage_select_and_load(&romstage))
+		if (legacy_romstage_select_and_load(&romstage) != CB_SUCCESS)
 			goto fail;
 	} else {
 		if (cbfs_prog_stage_load(&romstage))
 			goto fail;
 	}
 
-	timestamp_add_now(TS_END_COPYROM);
+	timestamp_add_now(TS_COPYROM_END);
 
 	console_time_report();
 
@@ -75,18 +72,25 @@ static int load_relocatable_ramstage(struct prog *ramstage)
 
 	return rmodule_stage_load(&rmod_ram);
 }
+void preload_ramstage(void)
+{
+	if (!CONFIG(CBFS_PRELOAD))
+		return;
 
-void run_ramstage(void)
+	printk(BIOS_DEBUG, "Preloading ramstage\n");
+
+	cbfs_preload(CONFIG_CBFS_PREFIX "/ramstage");
+}
+void __noreturn run_ramstage(void)
 {
 	struct prog ramstage =
 		PROG_INIT(PROG_RAMSTAGE, CONFIG_CBFS_PREFIX "/ramstage");
 
-	if (ENV_POSTCAR)
-		timestamp_add_now(TS_END_POSTCAR);
-
 	/* Call "end of romstage" here if postcar stage doesn't exist */
-	if (ENV_ROMSTAGE)
-		timestamp_add_now(TS_END_ROMSTAGE);
+	if (ENV_POSTCAR)
+		timestamp_add_now(TS_POSTCAR_END);
+	else
+		timestamp_add_now(TS_ROMSTAGE_END);
 
 	/*
 	 * Only x86 systems using ramstage stage cache currently take the same
@@ -97,7 +101,7 @@ void run_ramstage(void)
 
 	vboot_run_logic();
 
-	timestamp_add_now(TS_START_COPYRAM);
+	timestamp_add_now(TS_COPYRAM_START);
 
 	if (ENV_X86) {
 		if (load_relocatable_ramstage(&ramstage))
@@ -109,7 +113,7 @@ void run_ramstage(void)
 
 	stage_cache_add(STAGE_RAMSTAGE, &ramstage);
 
-	timestamp_add_now(TS_END_COPYRAM);
+	timestamp_add_now(TS_COPYRAM_END);
 
 	console_time_report();
 
@@ -127,81 +131,47 @@ fail:
 static struct prog global_payload =
 	PROG_INIT(PROG_PAYLOAD, CONFIG_CBFS_PREFIX "/payload");
 
-static struct thread_handle payload_preload_handle;
-
-static enum cb_err payload_preload_thread_entry(void *arg)
-{
-	size_t size;
-	struct prog *payload = &global_payload;
-
-	printk(BIOS_DEBUG, "Preloading payload\n");
-
-	payload->cbfs_type = CBFS_TYPE_QUERY;
-
-	size = cbfs_type_load(prog_name(payload), _payload_preload_cache,
-			      REGION_SIZE(payload_preload_cache), &payload->cbfs_type);
-
-	if (!size) {
-		printk(BIOS_ERR, "ERROR: Preloading payload failed\n");
-		return CB_ERR;
-	}
-
-	printk(BIOS_DEBUG, "Preloading payload complete\n");
-
-	return CB_SUCCESS;
-}
-
 void payload_preload(void)
 {
-	struct thread_handle *handle = &payload_preload_handle;
-
-	if (!CONFIG(PAYLOAD_PRELOAD))
+	if (!CONFIG(CBFS_PRELOAD))
 		return;
 
-	if (thread_run(handle, payload_preload_thread_entry, NULL))
-		printk(BIOS_ERR, "ERROR: Failed to start payload preload thread\n");
+	cbfs_preload(global_payload.name);
 }
 
 void payload_load(void)
 {
 	struct prog *payload = &global_payload;
-	struct thread_handle *handle = &payload_preload_handle;
-	void *mapping = NULL;
-	void *buffer;
+	void *mapping;
 
 	timestamp_add_now(TS_LOAD_PAYLOAD);
 
 	if (prog_locate_hook(payload))
 		goto out;
 
-	if (CONFIG(PAYLOAD_PRELOAD) && thread_join(handle) == CB_SUCCESS) {
-		buffer = _payload_preload_cache;
-	} else {
-		payload->cbfs_type = CBFS_TYPE_QUERY;
-		mapping = cbfs_type_map(prog_name(payload), NULL, &payload->cbfs_type);
-		buffer = mapping;
-	}
+	payload->cbfs_type = CBFS_TYPE_QUERY;
+	mapping = cbfs_type_map(prog_name(payload), NULL, &payload->cbfs_type);
 
-	if (!buffer)
+	if (!mapping)
 		goto out;
 
 	switch (prog_cbfs_type(payload)) {
 	case CBFS_TYPE_SELF: /* Simple ELF */
-		selfload_mapped(payload, buffer, BM_MEM_RAM);
+		selfload_mapped(payload, mapping, BM_MEM_RAM);
 		break;
-	case CBFS_TYPE_FIT: /* Flattened image tree */
+	case CBFS_TYPE_FIT_PAYLOAD: /* Flattened image tree */
 		if (CONFIG(PAYLOAD_FIT_SUPPORT)) {
-			fit_payload(payload, buffer);
+			fit_payload(payload, mapping);
 			break;
-		} /* else fall-through */
+		}
+		__fallthrough;
 	default:
 		die_with_post_code(POST_INVALID_ROM,
 				   "Unsupported payload type %d.\n", payload->cbfs_type);
 		break;
 	}
 
-	if (mapping)
-		cbfs_unmap(mapping);
+	cbfs_unmap(mapping);
 out:
 	if (prog_entry(payload) == NULL)
 		die_with_post_code(POST_INVALID_ROM, "Payload not loaded.\n");
