@@ -1,15 +1,12 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <assert.h>
-#include <stddef.h>
-#include <stdint.h>
-#include <stdlib.h>
 #include <bootstate.h>
-#include <commonlib/bsd/compiler.h>
 #include <console/console.h>
 #include <smp/node.h>
 #include <thread.h>
 #include <timer.h>
+#include <types.h>
 
 static u8 thread_stacks[CONFIG_STACK_SIZE * CONFIG_NUM_THREADS] __aligned(sizeof(uint64_t));
 static bool initialized;
@@ -283,15 +280,14 @@ int thread_run(struct thread_handle *handle, enum cb_err (*func)(void *), void *
 	current = current_thread();
 
 	if (!thread_can_yield(current)) {
-		printk(BIOS_ERR,
-		       "ERROR: %s() called from non-yielding context!\n", __func__);
+		printk(BIOS_ERR, "%s() called from non-yielding context!\n", __func__);
 		return -1;
 	}
 
 	t = get_free_thread();
 
 	if (t == NULL) {
-		printk(BIOS_ERR, "ERROR: %s: No more threads!\n", __func__);
+		printk(BIOS_ERR, "%s: No more threads!\n", __func__);
 		return -1;
 	}
 
@@ -318,15 +314,14 @@ int thread_run_until(struct thread_handle *handle, enum cb_err (*func)(void *),
 	current = current_thread();
 
 	if (!thread_can_yield(current)) {
-		printk(BIOS_ERR,
-		       "ERROR: %s() called from non-yielding context!\n", __func__);
+		printk(BIOS_ERR, "%s() called from non-yielding context!\n", __func__);
 		return -1;
 	}
 
 	t = get_free_thread();
 
 	if (t == NULL) {
-		printk(BIOS_ERR, "ERROR: %s: No more threads!\n", __func__);
+		printk(BIOS_ERR, "%s: No more threads!\n", __func__);
 		return -1;
 	}
 
@@ -398,14 +393,14 @@ enum cb_err thread_join(struct thread_handle *handle)
 	if (handle->state == THREAD_UNINITIALIZED)
 		return CB_ERR_ARG;
 
-	stopwatch_init(&sw);
-
 	printk(BIOS_SPEW, "waiting for thread\n");
 
+	stopwatch_init(&sw);
+
 	while (handle->state != THREAD_DONE)
 		assert(thread_yield() == 0);
 
-	printk(BIOS_SPEW, "took %lu us\n", stopwatch_duration_usecs(&sw));
+	printk(BIOS_SPEW, "took %lld us\n", stopwatch_duration_usecs(&sw));
 
 	return handle->error;
 }
@@ -420,7 +415,7 @@ void thread_mutex_lock(struct thread_mutex *mutex)
 		assert(thread_yield() == 0);
 	mutex->locked = true;
 
-	printk(BIOS_SPEW, "took %lu us to acquire mutex\n", stopwatch_duration_usecs(&sw));
+	printk(BIOS_SPEW, "took %lld us to acquire mutex\n", stopwatch_duration_usecs(&sw));
 }
 
 void thread_mutex_unlock(struct thread_mutex *mutex)
