@@ -7,12 +7,11 @@
 void bootblock_mainboard_early_init(void)
 {
 	int i;
-	u32 val;
 
 	/* Disable PCI-PCI bridge and release GPIO32/33 for other uses. */
 	pm_write8(0xea, 0x1);
 
 	/* On Larne, after LpcClkDrvSth is set, it needs some time to be stable, because of the buffer ICS551M */
 	for (i = 0; i < 200000; i++)
-		val = inb(0xcd6);
+		inb(0xcd6);
 }
