@@ -53,7 +53,7 @@
 #define BLDCFG_FCH_GPP_PORT1_PRESENT		TRUE
 #define BLDCFG_FCH_GPP_PORT2_PRESENT		CONFIG(BOARD_ASUS_F2A85_M_PRO)
 
-GPIO_CONTROL f2a85_m_gpio[] = {
+CONST GPIO_CONTROL f2a85_m_gpio[] = {
 	{-1}
 };
 #define BLDCFG_FCH_GPIO_CONTROL_LIST		(f2a85_m_gpio)
