@@ -8,10 +8,8 @@
 #include <device/pci_def.h>
 #include <device/pci_ops.h>
 #include <arch/ioapic.h>
-#include <stdint.h>
 #include <device/device.h>
 #include <stdlib.h>
-#include <string.h>
 #include <smbios.h>
 #include <types.h>
 #include "memory.h"
@@ -49,7 +47,7 @@ static void cpu_pci_domain_read_resources(struct device *dev)
 	int i440fx = (nbid == 0x1237);
 	int q35    = (nbid == 0x29c0);
 	struct resource *res;
-	unsigned long tomk = 0, high;
+	uint64_t tomk = 0;
 	int idx = 10;
 	FWCfgFile f;
 
@@ -67,12 +65,10 @@ static void cpu_pci_domain_read_resources(struct device *dev)
 				       list[i].address, list[i].length);
 				if (list[i].address == 0) {
 					tomk = list[i].length / 1024;
-					ram_resource(dev, idx++, 0, 640);
-					ram_resource(dev, idx++, 768, tomk - 768);
+					ram_from_to(dev, idx++, 0, 0xa0000);
+					ram_from_to(dev, idx++, 0xc0000, tomk * KiB);
 				} else {
-					ram_resource(dev, idx++,
-						     list[i].address / 1024,
-						     list[i].length / 1024);
+					ram_range(dev, idx++, list[i].address, list[i].length);
 				}
 				break;
 			case 2: /* reserved */
@@ -96,15 +92,16 @@ static void cpu_pci_domain_read_resources(struct device *dev)
 	if (!tomk) {
 		/* qemu older than 1.7, or reading etc/e820 failed. Fallback to cmos. */
 		tomk = qemu_get_memory_size();
-		high = qemu_get_high_memory_size();
-		printk(BIOS_DEBUG, "QEMU: cmos: %lu MiB RAM below 4G.\n", tomk / 1024);
-		printk(BIOS_DEBUG, "QEMU: cmos: %lu MiB RAM above 4G.\n", high / 1024);
+		uint64_t high = qemu_get_high_memory_size();
+		printk(BIOS_DEBUG, "QEMU: cmos: %llu MiB RAM below 4G.\n", tomk / 1024);
+		printk(BIOS_DEBUG, "QEMU: cmos: %llu MiB RAM above 4G.\n", high / 1024);
 
 		/* Report the memory regions. */
-		ram_resource(dev, idx++, 0, 640);
-		ram_resource(dev, idx++, 768, tomk - 768);
+		ram_from_to(dev, idx++, 0, 0xa0000);
+		ram_from_to(dev, idx++, 0xc0000, tomk * KiB);
+
 		if (high)
-			ram_resource(dev, idx++, 4 * 1024 * 1024, high);
+			upper_ram_end(dev, idx++, 4ull * GiB + high * KiB);
 	}
 
 	/* Reserve I/O ports used by QEMU */
@@ -121,10 +118,10 @@ static void cpu_pci_domain_read_resources(struct device *dev)
 	}
 
 	/* A segment is legacy VGA region */
-	mmio_resource(dev, idx++, 0xa0000 / KiB, (0xc0000 - 0xa0000) / KiB);
+	mmio_from_to(dev, idx++, 0xa0000, 0xc0000);
 
 	/* C segment to 1MB is reserved RAM (low tables) */
-	reserved_ram_resource(dev, idx++, 0xc0000 / KiB, (1 * MiB - 0xc0000) / KiB);
+	reserved_ram_from_to(dev, idx++, 0xc0000, 1 * MiB);
 
 	if (q35 && ((tomk * 1024) < 0xb0000000)) {
 		/*
@@ -222,7 +219,7 @@ static const char *qemu_acpi_name(const struct device *dev)
 	if (dev->path.type == DEVICE_PATH_DOMAIN)
 		return "PCI0";
 
-	if (dev->path.type != DEVICE_PATH_PCI || dev->bus->secondary != 0)
+	if (!is_pci_dev_on_bus(dev, 0))
 		return NULL;
 
 	return NULL;
@@ -249,7 +246,7 @@ extern const struct mp_ops mp_ops_with_smm;
 
 void mp_init_cpus(struct bus *cpu_bus)
 {
-	const struct mp_ops *ops = CONFIG(SMM_TSEG) ? &mp_ops_with_smm : &mp_ops_no_smm;
+	const struct mp_ops *ops = CONFIG(NO_SMM) ? &mp_ops_no_smm : &mp_ops_with_smm;
 
 	/* TODO: Handle mp_init_with_smm failure? */
 	mp_init_with_smm(cpu_bus, ops);
