@@ -7,7 +7,6 @@
 #include <gpio.h>
 #include <soc/nhlt.h>
 #include <variant/gpio.h>
-#include <vendorcode/google/chromeos/chromeos.h>
 #include "ec.h"
 
 static const char *oem_id_maxim = "INTEL";
@@ -96,7 +95,6 @@ static void mainboard_enable(struct device *dev)
 {
 	dev->ops->init = mainboard_init;
 	dev->ops->write_acpi_tables = mainboard_write_acpi_tables;
-	dev->ops->acpi_inject_dsdt = chromeos_dsdt_generator;
 }
 
 static void mainboard_chip_init(void *chip_info)
