@@ -6,23 +6,29 @@
 #include <commonlib/bsd/cb_err.h>
 #include <console/console.h>
 #include <gpio.h>
+#include <soc/socinfo.h>
 
 uint32_t board_id(void)
 {
 	static uint32_t id = UNDEFINED_STRAPPING_ID;
-	const gpio_t pins[] = {[2] = GPIO(50), [1] = GPIO(49), [0] = GPIO(48)};
+	gpio_t pins[3] = { 0 };
+	if (CONFIG(BOARD_GOOGLE_HEROBRINE_REV0)) {
+		pins[2] = GPIO(75);
+		pins[1] = GPIO(74);
+		pins[0] = GPIO(73);
+	} else {
+		pins[2] = GPIO(50);
+		pins[1] = GPIO(49);
+		pins[0] = GPIO(48);
+	}
 
 	if (id == UNDEFINED_STRAPPING_ID)
 		id = gpio_base3_value(pins, ARRAY_SIZE(pins));
 
-	if ((id == QCOM_SC7280_SKU1) || (id == QCOM_SC7280_SKU2) ||
-						(id == QCOM_SC7280_SKU3))
-		printk(BIOS_INFO, "BoardID :%d - "
+	printk(BIOS_INFO, "BoardID :%d - "
 				"Machine model: "
 				"Qualcomm Technologies, Inc. "
-				"sc7280 IDP SKU%d platform\n", id, (id+1));
-	else
-		printk(BIOS_ERR, "Invalid BoardId : %d\n", id);
+				"sc7280 platform\n", id);
 
 	return id;
 }
@@ -36,5 +42,10 @@ uint32_t ram_code(void)
 
 uint32_t sku_id(void)
 {
-	return google_chromeec_get_board_sku();
+	static uint32_t id = UNDEFINED_STRAPPING_ID;
+	/*Update modem status in 9th bit of sku id*/
+	uint32_t mask = 1 << 9;
+	id = google_chromeec_get_board_sku();
+	id = ((id & ~mask) | (socinfo_modem_supported() << 9));
+	return id;
 }
