@@ -38,7 +38,9 @@
 #define MAINBOARD_EC_S3_WAKE_EVENTS \
 	(MAINBOARD_EC_S5_WAKE_EVENTS |\
 	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_KEY_PRESSED) |\
-	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_MODE_CHANGE))
+	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_MODE_CHANGE) |\
+	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_AC_CONNECTED) |\
+	 EC_HOST_EVENT_MASK(EC_HOST_EVENT_AC_DISCONNECTED))
 
 /* Log EC wake events plus EC shutdown events */
 #define MAINBOARD_EC_LOG_EVENTS \
