@@ -7,7 +7,6 @@
 #if CONFIG(VGA_ROM_RUN)
 #include <x86emu/x86emu.h>
 #endif
-#include <acpi/acpi.h>
 #include <arch/io.h>
 #include <arch/interrupt.h>
 #include "onboard.h"
@@ -16,7 +15,6 @@
 #include <southbridge/intel/common/gpio.h>
 #include <smbios.h>
 #include <ec/google/chromeec/ec.h>
-#include <vendorcode/google/chromeos/chromeos.h>
 
 #if CONFIG(VGA_ROM_RUN)
 static int int15_handler(void)
@@ -170,7 +168,6 @@ static void mainboard_enable(struct device *dev)
 {
 	dev->ops->init = mainboard_init;
 	dev->ops->get_smbios_data = link_onboard_smbios_data;
-	dev->ops->acpi_inject_dsdt = chromeos_dsdt_generator;
 #if CONFIG(VGA_ROM_RUN)
 	/* Install custom int15 handler for VGA OPROM */
 	mainboard_interrupt_handlers(0x15, &int15_handler);
