@@ -1,11 +1,10 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
-#include <baseboard/variants.h>
 #include <bootmode.h>
 #include <boot/coreboot_tables.h>
 #include <ec/google/chromeec/ec.h>
 #include <gpio.h>
-#include <vendorcode/google/chromeos/chromeos.h>
+#include <types.h>
 #include <soc/gpio.h>
 #include <variant/gpio.h>
 
@@ -26,11 +25,8 @@ int get_write_protect_state(void)
 	return gpio_get(GPIO_PCH_WP);
 }
 
-void mainboard_chromeos_acpi_generate(void)
+int get_ec_is_trusted(void)
 {
-	const struct cros_gpio *gpios;
-	size_t num;
-
-	gpios = variant_cros_gpios(&num);
-	chromeos_acpi_gpio_generate(gpios, num);
+	/* EC is trusted if not in RW. */
+	return !gpio_get(GPIO_EC_IN_RW);
 }
