@@ -13,7 +13,6 @@
 
 void mainboard_memory_init_params(FSPM_UPD *mupd)
 {
-	FSP_M_CONFIG *mem_cfg = &mupd->FspmConfig;
 	const struct mb_cfg *board_cfg = variant_memory_params();
 	const struct mem_spd spd_info = {
 		.topo = MEM_TOPO_MEMORY_DOWN,
@@ -21,7 +20,7 @@ void mainboard_memory_init_params(FSPM_UPD *mupd)
 	};
 	bool half_populated = gpio_get(GPIO_MEM_CH_SEL);
 
-	memcfg_init(mem_cfg, board_cfg, &spd_info, half_populated);
+	memcfg_init(mupd, board_cfg, &spd_info, half_populated);
 	memcfg_variant_init(mupd);
 }
 
