@@ -4,9 +4,7 @@
 
 /* Enable PS/2 Mouse */
 #define SIO_EC_ENABLE_PS2M
-
-/* Enable DPTC support */
-#define EC_ENABLE_AMD_DPTC_SUPPORT
+#define SIO_EC_PS2M_IRQ Interrupt (ResourceConsumer, Level, ActiveLow, Shared) {12}
 
 #undef MAINBOARD_EC_S3_WAKE_EVENTS
 #undef MAINBOARD_EC_S3_DEVICE_EVENTS
