@@ -1,11 +1,10 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
-#include <baseboard/variants.h>
 #include <bootmode.h>
 #include <boot/coreboot_tables.h>
 #include <ec/google/chromeec/ec.h>
 #include <gpio.h>
-#include <vendorcode/google/chromeos/chromeos.h>
+#include <types.h>
 #include <soc/gpio.h>
 #include <variant/gpio.h>
 
@@ -25,16 +24,14 @@ int get_write_protect_state(void)
 	return 0;
 }
 
-void mainboard_chromeos_acpi_generate(void)
-{
-	const struct cros_gpio *gpios;
-	size_t num;
-
-	gpios = variant_cros_gpios(&num);
-	chromeos_acpi_gpio_generate(gpios, num);
-}
-
 int __weak get_lid_switch(void)
 {
 	return -1;
 }
+
+int get_ec_is_trusted(void)
+{
+	/* Do not have a Chrome EC involved in entering recovery mode;
+	   Always return trusted. */
+	return 1;
+}
