@@ -1,15 +1,13 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
+
 #include <assert.h>
-#include <console/console.h>
 #include <fsp/api.h>
 #include <soc/romstage.h>
 #include <spd_bin.h>
-#include <string.h>
 #include <soc/meminit.h>
 #include <baseboard/variants.h>
-#include <cbfs.h>
 
-void mainboard_memory_init_params(FSP_M_CONFIG *m_cfg)
+void mainboard_memory_init_params(FSPM_UPD *memupd)
 {
 	const struct mb_cfg *mem_config = variant_memory_params();
 	const bool half_populated = false;
@@ -19,5 +17,5 @@ void mainboard_memory_init_params(FSP_M_CONFIG *m_cfg)
 		.cbfs_index = variant_memory_sku(),
 	};
 
-	memcfg_init(m_cfg, mem_config, &lp5_spd_info, half_populated);
+	memcfg_init(memupd, mem_config, &lp5_spd_info, half_populated);
 }
