@@ -52,7 +52,7 @@
 #define BLDCFG_FCH_GPP_PORT0_PRESENT		TRUE
 #define BLDCFG_FCH_GPP_PORT1_PRESENT		TRUE
 
-GPIO_CONTROL ms7721_m_gpio[] = {
+CONST GPIO_CONTROL ms7721_m_gpio[] = {
 	{-1}
 };
 
