@@ -25,7 +25,7 @@ void mainboard_config_gpios(void)
 	num = ARRAY_SIZE(tagada_gpio_config);
 
 	if ((!table) || (!num)) {
-		printk(BIOS_ERR, "ERROR: No valid GPIO table found!\n");
+		printk(BIOS_ERR, "No valid GPIO table found!\n");
 		return;
 	}
 
