@@ -6,7 +6,7 @@
 
 static void mainboard_enable(struct device *dev)
 {
-	ram_resource(dev, 0, (uintptr_t)_dram/KiB, sdram_size_mb()*KiB);
+	ram_resource_kb(dev, 0, (uintptr_t)_dram/KiB, sdram_size_mb()*KiB);
 }
 
 struct chip_operations mainboard_ops = {
