@@ -26,5 +26,5 @@ void mainboard_memory_init_params(FSPM_UPD *mupd)
 	// Enable M.2 PCIE 4.0 and PEG1
 	mupd->FspmConfig.CpuPcieRpEnableMask = 0x3;
 
-	memcfg_init(&mupd->FspmConfig, &board_cfg, &spd_info, half_populated);
+	memcfg_init(mupd, &board_cfg, &spd_info, half_populated);
 }
