@@ -1,57 +1,70 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2011 Advanced Micro Devices, Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <console/console.h>
-#include <arch/io.h>
-#include <arch/acpi.h>
-#include <arch/acpigen.h>
+#include <device/pci_ops.h>
+#include <acpi/acpi.h>
+#include <acpi/acpigen.h>
 #include <stdint.h>
 #include <device/device.h>
 #include <device/pci.h>
 #include <device/pci_ids.h>
-#include <device/hypertransport.h>
-#include <stdlib.h>
 #include <string.h>
 #include <lib.h>
 #include <cpu/cpu.h>
-#include <cbmem.h>
-
-#include <cpu/x86/lapic.h>
+#include <cpu/amd/msr.h>
 #include <cpu/amd/mtrr.h>
-
-#include <northbridge/amd/agesa/agesawrapper.h>
+#include <northbridge/amd/nb_common.h>
 #include <northbridge/amd/agesa/state_machine.h>
 #include <northbridge/amd/agesa/agesa_helper.h>
-
 #include <sb_cimx.h>
 
 #define FX_DEVS 1
 
-static device_t __f0_dev[FX_DEVS];
-static device_t __f1_dev[FX_DEVS];
-static device_t __f2_dev[FX_DEVS];
-static device_t __f4_dev[FX_DEVS];
-static unsigned fx_devs = 0;
+static struct device *__f0_dev[FX_DEVS];
+static struct device *__f1_dev[FX_DEVS];
+static struct device *__f2_dev[FX_DEVS];
+static struct device *__f4_dev[FX_DEVS];
+static unsigned int fx_devs = 0;
 
-static device_t get_node_pci(u32 nodeid, u32 fn)
+static u32 get_io_addr_index(u32 nodeid, u32 linkn)
 {
-	if ((CONFIG_CDB + nodeid) < 32) {
-		return dev_find_slot(CONFIG_CBB, PCI_DEVFN(CONFIG_CDB + nodeid, fn));
-	} else {
-		return dev_find_slot(CONFIG_CBB-1, PCI_DEVFN(CONFIG_CDB + nodeid - 32, fn));
-	}
+	return 0;
+}
+
+static u32 get_mmio_addr_index(u32 nodeid, u32 linkn)
+{
+	return 0;
+}
+
+static void set_io_addr_reg(struct device *dev, u32 nodeid, u32 linkn, u32 reg,
+			    u32 io_min, u32 io_max)
+{
+
+	u32 tempreg;
+	/* io range allocation */
+	tempreg = (nodeid & 0xf) | ((nodeid & 0x30) << (8 - 4)) | (linkn << 4) |
+		  ((io_max & 0xf0) << (12 - 4)); //limit
+	pci_write_config32(__f1_dev[0], reg + 4, tempreg);
+
+	tempreg = 3 | ((io_min & 0xf0) << (12 - 4)); //base :ISA and VGA ?
+	pci_write_config32(__f1_dev[0], reg, tempreg);
+}
+
+static void set_mmio_addr_reg(u32 nodeid, u32 linkn, u32 reg, u32 index,
+			      u32 mmio_min, u32 mmio_max, u32 nodes)
+{
+
+	u32 tempreg;
+	/* io range allocation */
+	tempreg = (nodeid & 0xf) | (linkn << 4) | (mmio_max & 0xffffff00);
+	pci_write_config32(__f1_dev[0], reg + 4, tempreg);
+	tempreg = 3 | (nodeid & 0x30) | (mmio_min & 0xffffff00);
+	pci_write_config32(__f1_dev[0], reg, tempreg);
+}
+
+static struct device *get_node_pci(u32 nodeid, u32 fn)
+{
+	return pcidev_on_root(DEV_CDB + nodeid, fn);
 }
 
 static void get_fx_devs(void)
@@ -70,20 +83,20 @@ static void get_fx_devs(void)
 	}
 }
 
-static u32 f1_read_config32(unsigned reg)
+static u32 f1_read_config32(unsigned int reg)
 {
 	if (fx_devs == 0)
 		get_fx_devs();
 	return pci_read_config32(__f1_dev[0], reg);
 }
 
-static void f1_write_config32(unsigned reg, u32 value)
+static void f1_write_config32(unsigned int reg, u32 value)
 {
 	int i;
 	if (fx_devs == 0)
 		get_fx_devs();
 	for (i = 0; i < fx_devs; i++) {
-		device_t dev;
+		struct device *dev;
 		dev = __f1_dev[i];
 		if (dev && dev->enabled) {
 			pci_write_config32(dev, reg, value);
@@ -91,14 +104,39 @@ static void f1_write_config32(unsigned reg, u32 value)
 	}
 }
 
-static u32 amdfam14_nodeid(device_t dev)
+static int get_dram_base_limit(u32 nodeid, resource_t *basek, resource_t *limitk)
 {
-	return (dev->path.pci.devfn >> 3) - CONFIG_CDB;
+	u32 temp;
+
+	if (fx_devs == 0)
+		get_fx_devs();
+
+
+	temp = pci_read_config32(__f1_dev[nodeid], 0x40 + (nodeid << 3)); //[39:24] at [31:16]
+	if (!(temp & 1))
+		return 0; // this memory range is not enabled
+	/*
+	 * BKDG: {DramBase[35:24], 00_0000h} <= address[35:0] so shift left by 8 bits
+	 * for physical address and the convert to KiB by shifting 10 bits left
+	 */
+	*basek = ((temp & 0x0fff0000)) >> (10 - 8);
+	/*
+	 * BKDG address[35:0] <= {DramLimit[35:24], FF_FFFFh} converted as above but
+	 * ORed with 0xffff to get real limit before shifting.
+	 */
+	temp = pci_read_config32(__f1_dev[nodeid], 0x44 + (nodeid << 3)); //[39:24] at [31:16]
+	*limitk = ((temp & 0x0fff0000) | 0xffff) >> (10 - 8);
+	*limitk += 1; // round up last byte
+
+	return 1;
 }
 
-#include "amdfam14_conf.c"
+static u32 amdfam14_nodeid(struct device *dev)
+{
+	return (dev->path.pci.devfn >> 3) - DEV_CDB;
+}
 
-static void northbridge_init(device_t dev)
+static void northbridge_init(struct device *dev)
 {
 	printk(BIOS_DEBUG, "Northbridge init\n");
 }
@@ -114,15 +152,15 @@ static void set_vga_enable_reg(u32 nodeid, u32 linkn)
 
 }
 
-static int reg_useable(unsigned reg, device_t goal_dev, unsigned goal_nodeid,
-		       unsigned goal_link)
+static int reg_useable(unsigned int reg, struct device *goal_dev,
+		       unsigned int goal_nodeid, unsigned int goal_link)
 {
 	struct resource *res;
-	unsigned nodeid, link = 0;
+	unsigned int nodeid, link = 0;
 	int result;
 	res = 0;
 	for (nodeid = 0; !res && (nodeid < fx_devs); nodeid++) {
-		device_t dev;
+		struct device *dev;
 		dev = __f0_dev[nodeid];
 		if (!dev)
 			continue;
@@ -141,8 +179,8 @@ static int reg_useable(unsigned reg, device_t goal_dev, unsigned goal_nodeid,
 	return result;
 }
 
-static struct resource *amdfam14_find_iopair(device_t dev, unsigned nodeid,
-					     unsigned link)
+static struct resource *amdfam14_find_iopair(struct device *dev,
+			unsigned int nodeid, unsigned int link)
 {
 	struct resource *resource;
 	u32 result, reg;
@@ -156,7 +194,7 @@ static struct resource *amdfam14_find_iopair(device_t dev, unsigned nodeid,
 	/* Ext conf space */
 	if (!reg) {
 		/* Because of Extend conf space, we will never run out of reg,
-		 * but we need one index to differ them. So ,same node and same
+		 * but we need one index to differ them. So,same node and same
 		 *  link can have multi range
 		 */
 		u32 index = get_io_addr_index(nodeid, link);
@@ -168,7 +206,7 @@ static struct resource *amdfam14_find_iopair(device_t dev, unsigned nodeid,
 	return resource;
 }
 
-static struct resource *amdfam14_find_mempair(device_t dev, u32 nodeid,
+static struct resource *amdfam14_find_mempair(struct device *dev, u32 nodeid,
 					      u32 link)
 {
 	struct resource *resource;
@@ -192,7 +230,7 @@ static struct resource *amdfam14_find_mempair(device_t dev, u32 nodeid,
 	/* Ext conf space */
 	if (!reg) {
 		/* Because of Extend conf space, we will never run out of reg,
-		 * but we need one index to differ them. So ,same node and same
+		 * but we need one index to differ them. So,same node and same
 		 *  link can have multi range
 		 */
 		u32 index = get_mmio_addr_index(nodeid, link);
@@ -203,7 +241,7 @@ static struct resource *amdfam14_find_mempair(device_t dev, u32 nodeid,
 	return resource;
 }
 
-static void amdfam14_link_read_bases(device_t dev, u32 nodeid, u32 link)
+static void amdfam14_link_read_bases(struct device *dev, u32 nodeid, u32 link)
 {
 	struct resource *resource;
 
@@ -247,8 +285,9 @@ static void amdfam14_link_read_bases(device_t dev, u32 nodeid, u32 link)
 static u32 my_find_pci_tolm(struct bus *bus, u32 tolm)
 {
 	struct resource *min;
+	unsigned long mask_match = IORESOURCE_MEM | IORESOURCE_ASSIGNED;
 	min = 0;
-	search_bus_resources(bus, IORESOURCE_MEM, IORESOURCE_MEM, tolm_test,
+	search_bus_resources(bus, mask_match, mask_match, tolm_test,
 			     &min);
 	if (min && tolm > min->base) {
 		tolm = min->base;
@@ -259,7 +298,7 @@ static u32 my_find_pci_tolm(struct bus *bus, u32 tolm)
 #if CONFIG_HW_MEM_HOLE_SIZEK != 0
 
 struct hw_mem_hole_info {
-	unsigned hole_startk;
+	unsigned int hole_startk;
 	int node_id;
 };
 
@@ -270,10 +309,10 @@ static struct hw_mem_hole_info get_hw_mem_hole_info(void)
 	mem_hole.hole_startk = CONFIG_HW_MEM_HOLE_SIZEK;
 	mem_hole.node_id = -1;
 
-	struct dram_base_mask_t d;
+	resource_t basek, limitk;
 	u32 hole;
-	d = get_dram_base_mask(0);
-	if (d.mask & 1) {
+
+	if (get_dram_base_limit(0, &basek, &limitk)) {
 		hole = pci_read_config32(__f1_dev[0], 0xf0);
 		if (hole & 1) {	// we find the hole
 			mem_hole.hole_startk = (hole & (0xff << 24)) >> 10;
@@ -284,7 +323,7 @@ static struct hw_mem_hole_info get_hw_mem_hole_info(void)
 }
 #endif
 
-static void nb_read_resources(device_t dev)
+static void nb_read_resources(struct device *dev)
 {
 	u32 nodeid;
 	struct bus *link;
@@ -303,13 +342,14 @@ static void nb_read_resources(device_t dev)
 	 * It is not honored by the coreboot resource allocator if it is in
 	 * the CPU_CLUSTER.
 	 */
-	mmconf_resource(dev, 0xc0010058);
+	mmconf_resource(dev, MMIO_CONF_BASE);
 }
 
-static void set_resource(device_t dev, struct resource *resource, u32 nodeid)
+static void set_resource(struct device *dev, struct resource *resource,
+			 u32 nodeid)
 {
 	resource_t rbase, rend;
-	unsigned reg, link_num;
+	unsigned int reg, link_num;
 	char buf[50];
 
 	printk(BIOS_DEBUG, "\nFam14h - %s\n", __func__);
@@ -354,11 +394,11 @@ static void set_resource(device_t dev, struct resource *resource, u32 nodeid)
 	report_resource_stored(dev, resource, buf);
 }
 
-#if IS_ENABLED(CONFIG_CONSOLE_VGA_MULTI)
-extern device_t vga_pri;	// the primary vga device, defined in device.c
+#if CONFIG(CONSOLE_VGA_MULTI)
+extern struct device *vga_pri;	// the primary vga device, defined in device.c
 #endif
 
-static void create_vga_resource(device_t dev, unsigned nodeid)
+static void create_vga_resource(struct device *dev, unsigned int nodeid)
 {
 	struct bus *link;
 
@@ -368,7 +408,7 @@ static void create_vga_resource(device_t dev, unsigned nodeid)
 	 * we only deal with the 'first' vga card */
 	for (link = dev->link_list; link; link = link->next) {
 		if (link->bridge_ctrl & PCI_BRIDGE_CTL_VGA) {
-#if IS_ENABLED(CONFIG_CONSOLE_VGA_MULTI)
+#if CONFIG(CONSOLE_VGA_MULTI)
 			printk(BIOS_DEBUG,
 				"VGA: vga_pri bus num = %d bus range [%d,%d]\n",
 				 vga_pri->bus->secondary, link->secondary,
@@ -390,9 +430,9 @@ static void create_vga_resource(device_t dev, unsigned nodeid)
 	set_vga_enable_reg(nodeid, link->link_num);
 }
 
-static void nb_set_resources(device_t dev)
+static void nb_set_resources(struct device *dev)
 {
-	unsigned nodeid;
+	unsigned int nodeid;
 	struct bus *bus;
 	struct resource *res;
 
@@ -417,9 +457,9 @@ static void nb_set_resources(device_t dev)
 
 /* Domain/Root Complex related code */
 
-static void domain_read_resources(device_t dev)
+static void domain_read_resources(struct device *dev)
 {
-	unsigned reg;
+	unsigned int reg;
 
 	printk(BIOS_DEBUG, "\nFam14h - %s\n", __func__);
 
@@ -431,8 +471,8 @@ static void domain_read_resources(device_t dev)
 		limit = f1_read_config32(reg + 0x04);
 		/* Is this register allocated? */
 		if ((base & 3) != 0) {
-			unsigned nodeid, reg_link;
-			device_t reg_dev;
+			unsigned int nodeid, reg_link;
+			struct device *reg_dev;
 			if (reg < 0xc0) {	// mmio
 				nodeid = (limit & 0xf) + (base & 0x30);
 			} else {	// io
@@ -459,7 +499,7 @@ static void domain_read_resources(device_t dev)
 	pci_domain_read_resources(dev);
 }
 
-static void domain_set_resources(device_t dev)
+static void domain_set_resources(struct device *dev)
 {
 	printk(BIOS_DEBUG, "\nFam14h - %s\n", __func__);
 	printk(BIOS_DEBUG, "  amsr - incoming dev = %p\n", dev);
@@ -470,7 +510,6 @@ static void domain_set_resources(device_t dev)
 	struct bus *link;
 #if CONFIG_HW_MEM_HOLE_SIZEK != 0
 	struct hw_mem_hole_info mem_hole;
-	u32 reset_memhole = 1;
 #endif
 
 	pci_tolm = 0xffffffffUL;
@@ -499,42 +538,26 @@ static void domain_set_resources(device_t dev)
 	mem_hole = get_hw_mem_hole_info();
 
 	// Use hole_basek as mmio_basek, and we don't need to reset hole anymore
-	if ((mem_hole.node_id != -1) && (mmio_basek > mem_hole.hole_startk)) {
+	if ((mem_hole.node_id != -1) && (mmio_basek > mem_hole.hole_startk))
 		mmio_basek = mem_hole.hole_startk;
-		reset_memhole = 0;
-	}
 #endif
 
 	idx = 0x10;
-
-	struct dram_base_mask_t d;
 	resource_t basek, limitk, sizek;	// 4 1T
 
-	d = get_dram_base_mask(0);
+	if (get_dram_base_limit(0, &basek, &limitk)) {
+		sizek = limitk - basek;
 
-	if (d.mask & 1) {
-		basek = ((resource_t) ((u64) d.base)) << 8;
-		limitk = (resource_t) (((u64) d.mask << 8) | 0xFFFFFF);
-		printk(BIOS_DEBUG,
-			"adsr: (before) basek = %llx, limitk = %llx.\n", basek,
-			 limitk);
+		printk(BIOS_DEBUG, "adsr: basek = %llx, limitk = %llx, sizek = %llx.\n",
+				   basek, limitk, sizek);
 
-		/* Convert these values to multiples of 1K for ease of math. */
-		basek >>= 10;
-		limitk >>= 10;
-		sizek = limitk - basek + 1;
-
-		printk(BIOS_DEBUG,
-			"adsr: (after) basek = %llx, limitk = %llx, sizek = %llx.\n",
-			 basek, limitk, sizek);
-
-		/* see if we need a hole from 0xa0000 to 0xbffff */
-		if ((basek < 640) && (sizek > 768)) {
-			printk(BIOS_DEBUG,"adsr - 0xa0000 to 0xbffff resource.\n");
-			ram_resource(dev, (idx | 0), basek, 640 - basek);
+		/* See if we need a hole from 0xa0000 (640K) to 0xbffff (768K) */
+		if (basek < 640 && sizek > 768) {
+			printk(BIOS_DEBUG, "adsr - 0xa0000 to 0xbffff resource.\n");
+			ram_resource_kb(dev, (idx | 0), basek, 640 - basek);
 			idx += 0x10;
 			basek = 768;
-			sizek = limitk - 768;
+			sizek = limitk - basek;
 		}
 
 		printk(BIOS_DEBUG,
@@ -544,10 +567,10 @@ static void domain_set_resources(device_t dev)
 		/* split the region to accommodate pci memory space */
 		if ((basek < 4 * 1024 * 1024) && (limitk > mmio_basek)) {
 			if (basek <= mmio_basek) {
-				unsigned pre_sizek;
+				unsigned int pre_sizek;
 				pre_sizek = mmio_basek - basek;
 				if (pre_sizek > 0) {
-					ram_resource(dev, idx, basek,
+					ram_resource_kb(dev, idx, basek,
 						     pre_sizek);
 					idx += 0x10;
 					sizek -= pre_sizek;
@@ -562,7 +585,7 @@ static void domain_set_resources(device_t dev)
 			}
 		}
 
-		ram_resource(dev, (idx | 0), basek, sizek);
+		ram_resource_kb(dev, (idx | 0), basek, sizek);
 		idx += 0x10;
 		printk(BIOS_DEBUG,
 			"%d: mmio_basek=%08lx, basek=%08llx, limitk=%08llx\n", 0,
@@ -580,32 +603,6 @@ static void domain_set_resources(device_t dev)
 	printk(BIOS_DEBUG, "  adsr - leaving this lovely routine.\n");
 }
 
-static void domain_enable_resources(device_t dev)
-{
-#if IS_ENABLED(CONFIG_AGESA_LEGACY_WRAPPER)
-	/* Must be called after PCI enumeration and resource allocation */
-	printk(BIOS_DEBUG, "\nFam14h - %s\n", __func__);
-
-#if IS_ENABLED(CONFIG_AMD_SB_CIMX)
-	if (!acpi_is_wakeup_s3()) {
-		sb_After_Pci_Init();
-		sb_Mid_Post_Init();
-	} else {
-		sb_After_Pci_Restore_Init();
-	}
-#endif
-
-	if (!acpi_is_wakeup_s3()) {
-		/* Enable MMIO on AMD CPU Address Map Controller */
-		amd_initcpuio();
-
-		agesawrapper_amdinitmid();
-	}
-
-	printk(BIOS_DEBUG, "  ader - leaving domain_enable_resources.\n");
-#endif
-}
-
 static const char *domain_acpi_name(const struct device *dev)
 {
 	if (dev->path.type == DEVICE_PATH_DOMAIN)
@@ -619,15 +616,16 @@ static const char *domain_acpi_name(const struct device *dev)
 static void cpu_bus_scan(struct device *dev)
 {
 	struct bus *cpu_bus = dev->link_list;
-	device_t cpu;
+	struct device *cpu;
 	int apic_id, cores_found;
 
 	/* There is only one node for fam14, but there may be multiple cores. */
-	cpu = dev_find_slot(0, PCI_DEVFN(0x18, 0));
+	cpu = pcidev_on_root(0x18, 0);
 	if (!cpu)
-		printk(BIOS_ERR, "ERROR: %02x:%02x.0 not found", 0, 0x18);
+		printk(BIOS_ERR, "%02x:%02x.0 not found", 0, 0x18);
 
-	cores_found = (pci_read_config32(dev_find_slot(0,PCI_DEVFN(0x18,0x3)), 0xe8) >> 12) & 3;
+	cores_found = (pci_read_config32(pcidev_on_root(0x18, 0x3),
+					0xe8) >> 12) & 3;
 	printk(BIOS_DEBUG, "  AP siblings=%d\n", cores_found);
 
 	for (apic_id = 0; apic_id <= cores_found; apic_id++) {
@@ -637,14 +635,14 @@ static void cpu_bus_scan(struct device *dev)
 	}
 }
 
-static void cpu_bus_init(device_t dev)
+static void cpu_bus_init(struct device *dev)
 {
 	initialize_cpus(dev->link_list);
 }
 
 /* North Bridge Structures */
 
-static void northbridge_fill_ssdt_generator(device_t device)
+static void northbridge_fill_ssdt_generator(const struct device *device)
 {
 	msr_t msr;
 	char pscope[] = "\\_SB.PCI0";
@@ -685,7 +683,22 @@ static unsigned long acpi_fill_hest(acpi_hest_t *hest)
 	return (unsigned long)current;
 }
 
-static unsigned long agesa_write_acpi_tables(device_t device,
+static void patch_ssdt_processor_scope(acpi_header_t *ssdt)
+{
+	unsigned int len = ssdt->length - sizeof(acpi_header_t);
+	unsigned int i;
+
+	for (i = sizeof(acpi_header_t); i < len; i++) {
+		/* Search for _PR_ scope and replace it with _SB_ */
+		if (*(uint32_t *)((unsigned long)ssdt + i) == 0x5f52505f)
+			*(uint32_t *)((unsigned long)ssdt + i) = 0x5f42535f;
+	}
+	/* Recalculate checksum */
+	ssdt->checksum = 0;
+	ssdt->checksum = acpi_checksum((void *)ssdt, ssdt->length);
+}
+
+static unsigned long agesa_write_acpi_tables(const struct device *device,
 					     unsigned long current,
 					     acpi_rsdp_t *rsdp)
 {
@@ -696,19 +709,19 @@ static unsigned long agesa_write_acpi_tables(device_t device,
 	acpi_hest_t *hest;
 
 	/* HEST */
-	current = ALIGN(current, 8);
+	current = ALIGN_UP(current, 8);
 	hest = (acpi_hest_t *)current;
-	acpi_write_hest((void *)current, acpi_fill_hest);
-	acpi_add_table(rsdp, (void *)current);
-	current += ((acpi_header_t *)current)->length;
+	acpi_write_hest(hest, acpi_fill_hest);
+	acpi_add_table(rsdp, hest);
+	current += hest->header.length;
 
 	/* SRAT */
-	current = ALIGN(current, 8);
+	current = ALIGN_UP(current, 8);
 	printk(BIOS_DEBUG, "ACPI:  * SRAT at %lx\n", current);
-	srat = (acpi_srat_t *) agesawrapper_getlateinitptr (PICK_SRAT);
+	srat = (acpi_srat_t *)agesawrapper_getlateinitptr(PICK_SRAT);
 	if (srat != NULL) {
 		memcpy((void *)current, srat, srat->header.length);
-		srat = (acpi_srat_t *) current;
+		srat = (acpi_srat_t *)current;
 		current += srat->header.length;
 		acpi_add_table(rsdp, srat);
 	}
@@ -717,12 +730,12 @@ static unsigned long agesa_write_acpi_tables(device_t device,
 	}
 
 	/* SLIT */
-	current = ALIGN(current, 8);
+	current = ALIGN_UP(current, 8);
 	printk(BIOS_DEBUG, "ACPI:  * SLIT at %lx\n", current);
-	slit = (acpi_slit_t *) agesawrapper_getlateinitptr (PICK_SLIT);
+	slit = (acpi_slit_t *)agesawrapper_getlateinitptr(PICK_SLIT);
 	if (slit != NULL) {
 		memcpy((void *)current, slit, slit->header.length);
-		slit = (acpi_slit_t *) current;
+		slit = (acpi_slit_t *)current;
 		current += slit->header.length;
 		acpi_add_table(rsdp, slit);
 	}
@@ -731,12 +744,12 @@ static unsigned long agesa_write_acpi_tables(device_t device,
 	}
 
 	/* SSDT */
-	current = ALIGN(current, 16);
+	current = ALIGN_UP(current, 16);
 	printk(BIOS_DEBUG, "ACPI:  * AGESA ALIB SSDT at %lx\n", current);
-	alib = (acpi_header_t *)agesawrapper_getlateinitptr (PICK_ALIB);
+	alib = (acpi_header_t *)agesawrapper_getlateinitptr(PICK_ALIB);
 	if (alib != NULL) {
 		memcpy((void *)current, alib, alib->length);
-		alib = (acpi_header_t *) current;
+		alib = (acpi_header_t *)current;
 		current += alib->length;
 		acpi_add_table(rsdp, (void *)alib);
 	} else {
@@ -745,14 +758,17 @@ static unsigned long agesa_write_acpi_tables(device_t device,
 
 	/* The DSDT needs additional work for the AGESA SSDT Pstate table */
 	/* Keep the comment for a while. */
-	current = ALIGN(current, 16);
+	current = ALIGN_UP(current, 16);
 	printk(BIOS_DEBUG, "ACPI:  * AGESA SSDT Pstate at %lx\n", current);
-	ssdt = (acpi_header_t *)agesawrapper_getlateinitptr (PICK_PSTATE);
+	ssdt = (acpi_header_t *)agesawrapper_getlateinitptr(PICK_PSTATE);
 	if (ssdt != NULL) {
+		hexdump(ssdt, ssdt->length);
+		patch_ssdt_processor_scope(ssdt);
+		hexdump(ssdt, ssdt->length);
 		memcpy((void *)current, ssdt, ssdt->length);
-		ssdt = (acpi_header_t *) current;
+		ssdt = (acpi_header_t *)current;
 		current += ssdt->length;
-		acpi_add_table(rsdp,ssdt);
+		acpi_add_table(rsdp, ssdt);
 	} else {
 		printk(BIOS_DEBUG, "  AGESA SSDT Pstate table NULL. Skipping.\n");
 	}
@@ -764,15 +780,15 @@ static struct device_operations northbridge_operations = {
 	.read_resources = nb_read_resources,
 	.set_resources = nb_set_resources,
 	.enable_resources = pci_dev_enable_resources,
-	.acpi_fill_ssdt_generator = northbridge_fill_ssdt_generator,
+	.acpi_fill_ssdt = northbridge_fill_ssdt_generator,
 	.write_acpi_tables = agesa_write_acpi_tables,
 	.init = northbridge_init,
-	.enable = 0,.ops_pci = 0,
+	.enable = 0, .ops_pci = 0,
 };
 
 static const struct pci_driver northbridge_driver __pci_driver = {
 	.ops = &northbridge_operations,
-	.vendor = PCI_VENDOR_ID_AMD,
+	.vendor = PCI_VID_AMD,
 	.device = 0x1510,
 };
 
@@ -786,29 +802,19 @@ struct chip_operations northbridge_amd_agesa_family14_ops = {
 static struct device_operations pci_domain_ops = {
 	.read_resources = domain_read_resources,
 	.set_resources = domain_set_resources,
-	.enable_resources = domain_enable_resources,
-	.init = DEVICE_NOOP,
 	.scan_bus = pci_domain_scan_bus,
 	.acpi_name = domain_acpi_name,
 };
 
 static struct device_operations cpu_bus_ops = {
-	.read_resources = DEVICE_NOOP,
-	.set_resources = DEVICE_NOOP,
-	.enable_resources = DEVICE_NOOP,
+	.read_resources = noop_read_resources,
+	.set_resources = noop_set_resources,
 	.init = cpu_bus_init,
 	.scan_bus = cpu_bus_scan,
 };
 
 static void root_complex_enable_dev(struct device *dev)
 {
-	static int done = 0;
-
-	if (!done) {
-		setup_bsp_ramtop();
-		done = 1;
-	}
-
 	/* Set the operations if it is a special bus type */
 	if (dev->path.type == DEVICE_PATH_DOMAIN) {
 		dev->ops = &pci_domain_ops;
@@ -821,3 +827,25 @@ struct chip_operations northbridge_amd_agesa_family14_root_complex_ops = {
 	CHIP_NAME("AMD Family 14h Root Complex")
 	.enable_dev = root_complex_enable_dev,
 };
+
+/********************************************************************
+* Change the vendor / device IDs to match the generic VBIOS header.
+********************************************************************/
+u32 map_oprom_vendev(u32 vendev)
+{
+	u32 new_vendev = vendev;
+
+	switch (vendev) {
+	case 0x10029809:
+	case 0x10029808:
+	case 0x10029807:
+	case 0x10029806:
+	case 0x10029805:
+	case 0x10029804:
+	case 0x10029803:
+		new_vendev = 0x10029802;
+		break;
+	}
+
+	return new_vendev;
+}
