@@ -1,3 +1,5 @@
+/* SPDX-License-Identifier: GPL-2.0-only */
+
 /* This was originally for the e7500, modified for e7501
  * The primary differences are that 7501 apparently can
  * support single channel RAM (i haven't tested),
@@ -9,56 +11,44 @@
  * Steven James 02/06/2003
  */
 
-/* converted to C 6/2004 yhlu */
-
-
 #include <stdint.h>
 #include <device/pci_def.h>
 #include <arch/io.h>
-#include <arch/cpu.h>
+#include <device/mmio.h>
+#include <device/pci_ops.h>
+#include <device/smbus_host.h>
 #include <lib.h>
-#include <stdlib.h>
+#include <commonlib/helpers.h>
 #include <console/console.h>
-
-#include <cpu/x86/mtrr.h>
-#include <cpu/x86/cache.h>
-#include <cpu/x86/msr.h>
 #include <assert.h>
 #include <spd.h>
 #include <sdram_mode.h>
-#include <cbmem.h>
+#include <timestamp.h>
 
 #include "raminit.h"
 #include "e7505.h"
-#include "debug.h"
 
 /*-----------------------------------------------------------------------------
 Definitions:
 -----------------------------------------------------------------------------*/
 
-// Uncomment this to enable run-time checking of DIMM parameters
-// for dual-channel operation
-// Unfortunately the code seems to chew up several K of space.
-//#define VALIDATE_DIMM_COMPATIBILITY
-
-#if IS_ENABLED(CONFIG_DEBUG_RAM_SETUP)
+#if CONFIG(DEBUG_RAM_SETUP)
 #define RAM_DEBUG_MESSAGE(x)	printk(BIOS_DEBUG, x)
 #define RAM_DEBUG_HEX32(x)	printk(BIOS_DEBUG, "%08x", x)
 #define RAM_DEBUG_HEX8(x)	printk(BIOS_DEBUG, "%02x", x)
-#define DUMPNORTH()		dump_pci_device(MCHDEV)
 #else
 #define RAM_DEBUG_MESSAGE(x)
 #define RAM_DEBUG_HEX32(x)
 #define RAM_DEBUG_HEX8(x)
-#define DUMPNORTH()
 #endif
 
 #define E7501_SDRAM_MODE	(SDRAM_BURST_INTERLEAVED | SDRAM_BURST_4)
 #define SPD_ERROR		"Error reading SPD info\n"
 
-#define MCHDEV		PCI_DEV(0,0,0)
-#define RASDEV		PCI_DEV(0,0,1)
-#define D060DEV		PCI_DEV(0,6,0)
+#define MCHDEV		PCI_DEV(0, 0, 0)
+#define RASDEV		PCI_DEV(0, 0, 1)
+#define AGPDEV		PCI_DEV(0, 1, 0)
+#define D060DEV		PCI_DEV(0, 6, 0)
 
 // NOTE: This used to be 0x100000.
 //       That doesn't work on systems where A20M# is asserted, because
@@ -102,9 +92,8 @@ static const uint32_t refresh_rate_map[] = {
 	1, 7, 2, 1, 1, 3
 };
 
-#define MAX_SPD_REFRESH_RATE ((sizeof(refresh_rate_map) / sizeof(uint32_t)) - 1)
+#define MAX_SPD_REFRESH_RATE (ARRAY_SIZE(refresh_rate_map) - 1)
 
-#ifdef VALIDATE_DIMM_COMPATIBILITY
 // SPD parameters that must match for dual-channel operation
 static const uint8_t dual_channel_parameters[] = {
 	SPD_MEMORY_TYPE,
@@ -115,7 +104,6 @@ static const uint8_t dual_channel_parameters[] = {
 	SPD_PRIMARY_SDRAM_WIDTH,
 	SPD_NUM_BANKS_PER_SDRAM
 };
-#endif /* VALIDATE_DIMM_COMPATIBILITY */
 
 	/* Comments here are remains of e7501 or even 855PM.
 	 * They might be partially (in)correct for e7505.
@@ -255,7 +243,6 @@ static void mchtest_control(mchtst_cc cmd)
 	pci_write_config32(MCHDEV, MCHTST, dword);
 }
 
-
 /**
  *
  */
@@ -328,20 +315,18 @@ static struct dimm_size sdram_spd_get_page_size(uint16_t dimm_socket_address)
 	pgsz.side2 = 0;
 
 	// Side 1
-	value = spd_read_byte(dimm_socket_address, SPD_NUM_COLUMNS);
+	value = smbus_read_byte(dimm_socket_address, SPD_NUM_COLUMNS);
 	if (value < 0)
 		goto hw_err;
 	pgsz.side1 = value & 0xf;	// # columns in bank 1
 
 	/* Get the module data width and convert it to a power of two */
-	value =
-	    spd_read_byte(dimm_socket_address, SPD_MODULE_DATA_WIDTH_MSB);
+	value = smbus_read_byte(dimm_socket_address, SPD_MODULE_DATA_WIDTH_MSB);
 	if (value < 0)
 		goto hw_err;
 	module_data_width = (value & 0xff) << 8;
 
-	value =
-	    spd_read_byte(dimm_socket_address, SPD_MODULE_DATA_WIDTH_LSB);
+	value = smbus_read_byte(dimm_socket_address, SPD_MODULE_DATA_WIDTH_LSB);
 	if (value < 0)
 		goto hw_err;
 	module_data_width |= (value & 0xff);
@@ -349,7 +334,7 @@ static struct dimm_size sdram_spd_get_page_size(uint16_t dimm_socket_address)
 	pgsz.side1 += log2(module_data_width);
 
 	/* side two */
-	value = spd_read_byte(dimm_socket_address, SPD_NUM_DIMM_BANKS);
+	value = smbus_read_byte(dimm_socket_address, SPD_NUM_DIMM_BANKS);
 	if (value < 0)
 		goto hw_err;
 	if (value > 2)
@@ -357,8 +342,7 @@ static struct dimm_size sdram_spd_get_page_size(uint16_t dimm_socket_address)
 	if (value == 2) {
 
 		pgsz.side2 = pgsz.side1;	// Assume symmetric banks until we know differently
-		value =
-		    spd_read_byte(dimm_socket_address, SPD_NUM_COLUMNS);
+		value = smbus_read_byte(dimm_socket_address, SPD_NUM_COLUMNS);
 		if (value < 0)
 			goto hw_err;
 		if ((value & 0xf0) != 0) {
@@ -389,8 +373,7 @@ static struct dimm_size sdram_spd_get_width(uint16_t dimm_socket_address)
 	width.side1 = 0;
 	width.side2 = 0;
 
-	value =
-	    spd_read_byte(dimm_socket_address, SPD_PRIMARY_SDRAM_WIDTH);
+	value = smbus_read_byte(dimm_socket_address, SPD_PRIMARY_SDRAM_WIDTH);
 	die_on_spd_error(value);
 
 	width.side1 = value & 0x7f;	// Mask off bank 2 flag
@@ -399,23 +382,11 @@ static struct dimm_size sdram_spd_get_width(uint16_t dimm_socket_address)
 		width.side2 = width.side1 << 1;	// Bank 2 exists and is double-width
 	} else {
 		// If bank 2 exists, it's the same width as bank 1
-		value =
-		    spd_read_byte(dimm_socket_address, SPD_NUM_DIMM_BANKS);
+		value = smbus_read_byte(dimm_socket_address, SPD_NUM_DIMM_BANKS);
 		die_on_spd_error(value);
 
-#ifdef ROMCC_IF_BUG_FIXED
 		if (value == 2)
 			width.side2 = width.side1;
-#else
-		switch (value) {
-		case 2:
-			width.side2 = width.side1;
-			break;
-
-		default:
-			break;
-		}
-#endif
 	}
 
 	return width;
@@ -434,7 +405,7 @@ static struct dimm_size sdram_spd_get_width(uint16_t dimm_socket_address)
  * @param dimm_socket_address SMBus address of DIMM socket to interrogate.
  * @return log2(number of bits) for each side of the DIMM.
  */
-static struct dimm_size spd_get_dimm_size(unsigned dimm_socket_address)
+static struct dimm_size spd_get_dimm_size(unsigned int dimm_socket_address)
 {
 	int value;
 
@@ -443,7 +414,7 @@ static struct dimm_size spd_get_dimm_size(unsigned dimm_socket_address)
 
 	if (sz.side1 > 0) {
 
-		value = spd_read_byte(dimm_socket_address, SPD_NUM_ROWS);
+		value = smbus_read_byte(dimm_socket_address, SPD_NUM_ROWS);
 		die_on_spd_error(value);
 
 		sz.side1 += value & 0xf;
@@ -457,8 +428,7 @@ static struct dimm_size spd_get_dimm_size(unsigned dimm_socket_address)
 				sz.side2 += value;	// Symmetric
 		}
 
-		value =
-		    spd_read_byte(dimm_socket_address,
+		value = smbus_read_byte(dimm_socket_address,
 				  SPD_NUM_BANKS_PER_SDRAM);
 		die_on_spd_error(value);
 
@@ -471,8 +441,6 @@ static struct dimm_size spd_get_dimm_size(unsigned dimm_socket_address)
 	return sz;
 }
 
-#ifdef VALIDATE_DIMM_COMPATIBILITY
-
 /**
  * Determine whether two DIMMs have the same value for an SPD parameter.
  *
@@ -487,8 +455,8 @@ static uint8_t are_spd_values_equal(uint8_t spd_byte_number,
 				    uint16_t dimm1_address)
 {
 	uint8_t bEqual = 0;
-	int dimm0_value = spd_read_byte(dimm0_address, spd_byte_number);
-	int dimm1_value = spd_read_byte(dimm1_address, spd_byte_number);
+	int dimm0_value = smbus_read_byte(dimm0_address, spd_byte_number);
+	int dimm1_value = smbus_read_byte(dimm1_address, spd_byte_number);
 
 	if ((dimm0_value >= 0) && (dimm1_value >= 0)
 	    && (dimm0_value == dimm1_value))
@@ -496,7 +464,6 @@ static uint8_t are_spd_values_equal(uint8_t spd_byte_number,
 
 	return bEqual;
 }
-#endif
 
 /**
  * Scan for compatible DIMMs.
@@ -533,34 +500,29 @@ static uint8_t spd_get_supported_dimms(const struct mem_controller *ctrl)
 		uint16_t channel0_dimm = ctrl->channel0[i];
 		uint16_t channel1_dimm = ctrl->channel1[i];
 		uint8_t bDualChannel = 1;
-#ifdef VALIDATE_DIMM_COMPATIBILITY
 		struct dimm_size page_size;
 		struct dimm_size sdram_width;
-#endif
 		int spd_value;
 
 		if (channel0_dimm == 0)
 			continue;	// No such socket on this mainboard
 
-		if (spd_read_byte(channel0_dimm, SPD_MEMORY_TYPE) !=
+		if (smbus_read_byte(channel0_dimm, SPD_MEMORY_TYPE) !=
 		    SPD_MEMORY_TYPE_SDRAM_DDR)
 			continue;
 
-#ifdef VALIDATE_DIMM_COMPATIBILITY
-		if (spd_read_byte(channel0_dimm, SPD_MODULE_VOLTAGE) !=
+		if (smbus_read_byte(channel0_dimm, SPD_MODULE_VOLTAGE) !=
 		    SPD_VOLTAGE_SSTL2)
 			continue;	// Unsupported voltage
 
 		// E7501 does not support unregistered DIMMs
-		spd_value =
-		    spd_read_byte(channel0_dimm, SPD_MODULE_ATTRIBUTES);
+		spd_value = smbus_read_byte(channel0_dimm, SPD_MODULE_ATTRIBUTES);
 		if (!(spd_value & MODULE_REGISTERED) || (spd_value < 0))
 			continue;
 
 		// Must support burst = 4 for dual-channel operation on E7501
 		// NOTE: for single-channel, burst = 8 is required
-		spd_value =
-		    spd_read_byte(channel0_dimm,
+		spd_value = smbus_read_byte(channel0_dimm,
 				  SPD_SUPPORTED_BURST_LENGTHS);
 		if (!(spd_value & SPD_BURST_LENGTH_4) || (spd_value < 0))
 			continue;
@@ -594,23 +556,21 @@ static uint8_t spd_get_supported_dimms(const struct mem_controller *ctrl)
 			    && (sdram_width.side2 != 8))
 				continue;
 		}
-#endif
+
 		// Channel 0 DIMM looks compatible.
 		// Now see if it is paired with the proper DIMM on channel 1.
 
 		ASSERT(channel1_dimm != 0);	// No such socket on this mainboard??
 
 		// NOTE: unpopulated DIMMs cause read to fail
-		spd_value =
-		    spd_read_byte(channel1_dimm, SPD_MODULE_ATTRIBUTES);
+		spd_value = smbus_read_byte(channel1_dimm, SPD_MODULE_ATTRIBUTES);
 		if (!(spd_value & MODULE_REGISTERED) || (spd_value < 0)) {
 
 			printk(BIOS_DEBUG, "Skipping un-matched DIMMs - only dual-channel operation supported\n");
 			continue;
 		}
-#ifdef VALIDATE_DIMM_COMPATIBILITY
-		spd_value =
-		    spd_read_byte(channel1_dimm,
+
+		spd_value = smbus_read_byte(channel1_dimm,
 				  SPD_SUPPORTED_BURST_LENGTHS);
 		if (!(spd_value & SPD_BURST_LENGTH_4) || (spd_value < 0))
 			continue;
@@ -625,27 +585,13 @@ static uint8_t spd_get_supported_dimms(const struct mem_controller *ctrl)
 				break;
 			}
 		}
-#endif /* VALIDATE_DIMM_COMPATIBILITY */
 
-		// Code around ROMCC bug in optimization of "if" statements
-#ifdef ROMCC_IF_BUG_FIXED
 		if (bDualChannel) {
-			// Made it through all the checks, this DIMM pair is usable
-			dimm_mask |= ((1 << i) | (1 << (MAX_DIMM_SOCKETS_PER_CHANNEL + i)));
+			// This DIMM pair is usable
+			dimm_mask |= 1 << i;
+			dimm_mask |= 1 << (MAX_DIMM_SOCKETS_PER_CHANNEL + i);
 		} else
 			printk(BIOS_DEBUG, "Skipping un-matched DIMMs - only dual-channel operation supported\n");
-#else
-		switch (bDualChannel) {
-		case 0:
-			printk(BIOS_DEBUG, "Skipping un-matched DIMMs - only dual-channel operation supported\n");
-			break;
-
-		default:
-			// Made it through all the checks, this DIMM pair is usable
-			dimm_mask |= (1 << i) | (1 << (MAX_DIMM_SOCKETS_PER_CHANNEL + i));
-			break;
-		}
-#endif
 	}
 
 	return dimm_mask;
@@ -770,7 +716,7 @@ DIMM-independent configuration functions:
  *                   (0..MAX_DIMM_SOCKETS_PER_CHANNEL).
  * @return New multiple of 64 MB total DRAM in the system.
  */
-static uint8_t configure_dimm_row_boundaries(struct dimm_size dimm_log2_num_bits, uint8_t total_dram_64M_multiple, unsigned dimm_index)
+static uint8_t configure_dimm_row_boundaries(struct dimm_size dimm_log2_num_bits, uint8_t total_dram_64M_multiple, unsigned int dimm_index)
 {
 	int i;
 
@@ -838,6 +784,15 @@ static void configure_e7501_ram_addresses(const struct mem_controller
 {
 	int i;
 	uint8_t total_dram_64M_multiple = 0;
+	uint64_t tolm, tom;
+	uint16_t reg;
+
+	/* FIXME: Is there standard presence detect bit somewhere. */
+	const int agp_slot_disabled = 1;
+
+	/* Start with disabled remap range. */
+	uint16_t remapbase_r = 0x3ff;
+	uint16_t remaplimit_r = 0;
 
 	// Configure the E7501's DRAM row boundaries
 	// Start by zeroing out the temporary initial configuration
@@ -867,160 +822,45 @@ static void configure_e7501_ram_addresses(const struct mem_controller
 		    configure_dimm_row_boundaries(sz, total_dram_64M_multiple, i);
 	}
 
-	// Configure the Top Of Low Memory (TOLM) in the E7501
-	// This address must be a multiple of 128 MB that is less than 4 GB.
-	// NOTE: 16-bit wide TOLM register stores only the highest 5 bits of a 32-bit address
-	//               in the highest 5 bits.
-
-	// We set TOLM to the smaller of 0xC0000000 (3 GB) or the total DRAM in the system.
-	// This reserves addresses from 0xC0000000 - 0xFFFFFFFF for non-DRAM purposes
-	// such as flash and memory-mapped I/O.
-
-	// If there is more than 3 GB of DRAM, we define a remap window which
-	// makes the DRAM "behind" the reserved region available above the top of physical
-	// memory.
-
-	// NOTE: 0xC0000000 / (64 MB) == 0x30
-
-	if (total_dram_64M_multiple <= 0x30) {
-
-		// <= 3 GB total RAM
-
-		/* I should really adjust all of this in C after I have resources
-		 * to all of the pci devices.
-		 */
-
-		// Round up to 128MB granularity
-		// SJM: Is "missing" 64 MB of memory a potential issue? Should this round down?
-
-		uint8_t total_dram_128M_multiple =
-		    (total_dram_64M_multiple + 1) >> 1;
-
-		// Convert to high 16 bits of address
-		uint16_t top_of_low_memory =
-		    total_dram_128M_multiple << 11;
-
-		pci_write_config16(MCHDEV, TOLM,
-				   top_of_low_memory);
+	tom = total_dram_64M_multiple * 64ULL * MiB;
 
+	/* Reserve MMIO space. */
+	tolm = 4ULL * GiB - 512 * MiB;
+	if (agp_slot_disabled) {
+		/* Reduce apertures to 2 x 4 MiB. */
+		pci_write_config8(MCHDEV, APSIZE, 0x3F);
+		pci_write_config16(AGPDEV, APSIZE1, 0x3F);
 	} else {
-
-		// > 3 GB total RAM
-
-		// Set defaults for > 4 GB DRAM, i.e. remap a 1 GB (= 0x10 * 64 MB) range of memory
-		uint16_t remap_base = total_dram_64M_multiple;	// A[25:0] == 0
-		uint16_t remap_limit = total_dram_64M_multiple + 0x10 - 1;	// A[25:0] == 0xF
-
-		// Put TOLM at 3 GB
-
-		pci_write_config16(MCHDEV, TOLM, 0xc000);
-
-		// Define a remap window to make the RAM that would appear from 3 GB - 4 GB
-		// visible just beyond 4 GB or the end of physical memory, whichever is larger
-		// NOTE: 16-bit wide REMAP registers store only the highest 10 bits of a 36-bit address,
-		//               (i.e. a multiple of 64 MB) in the lowest 10 bits.
-		// NOTE: 0x100000000 / (64 MB) == 0x40
-
-		if (total_dram_64M_multiple < 0x40) {
-			remap_base = 0x40;	// 0x100000000
-			remap_limit =
-			    0x40 + (total_dram_64M_multiple - 0x30) - 1;
-		}
-
-		pci_write_config16(MCHDEV, REMAPBASE,
-				   remap_base);
-		pci_write_config16(MCHDEV, REMAPLIMIT,
-				   remap_limit);
+		/* Add MMIO reserve for 2 x 256 MiB apertures. */
+		tolm -= 512 * MiB;
 	}
-}
+	tolm = MIN(tolm, tom);
 
-/**
- * Execute ECC full-speed scrub once and leave scrubber disabled.
- *
- * NOTE: All cache and stack is lost during ECC scrub loop.
- */
-static inline void __attribute__((always_inline))
-		initialize_ecc(unsigned long ret_addr, unsigned long ret_addr2)
-{
-	uint16_t scrubbed = pci_read_config16(MCHDEV, MCHCFGNS) & 0x08;
+	/* The PCI memory hole overlaps memory setup the remap window. */
+	if (tolm < tom) {
+		uint64_t remapbase = MAX(tom, 4ULL * GiB);
+		uint64_t remaplimit = remapbase + (4ULL * GiB - tolm);
 
-	if (!scrubbed) {
-		RAM_DEBUG_MESSAGE("Initializing ECC state...\n");
+		remapbase_r = remapbase / (64 * MiB);
+		remaplimit_r = remaplimit / (64 * MiB);
 
-		/* ECC scrub flushes cache-lines and stack, need to
-		 * store return address from romstage.c:main().
-		 */
-		asm volatile(
-			"movd %0, %%xmm0;"
-			"movd (%0), %%xmm1;"
-			"movd %1, %%xmm2;"
-			"movd (%1), %%xmm3;"
-			:: "r" (ret_addr), "r" (ret_addr2) :
-		);
-
-		/* NOTE: All cache is lost during this loop.
-		 * Make sure PCI access does not use stack.
-		 */
-
-		pci_write_config16(MCHDEV, MCHCFGNS, 0x01);
-		do {
-			scrubbed = pci_read_config16(MCHDEV, MCHCFGNS);
-		} while (! (scrubbed & 0x08));
-		pci_write_config16(MCHDEV, MCHCFGNS, (scrubbed & ~0x07) | 0x04);
-
-		/* Some problem remains with XIP cache from ROM, so for
-		 * now, I disable XIP and also invalidate cache (again)
-		 * before the remaining small portion of romstage.
-		 *
-		 * Adding NOPs here has unexpected results, making
-		 * the first do_printk()/vtxprintf() after ECC scrub
-		 * fail midway. Sometimes vtxprintf() dumps strings
-		 * completely but with every 4th (fourth) character as "/".
-		 *
-		 * An inlined dump to console of the same string,
-		 * before vtxprintf() call, is successful. So the
-		 * source string should be completely in cache already.
-		 *
-		 * I need to review this again with CPU microcode
-		 * update applied pre-CAR.
-		 */
-
-		/* Disable and invalidate all cache. */
-		msr_t xip_mtrr = rdmsr(MTRR_PHYS_MASK(1));
-		xip_mtrr.lo &= ~MTRR_PHYS_MASK_VALID;
-		invd();
-		wrmsr(MTRR_PHYS_MASK(1), xip_mtrr);
-		invd();
-
-		RAM_DEBUG_MESSAGE("ECC state initialized.\n");
-
-		/* Recover IP for return from main. */
-		asm volatile(
-			"movd %%xmm0, %%edi;"
-			"movd %%xmm1, (%%edi);"
-			"movd %%xmm2, %%edi;"
-			"movd %%xmm3, (%%edi);"
-			 ::: "edi"
-		);
-
-#if IS_ENABLED(CONFIG_DEBUG_RAM_SETUP)
-		unsigned int a1, a2;
-		asm volatile("movd %%xmm2, %%eax;" : "=a" (a1) ::);
-		asm volatile("movd %%xmm3, %%eax;" : "=a" (a2) ::);
-		printk(BIOS_DEBUG, "return EIP @ %x = %x\n", a1, a2);
-		asm volatile("movd %%xmm0, %%eax;" : "=a" (a1) ::);
-		asm volatile("movd %%xmm1, %%eax;" : "=a" (a2) ::);
-		printk(BIOS_DEBUG, "return EIP @ %x = %x\n", a1, a2);
-#endif
+		/* Limit register is inclusive. */
+		remaplimit_r -= 1;
 	}
 
-	/* Clear the ECC error bits. */
-	pci_write_config8(RASDEV, DRAM_FERR, 0x03);
-	pci_write_config8(RASDEV, DRAM_NERR, 0x03);
+	/* Write the RAM configuration registers,
+	   preserving the reserved bits. */
+	reg = pci_read_config16(MCHDEV, TOLM) & 0x7ff;
+	reg |= (tolm / (128 * MiB)) << 11;
+	pci_write_config16(MCHDEV, TOLM, reg);
 
-	/* Clear DRAM Interface error bits. */
-	pci_write_config32(RASDEV, FERR_GLOBAL, 1 << 18);
-	pci_write_config32(RASDEV, NERR_GLOBAL, 1 << 18);
+	reg = pci_read_config16(MCHDEV, REMAPBASE) & 0xfc00;
+	reg |= remapbase_r;
+	pci_write_config16(MCHDEV, REMAPBASE, reg);
+
+	reg = pci_read_config16(MCHDEV, REMAPLIMIT) & 0xfc00;
+	reg |= remaplimit_r;
+	pci_write_config16(MCHDEV, REMAPLIMIT, reg);
 }
 
 /**
@@ -1062,24 +902,21 @@ static void configure_e7501_dram_timing(const struct mem_controller *ctrl,
 			dimm_socket_address =
 			    ctrl->channel1[i - MAX_DIMM_SOCKETS_PER_CHANNEL];
 
-		value =
-		    spd_read_byte(dimm_socket_address,
+		value = smbus_read_byte(dimm_socket_address,
 				  SPD_MIN_ROW_PRECHARGE_TIME);
 		if (value < 0)
 			goto hw_err;
 		if (value > slowest_row_precharge)
 			slowest_row_precharge = value;
 
-		value =
-		    spd_read_byte(dimm_socket_address,
+		value = smbus_read_byte(dimm_socket_address,
 				  SPD_MIN_RAS_TO_CAS_DELAY);
 		if (value < 0)
 			goto hw_err;
 		if (value > slowest_ras_cas_delay)
 			slowest_ras_cas_delay = value;
 
-		value =
-		    spd_read_byte(dimm_socket_address,
+		value = smbus_read_byte(dimm_socket_address,
 				  SPD_MIN_ACTIVE_TO_PRECHARGE_DELAY);
 		if (value < 0)
 			goto hw_err;
@@ -1195,8 +1032,7 @@ static void configure_e7501_cas_latency(const struct mem_controller *ctrl,
 			dimm_socket_address =
 			    ctrl->channel1[i - MAX_DIMM_SOCKETS_PER_CHANNEL];
 
-		value =
-		    spd_read_byte(dimm_socket_address,
+		value = smbus_read_byte(dimm_socket_address,
 				  SPD_ACCEPTABLE_CAS_LATENCIES);
 		if (value < 0)
 			goto hw_err;
@@ -1206,8 +1042,7 @@ static void configure_e7501_cas_latency(const struct mem_controller *ctrl,
 
 		// Can we support the highest CAS# latency?
 
-		value =
-		    spd_read_byte(dimm_socket_address,
+		value = smbus_read_byte(dimm_socket_address,
 				  SPD_MIN_CYCLE_TIME_AT_CAS_MAX);
 		if (value < 0)
 			goto hw_err;
@@ -1222,8 +1057,7 @@ static void configure_e7501_cas_latency(const struct mem_controller *ctrl,
 
 		current_cas_latency >>= 1;
 		if (current_cas_latency != 0) {
-			value =
-			    spd_read_byte(dimm_socket_address,
+			value = smbus_read_byte(dimm_socket_address,
 					  SPD_SDRAM_CYCLE_TIME_2ND);
 			if (value < 0)
 				goto hw_err;
@@ -1234,8 +1068,7 @@ static void configure_e7501_cas_latency(const struct mem_controller *ctrl,
 		// Can we support the next-highest CAS# latency (max - 1.0)?
 		current_cas_latency >>= 1;
 		if (current_cas_latency != 0) {
-			value =
-			    spd_read_byte(dimm_socket_address,
+			value = smbus_read_byte(dimm_socket_address,
 					  SPD_SDRAM_CYCLE_TIME_3RD);
 			if (value < 0)
 				goto hw_err;
@@ -1367,15 +1200,14 @@ static void configure_e7501_dram_controller_mode(const struct
 		// Disable ECC mode if any one of the DIMMs does not support ECC
 		// SJM: Should we just die here? E7501 datasheet says non-ECC DIMMs aren't supported.
 
-		value =
-		    spd_read_byte(dimm_socket_address,
+		value = smbus_read_byte(dimm_socket_address,
 				  SPD_DIMM_CONFIG_TYPE);
 		die_on_spd_error(value);
 		if (value != ERROR_SCHEME_ECC) {
 			controller_mode &= ~(3 << 20);
 		}
 
-		value = spd_read_byte(dimm_socket_address, SPD_REFRESH);
+		value = smbus_read_byte(dimm_socket_address, SPD_REFRESH);
 		die_on_spd_error(value);
 		value &= 0x7f;	// Mask off self-refresh bit
 		if (value > MAX_SPD_REFRESH_RATE) {
@@ -1394,22 +1226,6 @@ static void configure_e7501_dram_controller_mode(const struct
 		    refresh_frequency[system_refresh_mode])
 			system_refresh_mode = dimm_refresh_mode;
 
-#ifdef SUSPICIOUS_LOOKING_CODE
-// SJM NOTE: This code doesn't look right. SPD values are an order of magnitude smaller
-//                       than the clock period of the memory controller. Also, no other northbridge
-//                       looks at SPD_CMD_SIGNAL_INPUT_HOLD_TIME.
-
-		// Switch to 2 clocks for address/command if required by any one of the DIMMs
-		// NOTE: At 133 MHz, 1 clock == 7.52 ns
-		value =
-		    spd_read_byte(dimm_socket_address,
-				  SPD_CMD_SIGNAL_INPUT_HOLD_TIME);
-		die_on_spd_error(value);
-		if (value >= 0xa0) {	/* At 133MHz this constant should be 0x75 */
-			controller_mode &= ~(1 << 16);	/* Use two clock cycles instead of one */
-		}
-#endif
-
 		/* go to the next DIMM */
 	}
 
@@ -1862,65 +1678,43 @@ static void sdram_set_registers(const struct mem_controller *ctrl)
 	byte &= ~0x60;
 	pci_write_config8(MCHDEV, 0xd9, byte);
 
-#ifdef SUSPICIOUS_LOOKING_CODE
-	/* This will access D2:F0:0x50, is this correct??
-	 * Vendor BIOS reads Device ID before this is set.
-	 * Undocumented in the p64h2 PCI-X bridge datasheet.
-	 */
-	byte = pci_read_config8(PCI_DEV(0,2,0), 0x50);
-	byte &= 0xcf;
-	byte |= 0x30
-	pci_write_config8(PCI_DEV(0,2,0), 0x50, byte);
-#endif
-
 	uint8_t revision = pci_read_config8(MCHDEV, 0x08);
 	if (revision >= 3)
 		d060_control(D060_CMD_1);
 }
 
-/**
- *
- *
- */
-void e7505_mch_init(const struct mem_controller *memctrl)
-{
-	RAM_DEBUG_MESSAGE("Northbridge prior to SDRAM init:\n");
-	DUMPNORTH();
-
-	sdram_set_registers(memctrl);
-	sdram_set_spd_registers(memctrl);
-	sdram_enable(memctrl);
-}
-
-uintptr_t restore_top_of_low_cacheable(void)
-{
-	u32 tolm = (pci_read_config16(MCHDEV, TOLM) & ~0x7ff) << 16;
-	return tolm;
-}
-
-/**
- * Scrub and reset error counts for ECC dimms.
- *
- * NOTE: this will invalidate cache and disable XIP cache for the
- * short remaining part of romstage.
- */
-void e7505_mch_scrub_ecc(unsigned long ret_addr)
-{
-	unsigned long ret_addr2 = (unsigned long)((unsigned long*)&ret_addr-1);
-	if ((pci_read_config32(MCHDEV, DRC)>>20 & 3) == 2)
-		initialize_ecc(ret_addr, ret_addr2);
-}
-
-void e7505_mch_done(const struct mem_controller *memctrl)
-{
-	sdram_post_ecc(memctrl);
-
-	RAM_DEBUG_MESSAGE("Northbridge following SDRAM init:\n");
-	DUMPNORTH();
-}
-
-int e7505_mch_is_ready(void)
+static int e7505_mch_is_ready(void)
 {
 	uint32_t dword = pci_read_config32(MCHDEV, DRC);
 	return !!(dword & DRC_DONE);
 }
+
+void sdram_initialize(void)
+{
+	static const struct mem_controller memctrl[] = {
+		{
+			.d0 = PCI_DEV(0, 0, 0),
+			.d0f1 = PCI_DEV(0, 0, 1),
+			.channel0 = { 0x50, 0x52, 0, 0 },
+			.channel1 = { 0x51, 0x53, 0, 0 },
+		},
+	};
+
+	/* If this is a warm boot, some initialisation can be skipped */
+	if (!e7505_mch_is_ready()) {
+
+		/* The real MCH initialisation. */
+		timestamp_add_now(TS_INITRAM_START);
+
+		sdram_set_registers(memctrl);
+		sdram_set_spd_registers(memctrl);
+		sdram_enable(memctrl);
+
+		/* Hook for post ECC scrub settings and debug. */
+		sdram_post_ecc(memctrl);
+
+		timestamp_add_now(TS_INITRAM_END);
+	}
+
+	printk(BIOS_DEBUG, "SDRAM is up.\n");
+}
