@@ -21,6 +21,7 @@ static void mch_domain_read_resources(struct device *dev)
 	uint64_t uma_memory_base = 0, uma_memory_size = 0;
 	uint64_t tseg_memory_base = 0, tseg_memory_size = 0;
 	struct device *const d0f0 = pcidev_on_root(0, 0);
+	int idx = 3;
 
 	pci_domain_read_resources(dev);
 
@@ -61,7 +62,7 @@ static void mch_domain_read_resources(struct device *dev)
 
 	/* cbmem_top can be shifted downwards due to alignment.
 	   Mark the region between cbmem_top and tomk as unusable */
-	cbmem_topk = ((uint32_t)cbmem_top() / KiB);
+	cbmem_topk = ((uintptr_t)cbmem_top() / KiB);
 	delta_cbmem = tomk_stolen - cbmem_topk;
 	tomk_stolen -= delta_cbmem;
 
@@ -74,15 +75,15 @@ static void mch_domain_read_resources(struct device *dev)
 	printk(BIOS_INFO, " (%dM)\n", (uint32_t)(tomk_stolen / KiB));
 
 	/* Report the memory regions */
-	ram_resource(dev, 3, 0, 0xa0000 / KiB);
-	ram_resource(dev, 4, 1 * MiB / KiB, (tomk - 1 * MiB / KiB));
-	uma_resource(dev, 5, uma_memory_base / KiB, uma_memory_size / KiB);
-	mmio_resource(dev, 6, tseg_memory_base / KiB, tseg_memory_size / KiB);
-	uma_resource(dev, 7, cbmem_topk, delta_cbmem);
+	ram_resource_kb(dev, idx++, 0, 0xa0000 / KiB);
+	ram_resource_kb(dev, idx++, 1 * MiB / KiB, (tomk - 1 * MiB / KiB));
+	uma_resource_kb(dev, idx++, uma_memory_base / KiB, uma_memory_size / KiB);
+	mmio_resource_kb(dev, idx++, tseg_memory_base / KiB, tseg_memory_size / KiB);
+	uma_resource_kb(dev, idx++, cbmem_topk, delta_cbmem);
 	/* legacy VGA memory */
-	mmio_resource(dev, 8, 0xa0000 / KiB, (0xc0000 - 0xa0000) / KiB);
+	mmio_resource_kb(dev, idx++, 0xa0000 / KiB, (0xc0000 - 0xa0000) / KiB);
 	/* RAM to be used for option roms and BIOS */
-	reserved_ram_resource(dev, 9, 0xc0000 / KiB, (1 * MiB - 0xc0000) / KiB);
+	reserved_ram_resource_kb(dev, idx++, 0xc0000 / KiB, (1 * MiB - 0xc0000) / KiB);
 }
 
 static void mch_domain_set_resources(struct device *dev)
@@ -100,7 +101,7 @@ static const char *northbridge_acpi_name(const struct device *dev)
 	if (dev->path.type == DEVICE_PATH_DOMAIN)
 		return "PCI0";
 
-	if (dev->path.type != DEVICE_PATH_PCI || dev->bus->secondary != 0)
+	if (!is_pci_dev_on_bus(dev, 0))
 		return NULL;
 
 	switch (dev->path.pci.devfn) {
@@ -115,7 +116,7 @@ void northbridge_write_smram(u8 smram)
 {
 	struct device *dev = pcidev_on_root(0, 0);
 
-	if (dev == NULL)
+	if (!dev)
 		die("could not find pci 00:00.0!\n");
 
 	pci_write_config8(dev, SMRAM, smram);
@@ -150,7 +151,7 @@ static const unsigned short pci_device_ids[] = {
 
 static const struct pci_driver mc_driver __pci_driver = {
 	.ops    = &mc_ops,
-	.vendor = PCI_VENDOR_ID_INTEL,
+	.vendor = PCI_VID_INTEL,
 	.devices = pci_device_ids,
 };
 
