@@ -42,7 +42,7 @@ void vboot_sync_ec(void)
 	vb2_error_t retval = VB2_SUCCESS;
 	struct vb2_context *ctx;
 
-	timestamp_add_now(TS_START_EC_SYNC);
+	timestamp_add_now(TS_EC_SYNC_START);
 
 	ctx = vboot_get_context();
 	ctx->flags |= VB2_CONTEXT_EC_SYNC_SUPPORTED;
@@ -84,7 +84,7 @@ void vboot_sync_ec(void)
 		break;
 	}
 
-	timestamp_add_now(TS_END_EC_SYNC);
+	timestamp_add_now(TS_EC_SYNC_END);
 }
 
 /* Convert firmware image type into a flash offset */
@@ -174,7 +174,7 @@ static vb2_error_t ec_hash_image(enum vb2_firmware_selection select,
 		return VB2_ERROR_UNKNOWN;
 	}
 
-	printk(BIOS_INFO, "EC took %luus to calculate image hash\n",
+	printk(BIOS_INFO, "EC took %lldus to calculate image hash\n",
 		stopwatch_duration_usecs(&sw));
 
 	*hash = resp.hash_digest;
@@ -383,7 +383,7 @@ static vb2_error_t ec_get_expected_hash(enum vb2_firmware_selection select,
 	/* vboot has no API to return this memory, so must permanently leak a mapping here. */
 	const uint8_t *file = cbfs_map(filename, &size);
 
-	if (file == NULL)
+	if (!file)
 		return VB2_ERROR_UNKNOWN;
 
 	*hash = file;
@@ -460,7 +460,7 @@ vb2_error_t vb2ex_ec_vboot_done(struct vb2_context *ctx)
 		       "EC requests limited power usage. Request shutdown.\n");
 		return VB2_REQUEST_SHUTDOWN;
 	} else {
-		printk(BIOS_INFO, "Waited %luus to clear limit power flag.\n",
+		printk(BIOS_INFO, "Waited %lldus to clear limit power flag.\n",
 			stopwatch_duration_usecs(&sw));
 	}
 
@@ -541,7 +541,7 @@ vb2_error_t vb2ex_ec_jump_to_rw(void)
 	mdelay(50);
 	while (google_chromeec_hello()) {
 		if (stopwatch_expired(&sw)) {
-			printk(BIOS_ERR, "EC did not return from reboot after %luus\n",
+			printk(BIOS_ERR, "EC did not return from reboot after %lldus\n",
 			       stopwatch_duration_usecs(&sw));
 			return VB2_ERROR_UNKNOWN;
 		}
@@ -549,7 +549,7 @@ vb2_error_t vb2ex_ec_jump_to_rw(void)
 		mdelay(5);
 	}
 
-	printk(BIOS_INFO, "\nEC returned from reboot after %luus\n",
+	printk(BIOS_INFO, "\nEC returned from reboot after %lldus\n",
 	       stopwatch_duration_usecs(&sw));
 
 	return VB2_SUCCESS;
