@@ -16,8 +16,8 @@ uintptr_t agesa_write_acpi_tables(const struct device *device, uintptr_t current
 	current = add_agesa_fsp_acpi_table(AMD_FSP_ACPI_ALIB_HOB_GUID, "ALIB", rsdp, current);
 
 	/* IVRS */
-	current = ALIGN(current, 8);
-	ivrs = (acpi_ivrs_t *) current;
+	current = ALIGN_UP(current, 8);
+	ivrs = (acpi_ivrs_t *)current;
 	acpi_create_ivrs(ivrs, acpi_fill_ivrs);
 	current += ivrs->header.length;
 	acpi_add_table(rsdp, ivrs);
