@@ -8,18 +8,12 @@
 #include <console/console.h>
 #include <fsp/api.h>
 #include <program_loading.h>
-#include <timestamp.h>
+#include <romstage_common.h>
 
-asmlinkage void car_stage_entry(void)
+void __noreturn romstage_main(void)
 {
-	timestamp_add_now(TS_START_ROMSTAGE);
-
 	post_code(0x40);
 
-	console_init();
-
-	post_code(0x41);
-
 	/* Snapshot chipset state prior to any FSP call */
 	fill_chipset_state();
 
