@@ -2,6 +2,7 @@
 
 #include <amdblocks/psp_efs.h>
 #include <arch/mmio.h>
+#include <assert.h>
 #include <boot_device.h>
 #include <commonlib/region.h>
 #include <types.h>
@@ -24,7 +25,13 @@ bool read_efs_spi_settings(uint8_t *mode, uint8_t *speed)
 	if (!efs_is_valid())
 		return false;
 
+#ifndef SPI_MODE_FIELD
+	printk(BIOS_ERR, "Unknown cpu in psp_efs.h\n");
+	printk(BIOS_ERR, "SPI speed/mode not set.\n");
+	return false;
+#else
 	*mode = efs->SPI_MODE_FIELD;
 	*speed = efs->SPI_SPEED_FIELD;
 	return true;
+#endif
 }
