@@ -4,7 +4,6 @@
 #include <acpi/acpi_crat.h>
 #include <acpi/acpi_ivrs.h>
 #include <arch/cpu.h>
-#include <console/console.h>
 #include <cpu/amd/cpuid.h>
 #include <cpu/amd/msr.h>
 #include <FspGuids.h>
@@ -19,7 +18,6 @@
 #include <amdblocks/ioapic.h>
 #include <soc/data_fabric.h>
 #include <soc/pci_devs.h>
-#include <stdlib.h>
 #include <arch/mmio.h>
 
 static unsigned long gen_crat_hsa_entry(struct acpi_crat_header *crat, unsigned long current)
@@ -82,7 +80,7 @@ static unsigned long gen_crat_memory_entries(struct acpi_crat_header *crat,
 			if (memory_base == 0) {
 				current =
 					create_crat_memory_entry(0, 0ull, 0xa0000ull, current);
-				memory_base = (1 * 1024 * 1024);
+				memory_base = 1 * MiB;
 				memory_length = memory_base;
 				new_entries++;
 			}
@@ -549,7 +547,7 @@ uintptr_t agesa_write_acpi_tables(const struct device *device, uintptr_t current
 	struct acpi_crat_header *crat;
 
 	/* CRAT */
-	current = ALIGN(current, 8);
+	current = ALIGN_UP(current, 8);
 	crat = (struct acpi_crat_header *)current;
 	acpi_create_crat(crat, acpi_fill_crat);
 	current += crat->header.length;
@@ -559,8 +557,8 @@ uintptr_t agesa_write_acpi_tables(const struct device *device, uintptr_t current
 	current = add_agesa_fsp_acpi_table(AMD_FSP_ACPI_ALIB_HOB_GUID, "ALIB", rsdp, current);
 
 	/* IVRS */
-	current = ALIGN(current, 8);
-	ivrs = (acpi_ivrs_t *) current;
+	current = ALIGN_UP(current, 8);
+	ivrs = (acpi_ivrs_t *)current;
 	acpi_create_ivrs(ivrs, acpi_fill_ivrs);
 	current += ivrs->header.length;
 	acpi_add_table(rsdp, ivrs);
