@@ -8,6 +8,9 @@
 #include <device/device.h>
 #include <stdint.h>
 
+/* RTC Registers */
+#define RTC_DATE_ALARM		0x0d
+
 uintptr_t agesa_write_acpi_tables(const struct device *device, uintptr_t current,
 				  acpi_rsdp_t *rsdp);
 
