@@ -1,28 +1,30 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
+#include <acpi/acpi.h>
 #include <amdblocks/acpi.h>
+#include <amdblocks/agesawrapper.h>
+#include <amdblocks/agesawrapper_call.h>
 #include <amdblocks/biosram.h>
-#include <device/pci_ops.h>
+#include <amdblocks/psp.h>
 #include <arch/cpu.h>
 #include <arch/romstage.h>
-#include <acpi/acpi.h>
-#include <cpu/x86/msr.h>
-#include <cpu/x86/mtrr.h>
-#include <cpu/x86/smm.h>
-#include <cpu/amd/mtrr.h>
 #include <cbmem.h>
 #include <commonlib/helpers.h>
 #include <console/console.h>
+#include <cpu/amd/mtrr.h>
+#include <cpu/x86/msr.h>
+#include <cpu/x86/mtrr.h>
+#include <cpu/x86/smm.h>
 #include <device/device.h>
-#include <program_loading.h>
-#include <romstage_handoff.h>
+#include <device/pci_ops.h>
 #include <elog.h>
-#include <amdblocks/agesawrapper.h>
-#include <amdblocks/agesawrapper_call.h>
+#include <program_loading.h>
+#include <romstage_common.h>
+#include <romstage_handoff.h>
 #include <soc/northbridge.h>
 #include <soc/pci_devs.h>
 #include <soc/southbridge.h>
-#include <amdblocks/psp.h>
+#include <stdint.h>
 
 #include "chip.h"
 
@@ -46,19 +48,14 @@ static void bsp_agesa_call(void)
 	set_ap_entry_ptr(agesa_call); /* indicate the path to the AP */
 	agesa_call();
 }
-
-asmlinkage void car_stage_entry(void)
+void __noreturn romstage_main(void)
 {
-	struct postcar_frame pcf;
-	uintptr_t top_of_ram;
 	msr_t base, mask;
 	msr_t mtrr_cap = rdmsr(MTRR_CAP_MSR);
 	int vmtrrs = mtrr_cap.lo & MTRR_CAP_VCNT;
 	int s3_resume = acpi_is_wakeup_s3();
 	int i;
 
-	console_init();
-
 	soc_enable_psp_early();
 	if (CONFIG(SOC_AMD_PSP_SELECTABLE_SMU_FW))
 		psp_load_named_blob(BLOB_SMU_FW, "smu_fw");
@@ -121,28 +118,16 @@ asmlinkage void car_stage_entry(void)
 		smm_list_regions();
 
 	post_code(0x44);
-	if (postcar_frame_init(&pcf, 0))
-		die("Unable to initialize postcar frame.\n");
+	prepare_and_run_postcar();
+}
 
-	/*
-	 * We need to make sure ramstage will be run cached. At this point exact
-	 * location of ramstage in cbmem is not known. Instruct postcar to cache
-	 * 16 megs under cbmem top which is a safe bet to cover ramstage.
-	 */
-	top_of_ram = (uintptr_t) cbmem_top();
-	postcar_frame_add_mtrr(&pcf, top_of_ram - 16*MiB, 16*MiB,
-		MTRR_TYPE_WRBACK);
-
-	/* Cache the memory-mapped boot media. */
-	postcar_frame_add_romcache(&pcf, MTRR_TYPE_WRPROT);
+void fill_postcar_frame(struct postcar_frame *pcf)
+{
+	uintptr_t top_of_ram = (uintptr_t)cbmem_top();
+	postcar_frame_add_mtrr(pcf, top_of_ram - 16 * MiB, 16 * MiB, MTRR_TYPE_WRBACK);
 
 	/* Cache the TSEG region */
-	postcar_enable_tseg_cache(&pcf);
-
-	post_code(0x45);
-	run_postcar_phase(&pcf);
-
-	post_code(0x50);  /* Should never see this post code. */
+	postcar_enable_tseg_cache(pcf);
 }
 
 void SetMemParams(AMD_POST_PARAMS *PostParams)
@@ -151,7 +136,7 @@ void SetMemParams(AMD_POST_PARAMS *PostParams)
 	const struct device *dev = pcidev_path_on_root(GNB_DEVFN);
 
 	if (!dev || !dev->chip_info) {
-		printk(BIOS_ERR, "ERROR: Cannot find SoC devicetree config\n");
+		printk(BIOS_ERR, "Cannot find SoC devicetree config\n");
 		/* In case of a BIOS error, only attempt to set UMA. */
 		PostParams->MemConfig.UmaMode = CONFIG(GFXUMA) ?
 					UMA_AUTO : UMA_NONE;
@@ -189,7 +174,7 @@ void soc_customize_init_early(AMD_EARLY_PARAMS *InitEarly)
 	struct _PLATFORM_CONFIGURATION *platform;
 
 	if (!dev || !dev->chip_info) {
-		printk(BIOS_WARNING, "Warning: Cannot find SoC devicetree"
+		printk(BIOS_WARNING, "Cannot find SoC devicetree"
 					" config, STAPM unchanged\n");
 		return;
 	}
@@ -214,6 +199,5 @@ static void migrate_power_state(int is_recovery)
 		acpi_fill_pm_gpe_state(&state->gpe_state);
 		acpi_pm_gpe_add_events_print_events();
 	}
-	acpi_clear_pm_gpe_status();
 }
-ROMSTAGE_CBMEM_INIT_HOOK(migrate_power_state)
+CBMEM_CREATION_HOOK(migrate_power_state);
