@@ -96,6 +96,12 @@ struct soc_intel_apollolake_config {
 	/* Select the eMMC max speed allowed. */
 	uint8_t emmc_host_max_speed;
 
+	/* Sata Ports Hot Plug */
+	uint8_t SataPortsHotPlug[2];
+
+	/* Sata Ports Enable */
+	uint8_t SataPortsEnable[2];
+
 	/* Specifies on which IRQ the SCI will internally appear. */
 	uint8_t sci_irq;
 
@@ -106,6 +112,16 @@ struct soc_intel_apollolake_config {
 	uint8_t gpe0_dw2; /* GPE0_95_64 STS/EN */
 	uint8_t gpe0_dw3; /* GPE0_127_96 STS/EN */
 
+	/* LPC fixed enables and ranges */
+	uint16_t lpc_iod;
+	uint16_t lpc_ioe;
+
+	/* Generic IO decode ranges */
+	uint32_t gen1_dec;
+	uint32_t gen2_dec;
+	uint32_t gen3_dec;
+	uint32_t gen4_dec;
+
 	/* Configure LPSS S0ix Enable */
 	uint8_t lpss_s0ix_enable;
 
@@ -122,6 +138,9 @@ struct soc_intel_apollolake_config {
 	uint8_t hdaudio_pwr_gate_enable;
 	uint8_t hdaudio_bios_config_lockdown;
 
+	/* Enhanced C-states */
+	int enhanced_cstates;
+
 	/* SLP S3 minimum assertion width. */
 	int slp_s3_assertion_width_usecs;
 
@@ -190,6 +209,12 @@ struct soc_intel_apollolake_config {
 	 * 0:Enable (default), 1:Disable.
 	 */
 	uint8_t disable_xhci_lfps_pm;
+
+	/* SATA Aggressive Link Power Management */
+	uint8_t DisableSataSalpSupport;
+
+	/* Sata Power Optimisation */
+	uint8_t SataPwrOptimizeDisable;
 };
 
 typedef struct soc_intel_apollolake_config config_t;
