@@ -28,11 +28,6 @@ struct __packed global_nvs {
 	uint8_t		scdo; /* 0x2A - GPIO pad offset relative to the community */
 	uint8_t		uior; /* 0x2B - UART debug controller init on S3
 					 resume */
-	uint8_t		epcs; /* 0x2C - SGX Enabled status */
-	uint64_t	emna; /* 0x2D - 0x34 EPC base address */
-	uint64_t	elng; /* 0x35 - 0x3C EPC Length */
-	uint64_t	a4gb; /* 0x3D - 0x44 Base of above 4GB MMIO Resource */
-	uint64_t	a4gs; /* 0x45 - 0x4C Length of above 4GB MMIO Resource */
 };
 
 #endif	/* _SOC_APOLLOLAKE_NVS_H_ */
