@@ -152,6 +152,7 @@
 /* Memory mapped IO registers behind PMC_BASE_ADDRESS */
 #define PRSTS			0x1000
 #define GEN_PMCON1		0x1020
+#define GEN_PMCON_A		GEN_PMCON1
 #define  COLD_BOOT_STS		(1 << 27)
 #define  COLD_RESET_STS		(1 << 26)
 #define  WARM_RESET_STS		(1 << 25)
@@ -164,6 +165,11 @@
 				 WARM_RESET_STS | GLOBAL_RESET_STS | \
 				 SRS | MS4V)
 #define GEN_PMCON2		0x1024
+#	define LPC_LPB_CLK_CTRL		((1 << 11) | (1 << 12) | (1 << 13))
+#	define BIOS_PCI_EXP_EN		(1 << 10)
+#	define PWRBTN_LVL		(1 <<  9)
+#	define SMI_LOCK			(1 <<  4)
+#	define PER_SMI_SEL		(1 <<  0)
 #define GEN_PMCON3		0x1028
 #       define SLP_S3_ASSERT_WIDTH_SHIFT	10
 #       define SLP_S3_ASSERT_MASK	(0x3 << SLP_S3_ASSERT_WIDTH_SHIFT)
@@ -235,6 +241,9 @@ struct chipset_power_state {
 
 void pch_log_state(void);
 
+/* Get base address PMC memory mapped registers. */
+uint8_t *pmc_mmio_regs(void);
+
 /* STM Support */
 uint16_t get_pmbase(void);
 
