@@ -18,7 +18,7 @@
 void soc_add_fixed_mmio_resources(struct device *dev, int *index)
 {
 	static const struct sa_mmio_descriptor soc_fixed_resources[] = {
-		{ PCIEXBAR, CONFIG_MMCONF_BASE_ADDRESS, CONFIG_MMCONF_LENGTH,
+		{ PCIEXBAR, CONFIG_ECAM_MMCONF_BASE_ADDRESS, CONFIG_ECAM_MMCONF_LENGTH,
 				"PCIEXBAR" },
 		{ MCHBAR, MCH_BASE_ADDRESS, MCH_BASE_SIZE, "MCHBAR" },
 	};
@@ -32,13 +32,13 @@ void soc_add_fixed_mmio_resources(struct device *dev, int *index)
 		return;
 
 	if (MCHBAR32(GFXVTBAR) & VTBAR_ENABLED) {
-		mmio_resource(dev, *index,
+		mmio_resource_kb(dev, *index,
 			      (MCHBAR64(GFXVTBAR) & VTBAR_MASK) / KiB,
 			      VTBAR_SIZE / KiB);
 		(*index)++;
 	}
 	if (MCHBAR32(DEFVTBAR) & VTBAR_ENABLED) {
-		mmio_resource(dev, *index,
+		mmio_resource_kb(dev, *index,
 			      (MCHBAR64(DEFVTBAR) & VTBAR_MASK) / KiB,
 			      VTBAR_SIZE / KiB);
 		(*index)++;
