@@ -604,6 +604,6 @@ static const unsigned short pci_device_ids[] = {
 
 static const struct pci_driver igd_driver __pci_driver = {
 	.ops	 = &igd_ops,
-	.vendor	 = PCI_VENDOR_ID_INTEL,
+	.vendor	 = PCI_VID_INTEL,
 	.devices = pci_device_ids,
 };
