@@ -4,7 +4,6 @@
 #include <device/mmio.h>
 #include <device/device.h>
 #include <device/pci_ops.h>
-#include <intelblocks/dmi.h>
 #include <intelblocks/fast_spi.h>
 #include <intelblocks/gpio.h>
 #include <intelblocks/gspi.h>
@@ -34,11 +33,6 @@
 #define PCR_PSFX_TO_SHDW_PCIEN_IOEN	0x01
 #define PCR_PSFX_T0_SHDW_PCIEN	0x1C
 
-#define PCR_DMI_ACPIBA		0x27B4
-#define PCR_DMI_ACPIBDID	0x27B8
-#define PCR_DMI_PMBASEA		0x27AC
-#define PCR_DMI_PMBASEC		0x27B0
-
 static uint32_t get_pmc_reg_base(void)
 {
 	if (CONFIG(SOC_INTEL_CANNONLAKE_PCH_H))
