@@ -74,7 +74,7 @@ static void soc_pmc_read_resources(struct device *dev)
 	struct resource *res;
 
 	/* Add the fixed MMIO resource */
-	mmio_resource(dev, 0, PCH_PWRM_BASE_ADDRESS / KiB, PCH_PWRM_BASE_SIZE / KiB);
+	mmio_resource_kb(dev, 0, PCH_PWRM_BASE_ADDRESS / KiB, PCH_PWRM_BASE_SIZE / KiB);
 
 	/* Add the fixed I/O resource */
 	res = new_resource(dev, 1);
@@ -135,6 +135,22 @@ static void pmc_fill_ssdt(const struct device *dev)
 		generate_acpi_power_engine();
 }
 
+/*
+ * `pmc_final` function is native implementation of equivalent events performed by
+ * each FSP NotifyPhase() API invocations.
+ *
+ *
+ * Clear PMCON status bits (Global Reset/Power Failure/Host Reset Status bits)
+ *
+ * Perform the PMCON status bit clear operation from `.final`
+ * to cover any such chances where later boot stage requested a global
+ * reset and PMCON status bit remains set.
+ */
+static void pmc_final(struct device *dev)
+{
+	pmc_clear_pmcon_sts();
+}
+
 struct device_operations pmc_ops = {
 	.read_resources	  = soc_pmc_read_resources,
 	.set_resources	  = noop_set_resources,
@@ -144,4 +160,5 @@ struct device_operations pmc_ops = {
 	.acpi_fill_ssdt	  = pmc_fill_ssdt,
 #endif
 	.scan_bus	  = scan_static_bus,
+	.final		  = pmc_final,
 };
