@@ -5,7 +5,6 @@
 #include <console/console.h>
 #include <cpu/cpu.h>
 #include <cpu/x86/mtrr.h>
-#include <cpu/x86/msr.h>
 #include <cpu/x86/mp.h>
 #include <cpu/intel/microcode.h>
 #include <intelblocks/cfg.h>
@@ -15,12 +14,17 @@
 #include <intelblocks/msr.h>
 #include <soc/cpu.h>
 
+static void initialize_microcode(void)
+{
+	const void *microcode_patch = intel_microcode_find();
+	intel_microcode_load_unlocked(microcode_patch);
+}
+
 static void init_one_cpu(struct device *dev)
 {
 	soc_core_init(dev);
 
-	const void *microcode_patch = intel_microcode_find();
-	intel_microcode_load_unlocked(microcode_patch);
+	initialize_microcode();
 }
 
 static struct device_operations cpu_dev_ops = {
@@ -28,6 +32,8 @@ static struct device_operations cpu_dev_ops = {
 };
 
 static const struct cpu_device_id cpu_table[] = {
+	{ X86_VENDOR_INTEL, CPUID_METEORLAKE_A0_1},
+	{ X86_VENDOR_INTEL, CPUID_METEORLAKE_A0_2},
 	{ X86_VENDOR_INTEL, CPUID_SKYLAKE_C0 },
 	{ X86_VENDOR_INTEL, CPUID_SKYLAKE_D0 },
 	{ X86_VENDOR_INTEL, CPUID_SKYLAKE_HQ0 },
@@ -60,7 +66,8 @@ static const struct cpu_device_id cpu_table[] = {
 	{ X86_VENDOR_INTEL, CPUID_COMETLAKE_H_S_6_2_G0 },
 	{ X86_VENDOR_INTEL, CPUID_COMETLAKE_H_S_6_2_G1 },
 	{ X86_VENDOR_INTEL, CPUID_COMETLAKE_H_S_10_2_P0 },
-	{ X86_VENDOR_INTEL, CPUID_COMETLAKE_H_S_10_2_Q0_P1 },
+	{ X86_VENDOR_INTEL, CPUID_COMETLAKE_H_S_10_2_P1 },
+	{ X86_VENDOR_INTEL, CPUID_COMETLAKE_H_S_10_2_Q0 },
 	{ X86_VENDOR_INTEL, CPUID_TIGERLAKE_A0 },
 	{ X86_VENDOR_INTEL, CPUID_TIGERLAKE_B0 },
 	{ X86_VENDOR_INTEL, CPUID_TIGERLAKE_R0 },
@@ -68,10 +75,14 @@ static const struct cpu_device_id cpu_table[] = {
 	{ X86_VENDOR_INTEL, CPUID_ELKHARTLAKE_B0 },
 	{ X86_VENDOR_INTEL, CPUID_JASPERLAKE_A0 },
 	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_S_A0 },
-	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_A0 },
-	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_A1 },
-	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_A2 },
-	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_A3 },
+	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_J0 },
+	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_K0 },
+	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_L0 },
+	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_Q0 },
+	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_R0 },
+	{ X86_VENDOR_INTEL, CPUID_ALDERLAKE_N_A0 },
+	{ X86_VENDOR_INTEL, CPUID_RAPTORLAKE_P_J0 },
+	{ X86_VENDOR_INTEL, CPUID_RAPTORLAKE_P_Q0 },
 	{ 0, 0 },
 };
 
@@ -143,23 +154,63 @@ static void coreboot_init_cpus(void *unused)
 	if (CONFIG(USE_INTEL_FSP_MP_INIT))
 		return;
 
-	const void *microcode_patch = intel_microcode_find();
-	intel_microcode_load_unlocked(microcode_patch);
+	initialize_microcode();
 
 	init_cpus();
 }
 
+static void post_cpus_add_romcache(void)
+{
+	if (!CONFIG(BOOT_DEVICE_MEMORY_MAPPED))
+		return;
+
+	fast_spi_cache_bios_region();
+}
+
 static void wrapper_x86_setup_mtrrs(void *unused)
 {
 	x86_setup_mtrrs_with_detect();
 }
 
+static void wrapper_set_bios_done(void *unused)
+{
+	cpu_soc_bios_done();
+}
+
+static void wrapper_init_core_prmrr(void *unused)
+{
+	init_core_prmrr();
+}
+
+void before_post_cpus_init(void)
+{
+	/*
+	 * Ensure all APs finish the task and continue if coreboot decides to
+	 * perform multiprocessor initialization using native coreboot drivers
+	 * instead using FSP MP PPI implementation.
+	 *
+	 * Ignore if USE_COREBOOT_MP_INIT is not enabled.
+	 */
+	if (!CONFIG(USE_COREBOOT_MP_INIT))
+		return;
+
+	if (mp_run_on_all_cpus(wrapper_init_core_prmrr, NULL) != CB_SUCCESS)
+		printk(BIOS_ERR, "core PRMRR sync failure\n");
+
+	if (mp_run_on_all_cpus(wrapper_set_bios_done, NULL) != CB_SUCCESS)
+		printk(BIOS_ERR, "Set BIOS Done failure\n");
+
+	intel_reload_microcode();
+}
+
 /* Ensure to re-program all MTRRs based on DRAM resource settings */
 static void post_cpus_init(void *unused)
 {
-	if (mp_run_on_all_cpus(&wrapper_x86_setup_mtrrs, NULL) != CB_SUCCESS)
+	/* Ensure all APs finish the task and continue */
+	if (mp_run_on_all_cpus_synchronously(&wrapper_x86_setup_mtrrs, NULL) != CB_SUCCESS)
 		printk(BIOS_ERR, "MTRR programming failure\n");
 
+	post_cpus_add_romcache();
 	x86_mtrr_check();
 }
 
