@@ -2,21 +2,29 @@
 
 #define __SIMPLE_DEVICE__
 
+#include <acpi/acpi.h>
+#include <acpi/acpigen.h>
 #include <arch/romstage.h>
 #include <device/mmio.h>
 #include <assert.h>
+#include <device/pci.h>
 #include <device/pci_def.h>
+#include <device/pci_ids.h>
 #include <device/pci_ops.h>
+#include <console/console.h>
 #include <commonlib/helpers.h>
 #include <cpu/x86/mtrr.h>
 #include <fast_spi_def.h>
-#include <intelblocks/dmi.h>
 #include <intelblocks/fast_spi.h>
+#include <intelblocks/gpmr.h>
 #include <lib.h>
 #include <soc/pci_devs.h>
 #include <spi_flash.h>
 #include <spi-generic.h>
 
+#define FLASH_MMIO_SIZE		(16 * MiB)
+#define FLASH_BASE_ADDR		((0xffffffff - FLASH_MMIO_SIZE) + 1)
+
 /*
  * Get the FAST_SPIBAR.
  */
@@ -169,6 +177,23 @@ void fast_spi_pr_dlock(void)
 	write32(spibar + SPIBAR_DLOCK, dlock);
 }
 
+/*
+ * Set FAST_SPIBAR + VSCC0 (0xC4) register VCL (bit 30).
+ */
+void fast_spi_vscc0_lock(void)
+{
+	void *spibar = fast_spi_get_bar();
+
+	/*
+	 * SPI Flash Programming Guide Section 5.5.2 describes Vendor Component Lock (VCL).
+	 * It is recommended to set the VCL bit. VCL applies to both VSCC0 and VSCC1.
+	 * Without this bit being set, it is possible to modify Host/GbE VSCC register(s),
+	 * which might results in undesired host and integrated GbE Serial Flash
+	 * functionality.
+	 */
+	setbits32(spibar + SPIBAR_SFDP0_VSCC0, SPIBAR_VSCC0_VCL);
+}
+
 /*
  * Set FAST_SPIBAR Soft Reset Data Register value.
  */
@@ -196,6 +221,23 @@ void fast_spi_set_strap_msg_data(uint32_t soft_reset_data)
 	write32(spibar + SPIBAR_RESET_LOCK, ssl);
 }
 
+static void fast_spi_enable_cache_range(unsigned int base, unsigned int size)
+{
+	if (ENV_RAMSTAGE) {
+		mtrr_use_temp_range(base, size, MTRR_TYPE_WRPROT);
+		return;
+	}
+
+	const int type = MTRR_TYPE_WRPROT;
+	int mtrr = get_free_var_mtrr();
+	if (mtrr == -1) {
+		printk(BIOS_WARNING, "ROM caching failed due to no free MTRR available!\n");
+		return;
+	}
+
+	set_var_mtrr(mtrr, base, size, type);
+}
+
 /*
  * Returns bios_start and fills in size of the BIOS region.
  */
@@ -240,15 +282,11 @@ static void fast_spi_cache_ext_bios_window(void)
 {
 	size_t ext_bios_size;
 	uintptr_t ext_bios_base;
-	const int type = MTRR_TYPE_WRPROT;
 
 	if (!fast_spi_ext_bios_cache_range(&ext_bios_base, &ext_bios_size))
 		return;
 
-	int mtrr = get_free_var_mtrr();
-	if (mtrr == -1)
-		return;
-	set_var_mtrr(mtrr, ext_bios_base, ext_bios_size, type);
+	fast_spi_enable_cache_range(ext_bios_base, ext_bios_size);
 }
 
 void fast_spi_cache_ext_bios_postcar(struct postcar_frame *pcf)
@@ -267,7 +305,6 @@ void fast_spi_cache_bios_region(void)
 {
 	size_t bios_size;
 	uint32_t alignment;
-	const int type = MTRR_TYPE_WRPROT;
 	uintptr_t base;
 
 	/* Only the IFD BIOS region is memory mapped (at top of 4G) */
@@ -286,16 +323,7 @@ void fast_spi_cache_bios_region(void)
 	bios_size = ALIGN_UP(bios_size, alignment);
 	base = 4ULL*GiB - bios_size;
 
-	if (ENV_PAYLOAD_LOADER) {
-		mtrr_use_temp_range(base, bios_size, type);
-	} else {
-		int mtrr = get_free_var_mtrr();
-
-		if (mtrr == -1)
-			return;
-
-		set_var_mtrr(mtrr, base, bios_size, type);
-	}
+	fast_spi_enable_cache_range(base, bios_size);
 
 	/* Check if caching is needed for extended bios region if supported */
 	fast_spi_cache_ext_bios_window();
@@ -326,7 +354,7 @@ static void fast_spi_enable_ext_bios(void)
 #endif
 
 	/* Configure Source decode for Extended BIOS Region */
-	if (dmi_enable_gpmr(CONFIG_EXT_BIOS_WIN_BASE, CONFIG_EXT_BIOS_WIN_SIZE,
+	if (enable_gpmr(CONFIG_EXT_BIOS_WIN_BASE, CONFIG_EXT_BIOS_WIN_SIZE,
 				soc_get_spi_psf_destination_id()) == CB_ERR)
 		return;
 
@@ -429,3 +457,117 @@ void fast_spi_disable_wp(void)
 	bios_cntl |= SPI_BIOS_CONTROL_WPD;
 	pci_write_config8(dev, SPI_BIOS_CONTROL, bios_cntl);
 }
+
+void fast_spi_clear_outstanding_status(void)
+{
+	void *spibar = fast_spi_get_bar();
+
+	/* Make sure all W1C status bits get cleared. */
+	write32(spibar + SPIBAR_HSFSTS_CTL, SPIBAR_HSFSTS_W1C_BITS);
+}
+
+
+/* As there is no official ACPI ID for this controller use the generic PNP ID for now. */
+static const char *fast_spi_acpi_hid(const struct device *dev)
+{
+	return "PNP0C02";
+}
+
+static const char *fast_spi_acpi_name(const struct device *dev)
+{
+	return "FSPI";
+}
+
+/*
+ * Generate an ACPI entry for the SPI controller. This way the allocated resources
+ * for the SPI controller can be communicated to the OS even if the device is
+ * not visible on PCI (because it is hidden) and therefore can not be probed by the OS.
+ */
+static void fast_spi_fill_ssdt(const struct device *dev)
+{
+	const char *scope = acpi_device_scope(dev);
+	const char *hid = fast_spi_acpi_hid(dev);
+	struct resource *res;
+
+	/* Do not add SSDT if the fast SPI device is hidden. */
+	if (dev->hidden || !CONFIG(FAST_SPI_GENERATE_SSDT))
+		return;
+	if (!scope || !hid)
+		return;
+
+	res = probe_resource(dev, PCI_BASE_ADDRESS_0);
+	if (!res)
+		return;
+
+	/* Scope */
+	acpigen_write_scope(scope);
+
+	/* Device */
+	acpigen_write_device(acpi_device_name(dev));
+	acpigen_write_name_string("_HID", hid);
+	acpi_device_write_uid(dev);
+	acpigen_write_name_string("_DDN", "ACPI Fast SPI");
+	acpigen_write_STA(acpi_device_status(dev));
+
+	/* Resources */
+	acpigen_write_name("_CRS");
+	acpigen_write_resourcetemplate_header();
+
+	/* Add BAR0 resource. */
+	acpigen_write_mem32fixed(1, res->base, res->size);
+
+	acpigen_write_resourcetemplate_footer();
+
+	acpigen_pop_len(); /* Device */
+	acpigen_pop_len(); /* Scope */
+}
+
+
+static void fast_spi_read_resources(struct device *dev)
+{
+	/* Read standard PCI resources. */
+	pci_dev_read_resources(dev);
+
+	/* Add SPI flash MMIO window as a reserved resource. */
+	mmio_resource_kb(dev, 0, FLASH_BASE_ADDR / KiB, FLASH_MMIO_SIZE / KiB);
+	/* Add extended SPI flash MMIO window as reserved resource if enabled. */
+	if (CONFIG(FAST_SPI_SUPPORTS_EXT_BIOS_WINDOW)) {
+		mmio_resource_kb(dev, 1, CONFIG_EXT_BIOS_WIN_BASE / KiB,
+				 CONFIG_EXT_BIOS_WIN_SIZE / KiB);
+	}
+}
+
+static struct device_operations fast_spi_dev_ops = {
+	.read_resources			= fast_spi_read_resources,
+	.set_resources			= pci_dev_set_resources,
+	.enable_resources		= pci_dev_enable_resources,
+	.acpi_fill_ssdt			= fast_spi_fill_ssdt,
+	.acpi_name			= fast_spi_acpi_name,
+};
+
+static const unsigned short pci_device_ids[] = {
+	PCI_DID_INTEL_ADP_M_N_HWSEQ_SPI,
+	PCI_DID_INTEL_ADP_P_HWSEQ_SPI,
+	PCI_DID_INTEL_ADP_S_HWSEQ_SPI,
+	PCI_DID_INTEL_APL_HWSEQ_SPI,
+	PCI_DID_INTEL_GLK_HWSEQ_SPI,
+	PCI_DID_INTEL_CMP_HWSEQ_SPI,
+	PCI_DID_INTEL_CMP_H_HWSEQ_SPI,
+	PCI_DID_INTEL_CNL_HWSEQ_SPI,
+	PCI_DID_INTEL_CNP_H_HWSEQ_SPI,
+	PCI_DID_INTEL_ICP_HWSEQ_SPI,
+	PCI_DID_INTEL_JSP_HWSEQ_SPI,
+	PCI_DID_INTEL_LWB_SPI,
+	PCI_DID_INTEL_LWB_SPI_SUPER,
+	PCI_DID_INTEL_MCC_SPI0,
+	PCI_DID_INTEL_MTL_HWSEQ_SPI,
+	PCI_DID_INTEL_SPR_HWSEQ_SPI,
+	PCI_DID_INTEL_TGP_SPI0,
+	0
+};
+
+static const struct pci_driver fast_spi __pci_driver = {
+	.ops				= &fast_spi_dev_ops,
+	.vendor				= PCI_VID_INTEL,
+	.devices			= pci_device_ids,
+};
