@@ -17,9 +17,6 @@
 #define MSR_BIOS_UPGD_TRIG	0x7a
 #define  SGX_ACTIVATE_BIT	(1)
 #define MSR_PMG_IO_CAPTURE_BASE	0xe4
-#define MSR_POWER_MISC		0x120
-#define  ENABLE_IA_UNTRUSTED	(1 << 6)
-#define  FLUSH_DL1_L2		(1 << 8)
 #define MSR_EMULATE_PM_TIMER	0x121
 #define  EMULATE_DELAY_OFFSET_VALUE	20
 #define  EMULATE_PM_TMR_EN	(1 << 16)
@@ -51,6 +48,7 @@
 #define MSR_PRMRR_VALID_CONFIG		0x1fb
 #define MSR_POWER_CTL			0x1fc
 #define  POWER_CTL_C1E_MASK		(1 << 1)
+#define MSR_PRMRR_BASE_0		0x2a0
 #define MSR_EVICT_CTL			0x2e0
 #define MSR_LT_CONTROL			0x2e7
 #define  LT_CONTROL_LOCK		(1 << 0)
@@ -96,6 +94,8 @@
 #define PKG_POWER_LIMIT_TIME_MASK	(0x7f)
 #define PKG_POWER_LIMIT_DUTYCYCLE_SHIFT 24
 #define PKG_POWER_LIMIT_DUTYCYCLE_MASK  (0x7f)
+
+#define MSR_CORE_MKTME_ACTIVATION	0x9ff
 /* SMM save state MSRs */
 #define SMBASE_MSR			0xc20
 #define IEDBASE_MSR			0xc22
@@ -108,13 +108,6 @@
 #define SMRR_LOCK_SUPPORTED		(1<<14)
 
 #define SGX_SUPPORTED	(1<<2)
-/* Intel SDM: Table 36-6.
- * CPUID Leaf 12H, Sub-Leaf Index 2 or Higher for enumeration of
- * SGX Resources. Same Table  mentions about return values of the CPUID */
-#define SGX_RESOURCE_ENUM_CPUID_LEAF	(0x12)
-#define SGX_RESOURCE_ENUM_CPUID_SUBLEAF	(0x2)
-#define SGX_RESOURCE_ENUM_BIT	(0x1)
-#define SGX_RESOURCE_MASK_LO	(0xfffff000UL)
-#define SGX_RESOURCE_MASK_HI	(0xfffffUL)
+#define TME_SUPPORTED	(1<<13)
 
 #endif	/* SOC_INTEL_COMMON_MSR_H */
