@@ -1,18 +1,4 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2017 Intel Corp.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 #ifndef _SOC_COMMON_BLOCK_LPC_DEF_H_
 #define _SOC_COMMON_BLOCK_LPC_DEF_H_
@@ -21,8 +7,14 @@
 #define  LPC_SCNT_EN			(1 << 7)
 #define  LPC_SCNT_MODE			(1 << 6)
 #define LPC_IO_DECODE			0x80
-#define  LPC_IOD_COMA_RANGE             (0 << 0) /* 0x3F8 - 0x3FF COMA*/
-#define  LPC_IOD_COMB_RANGE             (1 << 4) /* 0x2F8 - 0x2FF COMB*/
+#define  LPC_IOD_FDD_RANGE		(0 << 12)
+#define  LPC_IOD_FDD_RANGE_MASK		(1 << 12)
+#define  LPC_IOD_LPT_RANGE		(0 << 8)
+#define  LPC_IOD_LPT_RANGE_MASK		(3 << 8)
+#define  LPC_IOD_COMA_RANGE_MASK	(7 << 0)
+#define  LPC_IOD_COMB_RANGE_MASK	(7 << 4)
+#define  LPC_IOD_COMA_RANGE		(0 << 0) /* 0x3F8 - 0x3FF COMA */
+#define  LPC_IOD_COMB_RANGE		(1 << 4) /* 0x2F8 - 0x2FF COMB */
 /* Use IO_<peripheral>_<IO port> style macros defined in lpc_lib.h
  * to enable decoding of I/O locations for a peripheral. */
 #define LPC_IO_ENABLES			0x82
@@ -37,7 +29,8 @@
 #define LPC_LGMR_WINDOW_SIZE		(64 * KiB)
 #define LPC_BIOS_CNTL			0xdc
 #define  LPC_BC_BILD			(1 << 7) /* BILD */
-#define  LPC_BC_LE			(1 << 2) /* LE */
+#define  LPC_BC_LE			(1 << 1) /* LE */
+#define  LPC_BC_WPD			(1 << 0) /* WPD */
 #define  LPC_BC_EISS			(1 << 5) /* EISS */
 #define LPC_PCCTL			0xE0 /* PCI Clock Control */
 #define  LPC_PCCTL_CLKRUN_EN		(1 << 0)
