@@ -1,11 +1,14 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
+#define __SIMPLE_DEVICE__
+
 #include <device/pci_ops.h>
 #include <console/console.h>
 #include <device/device.h>
 #include <device/pci.h>
 #include <device/pci_ids.h>
 #include <intelblocks/p2sb.h>
+#include <intelblocks/p2sblib.h>
 #include <soc/iomap.h>
 #include <soc/p2sb.h>
 #include <soc/pci_devs.h>
@@ -13,29 +16,9 @@
 
 #define PCH_P2SB_EPMASK(mask_number) (PCH_P2SB_EPMASK0 + ((mask_number) * 4))
 
-#define HIDE_BIT (1 << 0)
-
-static bool p2sb_is_hidden(void)
-{
-	const uint16_t pci_vid = pci_read_config16(PCH_DEV_P2SB, PCI_VENDOR_ID);
-
-	if (pci_vid == 0xffff)
-		return true;
-	if (pci_vid == PCI_VENDOR_ID_INTEL)
-		return false;
-	printk(BIOS_ERR, "P2SB PCI_VENDOR_ID is invalid, unknown if hidden\n");
-	return true;
-}
-
 void p2sb_enable_bar(void)
 {
-	/* Enable PCR Base address in PCH */
-	pci_write_config32(PCH_DEV_P2SB, PCI_BASE_ADDRESS_0, P2SB_BAR);
-	pci_write_config32(PCH_DEV_P2SB, PCI_BASE_ADDRESS_1, 0);
-
-	/* Enable P2SB MSE */
-	pci_write_config16(PCH_DEV_P2SB, PCI_COMMAND,
-			  PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY);
+	p2sb_dev_enable_bar(PCH_DEV_P2SB, P2SB_BAR);
 }
 
 /*
@@ -56,7 +39,7 @@ void p2sb_configure_hpet(void)
 
 union p2sb_bdf p2sb_get_hpet_bdf(void)
 {
-	const bool was_hidden = p2sb_is_hidden();
+	const bool was_hidden = p2sb_dev_is_hidden(PCH_DEV_P2SB);
 	if (was_hidden)
 		p2sb_unhide();
 
@@ -75,7 +58,7 @@ void p2sb_set_hpet_bdf(union p2sb_bdf bdf)
 
 union p2sb_bdf p2sb_get_ioapic_bdf(void)
 {
-	const bool was_hidden = p2sb_is_hidden();
+	const bool was_hidden = p2sb_dev_is_hidden(PCH_DEV_P2SB);
 	if (was_hidden)
 		p2sb_unhide();
 
@@ -92,35 +75,14 @@ void p2sb_set_ioapic_bdf(union p2sb_bdf bdf)
 	pci_write_config16(PCH_DEV_P2SB, PCH_P2SB_IBDF, bdf.raw);
 }
 
-static void p2sb_set_hide_bit(int hide)
-{
-	const uint16_t reg = PCH_P2SB_E0 + 1;
-	const uint8_t mask = HIDE_BIT;
-	uint8_t val;
-
-	val = pci_read_config8(PCH_DEV_P2SB, reg);
-	val &= ~mask;
-	if (hide)
-		val |= mask;
-	pci_write_config8(PCH_DEV_P2SB, reg, val);
-}
-
 void p2sb_unhide(void)
 {
-	p2sb_set_hide_bit(0);
-
-	if (p2sb_is_hidden())
-		die_with_post_code(POST_HW_INIT_FAILURE,
-				   "Unable to unhide PCH_DEV_P2SB device !\n");
+	p2sb_dev_unhide(PCH_DEV_P2SB);
 }
 
 void p2sb_hide(void)
 {
-	p2sb_set_hide_bit(1);
-
-	if (!p2sb_is_hidden())
-		die_with_post_code(POST_HW_INIT_FAILURE,
-				   "Unable to hide PCH_DEV_P2SB device !\n");
+	p2sb_dev_hide(PCH_DEV_P2SB);
 }
 
 static void p2sb_configure_endpoints(int epmask_id, uint32_t mask)
@@ -166,40 +128,39 @@ static void read_resources(struct device *dev)
 	 * The following code makes sure that it doesn't change if the device
 	 * is visible and the resource allocator is being run.
 	 */
-	mmio_resource(dev, PCI_BASE_ADDRESS_0, P2SB_BAR / KiB, P2SB_SIZE / KiB);
+	mmio_resource_kb(dev, PCI_BASE_ADDRESS_0, P2SB_BAR / KiB, P2SB_SIZE / KiB);
 }
 
-static const struct device_operations device_ops = {
+const struct device_operations p2sb_ops = {
 	.read_resources		= read_resources,
 	.set_resources		= noop_set_resources,
 	.ops_pci		= &pci_dev_ops_pci,
 };
 
 static const unsigned short pci_device_ids[] = {
-	PCI_DEVICE_ID_INTEL_APL_P2SB,
-	PCI_DEVICE_ID_INTEL_GLK_P2SB,
-	PCI_DEVICE_ID_INTEL_LWB_P2SB,
-	PCI_DEVICE_ID_INTEL_LWB_P2SB_SUPER,
-	PCI_DEVICE_ID_INTEL_SKL_LP_P2SB,
-	PCI_DEVICE_ID_INTEL_SKL_P2SB,
-	PCI_DEVICE_ID_INTEL_KBL_P2SB,
-	PCI_DEVICE_ID_INTEL_CNL_P2SB,
-	PCI_DEVICE_ID_INTEL_CNP_H_P2SB,
-	PCI_DEVICE_ID_INTEL_ICL_P2SB,
-	PCI_DEVICE_ID_INTEL_CMP_P2SB,
-	PCI_DEVICE_ID_INTEL_CMP_H_P2SB,
-	PCI_DEVICE_ID_INTEL_TGL_P2SB,
-	PCI_DEVICE_ID_INTEL_TGL_H_P2SB,
-	PCI_DEVICE_ID_INTEL_EHL_P2SB,
-	PCI_DEVICE_ID_INTEL_JSP_P2SB,
-	PCI_DEVICE_ID_INTEL_ADP_P_P2SB,
-	PCI_DEVICE_ID_INTEL_ADP_S_P2SB,
-	PCI_DEVICE_ID_INTEL_ADP_M_P2SB,
+	PCI_DID_INTEL_MTL_SOC_P2SB,
+	PCI_DID_INTEL_RPP_P_P2SB,
+	PCI_DID_INTEL_APL_P2SB,
+	PCI_DID_INTEL_GLK_P2SB,
+	PCI_DID_INTEL_LWB_P2SB,
+	PCI_DID_INTEL_LWB_P2SB_SUPER,
+	PCI_DID_INTEL_CNL_P2SB,
+	PCI_DID_INTEL_CNP_H_P2SB,
+	PCI_DID_INTEL_ICL_P2SB,
+	PCI_DID_INTEL_CMP_P2SB,
+	PCI_DID_INTEL_CMP_H_P2SB,
+	PCI_DID_INTEL_TGL_P2SB,
+	PCI_DID_INTEL_TGL_H_P2SB,
+	PCI_DID_INTEL_EHL_P2SB,
+	PCI_DID_INTEL_JSP_P2SB,
+	PCI_DID_INTEL_ADP_P_P2SB,
+	PCI_DID_INTEL_ADP_S_P2SB,
+	PCI_DID_INTEL_ADP_M_P2SB,
 	0,
 };
 
 static const struct pci_driver pmc __pci_driver = {
-	.ops		= &device_ops,
-	.vendor		= PCI_VENDOR_ID_INTEL,
+	.ops		= &p2sb_ops,
+	.vendor		= PCI_VID_INTEL,
 	.devices	= pci_device_ids,
 };
