@@ -5,20 +5,26 @@
 #include <assert.h>
 #include <bootmode.h>
 #include <device/mmio.h>
+#include <device/pci.h>
 #include <cbmem.h>
 #include <cpu/x86/smm.h>
 #include <console/console.h>
 #include <halt.h>
+#include <intelblocks/pmc_ipc.h>
 #include <intelblocks/pmclib.h>
 #include <intelblocks/gpio.h>
 #include <intelblocks/tco.h>
 #include <option.h>
 #include <security/vboot/vboot_common.h>
+#include <soc/pci_devs.h>
 #include <soc/pm.h>
 #include <stdint.h>
 #include <string.h>
 #include <timer.h>
 
+#define PMC_IPC_BIOS_RST_COMPLETE		0xd0
+#define PMC_IPC_BIOS_RST_SUBID_PCI_ENUM_DONE	0
+
 static struct chipset_power_state power_state;
 
 /* List of Minimum Assertion durations in microseconds */
@@ -79,7 +85,7 @@ static void migrate_power_state(int is_recovery)
 	}
 	memcpy(ps_cbmem, &power_state, sizeof(*ps_cbmem));
 }
-ROMSTAGE_CBMEM_INIT_HOOK(migrate_power_state)
+CBMEM_CREATION_HOOK(migrate_power_state);
 
 static void print_num_status_bits(int num_bits, uint32_t status,
 				  const char *const bit_names[])
@@ -238,9 +244,9 @@ static uint16_t print_pm1_status(uint16_t pm1_sts)
 	if (!pm1_sts)
 		return 0;
 
-	printk(BIOS_SPEW, "PM1_STS: ");
+	printk(BIOS_DEBUG, "PM1_STS: ");
 	print_num_status_bits(ARRAY_SIZE(pm1_sts_bits), pm1_sts, pm1_sts_bits);
-	printk(BIOS_SPEW, "\n");
+	printk(BIOS_DEBUG, "\n");
 
 	return pm1_sts;
 }
@@ -397,6 +403,9 @@ static int pmc_prev_sleep_state(const struct chipset_power_state *ps)
 			if (CONFIG(HAVE_ACPI_RESUME))
 				prev_sleep_state = ACPI_S3;
 			break;
+		case ACPI_S4:
+			prev_sleep_state = ACPI_S4;
+			break;
 		case ACPI_S5:
 			prev_sleep_state = ACPI_S5;
 			break;
@@ -466,6 +475,11 @@ void pmc_global_reset_enable(bool enable)
 
 int platform_is_resuming(void)
 {
+	/* Read power state from PMC data structure */
+	if (ENV_RAMSTAGE)
+		return acpi_get_sleep_type() == ACPI_S3;
+
+	/* Read power state from PMC ABASE */
 	if (!(inw(ACPI_BASE_ADDRESS + PM1_STS) & WAK_STS))
 		return 0;
 
@@ -580,6 +594,44 @@ void pmc_gpe_init(void)
 	gpio_route_gpe(dw0, dw1, dw2);
 }
 
+#if ENV_RAMSTAGE
+static void pmc_clear_pmcon_sts_mmio(void)
+{
+	uint8_t *addr = pmc_mmio_regs();
+
+	clrbits32((addr + GEN_PMCON_A), MS4V);
+}
+
+static void pmc_clear_pmcon_sts_pci(void)
+{
+	struct device *dev = pcidev_path_on_root(PCH_DEVFN_PMC);
+	if (!dev)
+		return;
+
+	pci_and_config32(dev, GEN_PMCON_A, ~MS4V);
+}
+
+/*
+ * Clear PMC GEN_PMCON_A register status bits:
+ * SUS_PWR_FLR, GBL_RST_STS, HOST_RST_STS, PWR_FLR bits
+ * while retaining MS4V write-1-to-clear bit
+ */
+void pmc_clear_pmcon_sts(void)
+{
+	/*
+	 * Accessing PMC GEN_PMCON_A register differs between different Intel chipsets.
+	 * Typically, there are two possible ways to perform GEN_PMCON_A register programming
+	 * (like `pmc_clear_pmcon_sts()`) as:
+	 * 1. Using PCI configuration space when GEN_PMCON_A is a PCI configuration register.
+	 * 2. Using MMIO access when GEN_PMCON_A is a memory mapped register.
+	 */
+	if (CONFIG(SOC_INTEL_MEM_MAPPED_PM_CONFIGURATION))
+		pmc_clear_pmcon_sts_mmio();
+	else
+		pmc_clear_pmcon_sts_pci();
+}
+#endif
+
 void pmc_set_power_failure_state(const bool target_on)
 {
 	const unsigned int state = get_uint_option("power_on_after_fail",
@@ -611,7 +663,7 @@ void pmc_set_power_failure_state(const bool target_on)
 		pmc_soc_set_afterg3_en(target_on);
 		break;
 	default:
-		printk(BIOS_WARNING, "WARNING: Unknown power-failure state: %d\n", state);
+		printk(BIOS_WARNING, "Unknown power-failure state: %d\n", state);
 		break;
 	}
 }
@@ -742,3 +794,15 @@ enum pch_pmc_xtal pmc_get_xtal_freq(void)
 		return XTAL_UNKNOWN_FREQ;
 	}
 }
+
+void pmc_send_pci_enum_done(void)
+{
+	struct pmc_ipc_buffer req = { 0 };
+	struct pmc_ipc_buffer rsp;
+	uint32_t cmd;
+
+	cmd = pmc_make_ipc_cmd(PMC_IPC_BIOS_RST_COMPLETE,
+			 PMC_IPC_BIOS_RST_SUBID_PCI_ENUM_DONE, 0);
+	if (pmc_send_ipc_cmd(cmd, &req, &rsp) != CB_SUCCESS)
+		printk(BIOS_ERR, "PMC: Failed sending PCI Enumeration Done Command\n");
+}
