@@ -38,7 +38,7 @@ static void sd_fill_ssdt(const struct device *dev)
 }
 #endif
 
-static struct device_operations dev_ops = {
+struct device_operations sd_ops = {
 	.read_resources		= pci_dev_read_resources,
 	.set_resources		= pci_dev_set_resources,
 	.enable_resources	= pci_dev_enable_resources,
@@ -49,21 +49,20 @@ static struct device_operations dev_ops = {
 };
 
 static const unsigned short pci_device_ids[] = {
-	PCI_DEVICE_ID_INTEL_APL_SD,
-	PCI_DEVICE_ID_INTEL_CNL_SD,
-	PCI_DEVICE_ID_INTEL_GLK_SD,
-	PCI_DEVICE_ID_INTEL_SKL_SD,
-	PCI_DEVICE_ID_INTEL_CNP_H_SD,
-	PCI_DEVICE_ID_INTEL_ICL_SD,
-	PCI_DEVICE_ID_INTEL_CMP_SD,
-	PCI_DEVICE_ID_INTEL_CMP_H_SD,
-	PCI_DEVICE_ID_INTEL_MCC_SD,
-	PCI_DEVICE_ID_INTEL_JSP_SD,
+	PCI_DID_INTEL_APL_SD,
+	PCI_DID_INTEL_CNL_SD,
+	PCI_DID_INTEL_GLK_SD,
+	PCI_DID_INTEL_CNP_H_SD,
+	PCI_DID_INTEL_ICL_SD,
+	PCI_DID_INTEL_CMP_SD,
+	PCI_DID_INTEL_CMP_H_SD,
+	PCI_DID_INTEL_MCC_SD,
+	PCI_DID_INTEL_JSP_SD,
 	0
 };
 
 static const struct pci_driver pch_sd __pci_driver = {
-	.ops	= &dev_ops,
-	.vendor	= PCI_VENDOR_ID_INTEL,
+	.ops	= &sd_ops,
+	.vendor	= PCI_VID_INTEL,
 	.devices	= pci_device_ids
 };
