@@ -2,8 +2,6 @@
 
 #include <arch/hlt.h>
 #include <arch/io.h>
-#include <arch/mmio.h>
-#include <device/pci_ops.h>
 #include <console/console.h>
 #include <cpu/x86/cache.h>
 #include <cpu/x86/msr.h>
@@ -12,7 +10,9 @@
 #include <cpu/intel/em64t101_save_state.h>
 #include <cpu/intel/msr.h>
 #include <delay.h>
+#include <device/mmio.h>
 #include <device/pci_def.h>
+#include <device/pci_ops.h>
 #include <elog.h>
 #include <intelblocks/fast_spi.h>
 #include <intelblocks/pmclib.h>
@@ -311,6 +311,25 @@ static void southbridge_smi_store(
 	}
 }
 
+__weak const struct gpio_lock_config *soc_gpio_lock_config(size_t *num)
+{
+	*num = 0;
+	return NULL;
+}
+
+static void soc_lock_gpios(void)
+{
+	const struct gpio_lock_config *soc_gpios;
+	size_t soc_gpio_num;
+
+	/* get list of gpios from SoC */
+	soc_gpios = soc_gpio_lock_config(&soc_gpio_num);
+
+	/* Lock any soc requested gpios */
+	if (soc_gpio_num)
+		gpio_lock_pads(soc_gpios, soc_gpio_num);
+}
+
 static void finalize(void)
 {
 	static int finalize_done;
@@ -337,6 +356,10 @@ static void finalize(void)
 	 */
 	mainboard_smi_finalize();
 
+	/* Lock down all GPIOs that may have been requested by the SoC and/or the mainboard. */
+	if (CONFIG(SOC_INTEL_COMMON_BLOCK_SMM_LOCK_GPIO_PADS))
+		soc_lock_gpios();
+
 	/* Specific SOC SMI handler during ramstage finalize phase */
 	smihandler_soc_at_finalize();
 }
