@@ -33,23 +33,27 @@ static const struct device_operations device_ops = {
 };
 
 static const unsigned short pci_device_ids[] = {
-	PCI_DEVICE_ID_INTEL_APL_SRAM,
-	PCI_DEVICE_ID_INTEL_GLK_SRAM,
-	PCI_DEVICE_ID_INTEL_ICL_SRAM,
-	PCI_DEVICE_ID_INTEL_CMP_SRAM,
-	PCI_DEVICE_ID_INTEL_CMP_H_SRAM,
-	PCI_DEVICE_ID_INTEL_TGL_SRAM,
-	PCI_DEVICE_ID_INTEL_TGL_H_SRAM,
-	PCI_DEVICE_ID_INTEL_MCC_SRAM,
-	PCI_DEVICE_ID_INTEL_JSP_SRAM,
-	PCI_DEVICE_ID_INTEL_ADP_P_SRAM,
-	PCI_DEVICE_ID_INTEL_ADP_S_SRAM,
-	PCI_DEVICE_ID_INTEL_ADP_M_SRAM,
+	PCI_DID_INTEL_MTL_SOC_SRAM,
+	PCI_DID_INTEL_MTL_IOE_M_SRAM,
+	PCI_DID_INTEL_MTL_IOE_P_SRAM,
+	PCI_DID_INTEL_MTL_CRASHLOG_SRAM,
+	PCI_DID_INTEL_APL_SRAM,
+	PCI_DID_INTEL_GLK_SRAM,
+	PCI_DID_INTEL_ICL_SRAM,
+	PCI_DID_INTEL_CMP_SRAM,
+	PCI_DID_INTEL_CMP_H_SRAM,
+	PCI_DID_INTEL_TGP_PMC_CRASHLOG_SRAM,
+	PCI_DID_INTEL_TGL_H_SRAM,
+	PCI_DID_INTEL_MCC_SRAM,
+	PCI_DID_INTEL_JSP_SRAM,
+	PCI_DID_INTEL_ADP_S_PMC_CRASHLOG_SRAM,
+	PCI_DID_INTEL_ADP_P_PMC_CRASHLOG_SRAM,
+	PCI_DID_INTEL_ADP_N_PMC_CRASHLOG_SRAM,
 	0,
 };
 
 static const struct pci_driver sram __pci_driver = {
 	.ops		= &device_ops,
-	.vendor		= PCI_VENDOR_ID_INTEL,
+	.vendor		= PCI_VID_INTEL,
 	.devices	= pci_device_ids,
 };
