@@ -18,7 +18,7 @@ bool xdci_can_enable(unsigned int xdci_devfn)
 	return is_devfn_enabled(xdci_devfn);
 }
 
-static struct device_operations usb_xdci_ops = {
+struct device_operations usb_xdci_ops = {
 	.read_resources		= pci_dev_read_resources,
 	.set_resources		= pci_dev_set_resources,
 	.enable_resources	= pci_dev_enable_resources,
@@ -27,26 +27,26 @@ static struct device_operations usb_xdci_ops = {
 };
 
 static const unsigned short pci_device_ids[] = {
-	PCI_DEVICE_ID_INTEL_APL_XDCI,
-	PCI_DEVICE_ID_INTEL_CNL_LP_XDCI,
-	PCI_DEVICE_ID_INTEL_GLK_XDCI,
-	PCI_DEVICE_ID_INTEL_SPT_LP_XDCI,
-	PCI_DEVICE_ID_INTEL_CNP_H_XDCI,
-	PCI_DEVICE_ID_INTEL_ICP_LP_XDCI,
-	PCI_DEVICE_ID_INTEL_CMP_LP_XDCI,
-	PCI_DEVICE_ID_INTEL_CMP_H_XDCI,
-	PCI_DEVICE_ID_INTEL_TGP_LP_XDCI,
-	PCI_DEVICE_ID_INTEL_TGP_H_XDCI,
-	PCI_DEVICE_ID_INTEL_MCC_XDCI,
-	PCI_DEVICE_ID_INTEL_JSP_XDCI,
-	PCI_DEVICE_ID_INTEL_ADP_P_XDCI,
-	PCI_DEVICE_ID_INTEL_ADP_S_XDCI,
-	PCI_DEVICE_ID_INTEL_ADP_M_XDCI,
+	PCI_DID_INTEL_MTL_XDCI,
+	PCI_DID_INTEL_APL_XDCI,
+	PCI_DID_INTEL_CNL_LP_XDCI,
+	PCI_DID_INTEL_GLK_XDCI,
+	PCI_DID_INTEL_CNP_H_XDCI,
+	PCI_DID_INTEL_ICP_LP_XDCI,
+	PCI_DID_INTEL_CMP_LP_XDCI,
+	PCI_DID_INTEL_CMP_H_XDCI,
+	PCI_DID_INTEL_TGP_LP_XDCI,
+	PCI_DID_INTEL_TGP_H_XDCI,
+	PCI_DID_INTEL_MCC_XDCI,
+	PCI_DID_INTEL_JSP_XDCI,
+	PCI_DID_INTEL_ADP_P_XDCI,
+	PCI_DID_INTEL_ADP_S_XDCI,
+	PCI_DID_INTEL_ADP_M_XDCI,
 	0
 };
 
 static const struct pci_driver pch_usb_xdci __pci_driver = {
 	.ops	 = &usb_xdci_ops,
-	.vendor	 = PCI_VENDOR_ID_INTEL,
+	.vendor	 = PCI_VID_INTEL,
 	.devices	 = pci_device_ids,
 };
