@@ -1,19 +1,4 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2008-2009 coresystems GmbH
- * Copyright (C) 2014 Google Inc.
- * Copyright (C) 2015-2017 Intel Corporation.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <device/pci.h>
 #include <device/pci_ids.h>
@@ -22,7 +7,7 @@
 #include <soc/pm.h>
 #include <soc/ramstage.h>
 
-static void pmc_read_resources(device_t dev)
+static void pmc_read_resources(struct device *dev)
 {
 	unsigned int index = 0;
 	struct resource *res;
@@ -50,14 +35,14 @@ static void pmc_read_resources(device_t dev)
 }
 
 static struct device_operations device_ops = {
-	.read_resources		= &pmc_read_resources,
-	.set_resources		= &pci_dev_set_resources,
-	.enable_resources	= &pci_dev_enable_resources,
-	.scan_bus		= &scan_lpc_bus,
+	.read_resources		= pmc_read_resources,
+	.set_resources		= pci_dev_set_resources,
+	.enable_resources	= pci_dev_enable_resources,
+	.scan_bus		= scan_static_bus,
 };
 
 static const struct pci_driver pmc __pci_driver = {
 	.ops	= &device_ops,
-	.vendor	= PCI_VENDOR_ID_INTEL,
+	.vendor	= PCI_VID_INTEL,
 	.device	= QUARK_V_LPC_DEVICE_ID_0,
 };
