@@ -93,6 +93,14 @@ static void parse_devicetree(FSP_S_CONFIG *params)
  * regardless of whether or not they are used by the mainboard.
  */
 static const struct slot_irq_constraints irq_constraints[] = {
+	{
+		.slot = SA_DEV_SLOT_PEG,
+		.fns = {
+			FIXED_INT_PIRQ(SA_DEVFN_PEG1, PCI_INT_A, PIRQ_A),
+			FIXED_INT_PIRQ(SA_DEVFN_PEG2, PCI_INT_B, PIRQ_B),
+			FIXED_INT_PIRQ(SA_DEVFN_PEG3, PCI_INT_C, PIRQ_C),
+		},
+	},
 	{
 		.slot = SA_DEV_SLOT_IGD,
 		.fns = {
@@ -114,7 +122,7 @@ static const struct slot_irq_constraints irq_constraints[] = {
 	{
 		.slot = SA_DEV_SLOT_CPU_PCIE,
 		.fns = {
-			ANY_PIRQ(SA_DEVFN_CPU_PCIE),
+			FIXED_INT_PIRQ(SA_DEVFN_CPU_PCIE, PCI_INT_D, PIRQ_D),
 		},
 	},
 	{
@@ -137,6 +145,7 @@ static const struct slot_irq_constraints irq_constraints[] = {
 	{
 		.slot = PCH_DEV_SLOT_SIO0,
 		.fns = {
+			ANY_PIRQ(PCH_DEVFN_CT),
 			ANY_PIRQ(PCH_DEVFN_THC0),
 			ANY_PIRQ(PCH_DEVFN_THC1),
 		},
@@ -164,6 +173,7 @@ static const struct slot_irq_constraints irq_constraints[] = {
 		.slot = PCH_DEV_SLOT_XHCI,
 		.fns = {
 			ANY_PIRQ(PCH_DEVFN_XHCI),
+			DIRECT_IRQ(PCH_DEVFN_USBOTG),
 			FIXED_INT_ANY_PIRQ(PCH_DEVFN_CNVI_WIFI, PCI_INT_A),
 		},
 	},
@@ -402,6 +412,7 @@ void platform_fsp_silicon_init_params_cb(FSPS_UPD *supd)
 			config->PcieRpAdvancedErrorReporting[i];
 		params->PcieRpHotPlug[i] = config->PcieRpHotPlug[i];
 		params->PciePtm[i] = config->PciePtm[i];
+		params->PcieRpSlotImplemented[i] = config->PcieRpSlotImplemented[i];
 	}
 
 	/* Enable ClkReqDetect for enabled port */
@@ -474,7 +485,7 @@ void platform_fsp_silicon_init_params_cb(FSPS_UPD *supd)
 	params->SlowSlewRate[0] = config->SlowSlewRate;
 
 	/* Enable TCPU for processor thermal control */
-	params->Device4Enable = config->Device4Enable;
+	params->Device4Enable = is_devfn_enabled(SA_DEVFN_DPTF);
 
 	/* Set TccActivationOffset */
 	params->TccActivationOffset = config->tcc_offset;
@@ -530,10 +541,7 @@ void platform_fsp_silicon_init_params_cb(FSPS_UPD *supd)
 	/* USB4/TBT */
 	for (i = 0; i < ARRAY_SIZE(params->ITbtPcieRootPortEn); i++) {
 		dev = pcidev_on_root(SA_DEV_SLOT_TBT, i);
-		if (dev)
-			params->ITbtPcieRootPortEn[i] = dev->enabled;
-		else
-			params->ITbtPcieRootPortEn[i] = 0;
+		params->ITbtPcieRootPortEn[i] = is_dev_enabled(dev);
 	}
 
 	/* PCH FIVR settings override */
