@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
 #ifndef SOC_INTEL_XEON_SP_ITSS_H
-#define SOC_INTEL_XEONS_P_ITSS_H
+#define SOC_INTEL_XEON_SP_ITSS_H
 
 #define ITSS_MAX_IRQ	119
 #define IRQS_PER_IPC	32
