@@ -253,7 +253,7 @@ static const struct mux_sel mux_sels[] = {
 	{ .id = TOP_DSI_OCC_SEL, .sel = 1 },		/* 1: univpll_d3_d2 */
 	{ .id = TOP_SPMI_MST_SEL, .sel = 2 },		/* 2: ulposc1_d4 */
 	/* CLK_CFG_13 */
-	{ .id = TOP_SPINOR_SEL, .sel = 5 },		/* 5: mainpll_d7_d2 */
+	{ .id = TOP_SPINOR_SEL, .sel = 3 },		/* 3: univpll_d3_d8 */
 	{ .id = TOP_NNA_SEL, .sel = 14 },		/* 14: nnapll_ck */
 	{ .id = TOP_NNA1_SEL, .sel = 14 },		/* 14: nnapll_ck */
 	{ .id = TOP_NNA2_SEL, .sel = 15 },		/* 15: nna2pll_ck */
@@ -425,17 +425,48 @@ void mt_pll_init(void)
 	setbits32(&mtk_topckgen->clk_misc_cfg_0, ARMPLL_DIVIDER_PLL1_EN);
 	setbits32(&mtk_topckgen->clk_misc_cfg_0, ARMPLL_DIVIDER_PLL2_EN);
 
-	clrsetbits32(&mt8186_mcucfg->cpu_plldiv_cfg0, MCU_DIV_MASK, MCU_DIV_1);
-	clrsetbits32(&mt8186_mcucfg->cpu_plldiv_cfg1, MCU_DIV_MASK, MCU_DIV_1);
-	clrsetbits32(&mt8186_mcucfg->bus_plldiv_cfg, MCU_DIV_MASK, MCU_DIV_1);
+	clrsetbits32(&mtk_mcucfg->cpu_plldiv_cfg0, MCU_DIV_MASK, MCU_DIV_1);
+	clrsetbits32(&mtk_mcucfg->cpu_plldiv_cfg1, MCU_DIV_MASK, MCU_DIV_1);
+	clrsetbits32(&mtk_mcucfg->bus_plldiv_cfg, MCU_DIV_MASK, MCU_DIV_1);
 
-	clrsetbits32(&mt8186_mcucfg->cpu_plldiv_cfg0, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
-	clrsetbits32(&mt8186_mcucfg->cpu_plldiv_cfg1, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
-	clrsetbits32(&mt8186_mcucfg->bus_plldiv_cfg, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
+	clrsetbits32(&mtk_mcucfg->cpu_plldiv_cfg0, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
+	clrsetbits32(&mtk_mcucfg->cpu_plldiv_cfg1, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
+	clrsetbits32(&mtk_mcucfg->bus_plldiv_cfg, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
 
 	write32(&mt8186_infracfg_ao->infra_bus_dcm_ctrl, 0x805f0603);
 	write32(&mt8186_infracfg_ao->peri_bus_dcm_ctrl, 0xb07f0603);
 
+	/* dcm_infracfg_ao_audio_bus and dcm_infracfg_ao_icusb_bus */
+	SET32_BITFIELDS(&mt8186_infracfg_ao->peri_bus_dcm_ctrl,
+			INFRACFG_AO_AUDIO_BUS_REG0, 0,
+			INFRACFG_AO_ICUSB_BUS_REG0, 0,
+			INFRACFG_AO_AUDIO_BUS_REG0, 1,
+			INFRACFG_AO_ICUSB_BUS_REG0, 1);
+
+	/* dcm_infracfg_ao_infra_bus */
+	SET32_BITFIELDS(&mt8186_infracfg_ao->infra_bus_dcm_ctrl,
+			INFRACFG_AO_INFRA_BUS_REG0_0, 0,
+			INFRACFG_AO_INFRA_BUS_REG0_1, 0,
+			INFRACFG_AO_INFRA_BUS_REG0_2, 0,
+			INFRACFG_AO_INFRA_BUS_REG0_0, 0x603,
+			INFRACFG_AO_INFRA_BUS_REG0_1, 0xF,
+			INFRACFG_AO_INFRA_BUS_REG0_2, 1);
+
+	/* dcm_infracfg_ao_p2p_rx_clk */
+	SET32_BITFIELDS(&mt8186_infracfg_ao->p2p_rx_clk_on,
+			INFRACFG_AO_P2P_RX_CLK_REG0_MASK_0, 0,
+			INFRACFG_AO_P2P_RX_CLK_REG0_MASK_1, 0,
+			INFRACFG_AO_P2P_RX_CLK_REG0_MASK_1, 1);
+
+	/* dcm_infracfg_ao_peri_bus */
+	SET32_BITFIELDS(&mt8186_infracfg_ao->peri_bus_dcm_ctrl,
+			INFRACFG_AO_PERI_BUS_REG0_0, 0,
+			INFRACFG_AO_PERI_BUS_REG0_1, 0,
+			INFRACFG_AO_PERI_BUS_REG0_2, 0,
+			INFRACFG_AO_PERI_BUS_REG0_0, 3,
+			INFRACFG_AO_PERI_BUS_REG0_1, 0xFF07C,
+			INFRACFG_AO_PERI_BUS_REG0_2, 1);
+
 	for (i = 0; i < ARRAY_SIZE(mux_sels); i++)
 		mux_set_sel(&muxes[mux_sels[i].id], mux_sels[i].sel);
 
@@ -443,14 +474,17 @@ void mt_pll_init(void)
 	write32(&mt8186_infracfg_ao->module_sw_cg_0_clr, 0x00000030);
 	/* [7] DVFSRC_CG, [20] DEVICE_APC_CG */
 	write32(&mt8186_infracfg_ao->module_sw_cg_1_clr, 0x00100080);
-	/* [15] SEJ_F13M_CK_CG, [16] AES_TOP0_BCLK_CK_CG */
-	write32(&mt8186_infracfg_ao->module_sw_cg_3_clr, 0x00018000);
+	/*
+	 * [15] SEJ_F13M_CK_CG, [16] AES_TOP0_BCLK_CK_CG,
+	 * [22] FADSP_26M_CG, [23] FADSP_32K_CG, [27] FADSP_CK_CG
+	 */
+	write32(&mt8186_infracfg_ao->module_sw_cg_3_clr, 0x08C18000);
 }
 
 void mt_pll_raise_little_cpu_freq(u32 freq)
 {
 	/* switch clock source to intermediate clock */
-	clrsetbits32(&mt8186_mcucfg->cpu_plldiv_cfg0, MCU_MUX_MASK, MCU_MUX_SRC_26M);
+	clrsetbits32(&mtk_mcucfg->cpu_plldiv_cfg0, MCU_MUX_MASK, MCU_MUX_SRC_26M);
 
 	/* disable armpll_ll frequency output */
 	clrbits32(plls[APMIXED_ARMPLL_LL].reg, MT8186_PLL_EN);
@@ -463,13 +497,13 @@ void mt_pll_raise_little_cpu_freq(u32 freq)
 	udelay(PLL_EN_DELAY);
 
 	/* switch clock source back to armpll_ll */
-	clrsetbits32(&mt8186_mcucfg->cpu_plldiv_cfg0, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
+	clrsetbits32(&mtk_mcucfg->cpu_plldiv_cfg0, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
 }
 
 void mt_pll_raise_cci_freq(u32 freq)
 {
 	/* switch clock source to intermediate clock */
-	clrsetbits32(&mt8186_mcucfg->bus_plldiv_cfg, MCU_MUX_MASK, MCU_MUX_SRC_26M);
+	clrsetbits32(&mtk_mcucfg->bus_plldiv_cfg, MCU_MUX_MASK, MCU_MUX_SRC_26M);
 
 	/* disable ccipll frequency output */
 	clrbits32(plls[APMIXED_CCIPLL].reg, MT8186_PLL_EN);
@@ -482,7 +516,13 @@ void mt_pll_raise_cci_freq(u32 freq)
 	udelay(PLL_EN_DELAY);
 
 	/* switch clock source back to ccipll */
-	clrsetbits32(&mt8186_mcucfg->bus_plldiv_cfg, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
+	clrsetbits32(&mtk_mcucfg->bus_plldiv_cfg, MCU_MUX_MASK, MCU_MUX_SRC_PLL);
+}
+
+void mt_pll_set_usb_clock(void)
+{
+	/* enable usb macro control */
+	SET32_BITFIELDS(&mtk_topckgen->usb_top_cfg, USB_TOP_CFG_MACRO_CTRL, 3);
 }
 
 u32 mt_fmeter_get_freq_khz(enum fmeter_type type, u32 id)
@@ -511,7 +551,7 @@ u32 mt_fmeter_get_freq_khz(enum fmeter_type type, u32 id)
 		SET32_BITFIELDS(&mtk_topckgen->clk_misc_cfg_0,
 				CLK_MISC_CFG_0_METER_DIV, 0);
 	} else {
-		die("unsupport fmeter type\n");
+		die("unsupported fmeter type\n");
 	}
 
 	/* enable frequency meter */
