@@ -3,6 +3,8 @@
 #ifndef __SOC_MEDIATEK_MT8195_DRAMC_SOC_H__
 #define __SOC_MEDIATEK_MT8195_DRAMC_SOC_H__
 
+#include <soc/dramc_soc_common.h>
+
 typedef enum {
 	CHANNEL_A = 0,
 	CHANNEL_B,
@@ -29,22 +31,6 @@ typedef enum {
 	DRAM_DFS_SHUFFLE_MAX,
 } DRAM_DFS_SHUFFLE_TYPE_T;
 
-/*
- * Internal CBT mode enum
- * 1. Calibration flow uses vGet_Dram_CBT_Mode to
- *    differentiate between mixed vs non-mixed LP4
- * 2. Declared as dram_cbt_mode[RANK_MAX] internally to
- *    store each rank's CBT mode type
- */
-typedef enum {
-	CBT_NORMAL_MODE = 0,
-	CBT_BYTE_MODE1,
-} DRAM_CBT_MODE_T;
-
 #define DRAM_DFS_SHU_MAX	DRAM_DFS_SHUFFLE_MAX
 
-#define DQS_NUMBER_LP4 2
-#define DQS_BIT_NUMBER 8
-#define DQ_DATA_WIDTH_LP4 16
-
 #endif	/* __SOC_MEDIATEK_MT8195_DRAMC_SOC_H__ */
