@@ -6,7 +6,7 @@
 
 #define MTK_WDT_CLR_STATUS 0x230001FF
 
-void mtk_wdt_clr_status(uint32_t wdt_sta)
+void mtk_wdt_clr_status(void)
 {
 	write32(&mtk_wdt->wdt_mode, MTK_WDT_CLR_STATUS);
 }
