@@ -220,10 +220,10 @@ static int tegra_dc_sor_power_dplanes(struct tegra_dc_sor_data *sor,
 		case 4:
 			reg_val |= (NV_SOR_DP_PADCTL_PD_TXD_3_NO |
 				NV_SOR_DP_PADCTL_PD_TXD_2_NO);
-			/* fall through */
+			__fallthrough;
 		case 2:
 			reg_val |= NV_SOR_DP_PADCTL_PD_TXD_1_NO;
-			/* fall through */
+			__fallthrough;
 		case 1:
 			reg_val |= NV_SOR_DP_PADCTL_PD_TXD_0_NO;
 			break;
@@ -516,7 +516,7 @@ static void tegra_dc_sor_power_up(struct tegra_dc_sor_data *sor,
 static void dump_sor_reg(struct tegra_dc_sor_data *sor)
 {
 #define DUMP_REG(a) printk(BIOS_INFO, "%-32s  %03x  %08x\n",		\
-		#a, a, tegra_sor_readl(sor, a));
+		#a, a, tegra_sor_readl(sor, a))
 
 	DUMP_REG(NV_SOR_SUPER_STATE0);
 	DUMP_REG(NV_SOR_SUPER_STATE1);
@@ -581,8 +581,6 @@ static void dump_sor_reg(struct tegra_dc_sor_data *sor)
 	DUMP_REG(NV_SOR_DP_SPARE(0));
 	DUMP_REG(NV_SOR_DP_SPARE(1));
 	DUMP_REG(NV_SOR_DP_TPG);
-
-	return;
 }
 #endif
 
@@ -893,10 +891,10 @@ void tegra_sor_precharge_lanes(struct tegra_dc_sor_data *sor)
 	case 4:
 		val |= (NV_SOR_DP_PADCTL_PD_TXD_3_NO |
 			NV_SOR_DP_PADCTL_PD_TXD_2_NO);
-		/* fall through */
+		__fallthrough;
 	case 2:
 		val |= NV_SOR_DP_PADCTL_PD_TXD_1_NO;
-		/* fall through */
+		__fallthrough;
 	case 1:
 		val |= NV_SOR_DP_PADCTL_PD_TXD_0_NO;
 		break;
