@@ -8,6 +8,11 @@
 void bootblock_soc_init(void)
 {
 	clock_init();
-	quadspi_init(37500 * KHz);
+	/*
+	 * Through experimentation, we have determined
+	 * that a delay of 1/8 cycle is best for herobrine.
+	 * See b/190231148
+	 */
+	quadspi_init(75000 * KHz, 1);
 	qupv3_fw_init();
 }
