@@ -1,20 +1,8 @@
-/*
- * This file is part of the coreboot project.
- *
- * Copyright 2016 Rockchip Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <assert.h>
 #include <console/console.h>
+#include <device/mmio.h>
 #include <delay.h>
 #include <soc/addressmap.h>
 #include <soc/clock.h>
@@ -22,7 +10,6 @@
 #include <soc/i2c.h>
 #include <soc/soc.h>
 #include <stdint.h>
-#include <stdlib.h>
 #include <string.h>
 
 struct pll_div {
@@ -94,7 +81,7 @@ enum {
 	PLL_SSMOD_RESET_SHIFT		= 2,
 	PLL_SSMOD_DOWNSPEAD_MASK	= 1,
 	PLL_SSMOD_DOWNSPEAD_SHIFT	= 3,
-	PLL_SSMOD_DIVVAL_MASK		= 0Xf,
+	PLL_SSMOD_DIVVAL_MASK		= 0xf,
 	PLL_SSMOD_DIVVAL_SHIFT		= 4,
 	PLL_SSMOD_SPREADAMP_MASK	= 0x1f,
 	PLL_SSMOD_SPREADAMP_SHIFT	= 8,
@@ -262,14 +249,8 @@ enum {
 	CLK_I2C3_DIV_CON_SHIFT		= 0,
 
 	/* CRU_SOFTRST_CON4 */
-	RESETN_DDR0_REQ_MASK		= 1,
-	RESETN_DDR0_REQ_SHIFT		= 8,
-	RESETN_DDRPHY0_REQ_MASK		= 1,
-	RESETN_DDRPHY0_REQ_SHIFT	= 9,
-	RESETN_DDR1_REQ_MASK		= 1,
-	RESETN_DDR1_REQ_SHIFT		= 12,
-	RESETN_DDRPHY1_REQ_MASK		= 1,
-	RESETN_DDRPHY1_REQ_SHIFT	= 13,
+#define RESETN_DDR_REQ_SHIFT(ch)	(8 + (ch) * 4)
+#define RESETN_DDRPHY_REQ_SHIFT(ch)	(9 + (ch) * 4)
 };
 
 #define VCO_MAX_KHZ	(3200 * (MHz / KHz))
@@ -304,7 +285,7 @@ static void rkclk_set_pll(u32 *pll_con, const struct pll_div *div)
 	u32 output_khz = vco_khz / div->postdiv1 / div->postdiv2;
 
 	printk(BIOS_DEBUG, "PLL at %p: fbdiv=%d, refdiv=%d, postdiv1=%d, "
-			   "postdiv2=%d, vco=%u khz, output=%u khz\n",
+			   "postdiv2=%d, vco=%u kHz, output=%u kHz\n",
 			   pll_con, div->fbdiv, div->refdiv, div->postdiv1,
 			   div->postdiv2, vco_khz, output_khz);
 	assert(vco_khz >= VCO_MIN_KHZ && vco_khz <= VCO_MAX_KHZ &&
@@ -344,12 +325,12 @@ static void rkclk_set_pll(u32 *pll_con, const struct pll_div *div)
 /*
  * Configure the DPLL spread spectrum feature on memory clock.
  * Configure sequence:
- * 1. PLL been configured as frac mode, and DACPD should be set to 1’b0.
+ * 1. PLL been configured as frac mode, and DACPD should be set to 1'b0.
  * 2. Configure DOWNSPERAD, SPREAD, DIVVAL(option: configure xPLL_CON5 with
  *    extern wave table).
- * 3. set ssmod_disable_sscg = 1’b0, and set ssmod_bp = 1’b0.
- * 4. Assert RESET = 1’b1 to SSMOD.
- * 5. RESET = 1’b0 on SSMOD.
+ * 3. set ssmod_disable_sscg = 1'b0, and set ssmod_bp = 1'b0.
+ * 4. Assert RESET = 1'b1 to SSMOD.
+ * 5. RESET = 1'b0 on SSMOD.
  * 6. Adjust SPREAD/DIVVAL/DOWNSPREAD.
  */
 static void rkclk_set_dpllssc(struct pll_div *dpll_cfg)
@@ -373,9 +354,9 @@ static void rkclk_set_dpllssc(struct pll_div *dpll_cfg)
 	write32(&cru_ptr->dpll_con[3],
 		RK_CLRSETBITS(PLL_DSMPD_MASK << PLL_DSMPD_SHIFT,
 			      PLL_FRAC_MODE << PLL_DSMPD_SHIFT));
-	clrsetbits_le32(&cru_ptr->dpll_con[2],
-			PLL_FRACDIV_MASK << PLL_FRACDIV_SHIFT,
-			0 << PLL_FRACDIV_SHIFT);
+	clrsetbits32(&cru_ptr->dpll_con[2],
+		     PLL_FRACDIV_MASK << PLL_FRACDIV_SHIFT,
+		     0 << PLL_FRACDIV_SHIFT);
 
 	/*
 	 * Configure SSC divval.
@@ -385,13 +366,13 @@ static void rkclk_set_dpllssc(struct pll_div *dpll_cfg)
 	 * value of SPREAD.
 	 * SPREAD[4:0]	Center Spread	Down Spread
 	 *	0	0		0
-	 *	1	±0.1%		-0.10%
-	 *	2	±0.2%		-0.20%
-	 *	3	±0.3%		-0.30%
-	 *	4	±0.4%		-0.40%
-	 *	5	±0.5%		-0.50%
+	 *	1	+/-0.1%		-0.10%
+	 *	2	+/-0.2%		-0.20%
+	 *	3	+/-0.3%		-0.30%
+	 *	4	+/-0.4%		-0.40%
+	 *	5	+/-0.5%		-0.50%
 	 *	...
-	 *	31	±3.1%		-3.10%
+	 *	31	+/-3.1%		-3.10%
 	 */
 	write32(&cru_ptr->dpll_con[4],
 		RK_CLRSETBITS(PLL_SSMOD_DIVVAL_MASK << PLL_SSMOD_DIVVAL_SHIFT,
@@ -429,10 +410,10 @@ static int pll_para_config(u32 freq_hz, struct pll_div *div)
 		return -1;
 	}
 
-	postdiv1 = div_round_up(VCO_MIN_KHZ, freq_khz);
+	postdiv1 = DIV_ROUND_UP(VCO_MIN_KHZ, freq_khz);
 	if (postdiv1 > max_postdiv1) {
-		postdiv2 = div_round_up(postdiv1, max_postdiv1);
-		postdiv1 = div_round_up(postdiv1, postdiv2);
+		postdiv2 = DIV_ROUND_UP(postdiv1, max_postdiv1);
+		postdiv1 = DIV_ROUND_UP(postdiv1, postdiv2);
 	}
 
 	vco_khz = freq_khz * postdiv1 * postdiv2;
@@ -485,7 +466,7 @@ void rkclk_init(void)
 
 	/* some cru registers changed by bootrom, we'd better reset them to
 	 * reset/default values described in TRM to avoid confusion in kernel.
-	 * Please consider these threee lines as a fix of bootrom bug.
+	 * Please consider these three lines as a fix of bootrom bug.
 	 */
 	write32(&cru_ptr->clksel_con[12], 0xffff4101);
 	write32(&cru_ptr->clksel_con[19], 0xffff033f);
@@ -605,9 +586,9 @@ void rkclk_configure_cpu(enum apll_frequencies freq, enum cpu_cluster cluster)
 	apll_hz = apll_cfgs[freq]->freq;
 	rkclk_set_pll(pll_con, apll_cfgs[freq]);
 
-	aclkm_div = div_round_up(apll_hz, ACLKM_CORE_HZ) - 1;
-	pclk_dbg_div = div_round_up(apll_hz, PCLK_DBG_HZ) - 1;
-	atclk_div = div_round_up(apll_hz, ATCLK_CORE_HZ) - 1;
+	aclkm_div = DIV_ROUND_UP(apll_hz, ACLKM_CORE_HZ) - 1;
+	pclk_dbg_div = DIV_ROUND_UP(apll_hz, PCLK_DBG_HZ) - 1;
+	atclk_div = DIV_ROUND_UP(apll_hz, ATCLK_CORE_HZ) - 1;
 
 	write32(&cru_ptr->clksel_con[con_base],
 		RK_CLRSETBITS(ACLKM_CORE_DIV_CON_MASK <<
@@ -659,10 +640,17 @@ void rkclk_configure_ddr(unsigned int hz)
 	}
 	rkclk_set_pll(&cru_ptr->dpll_con[0], &dpll_cfg);
 
-	if (IS_ENABLED(CONFIG_RK3399_SPREAD_SPECTRUM_DDR))
+	if (CONFIG(RK3399_SPREAD_SPECTRUM_DDR))
 		rkclk_set_dpllssc(&dpll_cfg);
 }
 
+void rkclk_ddr_reset(u32 ch, u32 ctl, u32 phy)
+{
+	write32(&cru_ptr->softrst_con[4], RK_CLRSETBITS(
+		1 << RESETN_DDR_REQ_SHIFT(ch) | 1 << RESETN_DDRPHY_REQ_SHIFT(ch),
+		ctl << RESETN_DDR_REQ_SHIFT(ch) | phy << RESETN_DDRPHY_REQ_SHIFT(ch)));
+}
+
 #define SPI_CLK_REG_VALUE(bus, clk_div) \
 		RK_CLRSETBITS(CLK_SPI_PLL_SEL_MASK << \
 					CLK_SPI ##bus## _PLL_SEL_SHIFT | \
@@ -804,7 +792,7 @@ void rkclk_configure_i2s(unsigned int hz)
 	int v;
 
 	/**
-	 * clk_i2s0_sel: divider ouput from fraction
+	 * clk_i2s0_sel: divider output from fraction
 	 * clk_i2s0_pll_sel source clock: cpll
 	 * clk_i2s0_div_con: 1 (div+1)
 	 */
