@@ -95,7 +95,7 @@ static void hudson_lpc_read_resources(struct device *dev)
 		     IORESOURCE_ASSIGNED | IORESOURCE_FIXED;
 
 	/* Add a memory resource for the SPI BAR. */
-	fixed_mem_resource(dev, 2, SPI_BASE_ADDRESS / 1024, 1, IORESOURCE_SUBTRACTIVE);
+	mmio_range(dev, 2, SPI_BASE_ADDRESS, 1 * KiB);
 
 	res = new_resource(dev, 3); /* IOAPIC */
 	res->base = IO_APIC_ADDR;
@@ -244,7 +244,7 @@ static void hudson_lpc_enable_childrens_resources(struct device *dev)
 					default:
 						rsize = 0;
 						/* try AGESA allocated region in region 0 */
-						if ((var_num > 0) && ((base >=reg_var[0]) &&
+						if ((var_num > 0) && ((base >= reg_var[0]) &&
 								((base + res->size) <= (reg_var[0] + reg_size[0]))))
 							rsize = reg_size[0];
 					}
@@ -290,10 +290,10 @@ static void hudson_lpc_enable_childrens_resources(struct device *dev)
 	switch (var_num) {
 	case 3:
 		pci_write_config16(dev, 0x90, reg_var[2]);
-		/* fall through */
+		__fallthrough;
 	case 2:
 		pci_write_config16(dev, 0x66, reg_var[1]);
-		/* fall through */
+		__fallthrough;
 	case 1:
 		pci_write_config16(dev, 0x64, reg_var[0]);
 		break;
@@ -343,6 +343,6 @@ static struct device_operations lpc_ops = {
 };
 static const struct pci_driver lpc_driver __pci_driver = {
 	.ops = &lpc_ops,
-	.vendor = PCI_VENDOR_ID_AMD,
-	.device = PCI_DEVICE_ID_AMD_SB900_LPC,
+	.vendor = PCI_VID_AMD,
+	.device = PCI_DID_AMD_SB900_LPC,
 };
