@@ -7,11 +7,11 @@
 #include <cf9_reset.h>
 #include <reset.h>
 
-#define HT_INIT_CONTROL		0x6C
-#define HTIC_BIOSR_Detect	(1<<5)
+#define HT_INIT_CONTROL		0x6c
+#define HTIC_BIOSR_Detect	(1 << 5)
 
 #define DEV_CDB 0x18
-#define NODE_PCI(x, fn)	(((DEV_CDB+x)<32)?(PCI_DEV(0,(DEV_CDB+x),fn)):(PCI_DEV((0-1),(DEV_CDB+x-32),fn)))
+#define NODE_PCI(x, fn)	(((DEV_CDB + x) < 32) ? (PCI_DEV(0, (DEV_CDB + x), fn)) : (PCI_DEV((0 - 1), (DEV_CDB + x - 32), fn)))
 
 void cf9_reset_prepare(void)
 {
