@@ -18,7 +18,6 @@
 #include <device/pci_ids.h>
 #include <device/pci_def.h>
 #include <string.h>
-#include <delay.h>
 #include <elog.h>
 #include <halt.h>
 #include <option.h>
@@ -286,7 +285,7 @@ static struct device_operations device_ops = {
 
 static const struct pci_driver intel_me __pci_driver = {
 	.ops	= &device_ops,
-	.vendor	= PCI_VENDOR_ID_INTEL,
+	.vendor	= PCI_VID_INTEL,
 	.device	= 0x1e3a,
 };
 
