@@ -53,6 +53,6 @@ static const unsigned short pci_device_ids[] = {
 
 static const struct pci_driver pch_pci __pci_driver = {
 	.ops		= &device_ops,
-	.vendor		= PCI_VENDOR_ID_INTEL,
+	.vendor		= PCI_VID_INTEL,
 	.devices	= pci_device_ids,
 };
