@@ -89,7 +89,7 @@ static void i82801ix_pirq_init(struct device *dev)
 	for (irq_dev = all_devices; irq_dev; irq_dev = irq_dev->next) {
 		u8 int_pin = 0, int_line = 0;
 
-		if (!irq_dev->enabled || irq_dev->path.type != DEVICE_PATH_PCI)
+		if (!is_enabled_pci(irq_dev))
 			continue;
 
 		int_pin = pci_read_config8(irq_dev, PCI_INTERRUPT_PIN);
@@ -378,7 +378,7 @@ static void lpc_init(struct device *dev)
 	/* Don't allow evil boot loaders, kernels, or
 	 * userspace applications to deceive us:
 	 */
-	if (CONFIG(HAVE_SMI_HANDLER) && !CONFIG(PARALLEL_MP))
+	if (CONFIG(SMM_LEGACY_ASEG))
 		aseg_smm_lock();
 }
 
@@ -465,17 +465,17 @@ static struct device_operations device_ops = {
 };
 
 static const unsigned short pci_device_ids[] = {
-	PCI_DEVICE_ID_INTEL_82801IH_LPC,   /* ICH9DH  */
-	PCI_DEVICE_ID_INTEL_82801IO_LPC,   /* ICH9DO  */
-	PCI_DEVICE_ID_INTEL_82801IR_LPC,   /* ICH9R   */
-	PCI_DEVICE_ID_INTEL_82801IEM_LPC,  /* ICH9M-E */
-	PCI_DEVICE_ID_INTEL_82801IB_LPC,   /* ICH9    */
-	PCI_DEVICE_ID_INTEL_82801IBM_LPC,  /* ICH9M   */
+	PCI_DID_INTEL_82801IH_LPC,   /* ICH9DH  */
+	PCI_DID_INTEL_82801IO_LPC,   /* ICH9DO  */
+	PCI_DID_INTEL_82801IR_LPC,   /* ICH9R   */
+	PCI_DID_INTEL_82801IEM_LPC,  /* ICH9M-E */
+	PCI_DID_INTEL_82801IB_LPC,   /* ICH9    */
+	PCI_DID_INTEL_82801IBM_LPC,  /* ICH9M   */
 	0
 };
 
 static const struct pci_driver ich9_lpc __pci_driver = {
 	.ops		= &device_ops,
-	.vendor		= PCI_VENDOR_ID_INTEL,
+	.vendor		= PCI_VID_INTEL,
 	.devices	= pci_device_ids,
 };
