@@ -9,7 +9,7 @@
 
 #include <soc/nvs.h>
 
-#if !CONFIG(SMM_TSEG)
+#if CONFIG(SMM_LEGACY_ASEG)
 /* For qemu/x86-q35 to build properly. */
 struct global_nvs *gnvs;
 #endif
@@ -62,10 +62,6 @@ void southbridge_smi_monitor(void)
 		if (!(trap_cycle & (1 << 24))) { // It's a write
 			printk(BIOS_DEBUG, "SMI1 command\n");
 			data = RCBA32(0x1e18);
-			data &= mask;
-			// if (smi1)
-			// 	southbridge_smi_command(data);
-			// return;
 		}
 		// Fall through to debug
 	}
