@@ -120,7 +120,7 @@ static void pch_pirq_init(struct device *dev)
 	for (irq_dev = all_devices; irq_dev; irq_dev = irq_dev->next) {
 		u8 int_pin = 0, int_line = 0;
 
-		if (!irq_dev->enabled || irq_dev->path.type != DEVICE_PATH_PCI)
+		if (!is_enabled_pci(irq_dev))
 			continue;
 
 		int_pin = pci_read_config8(irq_dev, PCI_INTERRUPT_PIN);
@@ -624,7 +624,7 @@ static void pch_lpc_add_mmio_resources(struct device *dev)
 	 */
 	res = new_resource(dev, OIC);
 	res->base = default_decode_base;
-	res->size = 0 - default_decode_base;
+	res->size = 4ull * GiB - default_decode_base;
 	res->flags = IORESOURCE_MEM | IORESOURCE_ASSIGNED | IORESOURCE_FIXED;
 
 	/* RCBA */
@@ -691,8 +691,8 @@ static void pch_lpc_add_gen_io_resources(struct device *dev, int reg_value,
 					 int index)
 {
 	/*
-	 * Check if the register is enabled. If so and the base exceeds the
-	 * device's default, claim range and add the resource.
+	 * Check if the register is enabled. If so, and the base exceeds the
+	 * device's default claim range, add the resource.
 	 */
 	if (reg_value & 1) {
 		u16 base = reg_value & 0xfffc;
@@ -816,30 +816,30 @@ static struct device_operations device_ops = {
 
 /* IDs for LPC device of Intel 8 Series Chipset (Lynx Point) */
 static const unsigned short pci_device_ids[] = {
-	PCI_DEVICE_ID_INTEL_LPT_MOBILE_SAMPLE,
-	PCI_DEVICE_ID_INTEL_LPT_DESKTOP_SAMPLE,
-	PCI_DEVICE_ID_INTEL_LPT_Z87,
-	PCI_DEVICE_ID_INTEL_LPT_Z85,
-	PCI_DEVICE_ID_INTEL_LPT_HM86,
-	PCI_DEVICE_ID_INTEL_LPT_H87,
-	PCI_DEVICE_ID_INTEL_LPT_HM87,
-	PCI_DEVICE_ID_INTEL_LPT_Q85,
-	PCI_DEVICE_ID_INTEL_LPT_Q87,
-	PCI_DEVICE_ID_INTEL_LPT_QM87,
-	PCI_DEVICE_ID_INTEL_LPT_B85,
-	PCI_DEVICE_ID_INTEL_LPT_C222,
-	PCI_DEVICE_ID_INTEL_LPT_C224,
-	PCI_DEVICE_ID_INTEL_LPT_C226,
-	PCI_DEVICE_ID_INTEL_LPT_H81,
-	PCI_DEVICE_ID_INTEL_LPT_LP_SAMPLE,
-	PCI_DEVICE_ID_INTEL_LPT_LP_PREMIUM,
-	PCI_DEVICE_ID_INTEL_LPT_LP_MAINSTREAM,
-	PCI_DEVICE_ID_INTEL_LPT_LP_VALUE,
+	PCI_DID_INTEL_LPT_MOBILE_SAMPLE,
+	PCI_DID_INTEL_LPT_DESKTOP_SAMPLE,
+	PCI_DID_INTEL_LPT_Z87,
+	PCI_DID_INTEL_LPT_Z85,
+	PCI_DID_INTEL_LPT_HM86,
+	PCI_DID_INTEL_LPT_H87,
+	PCI_DID_INTEL_LPT_HM87,
+	PCI_DID_INTEL_LPT_Q85,
+	PCI_DID_INTEL_LPT_Q87,
+	PCI_DID_INTEL_LPT_QM87,
+	PCI_DID_INTEL_LPT_B85,
+	PCI_DID_INTEL_LPT_C222,
+	PCI_DID_INTEL_LPT_C224,
+	PCI_DID_INTEL_LPT_C226,
+	PCI_DID_INTEL_LPT_H81,
+	PCI_DID_INTEL_LPT_LP_SAMPLE,
+	PCI_DID_INTEL_LPT_LP_PREMIUM,
+	PCI_DID_INTEL_LPT_LP_MAINSTREAM,
+	PCI_DID_INTEL_LPT_LP_VALUE,
 	0
 };
 
 static const struct pci_driver pch_lpc __pci_driver = {
 	.ops	 = &device_ops,
-	.vendor	 = PCI_VENDOR_ID_INTEL,
+	.vendor	 = PCI_VID_INTEL,
 	.devices = pci_device_ids,
 };
