@@ -40,9 +40,9 @@ static const char *const me_bios_path_values[] = {
 /* MMIO base address for MEI interface */
 static u8 *mei_base_address;
 
-static void mei_dump(void *ptr, int dword, int offset, const char *type)
+static void mei_dump(u32 dword, int offset, const char *type)
 {
-	struct mei_csr *csr;
+	union mei_csr csr;
 
 	if (!CONFIG(DEBUG_INTEL_ME))
 		return;
@@ -52,16 +52,12 @@ static void mei_dump(void *ptr, int dword, int offset, const char *type)
 	switch (offset) {
 	case MEI_H_CSR:
 	case MEI_ME_CSR_HA:
-		csr = ptr;
-		if (!csr) {
-			printk(BIOS_SPEW, "ERROR: 0x%08x\n", dword);
-			break;
-		}
+		csr.raw = dword;
 		printk(BIOS_SPEW, "cbd=%u cbrp=%02u cbwp=%02u ready=%u "
-		       "reset=%u ig=%u is=%u ie=%u\n", csr->buffer_depth,
-		       csr->buffer_read_ptr, csr->buffer_write_ptr,
-		       csr->ready, csr->reset, csr->interrupt_generate,
-		       csr->interrupt_status, csr->interrupt_enable);
+		       "reset=%u ig=%u is=%u ie=%u\n", csr.buffer_depth,
+		       csr.buffer_read_ptr, csr.buffer_write_ptr,
+		       csr.ready, csr.reset, csr.interrupt_generate,
+		       csr.interrupt_status, csr.interrupt_enable);
 		break;
 	case MEI_ME_CB_RW:
 	case MEI_H_CB_WW:
@@ -77,64 +73,47 @@ static void mei_dump(void *ptr, int dword, int offset, const char *type)
  * ME/MEI access helpers using memcpy to avoid aliasing.
  */
 
-static inline void mei_read_dword_ptr(void *ptr, int offset)
+static inline union mei_csr read_host_csr(void)
 {
-	u32 dword = read32(mei_base_address + offset);
-	memcpy(ptr, &dword, sizeof(dword));
-	mei_dump(ptr, dword, offset, "READ");
+	union mei_csr csr = { .raw = read32(mei_base_address + MEI_H_CSR) };
+	mei_dump(csr.raw, MEI_H_CSR, "READ");
+	return csr;
 }
 
-static inline void mei_write_dword_ptr(void *ptr, int offset)
+static inline void write_host_csr(union mei_csr csr)
 {
-	u32 dword = 0;
-	memcpy(&dword, ptr, sizeof(dword));
-	write32(mei_base_address + offset, dword);
-	mei_dump(ptr, dword, offset, "WRITE");
+	write32(mei_base_address + MEI_H_CSR, csr.raw);
+	mei_dump(csr.raw, MEI_H_CSR, "WRITE");
 }
 
-static inline void pci_read_dword_ptr(struct device *dev, void *ptr, int offset)
+static inline union mei_csr read_me_csr(void)
 {
-	u32 dword = pci_read_config32(dev, offset);
-	memcpy(ptr, &dword, sizeof(dword));
-	mei_dump(ptr, dword, offset, "PCI READ");
-}
-
-static inline void read_host_csr(struct mei_csr *csr)
-{
-	mei_read_dword_ptr(csr, MEI_H_CSR);
-}
-
-static inline void write_host_csr(struct mei_csr *csr)
-{
-	mei_write_dword_ptr(csr, MEI_H_CSR);
-}
-
-static inline void read_me_csr(struct mei_csr *csr)
-{
-	mei_read_dword_ptr(csr, MEI_ME_CSR_HA);
+	union mei_csr csr = { .raw = read32(mei_base_address + MEI_ME_CSR_HA) };
+	mei_dump(csr.raw, MEI_ME_CSR_HA, "READ");
+	return csr;
 }
 
 static inline void write_cb(u32 dword)
 {
 	write32(mei_base_address + MEI_H_CB_WW, dword);
-	mei_dump(NULL, dword, MEI_H_CB_WW, "WRITE");
+	mei_dump(dword, MEI_H_CB_WW, "WRITE");
 }
 
 static inline u32 read_cb(void)
 {
 	u32 dword = read32(mei_base_address + MEI_ME_CB_RW);
-	mei_dump(NULL, dword, MEI_ME_CB_RW, "READ");
+	mei_dump(dword, MEI_ME_CB_RW, "READ");
 	return dword;
 }
 
 /* Wait for ME ready bit to be asserted */
 static int mei_wait_for_me_ready(void)
 {
-	struct mei_csr me;
+	union mei_csr me;
 	unsigned int try = ME_RETRY;
 
 	while (try--) {
-		read_me_csr(&me);
+		me = read_me_csr();
 		if (me.ready)
 			return 0;
 		udelay(ME_DELAY);
@@ -146,31 +125,31 @@ static int mei_wait_for_me_ready(void)
 
 static void mei_reset(void)
 {
-	struct mei_csr host;
+	union mei_csr host;
 
 	if (mei_wait_for_me_ready() < 0)
 		return;
 
 	/* Reset host and ME circular buffers for next message */
-	read_host_csr(&host);
+	host = read_host_csr();
 	host.reset = 1;
 	host.interrupt_generate = 1;
-	write_host_csr(&host);
+	write_host_csr(host);
 
 	if (mei_wait_for_me_ready() < 0)
 		return;
 
 	/* Re-init and indicate host is ready */
-	read_host_csr(&host);
+	host = read_host_csr();
 	host.interrupt_generate = 1;
 	host.ready = 1;
 	host.reset = 0;
-	write_host_csr(&host);
+	write_host_csr(host);
 }
 
-static int mei_send_packet(struct mei_header *mei, void *req_data)
+static int mei_send_packet(union mei_header *mei, void *req_data)
 {
-	struct mei_csr host;
+	union mei_csr host;
 	unsigned int ndata, n;
 	u32 *data;
 
@@ -190,11 +169,11 @@ static int mei_send_packet(struct mei_header *mei, void *req_data)
 	 * Make sure there is still room left in the circular buffer.
 	 * Reset the buffer pointers if the requested message will not fit.
 	 */
-	read_host_csr(&host);
+	host = read_host_csr();
 	if ((host.buffer_depth - host.buffer_write_ptr) < ndata) {
 		printk(BIOS_ERR, "ME: circular buffer full, resetting...\n");
 		mei_reset();
-		read_host_csr(&host);
+		host = read_host_csr();
 	}
 
 	/* Ensure the requested length will fit in the circular buffer. */
@@ -205,7 +184,7 @@ static int mei_send_packet(struct mei_header *mei, void *req_data)
 	}
 
 	/* Write MEI header */
-	mei_write_dword_ptr(mei, MEI_H_CB_WW);
+	write_cb(mei->raw);
 	ndata--;
 
 	/* Write message data */
@@ -214,9 +193,9 @@ static int mei_send_packet(struct mei_header *mei, void *req_data)
 		write_cb(*data++);
 
 	/* Generate interrupt to the ME */
-	read_host_csr(&host);
+	host = read_host_csr();
 	host.interrupt_generate = 1;
-	write_host_csr(&host);
+	write_host_csr(host);
 
 	/* Make sure ME is ready after sending request data */
 	return mei_wait_for_me_ready();
@@ -225,11 +204,11 @@ static int mei_send_packet(struct mei_header *mei, void *req_data)
 static int mei_send_data(u8 me_address, u8 host_address,
 			 void *req_data, int req_bytes)
 {
-	struct mei_header header = {
+	union mei_header header = {
 		.client_address = me_address,
 		.host_address = host_address,
 	};
-	struct mei_csr host;
+	union mei_csr host;
 	int current = 0;
 	u8 *req_ptr = req_data;
 
@@ -237,7 +216,7 @@ static int mei_send_data(u8 me_address, u8 host_address,
 		int remain = req_bytes - current;
 		int buf_len;
 
-		read_host_csr(&host);
+		host = read_host_csr();
 		buf_len = host.buffer_depth - host.buffer_write_ptr;
 
 		if (buf_len > remain) {
@@ -261,7 +240,7 @@ static int mei_send_data(u8 me_address, u8 host_address,
 static int mei_send_header(u8 me_address, u8 host_address,
 			   void *header, int header_len, int complete)
 {
-	struct mei_header mei = {
+	union mei_header mei = {
 		.client_address = me_address,
 		.host_address   = host_address,
 		.length         = header_len,
@@ -273,8 +252,8 @@ static int mei_send_header(u8 me_address, u8 host_address,
 static int mei_recv_msg(void *header, int header_bytes,
 			void *rsp_data, int rsp_bytes)
 {
-	struct mei_header mei_rsp;
-	struct mei_csr me, host;
+	union mei_header mei_rsp;
+	union mei_csr me, host;
 	unsigned int ndata, n;
 	unsigned int expected;
 	u32 *data;
@@ -293,7 +272,7 @@ static int mei_recv_msg(void *header, int header_bytes,
 	 * expected number of dwords are present in the circular buffer.
 	 */
 	for (n = ME_RETRY; n; --n) {
-		read_me_csr(&me);
+		me = read_me_csr();
 		if ((me.buffer_write_ptr - me.buffer_read_ptr) >= expected)
 			break;
 		udelay(ME_DELAY);
@@ -306,7 +285,7 @@ static int mei_recv_msg(void *header, int header_bytes,
 	}
 
 	/* Read and verify MEI response header from the ME */
-	mei_read_dword_ptr(&mei_rsp, MEI_ME_CB_RW);
+	mei_rsp.raw = read_cb();
 	if (!mei_rsp.is_complete) {
 		printk(BIOS_ERR, "ME: response is not complete\n");
 		return -1;
@@ -341,10 +320,10 @@ static int mei_recv_msg(void *header, int header_bytes,
 		*data++ = read_cb();
 
 	/* Tell the ME that we have consumed the response */
-	read_host_csr(&host);
+	host = read_host_csr();
 	host.interrupt_status = 1;
 	host.interrupt_generate = 1;
-	write_host_csr(&host);
+	write_host_csr(host);
 
 	return mei_wait_for_me_ready();
 }
@@ -417,14 +396,14 @@ static inline int mei_sendrecv_icc(struct icc_header *icc,
  */
 static void intel_me_mbp_give_up(struct device *dev)
 {
-	struct mei_csr csr;
+	union mei_csr csr;
 
 	pci_write_config32(dev, PCI_ME_H_GS2, PCI_ME_MBP_GIVE_UP);
 
-	read_host_csr(&csr);
+	csr = read_host_csr();
 	csr.reset = 1;
 	csr.interrupt_generate = 1;
-	write_host_csr(&csr);
+	write_host_csr(csr);
 }
 
 /*
@@ -434,11 +413,11 @@ static void intel_me_mbp_give_up(struct device *dev)
 static void intel_me_mbp_clear(struct device *dev)
 {
 	int count;
-	struct me_hfs2 hfs2;
+	union me_hfs2 hfs2;
 
 	/* Wait for the mbp_cleared indicator */
 	for (count = ME_RETRY; count > 0; --count) {
-		pci_read_dword_ptr(dev, &hfs2, PCI_ME_HFS2);
+		hfs2.raw = pci_read_config32(dev, PCI_ME_HFS2);
 		if (hfs2.mbp_cleared)
 			break;
 		udelay(ME_DELAY);
@@ -452,7 +431,7 @@ static void intel_me_mbp_clear(struct device *dev)
 	}
 }
 
-static void me_print_fw_version(mbp_fw_version_name *vers_name)
+static void me_print_fw_version(struct mbp_fw_version_name *vers_name)
 {
 	if (!vers_name) {
 		printk(BIOS_ERR, "ME: mbp missing version report\n");
@@ -471,7 +450,7 @@ static inline void print_cap(const char *name, int state)
 }
 
 /* Get ME Firmware Capabilities */
-static int mkhi_get_fwcaps(mbp_mefwcaps *cap)
+static int mkhi_get_fwcaps(struct mbp_mefwcaps *cap)
 {
 	u32 rule_id = 0;
 	struct me_fwcaps cap_msg;
@@ -491,9 +470,9 @@ static int mkhi_get_fwcaps(mbp_mefwcaps *cap)
 }
 
 /* Get ME Firmware Capabilities */
-static void me_print_fwcaps(mbp_mefwcaps *cap)
+static void me_print_fwcaps(struct mbp_mefwcaps *cap)
 {
-	mbp_mefwcaps local_caps;
+	struct mbp_mefwcaps local_caps;
 	if (!cap) {
 		cap = &local_caps;
 		printk(BIOS_ERR, "ME: mbp missing fwcaps report\n");
@@ -539,7 +518,7 @@ static int mkhi_end_of_post(void)
 
 void intel_me_finalize(struct device *dev)
 {
-	struct me_hfs hfs;
+	union me_hfs hfs;
 	u32 reg32;
 
 	reg32 = pci_read_config32(dev, PCI_BASE_ADDRESS_0);
@@ -553,8 +532,7 @@ void intel_me_finalize(struct device *dev)
 	intel_me_mbp_clear(dev);
 
 	/* Make sure ME is in a mode that expects EOP */
-	reg32 = pci_read_config32(dev, PCI_ME_HFS);
-	memcpy(&hfs, &reg32, sizeof(u32));
+	hfs.raw = pci_read_config32(dev, PCI_ME_HFS);
 
 	/* Abort and leave device alone if not normal mode */
 	if (hfs.fpt_bad ||
@@ -596,17 +574,14 @@ static int me_icc_set_clock_enables(u32 mask)
 }
 
 /* Determine the path that we should take based on ME status */
-static me_bios_path intel_me_path(struct device *dev)
+static enum me_bios_path intel_me_path(struct device *dev)
 {
-	me_bios_path path = ME_DISABLE_BIOS_PATH;
-	struct me_hfs hfs;
-	struct me_hfs2 hfs2;
-
-	pci_read_dword_ptr(dev, &hfs, PCI_ME_HFS);
-	pci_read_dword_ptr(dev, &hfs2, PCI_ME_HFS2);
+	enum me_bios_path path = ME_DISABLE_BIOS_PATH;
+	union me_hfs hfs = { .raw = pci_read_config32(dev, PCI_ME_HFS) };
+	union me_hfs2 hfs2 = { .raw = pci_read_config32(dev, PCI_ME_HFS2) };
 
 	/* Check and dump status */
-	intel_me_status(&hfs, &hfs2);
+	intel_me_status(hfs, hfs2);
 
 	/* Check Current Working State */
 	switch (hfs.working_state) {
@@ -667,7 +642,7 @@ static me_bios_path intel_me_path(struct device *dev)
 static int intel_mei_setup(struct device *dev)
 {
 	struct resource *res;
-	struct mei_csr host;
+	union mei_csr host;
 
 	/* Find the MMIO base for the ME interface */
 	res = probe_resource(dev, PCI_BASE_ADDRESS_0);
@@ -681,11 +656,11 @@ static int intel_mei_setup(struct device *dev)
 	pci_or_config16(dev, PCI_COMMAND, PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY);
 
 	/* Clean up status for next message */
-	read_host_csr(&host);
+	host = read_host_csr();
 	host.interrupt_generate = 1;
 	host.ready = 1;
 	host.reset = 0;
-	write_host_csr(&host);
+	write_host_csr(host);
 
 	return 0;
 }
@@ -693,11 +668,10 @@ static int intel_mei_setup(struct device *dev)
 /* Read the Extend register hash of ME firmware */
 static int intel_me_extend_valid(struct device *dev)
 {
-	struct me_heres status;
+	union me_heres status = { .raw = pci_read_config32(dev, PCI_ME_HERES) };
 	u32 extend[8] = {0};
 	int i, count = 0;
 
-	pci_read_dword_ptr(dev, &status, PCI_ME_HERES);
 	if (!status.extend_feature_present) {
 		printk(BIOS_ERR, "ME: Extend Feature not present\n");
 		return -1;
@@ -738,8 +712,7 @@ static int intel_me_extend_valid(struct device *dev)
 
 static u32 me_to_host_words_pending(void)
 {
-	struct mei_csr me;
-	read_me_csr(&me);
+	union mei_csr me = read_me_csr();
 	if (!me.ready)
 		return 0;
 	return (me.buffer_write_ptr - me.buffer_read_ptr) &
@@ -747,7 +720,7 @@ static u32 me_to_host_words_pending(void)
 }
 
 struct mbp_payload {
-	mbp_header header;
+	union mbp_header header;
 	u32 data[0];
 };
 
@@ -757,17 +730,15 @@ struct mbp_payload {
  * Return -1 to indicate a problem (give up)
  * Return 0 to indicate success (send LOCK+EOP)
  */
-static int intel_me_read_mbp(me_bios_payload *mbp_data, struct device *dev)
+static int intel_me_read_mbp(struct me_bios_payload *mbp_data, struct device *dev)
 {
-	mbp_header mbp_hdr;
+	union mbp_header mbp_hdr;
 	u32 me2host_pending;
-	struct mei_csr host;
-	struct me_hfs2 hfs2;
+	union mei_csr host;
+	union me_hfs2 hfs2 = { .raw = pci_read_config32(dev, PCI_ME_HFS2) };
 	struct mbp_payload *mbp;
 	int i;
 
-	pci_read_dword_ptr(dev, &hfs2, PCI_ME_HFS2);
-
 	if (!hfs2.mbp_rdy) {
 		printk(BIOS_ERR, "ME: MBP not ready\n");
 		goto mbp_failure;
@@ -780,7 +751,7 @@ static int intel_me_read_mbp(me_bios_payload *mbp_data, struct device *dev)
 	}
 
 	/* we know for sure that at least the header is there */
-	mei_read_dword_ptr(&mbp_hdr, MEI_ME_CB_RW);
+	mbp_hdr.raw = read_cb();
 
 	if ((mbp_hdr.num_entries > (mbp_hdr.mbp_size / 2)) ||
 	    (me2host_pending < mbp_hdr.mbp_size)) {
@@ -799,14 +770,14 @@ static int intel_me_read_mbp(me_bios_payload *mbp_data, struct device *dev)
 
 	i = 0;
 	while (i != me2host_pending) {
-		mei_read_dword_ptr(&mbp->data[i], MEI_ME_CB_RW);
+		mbp->data[i] = read_cb();
 		i++;
 	}
 
 	/* Signal to the ME that the host has finished reading the MBP. */
-	read_host_csr(&host);
+	host = read_host_csr();
 	host.interrupt_generate = 1;
-	write_host_csr(&host);
+	write_host_csr(host);
 
 	/* Dump out the MBP contents. */
 	if (CONFIG_DEFAULT_CONSOLE_LOGLEVEL >= BIOS_DEBUG) {
@@ -826,7 +797,7 @@ static int intel_me_read_mbp(me_bios_payload *mbp_data, struct device *dev)
 		}
 	/* Setup the pointers in the me_bios_payload structure. */
 	for (i = 0; i < mbp->header.mbp_size - 1;) {
-		mbp_item_header *item = (void *)&mbp->data[i];
+		struct mbp_item_header *item = (void *)&mbp->data[i];
 
 		switch (MBP_MAKE_IDENT(item->app_id, item->item_id)) {
 		case MBP_IDENT(KERNEL, FW_VER):
@@ -879,8 +850,8 @@ mbp_failure:
 static void intel_me_init(struct device *dev)
 {
 	struct southbridge_intel_lynxpoint_config *config = dev->chip_info;
-	me_bios_path path = intel_me_path(dev);
-	me_bios_payload mbp_data;
+	enum me_bios_path path = intel_me_path(dev);
+	struct me_bios_payload mbp_data;
 
 	/* Do initial setup and determine the BIOS path */
 	printk(BIOS_NOTICE, "ME: BIOS path: %s\n", me_bios_path_values[path]);
@@ -949,13 +920,13 @@ static struct device_operations device_ops = {
 };
 
 static const unsigned short pci_device_ids[] = {
-	PCI_DEVICE_ID_INTEL_LPT_H_MEI,
-	PCI_DEVICE_ID_INTEL_LPT_LP_MEI,
+	PCI_DID_INTEL_LPT_H_MEI,
+	PCI_DID_INTEL_LPT_LP_MEI,
 	0
 };
 
 static const struct pci_driver intel_me __pci_driver = {
 	.ops     = &device_ops,
-	.vendor  = PCI_VENDOR_ID_INTEL,
+	.vendor  = PCI_VID_INTEL,
 	.devices = pci_device_ids,
 };
