@@ -342,14 +342,14 @@ static struct device_operations usb_xhci_ops = {
 };
 
 static const unsigned short pci_device_ids[] = {
-	PCI_DEVICE_ID_INTEL_LPT_H_XHCI,
-	PCI_DEVICE_ID_INTEL_LPT_LP_XHCI,
+	PCI_DID_INTEL_LPT_H_XHCI,
+	PCI_DID_INTEL_LPT_LP_XHCI,
 	0
 };
 
 static const struct pci_driver pch_usb_xhci __pci_driver = {
 	.ops	 = &usb_xhci_ops,
-	.vendor	 = PCI_VENDOR_ID_INTEL,
+	.vendor	 = PCI_VID_INTEL,
 	.devices = pci_device_ids,
 };
 #endif /* !__SIMPLE_DEVICE__ */
