@@ -375,7 +375,7 @@ LibAmdMsrRead (
 VOID
 LibAmdMsrWrite (
   IN       UINT32 MsrAddress,
-  IN       UINT64 *Value,
+  CONST IN       UINT64 *Value,
   IN OUT   AMD_CONFIG_PARAMS *ConfigPtr
   )
 {
