@@ -91,7 +91,7 @@ BOOLEAN
 STATIC
 MemMSetCSRNb (
   IN OUT   MEM_NB_BLOCK *NBPtr,
-  IN       PCI_SPECIAL_CASE *SpecialCases,
+  CONST IN       PCI_SPECIAL_CASE *SpecialCases,
   IN       PCI_ADDR PciAddr,
   IN       UINT32 Value
   );
@@ -471,7 +471,7 @@ BOOLEAN
 STATIC
 MemMSetCSRNb (
   IN OUT   MEM_NB_BLOCK *NBPtr,
-  IN       PCI_SPECIAL_CASE *SpecialCases,
+  CONST IN       PCI_SPECIAL_CASE *SpecialCases,
   IN       PCI_ADDR PciAddr,
   IN       UINT32 Value
   )
