@@ -354,7 +354,7 @@ unsigned int dramc_set_vcore_voltage(unsigned int vcore)
 	return mtk_regulator_set_voltage(&reg_vcore, vcore, MAX_VCORE);
 #elif defined(FOR_COREBOOT)
 	dramc_debug("%s set vcore to %u\n", __func__, vcore);
-	mainboard_set_regulator_vol(MTK_REGULATOR_VCORE, vcore);
+	mainboard_set_regulator_voltage(MTK_REGULATOR_VCORE, vcore);
 	return 0;
 #else
 	return 0;
@@ -366,7 +366,7 @@ unsigned int dramc_get_vcore_voltage(void)
 #ifdef MTK_PMIC_MT6359
 	return mtk_regulator_get_voltage(&reg_vcore);
 #elif defined(FOR_COREBOOT)
-	return mainboard_get_regulator_vol(MTK_REGULATOR_VCORE);
+	return mainboard_get_regulator_voltage(MTK_REGULATOR_VCORE);
 #else
 	return 0;
 #endif
@@ -377,7 +377,7 @@ unsigned int dramc_set_vmdd_voltage(unsigned int ddr_type, unsigned int vdram)
 #ifdef MTK_PMIC_MT6359
 	mtk_regulator_set_voltage(&reg_vdram, vdram, MAX_VDRAM);
 #elif defined(FOR_COREBOOT)
-	mainboard_set_regulator_vol(MTK_REGULATOR_VDD2, vdram);
+	mainboard_set_regulator_voltage(MTK_REGULATOR_VDD2, vdram);
 #endif
 	return 0;
 }
@@ -387,7 +387,7 @@ unsigned int dramc_get_vmdd_voltage(unsigned int ddr_type)
 #ifdef MTK_PMIC_MT6359
 	return mtk_regulator_get_voltage(&reg_vdram);
 #elif defined(FOR_COREBOOT)
-	return mainboard_get_regulator_vol(MTK_REGULATOR_VDD2);
+	return mainboard_get_regulator_voltage(MTK_REGULATOR_VDD2);
 #else
 	return 0;
 #endif
@@ -398,7 +398,7 @@ unsigned int dramc_set_vmddq_voltage(unsigned int ddr_type, unsigned int vddq)
 #ifdef MTK_PMIC_MT6359
 	mtk_regulator_set_voltage(&reg_vddq, vddq, MAX_VDDQ);
 #elif defined(FOR_COREBOOT)
-	mainboard_set_regulator_vol(MTK_REGULATOR_VDDQ, vddq);
+	mainboard_set_regulator_voltage(MTK_REGULATOR_VDDQ, vddq);
 #endif
 	return 0;
 }
@@ -408,7 +408,7 @@ unsigned int dramc_get_vmddq_voltage(unsigned int ddr_type)
 #ifdef MTK_PMIC_MT6359
 	return mtk_regulator_get_voltage(&reg_vddq);
 #elif defined(FOR_COREBOOT)
-	return mainboard_get_regulator_vol(MTK_REGULATOR_VDDQ);
+	return mainboard_get_regulator_voltage(MTK_REGULATOR_VDDQ);
 #else
 	return 0;
 #endif
@@ -419,7 +419,7 @@ unsigned int dramc_set_vmddr_voltage(unsigned int vmddr)
 #ifdef MTK_PMIC_MT6359
 	return mtk_regulator_set_voltage(&reg_vmddr, vmddr, MAX_VMDDR);
 #elif defined(FOR_COREBOOT)
-	mainboard_set_regulator_vol(MTK_REGULATOR_VMDDR, vmddr);
+	mainboard_set_regulator_voltage(MTK_REGULATOR_VMDDR, vmddr);
 	return 0;
 #else
 	return 0;
@@ -431,7 +431,7 @@ unsigned int dramc_get_vmddr_voltage(void)
 #ifdef MTK_PMIC_MT6359
 	return mtk_regulator_get_voltage(&reg_vmddr);
 #elif defined(FOR_COREBOOT)
-	return mainboard_get_regulator_vol(MTK_REGULATOR_VMDDR);
+	return mainboard_get_regulator_voltage(MTK_REGULATOR_VMDDR);
 #else
 	return 0;
 #endif
@@ -442,7 +442,7 @@ unsigned int dramc_set_vio18_voltage(unsigned int vio18)
 #ifdef MTK_PMIC_MT6359
 	return mtk_regulator_set_voltage(&reg_vio18, vio18, MAX_VIO18);
 #elif defined(FOR_COREBOOT)
-	mainboard_set_regulator_vol(MTK_REGULATOR_VDD1, vio18);
+	mainboard_set_regulator_voltage(MTK_REGULATOR_VDD1, vio18);
 	return 0;
 #else
 	return 0;
@@ -455,7 +455,7 @@ unsigned int dramc_get_vio18_voltage(void)
 #ifdef MTK_PMIC_MT6359
 	return mtk_regulator_get_voltage(&reg_vio18);
 #elif defined(FOR_COREBOOT)
-	return mainboard_get_regulator_vol(MTK_REGULATOR_VDD1);
+	return mainboard_get_regulator_voltage(MTK_REGULATOR_VDD1);
 #else
 	return 0;
 #endif
