@@ -33,13 +33,14 @@ static struct typedesc_t filetypes[] unused = {
 	{CBFS_TYPE_CBFSHEADER, "cbfs header"},
 	{CBFS_TYPE_STAGE, "stage"},
 	{CBFS_TYPE_SELF, "simple elf"},
-	{CBFS_TYPE_FIT, "fit"},
+	{CBFS_TYPE_FIT_PAYLOAD, "fit_payload"},
 	{CBFS_TYPE_OPTIONROM, "optionrom"},
 	{CBFS_TYPE_BOOTSPLASH, "bootsplash"},
 	{CBFS_TYPE_RAW, "raw"},
 	{CBFS_TYPE_VSA, "vsa"},
 	{CBFS_TYPE_MBI, "mbi"},
 	{CBFS_TYPE_MICROCODE, "microcode"},
+	{CBFS_TYPE_INTEL_FIT, "intel_fit"},
 	{CBFS_TYPE_FSP, "fsp"},
 	{CBFS_TYPE_MRC, "mrc"},
 	{CBFS_TYPE_CMOS_DEFAULT, "cmos_default"},
@@ -54,7 +55,7 @@ static struct typedesc_t filetypes[] unused = {
 	{0, NULL}
 };
 
-#define CBFS_SUBHEADER(_p) ( (void *) ((((uint8_t *) (_p)) + ntohl((_p)->offset))) )
+#define CBFS_SUBHEADER(_p) ((void *) ((((uint8_t *) (_p)) + be32toh((_p)->offset))))
 
 static inline size_t cbfs_file_attr_hash_size(enum vb2_hash_algorithm algo)
 {
