@@ -10,11 +10,10 @@
 #include <assert.h>
 
 #include <commonlib/bsd/cbfs_serialized.h>
+#include <commonlib/bsd/sysincludes.h>
 #include <commonlib/helpers.h>
 #include <console/console.h>
 
-#include "swab.h"
-
 /*
  * There are two address spaces that this tool deals with - SPI flash address space and host
  * address space. This macros checks if the address is greater than 2GiB under the assumption
