@@ -1,6 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <inttypes.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -17,7 +18,8 @@
 #include <libgen.h>
 #include <assert.h>
 #include <regex.h>
-#include <commonlib/cbmem_id.h>
+#include <commonlib/bsd/cbmem_id.h>
+#include <commonlib/loglevel.h>
 #include <commonlib/timestamp_serialized.h>
 #include <commonlib/tcpa_log_serialized.h>
 #include <commonlib/coreboot_tables.h>
@@ -27,6 +29,10 @@
 #include <sys/sysctl.h>
 #endif
 
+#if defined(__i386__) || defined(__x86_64__)
+#include <x86intrin.h>
+#endif
+
 #define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))
 
 typedef uint8_t u8;
@@ -55,6 +61,9 @@ static int verbose = 0;
 static int mem_fd;
 static struct mapping lbtable_mapping;
 
+/* TSC frequency from the LB_TAG_TSC_INFO record. 0 if not present. */
+static uint32_t tsc_freq_khz = 0;
+
 static void die(const char *msg)
 {
 	if (msg)
@@ -78,9 +87,9 @@ static inline size_t size_to_mib(size_t sz)
 }
 
 /* Return mapping of physical address requested. */
-static const void *mapping_virt(const struct mapping *mapping)
+static void *mapping_virt(const struct mapping *mapping)
 {
-	const char *v = mapping->virt;
+	char *v = mapping->virt;
 
 	if (v == NULL)
 		return NULL;
@@ -89,8 +98,8 @@ static const void *mapping_virt(const struct mapping *mapping)
 }
 
 /* Returns virtual address on success, NULL on error. mapping is filled in. */
-static const void *map_memory(struct mapping *mapping, unsigned long long phys,
-				size_t sz)
+static void *map_memory_with_prot(struct mapping *mapping,
+				  unsigned long long phys, size_t sz, int prot)
 {
 	void *v;
 	unsigned long long page_size;
@@ -112,7 +121,7 @@ static const void *map_memory(struct mapping *mapping, unsigned long long phys,
 			phys);
 	}
 
-	v = mmap(NULL, mapping->virt_size, PROT_READ, MAP_SHARED, mem_fd,
+	v = mmap(NULL, mapping->virt_size, prot, MAP_SHARED, mem_fd,
 			phys - mapping->offset);
 
 	if (v == MAP_FAILED) {
@@ -130,6 +139,14 @@ static const void *map_memory(struct mapping *mapping, unsigned long long phys,
 	return mapping_virt(mapping);
 }
 
+/* Convenience helper for the common case of read-only mappings. */
+static const void *map_memory(struct mapping *mapping, unsigned long long phys,
+			      size_t sz)
+{
+	return map_memory_with_prot(mapping, phys, sz, PROT_READ);
+}
+
+
 /* Returns 0 on success, < 0 on error. mapping is cleared if successful. */
 static int unmap_memory(struct mapping *mapping)
 {
@@ -218,7 +235,7 @@ static int find_cbmem_entry(uint32_t id, uint64_t *addr, size_t *size)
 
 	while (offset < mapping_size(&lbtable_mapping)) {
 		const struct lb_record *lbr;
-		const struct lb_cbmem_entry *lbe;
+		struct lb_cbmem_entry lbe;
 
 		lbr = (const void *)(table + offset);
 		offset += lbr->size;
@@ -226,12 +243,12 @@ static int find_cbmem_entry(uint32_t id, uint64_t *addr, size_t *size)
 		if (lbr->tag != LB_TAG_CBMEM_ENTRY)
 			continue;
 
-		lbe = (const void *)lbr;
-		if (lbe->id != id)
+		aligned_memcpy(&lbe, lbr, sizeof(lbe));
+		if (lbe.id != id)
 			continue;
 
-		*addr = lbe->address;
-		*size = lbe->entry_size;
+		*addr = lbe.address;
+		*size = lbe.entry_size;
 		ret = 0;
 		break;
 	}
@@ -325,6 +342,10 @@ static int parse_cbtable_entries(const struct mapping *table_mapping)
 			    parse_cbmem_ref((struct lb_cbmem_ref *)lbr_p);
 			continue;
 		}
+		case LB_TAG_TSC_INFO:
+			debug("    Found TSC info.\n");
+			tsc_freq_khz = ((struct lb_tsc_info *)lbr_p)->freq_khz;
+			continue;
 		case LB_TAG_FORWARD: {
 			int ret;
 			/*
@@ -527,6 +548,24 @@ static void print_norm(u64 v)
 	}
 }
 
+static uint64_t timestamp_get(uint64_t table_tick_freq_mhz)
+{
+#if defined(__i386__) || defined(__x86_64__)
+	uint64_t tsc = __rdtsc();
+
+	/* No tick frequency specified means raw TSC values. */
+	if (!table_tick_freq_mhz)
+		return tsc;
+
+	if (tsc_freq_khz)
+		return tsc * table_tick_freq_mhz * 1000 / tsc_freq_khz;
+#else
+	(void)table_tick_freq_mhz;
+#endif
+	die("Don't know how to obtain timestamps on this platform.\n");
+	return 0;
+}
+
 static const char *timestamp_name(uint32_t id)
 {
 	for (size_t i = 0; i < ARRAY_SIZE(timestamp_ids); i++) {
@@ -536,6 +575,15 @@ static const char *timestamp_name(uint32_t id)
 	return "<unknown>";
 }
 
+static uint32_t timestamp_enum_name_to_id(const char *name)
+{
+	for (size_t i = 0; i < ARRAY_SIZE(timestamp_ids); i++) {
+		if (!strcmp(timestamp_ids[i].enum_name, name))
+			return timestamp_ids[i].id;
+	}
+	return 0;
+}
+
 static uint64_t timestamp_print_parseable_entry(uint32_t id, uint64_t stamp,
 						uint64_t prev_stamp)
 {
@@ -589,14 +637,72 @@ static int compare_timestamp_entries(const void *a, const void *b)
 	return 0;
 }
 
+static int find_matching_end(struct timestamp_table *sorted_tst_p, uint32_t start, uint32_t end)
+{
+	uint32_t id = sorted_tst_p->entries[start].entry_id;
+	uint32_t possible_match = 0;
+
+	for (uint32_t i = 0; i < ARRAY_SIZE(timestamp_ids); ++i) {
+		if (timestamp_ids[i].id == id) {
+			possible_match = timestamp_ids[i].id_end;
+			break;
+		}
+	}
+
+	/* No match found or timestamp not defined in IDs table */
+	if (!possible_match)
+		return -1;
+
+	for (uint32_t i = start + 1; i < end; i++)
+		if (sorted_tst_p->entries[i].entry_id == possible_match)
+			return i;
+
+	return -1;
+}
+
+static const char *get_timestamp_name(const uint32_t id)
+{
+	for (uint32_t i = 0; i < ARRAY_SIZE(timestamp_ids); i++)
+		if (timestamp_ids[i].id == id)
+			return timestamp_ids[i].enum_name;
+
+	return "UNKNOWN";
+}
+
+struct ts_range_stack {
+	const char *name;
+	const char *end_name;
+	uint32_t end;
+};
+
+static void print_with_path(struct ts_range_stack *range_stack, const int stacklvl,
+			    const uint64_t stamp, const char *last_part)
+{
+	for (int i = 1; i <= stacklvl; ++i) {
+		printf("%s -> %s", range_stack[i].name, range_stack[i].end_name);
+		if (i < stacklvl || last_part)
+			putchar(';');
+	}
+	if (last_part)
+		printf("%s", last_part);
+	printf(" %llu\n", (long long)arch_convert_raw_ts_entry(stamp));
+}
+
+enum timestamps_print_type {
+	TIMESTAMPS_PRINT_NONE,
+	TIMESTAMPS_PRINT_NORMAL,
+	TIMESTAMPS_PRINT_MACHINE_READABLE,
+	TIMESTAMPS_PRINT_STACKED,
+};
+
 /* dump the timestamp table */
-static void dump_timestamps(int mach_readable)
+static void dump_timestamps(enum timestamps_print_type output_type)
 {
 	const struct timestamp_table *tst_p;
 	struct timestamp_table *sorted_tst_p;
 	size_t size;
-	uint64_t prev_stamp;
-	uint64_t total_time;
+	uint64_t prev_stamp = 0;
+	uint64_t total_time = 0;
 	struct mapping timestamp_mapping;
 
 	if (timestamps.tag != LB_TAG_TIMESTAMPS) {
@@ -611,7 +717,7 @@ static void dump_timestamps(int mach_readable)
 
 	timestamp_set_tick_freq(tst_p->tick_freq_mhz);
 
-	if (!mach_readable)
+	if (output_type == TIMESTAMPS_PRINT_NORMAL)
 		printf("%d entries total:\n\n", tst_p->num_entries);
 	size += tst_p->num_entries * sizeof(tst_p->entries[0]);
 
@@ -621,41 +727,81 @@ static void dump_timestamps(int mach_readable)
 	if (!tst_p)
 		die("Unable to map full timestamp table\n");
 
-	/* Report the base time within the table. */
-	prev_stamp = 0;
-	if (mach_readable)
-		timestamp_print_parseable_entry(0,  tst_p->base_time,
-						prev_stamp);
-	else
-		timestamp_print_entry(0,  tst_p->base_time, prev_stamp);
-	prev_stamp = tst_p->base_time;
-
-	sorted_tst_p = malloc(size);
+	sorted_tst_p = malloc(size + sizeof(struct timestamp_entry));
 	if (!sorted_tst_p)
 		die("Failed to allocate memory");
 	aligned_memcpy(sorted_tst_p, tst_p, size);
 
+	/*
+	 * Insert a timestamp to represent the base time (start of coreboot),
+	 * in case we have to rebase for negative timestamps below.
+	 */
+	sorted_tst_p->entries[tst_p->num_entries].entry_id = 0;
+	sorted_tst_p->entries[tst_p->num_entries].entry_stamp = 0;
+	sorted_tst_p->num_entries += 1;
+
 	qsort(&sorted_tst_p->entries[0], sorted_tst_p->num_entries,
 	      sizeof(struct timestamp_entry), compare_timestamp_entries);
 
-	total_time = 0;
+	/*
+	 * If there are negative timestamp entries, rebase all of the
+	 * timestamps to the lowest one in the list.
+	 */
+	if (sorted_tst_p->entries[0].entry_stamp < 0) {
+		sorted_tst_p->base_time = -sorted_tst_p->entries[0].entry_stamp;
+		prev_stamp = 0;
+	} else {
+		prev_stamp = tst_p->base_time;
+	}
+
+	struct ts_range_stack range_stack[20];
+	range_stack[0].end = sorted_tst_p->num_entries;
+	int stacklvl = 0;
+
 	for (uint32_t i = 0; i < sorted_tst_p->num_entries; i++) {
 		uint64_t stamp;
 		const struct timestamp_entry *tse = &sorted_tst_p->entries[i];
 
 		/* Make all timestamps absolute. */
 		stamp = tse->entry_stamp + sorted_tst_p->base_time;
-		if (mach_readable)
-			total_time +=
-				timestamp_print_parseable_entry(tse->entry_id,
-							stamp, prev_stamp);
-		else
-			total_time += timestamp_print_entry(tse->entry_id,
-							stamp, prev_stamp);
+		if (output_type == TIMESTAMPS_PRINT_MACHINE_READABLE) {
+			timestamp_print_parseable_entry(tse->entry_id, stamp, prev_stamp);
+		} else if (output_type == TIMESTAMPS_PRINT_NORMAL) {
+			total_time += timestamp_print_entry(tse->entry_id, stamp, prev_stamp);
+		} else if (output_type == TIMESTAMPS_PRINT_STACKED) {
+			bool end_of_range = false;
+			/* Iterate over stacked entries to pop all ranges, which are closed by
+			   current element. For example, assuming two ranges: (TS_A, TS_C),
+			   (TS_B, TS_C) it will pop all of them instead of just last one. */
+			while (stacklvl > 0 && range_stack[stacklvl].end == i) {
+				end_of_range = true;
+				stacklvl--;
+			}
+
+			int match =
+				find_matching_end(sorted_tst_p, i, range_stack[stacklvl].end);
+			if (match != -1) {
+				const uint64_t match_stamp =
+					sorted_tst_p->entries[match].entry_stamp
+					+ sorted_tst_p->base_time;
+				stacklvl++;
+				assert(stacklvl < (int)ARRAY_SIZE(range_stack));
+				range_stack[stacklvl].name = get_timestamp_name(tse->entry_id);
+				range_stack[stacklvl].end_name = get_timestamp_name(
+					sorted_tst_p->entries[match].entry_id);
+				range_stack[stacklvl].end = match;
+				print_with_path(range_stack, stacklvl, match_stamp - stamp,
+						NULL);
+			} else if (!end_of_range) {
+				print_with_path(range_stack, stacklvl, stamp - prev_stamp,
+						get_timestamp_name(tse->entry_id));
+			}
+			/* else: No match && end_of_range == true */
+		}
 		prev_stamp = stamp;
 	}
 
-	if (!mach_readable) {
+	if (output_type == TIMESTAMPS_PRINT_NORMAL) {
 		printf("\nTotal Time: ");
 		print_norm(total_time);
 		printf("\n");
@@ -665,6 +811,38 @@ static void dump_timestamps(int mach_readable)
 	free(sorted_tst_p);
 }
 
+/* add a timestamp entry */
+static void timestamp_add_now(uint32_t timestamp_id)
+{
+	struct timestamp_table *tst_p;
+	struct mapping timestamp_mapping;
+
+	if (timestamps.tag != LB_TAG_TIMESTAMPS) {
+		die("No timestamps found in coreboot table.\n");
+	}
+
+	tst_p = map_memory_with_prot(&timestamp_mapping, timestamps.cbmem_addr,
+				     timestamps.size, PROT_READ | PROT_WRITE);
+	if (!tst_p)
+		die("Unable to map timestamp table\n");
+
+	/*
+	 * Note that coreboot sizes the cbmem entry in the table according to
+	 * max_entries, so it's OK to just add more entries if there's room.
+	 */
+	if (tst_p->num_entries >= tst_p->max_entries) {
+		die("Not enough space to add timestamp.\n");
+	} else {
+		int64_t time =
+			timestamp_get(tst_p->tick_freq_mhz) - tst_p->base_time;
+		tst_p->entries[tst_p->num_entries].entry_id = timestamp_id;
+		tst_p->entries[tst_p->num_entries].entry_stamp = time;
+		tst_p->num_entries += 1;
+	}
+
+	unmap_memory(&timestamp_mapping);
+}
+
 /* dump the tcpa log table */
 static void dump_tcpa_log(void)
 {
@@ -721,8 +899,31 @@ enum console_print_type {
 	CONSOLE_PRINT_PREVIOUS,
 };
 
+static int parse_loglevel(char *arg, int *print_unknown_logs)
+{
+	if (arg[0] == '+') {
+		*print_unknown_logs = 1;
+		arg++;
+	} else {
+		*print_unknown_logs = 0;
+	}
+
+	char *endptr;
+	int loglevel = strtol(arg, &endptr, 0);
+	if (*endptr == '\0' && loglevel >= BIOS_EMERG && loglevel <= BIOS_LOG_PREFIX_MAX_LEVEL)
+		return loglevel;
+
+	/* Only match first 3 characters so `NOTE` and `NOTICE` both match. */
+	for (int i = BIOS_EMERG; i <= BIOS_LOG_PREFIX_MAX_LEVEL; i++)
+		if (!strncasecmp(arg, bios_log_prefix[i], 3))
+			return i;
+
+	*print_unknown_logs = 1;
+	return BIOS_NEVER;
+}
+
 /* dump the cbmem console */
-static void dump_console(enum console_print_type type)
+static void dump_console(enum console_print_type type, int max_loglevel, int print_unknown_logs)
 {
 	const struct cbmem_console *console_p;
 	char *console_c;
@@ -776,7 +977,8 @@ static void dump_console(enum console_print_type type)
 	/* Slight memory corruption may occur between reboots and give us a few
 	   unprintable characters like '\0'. Replace them with '?' on output. */
 	for (cursor = 0; cursor < size; cursor++)
-		if (!isprint(console_c[cursor]) && !isspace(console_c[cursor]))
+		if (!isprint(console_c[cursor]) && !isspace(console_c[cursor])
+		    && !BIOS_LOG_IS_MARKER(console_c[cursor]))
 			console_c[cursor] = '?';
 
 	/* We detect the reboot cutoff by looking for a bootblock, romstage or
@@ -786,8 +988,8 @@ static void dump_console(enum console_print_type type)
 	cursor = previous = 0;
 	if (type != CONSOLE_PRINT_FULL) {
 #define BANNER_REGEX(stage) \
-		"\n\ncoreboot-[^\n]* " stage " starting.*\\.\\.\\.\n"
-#define OVERFLOW_REGEX(stage) "\n\\*\\*\\* Pre-CBMEM " stage " console overflow"
+		"\n\n.?coreboot-[^\n]* " stage " starting.*\\.\\.\\.\n"
+#define OVERFLOW_REGEX(stage) "\n.?\\*\\*\\* Pre-CBMEM " stage " console overflow"
 		const char *regex[] = { BANNER_REGEX("verstage-before-bootblock"),
 					BANNER_REGEX("bootblock"),
 					BANNER_REGEX("verstage"),
@@ -799,7 +1001,8 @@ static void dump_console(enum console_print_type type)
 		for (size_t i = 0; !cursor && i < ARRAY_SIZE(regex); i++) {
 			regex_t re;
 			regmatch_t match;
-			assert(!regcomp(&re, regex[i], 0));
+			int res = regcomp(&re, regex[i], REG_EXTENDED);
+			assert(res == 0);
 
 			/* Keep looking for matches so we find the last one. */
 			while (!regexec(&re, console_c + cursor, 1, &match, 0)) {
@@ -815,7 +1018,33 @@ static void dump_console(enum console_print_type type)
 		cursor = previous;
 	}
 
-	puts(console_c + cursor);
+	char c;
+	int suppressed = 0;
+	int tty = isatty(fileno(stdout));
+	while ((c = console_c[cursor++])) {
+		if (BIOS_LOG_IS_MARKER(c)) {
+			int lvl = BIOS_LOG_MARKER_TO_LEVEL(c);
+			if (lvl > max_loglevel) {
+				suppressed = 1;
+				continue;
+			}
+			suppressed = 0;
+			if (tty)
+				printf(BIOS_LOG_ESCAPE_PATTERN, bios_log_escape[lvl]);
+			printf(BIOS_LOG_PREFIX_PATTERN, bios_log_prefix[lvl]);
+		} else {
+			if (!suppressed)
+				putchar(c);
+			if (c == '\n') {
+				if (tty && !suppressed)
+					printf(BIOS_LOG_ESCAPE_RESET);
+				suppressed = !print_unknown_logs;
+			}
+		}
+	}
+	if (tty)
+		printf(BIOS_LOG_ESCAPE_RESET);
+
 	free(console_c);
 	unmap_memory(&console_mapping);
 }
@@ -865,7 +1094,7 @@ static void dump_cbmem_hex(void)
 		return;
 	}
 
-	hexdump(unpack_lb64(cbmem.start), unpack_lb64(cbmem.size));
+	hexdump(cbmem.start, cbmem.size);
 }
 
 static void rawdump(uint64_t base, uint64_t size)
@@ -899,7 +1128,7 @@ static void dump_cbmem_raw(unsigned int id)
 
 	while (offset < mapping_size(&lbtable_mapping)) {
 		const struct lb_record *lbr;
-		const struct lb_cbmem_entry *lbe;
+		struct lb_cbmem_entry lbe;
 
 		lbr = (const void *)(table + offset);
 		offset += lbr->size;
@@ -907,11 +1136,11 @@ static void dump_cbmem_raw(unsigned int id)
 		if (lbr->tag != LB_TAG_CBMEM_ENTRY)
 			continue;
 
-		lbe = (const void *)lbr;
-		if (lbe->id == id) {
-			debug("found id for raw dump %0x", lbe->id);
-			base = lbe->address;
-			size = lbe->entry_size;
+		aligned_memcpy(&lbe, lbr, sizeof(lbe));
+		if (lbe.id == id) {
+			debug("found id for raw dump %0x", lbe.id);
+			base = lbe.address;
+			size = lbe.entry_size;
 			break;
 		}
 	}
@@ -982,7 +1211,7 @@ static void dump_cbmem_toc(void)
 
 	while (offset < mapping_size(&lbtable_mapping)) {
 		const struct lb_record *lbr;
-		const struct lb_cbmem_entry *lbe;
+		struct lb_cbmem_entry lbe;
 
 		lbr = (const void *)(table + offset);
 		offset += lbr->size;
@@ -990,8 +1219,8 @@ static void dump_cbmem_toc(void)
 		if (lbr->tag != LB_TAG_CBMEM_ENTRY)
 			continue;
 
-		lbe = (const void *)lbr;
-		cbmem_print_entry(i, lbe->id, lbe->address, lbe->entry_size);
+		aligned_memcpy(&lbe, lbr, sizeof(lbe));
+		cbmem_print_entry(i, lbe.id, lbe.address, lbe.entry_size);
 		i++;
 	}
 }
@@ -1101,12 +1330,15 @@ static void print_usage(const char *name, int exit_code)
 	     "   -c | --console:                   print cbmem console\n"
 	     "   -1 | --oneboot:                   print cbmem console for last boot only\n"
 	     "   -2 | --2ndtolast:                 print cbmem console for the boot that came before the last one only\n"
+	     "   -B | --loglevel:                  maximum loglevel to print; prefix `+` (e.g. -B +INFO) to also print lines that have no level\n"
 	     "   -C | --coverage:                  dump coverage information\n"
 	     "   -l | --list:                      print cbmem table of contents\n"
 	     "   -x | --hexdump:                   print hexdump of cbmem area\n"
 	     "   -r | --rawdump ID:                print rawdump of specific ID (in hex) of cbtable\n"
 	     "   -t | --timestamps:                print timestamp information\n"
 	     "   -T | --parseable-timestamps:      print parseable timestamps\n"
+	     "   -S | --stacked-timestamps:        print stacked timestamps (e.g. for flame graph tools)\n"
+	     "   -a | --add-timestamp ID:          append timestamp with ID\n"
 	     "   -L | --tcpa-log                   print TCPA log\n"
 	     "   -V | --verbose:                   verbose (debugging) output\n"
 	     "   -v | --version:                   print the version\n"
@@ -1238,22 +1470,27 @@ int main(int argc, char** argv)
 	int print_list = 0;
 	int print_hexdump = 0;
 	int print_rawdump = 0;
-	int print_timestamps = 0;
 	int print_tcpa_log = 0;
-	int machine_readable_timestamps = 0;
+	enum timestamps_print_type timestamp_type = TIMESTAMPS_PRINT_NONE;
 	enum console_print_type console_type = CONSOLE_PRINT_FULL;
 	unsigned int rawdump_id = 0;
+	int max_loglevel = BIOS_NEVER;
+	int print_unknown_logs = 1;
+	uint32_t timestamp_id = 0;
 
 	int opt, option_index = 0;
 	static struct option long_options[] = {
 		{"console", 0, 0, 'c'},
 		{"oneboot", 0, 0, '1'},
 		{"2ndtolast", 0, 0, '2'},
+		{"loglevel", required_argument, 0, 'B'},
 		{"coverage", 0, 0, 'C'},
 		{"list", 0, 0, 'l'},
 		{"tcpa-log", 0, 0, 'L'},
 		{"timestamps", 0, 0, 't'},
 		{"parseable-timestamps", 0, 0, 'T'},
+		{"stacked-timestamps", 0, 0, 'S'},
+		{"add-timestamp", required_argument, 0, 'a'},
 		{"hexdump", 0, 0, 'x'},
 		{"rawdump", required_argument, 0, 'r'},
 		{"verbose", 0, 0, 'V'},
@@ -1261,7 +1498,7 @@ int main(int argc, char** argv)
 		{"help", 0, 0, 'h'},
 		{0, 0, 0, 0}
 	};
-	while ((opt = getopt_long(argc, argv, "c12CltTLxVvh?r:",
+	while ((opt = getopt_long(argc, argv, "c12B:CltTSa:LxVvh?r:",
 				  long_options, &option_index)) != EOF) {
 		switch (opt) {
 		case 'c':
@@ -1278,6 +1515,9 @@ int main(int argc, char** argv)
 			console_type = CONSOLE_PRINT_PREVIOUS;
 			print_defaults = 0;
 			break;
+		case 'B':
+			max_loglevel = parse_loglevel(optarg, &print_unknown_logs);
+			break;
 		case 'C':
 			print_coverage = 1;
 			print_defaults = 0;
@@ -1300,14 +1540,24 @@ int main(int argc, char** argv)
 			rawdump_id = strtoul(optarg, NULL, 16);
 			break;
 		case 't':
-			print_timestamps = 1;
+			timestamp_type = TIMESTAMPS_PRINT_NORMAL;
 			print_defaults = 0;
 			break;
 		case 'T':
-			print_timestamps = 1;
-			machine_readable_timestamps = 1;
+			timestamp_type = TIMESTAMPS_PRINT_MACHINE_READABLE;
 			print_defaults = 0;
 			break;
+		case 'S':
+			timestamp_type = TIMESTAMPS_PRINT_STACKED;
+			print_defaults = 0;
+			break;
+		case 'a':
+			print_defaults = 0;
+			timestamp_id = timestamp_enum_name_to_id(optarg);
+			/* Parse numeric value if name is unknown */
+			if (timestamp_id == 0)
+				timestamp_id = strtoul(optarg, NULL, 0);
+			break;
 		case 'V':
 			verbose = 1;
 			break;
@@ -1330,7 +1580,7 @@ int main(int argc, char** argv)
 		print_usage(argv[0], 1);
 	}
 
-	mem_fd = open("/dev/mem", O_RDONLY, 0);
+	mem_fd = open("/dev/mem", timestamp_id ? O_RDWR : O_RDONLY, 0);
 	if (mem_fd < 0) {
 		fprintf(stderr, "Failed to gain memory access: %s\n",
 			strerror(errno));
@@ -1403,7 +1653,7 @@ int main(int argc, char** argv)
 		die("Table not found.\n");
 
 	if (print_console)
-		dump_console(console_type);
+		dump_console(console_type, max_loglevel, print_unknown_logs);
 
 	if (print_coverage)
 		dump_coverage();
@@ -1417,8 +1667,14 @@ int main(int argc, char** argv)
 	if (print_rawdump)
 		dump_cbmem_raw(rawdump_id);
 
-	if (print_defaults || print_timestamps)
-		dump_timestamps(machine_readable_timestamps);
+	if (timestamp_id)
+		timestamp_add_now(timestamp_id);
+
+	if (print_defaults)
+		timestamp_type = TIMESTAMPS_PRINT_NORMAL;
+
+	if (timestamp_type != TIMESTAMPS_PRINT_NONE)
+		dump_timestamps(timestamp_type);
 
 	if (print_tcpa_log)
 		dump_tcpa_log();
