@@ -76,7 +76,8 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_ICH9ME		0x2917
 #define PCI_DEVICE_ID_INTEL_ICH10DO		0x3a14
 #define PCI_DEVICE_ID_INTEL_ICH10R		0x3a16
-#define PCI_DEVICE_ID_INTEL_ICH10               0x3a18
+#define PCI_DEVICE_ID_INTEL_ICH10		0x3a18
+#define PCI_DEVICE_ID_INTEL_ICH10D		0x3a1a
 #define PCI_DEVICE_ID_INTEL_3400_DESKTOP	0x3b00
 #define PCI_DEVICE_ID_INTEL_3400_MOBILE		0x3b01
 #define PCI_DEVICE_ID_INTEL_P55			0x3b02
@@ -148,6 +149,11 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_CANNONPOINT_LP_U_PREM	0x9d84
 #define PCI_DEVICE_ID_INTEL_COMETPOINT_LP_U_PREM	0x0284
 #define PCI_DEVICE_ID_INTEL_COMETPOINT_LP_U_BASE	0x0285
+#define PCI_DEVICE_ID_INTEL_TIGERPOINT_U_SUPER	0xa081
+#define PCI_DEVICE_ID_INTEL_TIGERPOINT_U_PREM	0xa082
+#define PCI_DEVICE_ID_INTEL_TIGERPOINT_U_BASE	0xa083
+#define PCI_DEVICE_ID_INTEL_TIGERPOINT_Y_SUPER	0xa086
+#define PCI_DEVICE_ID_INTEL_TIGERPOINT_Y_PREM	0xa087
 #define PCI_DEVICE_ID_INTEL_H110		0xa143
 #define PCI_DEVICE_ID_INTEL_H170		0xa144
 #define PCI_DEVICE_ID_INTEL_Z170		0xa145
@@ -205,6 +211,30 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_HM370		0xa30d
 #define PCI_DEVICE_ID_INTEL_CM246		0xa30e
 
+#define PCI_DEVICE_ID_INTEL_Q570		0x4384
+#define PCI_DEVICE_ID_INTEL_Z590		0x4385
+#define PCI_DEVICE_ID_INTEL_H570		0x4386
+#define PCI_DEVICE_ID_INTEL_B560		0x4387
+#define PCI_DEVICE_ID_INTEL_H510		0x4388
+#define PCI_DEVICE_ID_INTEL_WM590		0x4389
+#define PCI_DEVICE_ID_INTEL_QM580		0x438a
+#define PCI_DEVICE_ID_INTEL_HM570		0x438b
+#define PCI_DEVICE_ID_INTEL_C252		0x438c
+#define PCI_DEVICE_ID_INTEL_C256		0x438d
+#define PCI_DEVICE_ID_INTEL_W580		0x438f
+
+#define PCI_DEVICE_ID_INTEL_H610		0x7a87
+#define PCI_DEVICE_ID_INTEL_B660		0x7a86
+#define PCI_DEVICE_ID_INTEL_H670		0x7a85
+#define PCI_DEVICE_ID_INTEL_Q670		0x7a83
+#define PCI_DEVICE_ID_INTEL_Z690		0x7a84
+#define PCI_DEVICE_ID_INTEL_W680		0x7a88
+#define PCI_DEVICE_ID_INTEL_W685		0x7a8a
+#define PCI_DEVICE_ID_INTEL_WM690		0x7a8d
+#define PCI_DEVICE_ID_INTEL_HM670		0x7a8c
+#define PCI_DEVICE_ID_INTEL_WM790		0x7a0d
+#define PCI_DEVICE_ID_INTEL_HM770		0x7a0c
+
 #define PCI_DEVICE_ID_INTEL_82810		0x7120
 #define PCI_DEVICE_ID_INTEL_82810_DC	0x7122
 #define PCI_DEVICE_ID_INTEL_82810E_DC	0x7124
@@ -247,6 +277,12 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_C226		0x8c56
 #define PCI_DEVICE_ID_INTEL_H81			0x8c5c
 
+#define PCI_DEVICE_ID_INTEL_C9_MOBILE		0x8cc1
+#define PCI_DEVICE_ID_INTEL_C9_DESKTOP		0x8cc2
+#define PCI_DEVICE_ID_INTEL_HM97		0x8cc3
+#define PCI_DEVICE_ID_INTEL_Z97			0x8cc4
+#define PCI_DEVICE_ID_INTEL_H97			0x8cc6
+
 #define PCI_DEVICE_ID_INTEL_82X58		0x3405
 #define PCI_DEVICE_ID_INTEL_SCH_POULSBO	0x8100
 #define PCI_DEVICE_ID_INTEL_ATOM_DXXX	0xa000
@@ -257,6 +293,10 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_I5000V		0x25d4
 #define PCI_DEVICE_ID_INTEL_I5000P		0x25d8
 
+#define PCI_DEVICE_ID_INTEL_ADL_P		0x5182
+#define PCI_DEVICE_ID_INTEL_ADL_M		0x5187
+#define PCI_DEVICE_ID_INTEL_RPL_P		0x519d
+
 /* untested, but almost identical to D-series */
 #define PCI_DEVICE_ID_INTEL_ATOM_NXXX	0xa010
 
@@ -276,6 +316,7 @@ static inline uint32_t inl(unsigned port)
 
 #define PCI_DEVICE_ID_INTEL_APL_LPC		0x5ae8
 #define PCI_DEVICE_ID_INTEL_DNV_LPC		0x19dc
+#define PCI_DEVICE_ID_INTEL_GLK_LPC		0x31E8
 
 /* Intel starts counting these generations with the integration of the DRAM controller */
 #define PCI_DEVICE_ID_INTEL_CORE_0TH_GEN	0xd132 /* Nehalem */
@@ -292,6 +333,8 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_CORE_4TH_GEN_E3	0x0c08 /* Haswell (Xeon E3 v3) */
 #define PCI_DEVICE_ID_INTEL_CORE_4TH_GEN_U	0x0a04 /* Haswell-ULT */
 #define PCI_DEVICE_ID_INTEL_CORE_5TH_GEN_U	0x1604 /* Broadwell-ULT */
+#define PCI_DEVICE_ID_INTEL_CORE_5TH_GEN_D	0x1610 /* Broadwell (Desktop) */
+#define PCI_DEVICE_ID_INTEL_CORE_5TH_GEN_M	0x1614 /* Broadwell (Mobile) */
 #define PCI_DEVICE_ID_INTEL_CORE_6TH_GEN_D2	0x190f /* Skylake (Desktop) */
 #define PCI_DEVICE_ID_INTEL_CORE_6TH_GEN_U	0x1904 /* Skylake (Mobile) */
 #define PCI_DEVICE_ID_INTEL_CORE_6TH_GEN_Y	0x190c /* Skylake (Mobile) */
@@ -309,9 +352,26 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_CORE_CML_U1		0x9b51 /* Cometlake U (Mobile) */
 #define PCI_DEVICE_ID_INTEL_CORE_CML_U2		0x9b61 /* Cometlake U (Mobile) */
 #define PCI_DEVICE_ID_INTEL_CORE_CML_U3		0x9b71 /* Cometlake U (Mobile) */
+#define PCI_DEVICE_ID_INTEL_CORE_TGL_ID_U_2	0x9a04 /* Tigerlake UP3 2 Cores */
+#define PCI_DEVICE_ID_INTEL_CORE_TGL_ID_U_4	0x9a14 /* Tigerlake UP3 4 Cores */
+#define PCI_DEVICE_ID_INTEL_CORE_TGL_ID_Y_2	0x9a02 /* Tigerlake UP4 2 Cores */
+#define PCI_DEVICE_ID_INTEL_CORE_TGL_ID_Y_4	0x9a12 /* Tigerlake UP4 4 Cores */
+#define PCI_DEVICE_ID_INTEL_CORE_TGL_ID_H_4	0x9a16 /* Tigerlake H 4 Cores */
+#define PCI_DEVICE_ID_INTEL_CORE_TGL_ID_H_6	0x9a26 /* Tigerlake H 6 Cores */
+#define PCI_DEVICE_ID_INTEL_CORE_TGL_ID_H_8	0x9a36 /* Tigerlake H 8 Cores */
 #define PCI_DEVICE_ID_INTEL_HEWITTLAKE		0x6f00 /* Hewitt Lake */
 #define PCI_DEVICE_ID_INTEL_SAPPHIRERAPIDS_SP	0x09a2 /* Sapphire Rapids SP */
-
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_S_8_8	0x4660 /* Alderlake S LGA 8+8 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_S_8_4	0x4668 /* Alderlake S LGA 8+4 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_S_6_4	0x4648 /* Alderlake S LGA 6+4 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_S_8_0	0x4670 /* Alderlake S LGA 8+0 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_S_6_0	0x4650 /* Alderlake S LGA 6+0 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_HX_8_8	0x4637 /* Alderlake HX 8+8 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_HX_6_8	0x463B /* Alderlake HX 6+8 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_HX_4_8	0x4623 /* Alderlake HX 4+8 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_P_6_8	0x4641 /* Alderlake P 6+8 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_P_4_8	0x4621 /* Alderlake P 4+8 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_P_2_8	0x4601 /* Alderlake P 2+8 */
 
 /* Intel GPUs */
 #define PCI_DEVICE_ID_INTEL_G35_EXPRESS		0x2982
@@ -352,6 +412,7 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_HD_4600		0x0412
 #define PCI_DEVICE_ID_INTEL_HD_4600_1		0x0416
 #define PCI_DEVICE_ID_INTEL_HD_4400		0x041E
+#define PCI_DEVICE_ID_INTEL_HD_4400_1		0x0A16
 #define PCI_DEVICE_ID_INTEL_HD_5000		0x0422
 #define PCI_DEVICE_ID_INTEL_HD_5000_1		0x0426
 #define PCI_DEVICE_ID_INTEL_HD_5000_2		0x042A
@@ -381,6 +442,16 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_IRIS_PLUS_655	0x3EA5
 #define PCI_DEVICE_ID_INTEL_IRIS_PLUS_G7	0x8A52
 #define PCI_DEVICE_ID_INTEL_UHD_GRAPHICS	0x9b41
+#define PCI_DEVICE_ID_INTEL_TGL_GT2_Y		0x9A40
+#define PCI_DEVICE_ID_INTEL_TGL_GT2_UY		0x9A49
+#define PCI_DEVICE_ID_INTEL_TGL_GT1		0x9A60
+#define PCI_DEVICE_ID_INTEL_TGL_GT1_2		0x9A68
+#define PCI_DEVICE_ID_INTEL_TGL_GT2_ULT_1	0x9A78
+#define PCI_DEVICE_ID_INTEL_TGL_GT2_ULT_2	0x9A70
+#define PCI_DEVICE_ID_INTEL_ADL_S_GT1		0x4680
+#define PCI_DEVICE_ID_INTEL_ADL_S_GT1_2		0x4682
+#define PCI_DEVICE_ID_INTEL_ADL_S_GT1_3		0x4690
+#define PCI_DEVICE_ID_INTEL_ADL_S_GT1_4		0x4692
 
 #if !defined(__DARWIN__) && !defined(__FreeBSD__)
 typedef struct { uint32_t hi, lo; } msr_t;
